Package: RNeXML
Type: Package
Title: Semantically Rich I/O for the 'NeXML' Format
Version: 2.4.11
Authors@R: c(person("Carl", "Boettiger", role = c("cre", "aut"),
    email="cboettig@gmail.com", comment = c(ORCID="0000-0002-1642-628X")), 
    person("Scott", "Chamberlain", role ="aut", comment = c(ORCID="0000-0003-1444-9135")), 
    person("Hilmar", "Lapp", role = "aut", comment = c(ORCID="0000-0001-9107-0714")), 
    person("Kseniia","Shumelchyk", role = "aut"), 
    person("Rutger", "Vos", role = "aut", comment = c(ORCID="0000-0001-9254-7318")))
Description: Provides access to phyloinformatic data in 'NeXML' format.  The
    package should add new functionality to R such as the possibility to
    manipulate 'NeXML' objects in more various and refined way and compatibility
    with 'ape' objects.
URL: https://docs.ropensci.org/RNeXML/,
        https://github.com/ropensci/RNeXML
BugReports: https://github.com/ropensci/RNeXML/issues
License: BSD_3_clause + file LICENSE
VignetteBuilder: knitr
Suggests: spelling, rdflib, geiger (>= 2.0), phytools (>= 0.3.93),
        knitr (>= 1.5), rfigshare (>= 0.3.0), knitcitations (>= 1.0.1),
        testthat (>= 0.10.0), rmarkdown (>= 0.3.3), xslt, covr,
        taxalight, progress
Depends: R (>= 3.0.0), ape (>= 3.1), methods (>= 3.0.0)
Imports: XML (>= 3.95), plyr (>= 1.8), reshape2 (>= 1.2.2), httr (>=
        0.3), uuid (>= 0.1-1), dplyr (>= 0.7.0), tidyr (>= 0.3.1),
        stringr (>= 1.0), stringi, xml2, rlang
Collate: 'S4-utils.R' 'classes.R' 'add_basic_meta.R' 'add_characters.R'
        'add_meta.R' 'add_namespaces.R' 'nexmlTree.R' 'add_trees.R'
        'character_classes.R' 'concatenate_nexml.R' 'constructors.R'
        'deprecated.R' 'get_basic_metadata.R' 'get_characters.R'
        'get_level.R' 'get_metadata.R' 'get_namespaces.R' 'get_rdf.R'
        'get_taxa.R' 'get_taxa_meta.R' 'get_trees.R'
        'internal_get_node_maps.R' 'internal_isEmpty.R'
        'internal_name_by_id.R' 'internal_nexml_id.R' 'meta.R'
        'nexml_add.R' 'nexml_get.R' 'nexml_methods.R' 'nexml_publish.R'
        'nexml_read.R' 'nexml_validate.R' 'nexml_write.R'
        'prefixed-uris.R' 'simmap.R' 'taxize_nexml.R' 'tbl_df.R'
        'utils.R'
RoxygenNote: 7.2.3
X-schema.org-applicationCategory: Data Publication
X-schema.org-keywords: metadata, nexml, phylogenetics, linked-data
X-schema.org-isPartOf: https://ropensci.org
Language: en-US
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2023-02-01 16:56:11 UTC; cboettig
Author: Carl Boettiger [cre, aut] (<https://orcid.org/0000-0002-1642-628X>),
  Scott Chamberlain [aut] (<https://orcid.org/0000-0003-1444-9135>),
  Hilmar Lapp [aut] (<https://orcid.org/0000-0001-9107-0714>),
  Kseniia Shumelchyk [aut],
  Rutger Vos [aut] (<https://orcid.org/0000-0001-9254-7318>)
Maintainer: Carl Boettiger <cboettig@gmail.com>
Repository: CRAN
Date/Publication: 2023-02-01 17:50:07 UTC
Built: R 4.6.0; ; 2026-04-29 09:52:42 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(nexml_read,XMLInternalDocument)
S3method(nexml_read,XMLInternalNode)
S3method(nexml_read,character)
export(add_basic_meta)
export(add_characters)
export(add_meta)
export(add_namespaces)
export(add_trees)
export(expand_prefix)
export(flatten_multiphylo)
export(get_all_meta)
export(get_characters)
export(get_characters_list)
export(get_citation)
export(get_flat_trees)
export(get_level)
export(get_license)
export(get_meta)
export(get_metadata)
export(get_metadata_values)
export(get_namespaces)
export(get_otu)
export(get_otus_list)
export(get_rdf)
export(get_taxa)
export(get_taxa_list)
export(get_trees)
export(get_trees_list)
export(meta)
export(nexml)
export(nexml.cell)
export(nexml.char)
export(nexml.characters)
export(nexml.edge)
export(nexml.format)
export(nexml.matrix)
export(nexml.member)
export(nexml.node)
export(nexml.otu)
export(nexml.otus)
export(nexml.polymorphic_states)
export(nexml.row)
export(nexml.seq)
export(nexml.state)
export(nexml.states)
export(nexml.tree)
export(nexml.trees)
export(nexml.uncertain_state)
export(nexml.uncertain_states)
export(nexml_add)
export(nexml_figshare)
export(nexml_get)
export(nexml_publish)
export(nexml_read)
export(nexml_to_simmap)
export(nexml_validate)
export(nexml_write)
export(read.nexml)
export(reset_id_counter)
export(simmap_to_nexml)
export(taxize_nexml)
export(write.nexml)
exportClasses(nexml)
exportMethods("slot<-")
exportMethods(slot)
exportMethods(summary)
import(XML)
import(ape)
import(httr)
import(methods)
import(reshape2)
import(uuid)
importClassesFrom(methods,MethodWithNext)
importFrom(dplyr,"%>%")
importFrom(dplyr,any_of)
importFrom(dplyr,bind_rows)
importFrom(dplyr,coalesce)
importFrom(dplyr,left_join)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_at)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(methods,addNextMethod)
importFrom(methods,cacheMethod)
importFrom(methods,className)
importFrom(methods,getClassDef)
importFrom(methods,getGeneric)
importFrom(methods,new)
importFrom(methods,packageSlot)
importFrom(methods,selectMethod)
importFrom(plyr,arrange)
importFrom(plyr,compact)
importFrom(rlang,":=")
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(stringi,stri_match_first_regex)
importFrom(stringi,stri_replace_first_regex)
importFrom(stringr,str_replace)
importFrom(tidyr,spread)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,object.size)
importFrom(utils,packageName)
importFrom(uuid,UUIDgenerate)
importFrom(xml2,read_xml)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# RNeXML <img src="man/figures/logo.svg" align="right" alt="" width="120" />

[![DOI](https://zenodo.org/badge/11856817.svg)](https://zenodo.org/badge/latestdoi/11856817)
[![Build
Status](https://api.travis-ci.org/ropensci/RNeXML.png)](https://travis-ci.org/ropensci/RNeXML)
[![Windows build
status](https://ci.appveyor.com/api/projects/status/dhiwp5blx2ns2yba/branch/master?svg=true)](https://ci.appveyor.com/project/cboettig/rnexml/branch/master)
[![CRAN
status](https://www.r-pkg.org/badges/version/RNeXML)](https://cran.r-project.org/package=RNeXML)
[![codecov.io](https://codecov.io/github/ropensci/RNeXML/coverage.svg?branch=master)](https://codecov.io/github/ropensci/RNeXML?branch=master)
![downloads](http://cranlogs.r-pkg.org/badges/grand-total/RNeXML)

  - Maintainer: Carl Boettiger
  - Authors: Carl Boettiger, Scott Chamberlain, Hilmar Lapp, Kseniia
    Shumelchyk, Rutger Vos
  - License: BSD-3
  - [Issues](https://github.com/ropensci/RNeXML/issues): Bug reports,
    feature requests, and development discussion.

An extensive and rapidly growing collection of richly annotated
phylogenetics data is now available in the NeXML format. NeXML relies on
state-of-the-art data exchange technology to provide a format that can
be both validated and extended, providing a data quality assurance and
adaptability to the future that is lacking in other formats. See [Vos et
al
2012](https://doi.org/10.1093/sysbio/sys025 "NeXML: Rich, Extensible, and Verifiable Representation of Comparative Data and Metadata.")
for further details on the NeXML format.

## How to cite

RNeXML has been published in the following article:

> Boettiger C, Chamberlain S, Vos R and Lapp H (2016). “RNeXML: A
> Package for Reading and Writing Richly Annotated Phylogenetic,
> Character, and Trait Data in R.” *Methods in Ecology and Evolution*,
> **7**, pp. 352-357.
> doi:10.1111/2041-210X.12469

Although the published version of the article is pay-walled, the source
of the manuscript, and a much better rendered PDF, are included in this
package (in the `manuscripts` folder). You can also find it [freely
available on arXiv](https://arxiv.org/abs/1506.02722).

## Installation

The latest stable release of RNeXML is on CRAN, and can be installed
with the usual `install.packages("RNeXML")` command. Some of the more
specialized functionality described in the Vignettes (such as RDF
manipulation) requires additional packages which can be installed using:

``` r
install.packages("RNeXML", deps = TRUE)
```

The development version can be installed using:

``` r
remotes::install_github("ropensci/RNeXML")
```

## Getting Started

See the vignettes below for both a general quick start and an overview
of more specialized features.

  - [A Brief Introduction to
    RNeXML](https://docs.ropensci.org/RNeXML/articles/intro)
  - [RNeXML: A Package for Reading and Writing Richly Annotated
    Phylogenetic, Character, and Trait Data in
    R](https://github.com/ropensci/RNeXML/tree/master/manuscripts)
    (published in MEE).
  - [Handling Metadata in
    RNeXML](https://docs.ropensci.org/RNeXML/articles/metadata)
  - [The `nexml` S4
    Object](https://docs.ropensci.org/RNeXML/articles/S4)
  - [Semantic data & SPARQL with
    RNeXML](https://docs.ropensci.org/RNeXML/articles/sparql)
  - [Extending NeXML: an example based on
    simmap](https://docs.ropensci.org/RNeXML/articles/simmap)

-----

[![ropensci
footer](https://ropensci.org//public_images/github_footer.png)](https://ropensci.org/)

This directory and its subfolders contains the latest version of the nexml schema
from the subversion repository. The schema is designed following the 'Venetian
Blinds' design pattern. With this design approach we disassemble the problem 
into individual components. Instead of creating element declarations, we create type 
definitions.

This design has:

    * maximized reuse
    * maximized the potential to hide (localize) namespaces.

The Venetian Blind design espouses these guidelines:

    * Design your schema to maximize the potential for hiding (localizing) namespace complexities.
    * Use elementFormDefault to act as a switch for controlling namespace exposure - if you want element namespaces exposed in instance documents, simply turn the elementFormDefault switch to "on" (i.e, set elementFormDefault= "qualified"); if you don't want element namespaces exposed in instance documents, simply turn the elementFormDefault switch to "off" (i.e., set elementFormDefault="unqualified").
    * Design your schema to maximize reuse.
    * Use type definitions as the main form of component reuse.
    * Nest element declarations within type definitions.
    
For more about this (and other) schema design patterns and best practices, visit
http://www.xfront.com/GlobalVersusLocal.html#ThirdDesign

library(xslt)
library(xml2)
library(jsonld)
library(rdflib)
library(magrittr)

## Apply stylesheet to map NeXML RDFa -> RDF
## Does not map the full NeXML data
download.file("https://raw.githubusercontent.com/ropensci/RNeXML/master/inst/examples/RDFa2RDFXML.xsl", "toRDF.xsl")
download.file("https://raw.githubusercontent.com/ropensci/RNeXML/master/inst/examples/multitrees.xml", "example.xml")
rdf <- xslt::xml_xslt(read_xml("example.xml"), read_xml("toRDF.xsl"))

## Save RDF to file
write_xml(rdf, "rdf.xml")

rdf_parse("rdf.xml")

## JSON-LD RDF parser only speaks nquads, so use rdflib to convert:
rdflib::rdf_serialize(rdf_parse("rdf.xml"), "example.nquads", "nquads")
jsonld::jsonld_from_rdf("example.nquads")

## ----supplement-compile-settings, include=FALSE--------------------------
library("methods")
library("knitr")
opts_chunk$set(tidy = FALSE, warning = FALSE, message = FALSE, 
               cache = FALSE, comment = NA, verbose = TRUE, eval=require("rrdf"))
basename <- 'sparql' 



## ----include=FALSE-------------------------------------------------------
#  library("RNeXML")

## ------------------------------------------------------------------------
#  library("rrdf")
#  library("XML")
#  library("phytools")
#  library("RNeXML")

## ------------------------------------------------------------------------
#  nexml <- nexml_read(system.file("examples/primates.xml", package="RNeXML"))

## ------------------------------------------------------------------------
#  rdf <- get_rdf(system.file("examples/primates.xml", package="RNeXML"))
#  tmp <- tempfile()  # so we must write the XML out first
#  saveXML(rdf, tmp)
#  graph <- load.rdf(tmp)

## ------------------------------------------------------------------------
#  root <- sparql.rdf(graph,
#  "SELECT ?uri WHERE {
#      ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#rank> <http://rs.tdwg.org/ontology/voc/TaxonRank#Order> .
#      ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> ?uri
#  }")

## ------------------------------------------------------------------------
#  get_name <- function(id) {
#    max <- length(nexml@otus[[1]]@otu)
#    for(i in 1:max) {
#      if ( nexml@otus[[1]]@otu[[i]]@id == id ) {
#        label <- nexml@otus[[1]]@otu[[i]]@label
#        label <- gsub(" ","_",label)
#        return(label)
#      }
#    }
#  }

## ------------------------------------------------------------------------
#  recurse <- function(node){
#  
#      # fetch the taxonomic rank and id string
#      rank_query <- paste0(
#          "SELECT ?rank ?id WHERE {
#              ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> <",node,"> .
#              ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#rank> ?rank
#            }")
#      result <- sparql.rdf(graph, rank_query)
#  
#      # get the local ID, strip URI part
#      id <- result[2]
#      id <- gsub("^.+#", "", id, perl = TRUE)
#  
#      # if rank is terminal, return the name
#      if (result[1] == "http://rs.tdwg.org/ontology/voc/TaxonRank#Species") {
#          return(get_name(id))
#      }
#  
#      # recurse deeper
#      else {
#          child_query <- paste0(
#              "SELECT ?uri WHERE {
#                  ?id <http://www.w3.org/2000/01/rdf-schema#subClassOf> <",node,"> .
#                  ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> ?uri
#              }")
#          children <- sparql.rdf(graph, child_query)
#  
#          return(paste("(",
#                       paste(sapply(children, recurse),
#                             sep = ",", collapse = "," ),
#                       ")",
#                       get_name(id), # label interior nodes
#                       sep = "", collapse = ""))
#      }
#  }
#  

## ------------------------------------------------------------------------
#  newick <- paste(recurse(root), ";", sep = "", collapse = "")
#  tree <- read.newick(text = newick)
#  collapsed <- collapse.singles(tree)
#  plot(collapsed,
#       type='cladogram',
#       show.tip.label=FALSE,
#       show.node.label=TRUE,
#       cex=0.75,
#       edge.color='grey60',
#       label.offset=-9)

#ONLINE_VALIDATOR <- "http://162.13.187.155/nexml/phylows/validator"
#CANONICAL_SCHEMA <- "http://162.13.187.155/nexml/xsd/nexml.xsd"
ONLINE_VALIDATOR <- "http://www.nexml.org/nexml/phylows/validator"
CANONICAL_SCHEMA <- "http://www.nexml.org/2009/nexml.xsd"
CANONICAL_SCHEMA <-  "https://github.com/nexml/nexml/raw/master/xsd/nexml.xsd"
#' validate nexml using the online validator tool
#' @param file path to the nexml file to validate
#' @param schema URL of schema (for fallback method only, set by default).
#' @param local logical, if TRUE we skip the online validator and rely only on pure XML-schema validation.  This may fail to detect invalid use of some semantic elements.
#' @details Requires an internet connection if local=FALSE.  see http://www.nexml.org/nexml/phylows/validator for more information in debugging invalid files
#' @return TRUE if the file is valid, FALSE or error message otherwise
#' @export
#' @import httr XML
#' @examples \dontrun{
#' data(bird.orders)
#' birds <- nexml_write(bird.orders, "birds_orders.xml")
#' nexml_validate("birds_orders.xml")
#' unlink("birds_orders.xml") # delete file to clean up
#' }
nexml_validate <- function(file, schema=system.file("xsd/nexml.xsd", package="RNeXML"), local = TRUE){
  
  if(local) {
    return( nexml_schema_validate(file, schema=schema) )
  }
  
  a = POST(ONLINE_VALIDATOR, body=list(file = upload_file(file)))
  if(a$status_code %in% c(200,201)){
    TRUE
  } else if(a$status_code == 504){
    warning("Online validator timed out, trying schema-only validation.")
    nexml_schema_validate(file, schema=schema)

  } else if(a$status_code == 400){
    warning(paste("Validation failed, error messages:",
         xpathSApply(htmlParse(content(a, "text")), 
                     "//li[contains(@class, 'error') or contains(@class, 'fatal')]", xmlValue)
         ))
    FALSE
  } else {
    warning(paste("Unable to reach validator. status code:", a$status_code, ".  Message:\n\n", content(a, "text")))
    NULL
  }
}




nexml_schema_validate <- function(file, schema=CANONICAL_SCHEMA){

      xml2::xml_validate(xml2::read_xml(file), xml2::read_xml(schema)) 
   
    
}
#xmlSchemaValidate(xmlSchemaParse(content(a, "text"), asText=TRUE), file)   # fails to get other remote resources

context("Geiger tests (may take 15+ minutes)")


has_geiger <- require(geiger)


test_that("We can write caudata data to nexml", {
  skip_if_not(has_geiger)
  skip_on_cran()
  
  data(caudata)
  nexml_write(trees = caudata$phy, characters = caudata$dat, file="tmp.xml")
  expect_true_or_null(nexml_validate("tmp.xml"))
  unlink("tmp.xml") # cleanup
})


test_that("We can write geospiza data to nexml", {
  skip_on_cran()
  skip_if_not(has_geiger)
  data(geospiza)
  nexml_write(trees = geospiza$phy, characters = geospiza$dat, file="tmp.xml")
  expect_true_or_null(nexml_validate("tmp.xml"))
  unlink("tmp.xml") # cleanup
})

test_that("We can write chelonia data to nexml", {
  skip_on_cran()
  skip_if_not(has_geiger)
  data(chelonia)
  nexml_write(trees = chelonia$phy, characters = chelonia$dat, file="tmp.xml")
  expect_true_or_null(nexml_validate("tmp.xml"))
  unlink("tmp.xml") # cleanup
})

test_that("We can write primates data to nexml", {
  skip_on_cran()
  skip_if_not(has_geiger)
  data(primates)
  nexml_write(trees = primates$phy, characters = primates$dat, file="tmp.xml")
  expect_true_or_null(nexml_validate("tmp.xml"))
  unlink("tmp.xml") # cleanup
})

test_that("We can write whales data to nexml", {
  skip_on_cran()
  skip_if_not(has_geiger)
  data(whales)
# taxa need to be rownames not separate column 
  skip_on_cran()
  whales$dat <- whales$richness[[2]]
  names(whales$dat) <- whales$richness[[1]] 
  nexml_write(trees = whales$phy, characters = whales$dat, file="tmp.xml")
  expect_true_or_null(nexml_validate("tmp.xml"))
  unlink("tmp.xml") # cleanup
})


test_that("We can write amphibia multiphylo to nexml. Two of these phylogenies each have nearly 3K taxa, so this may take around 12 minutes", {
# multiphylo, where two phylogenies have each nearly 3K taxa
  skip_on_cran()
  skip_if_not(has_geiger)
  data(amphibia)
  class(amphibia) <- "multiPhylo"
  runtime <- system.time(nexml_write(amphibia, file="tmp.xml")) # Slow! about 12 minutes

  expect_true_or_null(nexml_validate("tmp.xml"))
  unlink("tmp.xml") # cleanup
})

context("parsing")

# More lower-level parsing tests in inheritance

test_that("We can parse a NeXML file to an S4 RNeXML::tree object", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  f <- system.file("examples", "trees.xml", package="RNeXML")
  doc <- xmlParse(f)
  root <- xmlRoot(doc)
  nexml <- as(root, "nexml")  ## parse the XML into S4
  expect_is(nexml,"nexml")
})



test_that("We preserve existing namespace", {
  ## tests are too slow for CRAN
  skip_on_cran()
  
  f <- system.file("examples", "biophylo.xml", package="RNeXML")
  nex <- nexml_read(f)

  expect_gt(length(get_namespaces(nex)), length(get_namespaces(nexml())))
  ## Check that the new abbreviations are added
  expect_true(all(c("concept", "map") %in% names(get_namespaces(nex))))

  g <- tempfile()
  nexml_write(nex, g)
  expect_true_or_null(nexml_validate(g))

  nex2 <- nexml_read(g)
  ## check the namespaces remain there
  expect_gt(length(get_namespaces(nex2)), length(get_namespaces(nexml())))
  ## Check that the new abbreviations are added 
  expect_true(all(c("concept", "map") %in% names(get_namespaces(nex2))))
  expect_equal(get_namespaces(nex2)["concept"], get_namespaces(nex)["concept"])
  expect_equal(get_namespaces(nex2)["map"], get_namespaces(nex)["map"])

  f <- system.file("examples", "phenoscape.xml", package = "RNeXML")
  nex <- nexml_read(f)
  # check that the cdao namespace didn't get clobbered
  expect_true("cdao" %in% names(get_namespaces(nex)))
  expect_equivalent(get_namespaces(nex)["cdao"],
                    "http://www.evolutionaryontology.org/cdao/1.0/cdao.owl#")
  expect_true(get_namespaces(nex)["cdao"] != get_namespaces(nexml())["cdao"])
})

test_that("base namespace gets added if not present", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  doc <- xmlParse(system.file("examples", "no-base-ns.xml", package="RNeXML"))
  xmlroot <- xmlRoot(doc)

  prefixes <- names(xmlNamespaceDefinitions(doc))
  expect_false(any(prefixes == ""))

  nex <- nexml_read(doc)
  expect_true(any(names(get_namespaces(nex)) == ""))
  expect_equal(expand_prefix("/nexml", get_namespaces(nex)),
               expand_prefix("/nexml", get_namespaces(nexml())))
  expect_equal(expand_prefix("nexml", get_namespaces(nex)),
               expand_prefix("nex:nexml", get_namespaces(nex)))
})


test_that("files with rootedge can be parsed and roundtripped", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  f <- system.file("examples", "coal.xml", package = "RNeXML")
  nex <- read.nexml(f)

  tr <- nex@trees[[1]]@tree[[1]]
  testthat::expect_true(length(tr@rootedge@target) > 0 && nchar(tr@rootedge@target) > 0)
  testthat::expect_equal(sum(sapply(tr@node, slot, "id") == tr@rootedge@target), 1)
  xmlOut <- as(nex, "XMLInternalNode")

  nex2 <- read.nexml(xmlOut)
  tr2 <- nex2@trees[[1]]@tree[[1]]
  testthat::expect_true(length(tr2@rootedge@target) > 0 && nchar(tr2@rootedge@target) > 0)
  testthat::expect_equal(sum(sapply(tr2@node, slot, "id") == tr2@rootedge@target), 1)
  testthat::expect_equal(sum(sapply(tr2@node, slot, "id") == tr@rootedge@target), 1)
  testthat::expect_equal(saveXML(as(tr, "XMLInternalNode")),
                         saveXML(as(tr2, "XMLInternalNode")))

})

context("Set global (uuid) identifiers")

test_that("We can generate valid EML with uuid ids on all elements", {
  skip_on_cran()
  
  skip_if_not_installed("geiger")
  skip_if_not_installed("uuid")
  
  if(require("uuid")){
    options(uuid = TRUE)
    require(geiger)
    data(geospiza)
    add_trees(geospiza$phy)
    nexml <- add_characters(geospiza$dat)
    write.nexml(nexml, file = "geospiza.xml")

    expect_true_or_null(nexml_validate("geospiza.xml"))
    unlink("geospiza.xml")
  }
})

context("Online validator tool")
test_that("example file validates", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  f <- system.file("examples", "trees.xml", package="RNeXML")
  expect_true_or_null(nexml_validate(f)) # null if we cannot perform validation, don't fail
})

test_that("RNeXML-generated file validates", {
  ## tests are too slow for CRAN
  skip_on_cran()
  
  
  data(bird.orders)
  f <- nexml_write(bird.orders, file="test.xml") 
  o <- nexml_validate(f)
  if(!is.null(o)){
    expect_true(o)
  } else {
    expect_null(o)
  }
  unlink("test.xml")
})


test_that("Validation can fail gracefully", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  f <- system.file("examples/sparql.newick", package="RNeXML")
  
  expect_error(
  o <- nexml_validate(f)
  )
})

context("Comparative analysis")

test_that("We can extract tree and trait data to run fitContinuous and fitDiscrete", {
  skip_if_not_installed("geiger")
  
  ## tests are too slow for CRAN
  skip_on_cran()

  nexml <- read.nexml(system.file("examples", "comp_analysis.xml", package="RNeXML"))
  traits <- get_characters(nexml)
  tree <- get_trees(nexml)
  expect_is(tree, "phylo")

  
})


test_that("We can serialize tree and trait data for a comparative analysis", {
  ## tests are too slow for CRAN
  skip_on_cran()
  
  skip_if_not_installed("geiger")

  require("geiger")
  data(geospiza)
  add_trees(geospiza$phy)
  nexml <- add_characters(geospiza$dat)
  write.nexml(nexml, file = "geospiza.xml")
  expect_true_or_null(nexml_validate("geospiza.xml"))
  unlink("geospiza.xml")
})

context("ape")


test_that("From ape::phylo to RNeXML::nexml object", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
   data(bird.orders)
   expect_is(as(bird.orders, "nexml"), class="nexml") 
})

test_that("We can go from various orderings of ape::phylo to RNeXML::nexml", {
  ## tests are too slow for CRAN
  skip_on_cran()
  
  data(bird.orders)
  nexml <- as(bird.orders, "nexml")
  phy <- as(nexml, "phylo")

  ## Demonstrate that we now have a phylo object
  p <- plot(phy)
  expect_that(plot(phy), is_a("list"))
  expect_that(phy, is_a("phylo"))
})

test_that("From nexml to multiPhylo", {

  ## tests are too slow for CRAN
  skip_on_cran()

  # part of base testing, could be replaced with higher level, but why 
  f <- system.file("examples", "trees.xml", package="RNeXML")
  doc <- xmlParse(f)
  root <- xmlRoot(doc)
  nexml <- as(root, "nexml")  ## parse the XML into S4

  ## APE TEST:  Coerce the S4 into phylo S3 object
  expect_warning(phy <- as(nexml, "phylo"), "Multiple trees found, Returning multiPhylo object")
  
  expect_is(phy, "multiPhylo")

})

## This unit test is really not testing ape functions but just the higher-level nexml_write function...
test_that("We can serialize the various versions of the ape format", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  data(bird.orders)
  nexml <- as(bird.orders, "nexml")
  nexml_write(nexml, file = "test.xml")
  expect_true_or_null(nexml_validate("test.xml"))
  unlink("test.xml")
})





test_that("We can read and write NeXML to phylo and back without edge.lengths", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
           s <- "owls(((Strix_aluco,Asio_otus),Athene_noctua),Tyto_alba);"
           cat(s, file = "ex.tre", sep = "\n")
           owls <- read.tree("ex.tre")
           nexml_write(owls, file = "ex.xml")
           owls2 <- as(nexml_read("ex.xml"), "phylo")
           expect_equal(owls, owls2)
## FIXME what? 
           unlink("ex.tre")
           unlink("ex.xml")
})



test_that("Rooted trees remain rooted on conversions", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
          expect_true(is.rooted(bird.orders))
          expect_true(is.rooted(as(as(bird.orders, "nexml"), "phylo")))
          write.nexml(bird.orders, file = "tmp.xml")
          expect_true(is.rooted(as(read.nexml("tmp.xml"), "phylo")))
          unlink("tmp.xml")
})

test_that("Unrooted trees remain unrooted on conversions", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  phy <- unroot(bird.orders)
  expect_false(is.rooted(phy))
  expect_false(is.rooted(as(as(phy, "nexml"), "phylo")))
  write.nexml(phy, file = "tmp.xml")
  expect_false(is.rooted(as(read.nexml("tmp.xml"), "phylo")))
  unlink("tmp.xml")
})

test_that("We can convert trees with only some edge lengths into ape::phylo", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
          f <- system.file("examples", "some_missing_branchlengths.xml", package="RNeXML")
          expect_warning(a <- as(read.nexml(f), "phylo"), "Multiple trees found, Returning multiPhylo object")
          # We can parse it, goodness knows what anyone will do with it.  Better to hack off the branch lengths or convert to 0, but that's for the user.   
})


test_that("NeXML with rootedge can be converted to ape::phylo", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  f <- system.file("examples", "coal.xml", package = "RNeXML")
  nex <- read.nexml(f)
  tr <- nex@trees[[1]]@tree[[1]]
  phy <- as(nex, "phylo")
  testthat::expect_equal(Nnode(phy) + Ntip(phy), length(tr@node) + 1)
  testthat::expect_equal(Nedge(phy), length(tr@edge) + 1)
  newick <- write.tree(phy)
  testthat::expect_true(startsWith(newick, "(("))
  testthat::expect_true(grepl('):[0-9.]+);$', newick))
})

#' get namespaces
#' 
#' get namespaces
#' @param nexml a nexml object
#' @return a named character vector providing the URLs defining each
#' of the namespaces used in the nexml file.  Names correspond to 
#' the prefix abbreviations of the namespaces. 
#' @examples
#' comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
#' nex <- nexml_read(comp_analysis)
#' get_namespaces(nex)
#' @export 
get_namespaces <- function(nexml){
  nexml@namespaces
}

#' Extract rdf-xml from a NeXML file
#' 
#' Extract rdf-xml from a NeXML file
#' @param file the name of a nexml file, or otherwise a nexml object. 
#' @return an RDF-XML object (XMLInternalDocument).  This can be manipulated with
#'   tools from the XML R package, or converted into a triplestore for use with 
#'   SPARQL queries from the rdflib R package.  
#' @export
#' @import httr XML
#' @importFrom xml2 read_xml
get_rdf <- function(file){

  
  ## Soft dependency on xslt
  if (!requireNamespace("xslt", quietly = TRUE)) {
    stop("xslt package required to convert to rdf",
         call. = FALSE)
  }
  xml_xslt <- getExportedValue("xslt", "xml_xslt")
  
    if(is(file, "nexml")){
      who <- tempfile()
      nexml_write(x=file, file=who)
      file <- who
    }
    to_rdf <- system.file("examples", "RDFa2RDFXML.xsl", package="RNeXML")
    rdf <- 
    rdf <- xml_xslt(xml2::read_xml(file), xml2::read_xml(to_rdf))

  rdf  
}

context("rdf")

test_that("we can extract rdf-xml", {

  skip_if_not_installed("rdflib")
  skip_if_not_installed("xslt")
  skip_on_os("solaris")
  
  rdf <- get_rdf(system.file("examples/primates.xml", package="RNeXML"))
  tmp <- tempfile()  # so we must write the XML out first
  xml2::write_xml(rdf, tmp) 
  
  graph <- rdflib::rdf_parse(tmp)
  

  expect_is(graph, "rdf")
  
  rdflib::rdf_free(graph)
})

test_that("we can parse nested meta with blank nodes", {
  
  skip_if_not_installed("rdflib")
  skip_if_not_installed("xslt")
  skip_on_os("solaris")
  
  f <- system.file("examples", "meta_example.xml", package="RNeXML")
  nex <- read.nexml(f)
  tmp <- tempfile()
  xml2::write_xml(get_rdf(f), tmp)
  triples <- rdflib::rdf_parse(tmp)
  ## Check the blank node
  df <- rdflib::rdf_query(triples, 
                          "SELECT ?s ?p ?o WHERE 
                          { ?s <http://purl.org/dc/elements/1.1/source> ?source .
                          ?source ?p ?o
                          }")
  testthat::expect_equal(dim(df), c(3,3))
})

test_that("we can perform sparql queries with rdf", {
  skip_if_not_installed("rdflib")
  skip_if_not_installed("xslt")
  # skip_on_travis()
  skip_on_os("solaris")
  
  rdf <- get_rdf(system.file("examples/primates.xml", package="RNeXML"))
  tmp <- tempfile()  # so we must write the XML out first
  xml2::write_xml(rdf, tmp) 
  
  graph <- rdflib::rdf_parse(tmp)
  
  
  root <- rdflib::rdf_query(graph, 
                    "SELECT ?uri WHERE { 
    ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#rank> <http://rs.tdwg.org/ontology/voc/TaxonRank#Order> . 
    ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> ?uri    
}")
  
  expect_is(root, "data.frame")
  
  rdflib::rdf_free(graph)
  
})

## Helper functions 
name_by_id <- function(x)
  unname(sapply(x, function(i) if(length(i@id)>0) i@id else NULL))
name_by_id_or_label <- function(x)
  unname(sapply(x, function(i) if(length(i@label)>0) i@label else i@id))

get_by_id <- function(x, id){
  ids <- sapply(x, function(i) i@id)
  m <- match(id, ids)
  x[[m]]
}

#' Add metadata to a nexml file
#' 
#' @param meta a meta S4 object, e.g. ouput of the function \code{\link{meta}}, or a list of these meta objects
#' @param nexml (S4) object
#' @param level the level at which the metadata annotation should be added.
#' @param namespaces named character string for any additional namespaces that should be defined.  
#' @param i for otus, trees, characters: if there are multiple such blocks, which one should be annotated?  Default is first/only block.  
#' @param at_id the id of the element to be annotated.  Optional, advanced use only. 
#' @seealso \code{\link{meta}} \code{\link{add_trees}} \code{\link{add_characters}} \code{\link{add_basic_meta}}
#' @return the updated nexml object
#' @examples
#' ## Create a new nexml object with a single metadata element: 
#' modified <- meta(property = "prism:modificationDate", content = "2013-10-04")
#' nex <- add_meta(modified) # Note: 'prism' is defined in nexml_namespaces by default.  
#' 
#' ## Write multiple metadata elements, including a new namespace:  
#' website <- meta(href = "http://carlboettiger.info", 
#'                 rel = "foaf:homepage")              # meta can be link-style metadata
#' nex <- add_meta(list(modified,  website), 
#'                 namespaces = c(foaf = "http://xmlns.com/foaf/0.1/"))
#'
#' ## Append more metadata, and specify a level: 
#' history <- meta(property = "skos:historyNote",
#'                  content = "Mapped from the bird.orders data in the ape package using RNeXML")
#' data(bird.orders)
#' nex <- add_trees(bird.orders) # need to have created a trees block first
#' nex <- add_meta(history, 
#'                 nexml = nex,
#'                 level = "trees",
#'                 namespaces = c(skos = "http://www.w3.org/2004/02/skos/core#"))
#' 
#' @export add_meta
#' @include classes.R
#' 
add_meta <- function(meta, 
                     nexml=new("nexml"), 
                     level=c("nexml", "otus", "trees", "characters"), 
                     namespaces = NULL,
                     i = 1, 
                     at_id = NULL){
  level <- match.arg(level)
  if (is.null(meta))
    stop("meta object to add is null, with no default")
  if(!is.list(meta))
    meta <- list(meta)
  if(!all(sapply(meta, is, "meta")))
    stop("All elements in list must be of class 'meta' or a subclass")
 
  if(!is.null(at_id)){
    stop("function does not yet handle at_id assignments")
    # case not written yet
  } else if(level =="nexml"){ 
    nexml@meta <- c(nexml@meta, meta)
  } else if(level =="otus"){ 
    nexml@otus[[i]]@meta <- c(nexml@otus[[i]]@meta, meta)
  } else if(level == "trees"){
    nexml@trees[[i]]@meta <- c(nexml@trees[[i]]@meta, meta)
  } else if(level == "characters"){
    nexml@characters[[i]]@meta <- c(nexml@characters[[i]]@meta, meta)
  } 

  ## append additional namespaces
  nexml <- add_namespaces(namespaces, nexml)

  nexml
}

#' @include nexmlTree.R
NULL

#' extract all phylogenetic trees in ape format
#' 
#' extract all phylogenetic trees in ape format
#' @param nexml a representation of the nexml object from
#'   which the data is to be retrieved
#' @return returns a list of lists of multiphylo trees, even if all trees
#'  are in the same `trees` node (and hence the outer list will be of length
#'   1) or if there is only a single tree (and hence the inner list will also 
#'   be of length 1.  This ensures a consistent return type regardless of
#'    the number of trees present in the nexml file, and also preserves any 
#'    grouping of trees.  
#' @export
#' @examples
#' comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
#' nex <- nexml_read(comp_analysis)
#' get_trees_list(nex)
#' @seealso \code{\link{get_trees}} \code{\link{get_flat_trees}} \code{\link{get_item}}
get_trees_list <- function(nexml) as(nexml, "multiPhyloList")



#' extract a phylogenetic tree from the nexml
#' 
#' extract a phylogenetic tree from the nexml
#' @param nexml a representation of the nexml object from
#'  which the data is to be retrieved
#' @return an ape::phylo tree, if only one tree is represented.
#'  Otherwise returns a list of lists of multiphylo trees.
#'  To consistently receive the list of lists format (preserving 
#'  the hierarchical nature of the nexml), use \code{\link{get_trees_list}} instead.  
#' @export
#' @seealso \code{\link{get_trees}} \code{\link{get_flat_trees}} \code{\link{get_item}}
#' @examples
#' comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
#' nex <- nexml_read(comp_analysis)
#' get_trees(nex)
get_trees <- function(nexml) as(nexml, "phylo")


#' get_flat_trees  
#' 
#' extract a single multiPhylo object containing all trees in the nexml
#' @details Note that this method collapses any hierarchical structure that may have been present as multiple `trees` nodes in the original nexml (though such a feature is rarely used).  To preserve that structure, use \code{\link{get_trees}} instead.  
#' @return a multiPhylo object (list of ape::phylo objects).  See details.  
#' @param nexml a representation of the nexml object from  which the data is to be retrieved
#' @export
#' @seealso \code{\link{get_trees}} \code{\link{get_trees}} \code{\link{get_item}} 
#' @examples
#' comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
#' nex <- nexml_read(comp_analysis)
#' get_flat_trees(nex)
get_flat_trees <- function(nexml) flatten_multiphylo(get_trees_list(nexml))



####### Coercion methods ######### 

setAs("nexml", "multiPhyloList", function(from){
   map <- get_otu_maps(from) 
   unname(lapply(from@trees, 
           function(X){
             out <- unname(lapply(X@tree,  toPhylo, map[[X@otus]]))
             class(out) <- "multiPhylo"
             out
           }))
})


# Always collapses all trees nodes into a multiphylo
setAs("nexml", "multiPhylo", function(from){
   map <- get_otu_maps(from) 
   out <- unname(lapply(from@trees, 
           function(X){
             out <- unname(lapply(X@tree,  toPhylo, map[[X@otus]]))
             class(out) <- "multiPhylo"
             out
           }))
  flatten_multiphylo(out)
})


#' Flatten a multiphylo object
#' 
#' @details NeXML has the concept of multiple `<trees>` nodes, each with multiple child `<tree>` nodes.
#' This maps naturally to a list of multiphylo  objects.  Sometimes
#' this hierarchy conveys important structural information, so it is not discarded by default. 
#' Occasionally it is useful to flatten the structure though, hence this function.  Note that this
#' discards the original structure, and the nexml file must be parsed again to recover it.  
#' @param object a list of multiphylo objects 
#' @export
flatten_multiphylo <- function(object){
  out <- unlist(object, FALSE, FALSE)
  class(out) <- "multiPhylo"
  out
}


setAs("nexml", "phylo", function(from){ 
    if(length(from@trees[[1]]@tree) == 1){
      maps <- get_otu_maps(from)
      otus_id <- from@trees[[1]]@otus
      out <- toPhylo(from@trees[[1]]@tree[[1]], maps[[otus_id]])
    } else { 
      warning("Multiple trees found, Returning multiPhylo object")
      out <- as(from, "multiPhylo") 
    }
    if(length(out) == 1)
     out <- flatten_multiphylo(out)
   out 
  })



########### Main internal function for converting nexml to phylo ########



#' nexml to phylo 
#' 
#' nexml to phylo coercion 
#' @param tree an nexml tree element 
#' @param otus a character string of taxonomic labels, named by the otu ids.  
#' e.g. (from get_otu_maps for the otus set matching the relevant trees node. 
#' @return phylo object.  If a "reconstructions" annotation is found on the 
#' edges, return simmap maps slot as well.  
#' @importFrom plyr arrange
toPhylo <- function(tree, otus){
  otu <- NULL # Avoid CRAN NOTE as per http://stackoverflow.com/questions/8096313/no-visible-binding-for-global-variable-note-in-r-cmd-check
  ## Extract the nodes list
  nodes <- sapply(unname(tree@node), 
                  function(x) 
                    c(node = unname(x@id), 
                      otu = missing_as_na(x@otu)))
  # conversion (nor ape?) can't handle rootedge, to if there is one, add source
  if (is(tree@rootedge, "rootEdge")) {
    rootEdge <- tree@rootedge
    rootEdge@source <- nexml_id(prefix = "rn")
    newroot <- c(node = rootEdge@source, otu = NA)
    nodes <- cbind(unname(newroot), nodes)
  } else
    rootEdge <- NULL
  # If any edges have lengths, use this routine
  if(any(sapply(tree@edge, function(x) length(x@length) > 0)))
    edges <- sapply(unname(tree@edge), 
                    function(x) 
                      c(source = unname(x@source), 
                        target = unname(x@target), 
                        length = if(identical(x@length, numeric(0)))
                                  NA 
                                 else 
                                   unname(x@length), 
                        id = unname(x@id)))
  else # no edge lengths, use this routine 
    edges <- sapply(unname(tree@edge), 
                    function(x) 
                      c(source = unname(x@source), 
                        target = unname(x@target), 
                        id = unname(x@id)))
  # conversion (nor ape?) can't handle rootedge, to if there is one, add it
  if (! is.null(rootEdge)) {
    edges <- cbind(c(source = unname(rootEdge@source),
                     target = unname(rootEdge@target),
                     length = unname(rootEdge@length),
                     id = unname(rootEdge@id)),
                   edges)
  }

  nodes <- data.frame(t(nodes), stringsAsFactors=FALSE)
  names(nodes) <- c("node", "otu")

## Identifies tip.label based on being named with OTUs while others are NULL
## FIXME Should instead decide that these are tips based on the edge labels?
  nodes <- cbind(plyr::arrange(nodes, otu), id = 1:dim(nodes)[1])  # Also warns because arrange isn't quoting the column name.  

## NB: these ids are the ape:id numbers by which nodes are identified in ape::phylo
## Arbitrary ids are not supported - ape expecs the numbers 1:n, starting with tips. 

##  nodes$node lists tip taxa first (see arrange fn above), since
##  APE expects nodes numbered 1:n_tips to be to correspond to tips.
  source_nodes <- match(edges["source",], nodes$node)
  target_nodes <- match(edges["target",], nodes$node)

  ##  Define elements of a phylo class object ##
  #--------------------------------------------#

  ## define edge matrix
  edge <- unname(cbind(source_nodes, target_nodes))
  if("length" %in% rownames(edges))
    edge.length <- as.numeric(edges["length",])       
  else
    edge.length <- NULL

  ## define tip labels
  tip_otus <- as.character(na.omit(nodes$otu))   
  tip.label <- otus[tip_otus]

  # Count internal nodes
  Nnode <- max(edge) - length(tip.label)

  # assemble the phylo object, assign class and return.  
  phy = list(edge=edge, 
             tip.label = unname(tip.label), 
             Nnode = Nnode)
  if(!is.null(edge.length))
    phy$edge.length = edge.length # optional fields
  class(phy) = "phylo"



  ## Check for simmap  


  phy
}


## Helper function
missing_as_na <- function(x){
  if(length(x) == 0)
    NA
  else
    unname(x)
}

context("top level API")


test_that("read.nexml works", {
  ## The short version using an RNeXML API
  ## tests are too slow for CRAN
  skip_on_cran()
  f <- system.file("examples", "trees.xml", package="RNeXML")
  nex <- read.nexml(f) # check alias
  expect_is(nex, "nexml")
})


test_that("write.nexml works (from ape::phylo)", {
  ## The short version using an RNeXML API
  ## tests are too slow for CRAN
  skip_on_cran()
  data(bird.orders)
  nexml_write(bird.orders, file="example.xml")
  write.nexml(bird.orders, file="example.xml") # check alias too

## Check that that example is valid NeXML
  expect_true_or_null(nexml_validate("example.xml"))
  expect_is(nexml_read("example.xml", "nexml"), "nexml")

  unlink("example.xml") # cleanup

})


test_that("write.nexml can write multiple trees at once ", {
  ## tests are too slow for CRAN
  skip_on_cran()
  f <- system.file("examples", "trees.xml", package="RNeXML")
  nex <- nexml_read(f)
  trees <- get_trees(nex)

  ##  We can write a listOfmultiPhylo if the argument is named
  
  nexml_write(trees = trees, file="example.xml")
  expect_true_or_null(nexml_validate("example.xml"))

  # we can write a multiPhylo (or phylo) by attempting coercion on the first argument instead:  
  
  nexml_write(trees[[1]], file="example.xml")
  expect_true_or_null(nexml_validate("example.xml"))


  unlink("example.xml") # cleanup

})



test_that("We can get the right level of lists of trees ", {
  ## tests are too slow for CRAN
  skip_on_cran()

  f <- system.file("examples", "trees.xml", package="RNeXML")
  nex <- nexml_read(f)

  ## identical methods, Collapses length-1 lists 
  phy <- as(nex, "phylo")  ##
  phy2 <- get_trees(nex)
  phy3 <- nexml_get(nex, "trees")
  expect_identical(phy, phy2)
  expect_identical(phy3, phy2)

  ## Doesn't collapse the length-1 lists, returns list of multiPhylo always: 
  phy <- as(nex, "multiPhyloList")  ##
  phy2 <- get_trees_list(nex)
  phy3 <- nexml_get(nex, "trees_list")
  expect_identical(phy, phy2)
  expect_identical(phy3, phy2)

  ## Collapse to multiPhylo   
  phy <- as(nex, "multiPhylo")  ##
  phy2 <- get_trees(nex) # same because there are two trees in the same `trees` node.  
  expect_identical(phy, phy2)
  phy3 <- nexml_get(nex, "flat_trees") ## FIXME SOMETHING WRONG!
  expect_identical(phy3, phy2)
})

#setMethod("head", signature("nexml"), function(x, n=6L, ...){
#          write.nexml(x, "tmp123.xml")
#          txt <- readLines("tmp123.xml", n=n)
#          unlink("tmp123.xml")
#          cat(txt, "\n")
#})
#setMethod("tail", signature("nexml"), function(x, n=6L, ...){
#          write.nexml(x, "tmp123.xml")
#          txt <- readLines("tmp123.xml", n=-n)
#          unlink("tmp123.xml")
#          cat(txt, "\n")
#})
setMethod("show", signature("nexml"), function(object){
  s <- summary(object)

  tr_block_counts <- ""
  if (s$nblocks["trees"] > 0) {
    tr_block_counts <-
      paste0(sapply(1:s$nblocks["trees"],
                    function(i)
                      paste("\t   block", i, "contains",
                            s$ntrees[i], "phylogenetic tree(s)")),
             collapse = "\n")
    tr_block_counts <- paste0("where:\n", tr_block_counts)
  }

  m_block_counts <- ""
  if (s$nblocks["characters"] > 0) {
    m_block_counts <-
      paste0(
        sapply(1:s$nblocks["characters"],
               function(i) {
                 p1 <- paste("\t   block", i, "defines", s$ncharacters[i])
                 if (endsWith(object@characters[[i]]@`xsi:type`, "ContinuousCells"))
                   p1 <- paste(p1, "continuous character(s)")
                 else {
                   p1 <- paste(p1, "standard/discrete character(s),")
                   if (is.na(s$nstates[1, i]))
                     p1 <- paste(p1, "no state definitions")
                   else {
                     p1 <- paste(p1, "with")
                     if (s$nstates["Min.", i] == s$nstates["Max.", i])
                       p1 <- paste(p1, s$nstates["Max.", i], "states each")
                     else
                       p1 <- paste(p1, s$nstates["Min.", i], "to", s$nstates["Max.", i], "states")
                     p1 <- paste(p1,
                                 "\n\t    and ",
                                 sum(s$nnonstdstatedefs[i,]),
                                 "polymorphic or uncertain state(s) defined")
                   }
                 }
                 paste(p1,
                       "\n\t   matrix", i, "has", s$nmatrixrows[i], "row(s)")
               }),
        collapse = "\n")
    m_block_counts <- paste0("where:\n", m_block_counts)
  }

  cat(paste("A nexml object representing:\n",
            "\t", s$nblocks["trees"], "phylogenetic tree block(s),",
            tr_block_counts, "\n",
            "\t", s$nblocks["characters"], "character block(s),",
            m_block_counts, "\n",
            "\t", sum(s$notus), "taxonomic units in", s$nblocks["otus"], "block(s)",
            "\n  Taxa:\t", paste(head(get_taxa(object)$label), collapse = ", "), "..."))

  cat("\n  Metadata annotations:",
      paste0("\n\t", s$nmeta$nexml, " at top level"),
      sapply(names(s$nmeta)[-1],
             function(el)
               if (length(s$nmeta[[el]]) == 0)
                 ""
               else
                 paste0("\n\t",
                        sapply(1:length(s$nmeta[[el]]),
                               function(i) paste(s$nmeta[[el]][i], "in block", i)),
                        " at ", el, " level", collapse = "")))

  # description if provided
  descr <- get_metadata_values(object, props = "dc:description")
  if (all(!is.null(descr)) && any(nchar(descr) > 0))
    cat("\n\n", paste(descr, collapse = "\n"), sep = "")

  # authors if provided
  creator <- get_metadata_values(object, props = "dc:creator")
  if (all(!is.null(creator)) && any(nchar(creator) > 0))
    cat("\n\nAuthor(s):", paste(creator, collapse = ", "))

  # license if provided
  lic <- get_license(object)
  if (! is.na(lic))
    cat("\n\nLicense:", lic)

  cat("\n\nNeXML generated by", object@generator, "using",
      "schema version:", object@version,
      "\nSize:", capture.output(print(object.size(object), units="auto")), "\n")
})


# FIXME: consider showing author/title/citation information if available?
#' Summary method for nexml objects
#'
#' Generates a list of various counts of the major elements that comprise a
#' [nexml][nexml-class] object, such as number of different kinds of blocks,
#' characters, states, OTUs (taxa), etc.
#'
#' The [show][methods::show] method uses this summary for pretty-printing a
#' summary of the NeXML object, but it can be used on its own as well, in
#' particular for quick inspection of key properties of a NeXML file.
#' @param object the [nexml][nexml-class] object
#' @return A list with the following elements:
#'  - `nblocks` the number of trees, otus, and characters blocks
#'  - `ncharacters` the number of characters in each characters block
#'  - `nstates` summary statistics of the number of character states per state set
#'       defined for each characters block
#'  - `nnonstdstatedefs` the number of polymorphic and uncertain states defined
#'       for each character block
#'  - `nmatrixrows` the number of rows in the matrix for each character block
#'  - `ntrees` the number of trees contained in each trees block
#'  - `notus` the number of OTUs defined in each OTUs block
#'  - `nmeta` a list of the number of the number of metadata annotations at
#'       several levels, specifically:
#'       * `nexml` at the top (nexml) level
#'       * `otu` at the OTU level, for each OTUs block
#'       * `char` at the character level, for each characters block
#'       * `state` at the character state level, for each characters block
#' @examples
#' nex <- nexml_read(system.file("examples", "comp_analysis.xml", package = "RNeXML"))
#' s <- summary(nex)
#' # number of major blocks:
#' s$nblocks
#'
#' # each characters block defines 1 character:
#' s$ncharacters
#'
#' # summary stats of states per character (for morphological matrices there is
#' # typically one state set per character)
#' s$nstates # note that first block is of continuous type, so no stats there
#'
#' # pretty-printed summary:
#' nex # this is the same as show(nex)
#' @aliases summary.nexml
#' @export
setMethod("summary", signature("nexml"), function(object){
  # number of blocks of trees, OTUs, characters
  s <- list(
    nblocks = c(trees = length(object@trees),
                otus = length(object@otus),
                characters = length(object@characters))
  )

  # number of characters per character block
  nc <- sapply(object@characters, function(bl) length(bl@format@char))
  if (length(nc) > 0)
    names(nc) <- paste0("block.", seq(1, length(object@characters)))
  else
    nc <- as.integer(nc)
  s$ncharacters <- nc

  # number of states per character
  ns <- sapply(object@characters,
               function(bl) {
                 if (length(bl@format@states) == 0) {
                   bogusSumm <- summary(c(0))
                   NAs <- rep(NA, times = length(bogusSumm))
                   names(NAs) <- names(bogusSumm)
                   NAs
                 } else
                   summary(sapply(bl@format@states, function(x) length(x@state)))
               })
  if (length(dim(ns)) >= 2)
    colnames(ns) <- paste0("block.", seq(1, length(object@characters)))
  else
    ns <- as.integer(ns)
  s$nstates <- ns

  # number of state sets with polymorphic and uncertain states
  # (Note that morphological matrices typically define 1 state set for each
  # character, but molecular matrices will usually only define 1 state set.
  # Hence, the number of state sets may or may not say something about the
  # number of characters.)
  nsp <- sapply(object@characters,
                function(bl)
                  if (length(bl@format@states) == 0)
                    NA
                  else
                    sapply(bl@format@states,
                           function(x) length(x@polymorphic_state_set)))
  if (length(dim(nsp)) > 0)
    nsp <- apply(nsp, 2, function(x) sum(x > 0))
  nsu <- sapply(object@characters,
                function(bl)
                  if (length(bl@format@states) == 0)
                    NA
                  else
                    sapply(bl@format@states,
                           function(x) length(x@uncertain_state_set)))
  if (length(dim(nsu)) > 0)
    nsu <- apply(nsu, 2, function(x) sum(x > 0))
  s$nnonstdstatedefs <- matrix(c(nsp, nsu), ncol = 2, nrow = length(nsp), byrow = FALSE)
  colnames(s$nnonstdstatedefs) <- c("polymorphic", "uncertain")
  if (length(s$nnonstdstatedefs[, 1]) > 0)
    row.names(s$nnonstdstatedefs) <- paste0("block.", seq(1, length(object@characters)))

  # number of rows per matrix
  nr <- sapply(object@characters, function(bl) length(bl@matrix@row))
  if (length(nr) > 0)
    names(nr) <- paste0("block.", seq(1, length(object@characters)))
  else
    nr <- as.integer(nr)
  s$nmatrixrows <- nr

  # number of trees per trees block
  nt <- sapply(object@trees, function(bl) length(bl@tree))
  if (length(nt) > 0)
    names(nt) <- paste0("block.", seq(1, length(object@trees)))
  else
    nt <- as.integer(nt)
  s$ntrees <- nt

  # number of OTUs per OTUs block
  ntaxa <- sapply(object@otus, function(bl) length(bl@otu))
  if (length(ntaxa) > 0)
    names(ntaxa) <- paste0("block.", seq(1, length(object@otus)))
  else
    ntaxa <- as.integer(ntaxa)
  s$notus <- ntaxa

  # number of metadata annotations
  nmeta <- list(
    nexml = length(get_all_meta(object)),
    otu = sapply(object@otus,
                 function(bl) sum(sapply(bl@otu,
                                         function(x) length(get_all_meta(x))))),
    char = sapply(object@characters,
                  function(bl) sum(sapply(bl@format@char,
                                         function(x) length(get_all_meta(x))))),
    state = sapply(object@characters,
                   function(bl)
                     if (length(bl@format@states) == 0)
                       0
                     else
                       sum(unlist(
                         sapply(bl@format@states,
                                function(st)
                                  sapply(st@state,
                                         function(x) length(get_all_meta(x)))))))
  )
  for (elem in names(nmeta)[-1]) {
    if (length(nmeta[[elem]]) > 0) {
      if (elem %in% c("char", "state"))
        bltype <- "characters"
      else
        bltype <- "otus"
      names(nmeta[[elem]]) <- paste0("block.", seq(1, length(slot(object, bltype))))
    } else {
      nmeta[[elem]] <- as.integer(nmeta[[elem]])
    }
  }
  s$nmeta <- nmeta

  s
})

nexml_env = new.env(hash=TRUE)

# If no prefix is given, will use a UUID
# Generates an id number by appending a counter to the prefix
# Will keep track of the counter for each prefix for that session.  
#' @import uuid
nexml_id <- function(prefix = "",
                     use_uuid = getOption("uuid", FALSE)){
  if(use_uuid){
      uid <- paste0("uuid-", UUIDgenerate())
  } else {
    if((prefix %in% ls(envir=nexml_env)))
      id_counter <- get(prefix, envir=nexml_env)
    else {
      assign(prefix, 1, envir=nexml_env)
      id_counter <- 1
    } 

    uid <- paste0(prefix, id_counter)
    id_counter <- id_counter + 1
    assign(prefix, id_counter, envir=nexml_env)
  }
  uid
}

#' reset id counter
#' 
#' reset the id counter
#' @export
reset_id_counter <- function(){
  rm(list=ls(envir=nexml_env), envir=nexml_env)
}

# use an environment to store counter 

#' add elements to a new or existing nexml object
#' 
#' add elements to a new or existing nexml object
#' @param x the object to be added
#' @param nexml an existing nexml object onto which the object should be appended
#' @param type the type of object being provided.  
#' @param ... additional optional arguments to the add functions
#'
#' @return a nexml object with the additional data
#' @seealso \code{\link{add_trees}} \code{\link{add_characters}} \code{\link{add_meta}} \code{\link{add_namespaces}}    
#' @export
nexml_add <- function(x, 
                      nexml = new("nexml"), 
                      type = c("trees", 
                               "characters", 
                               "meta", 
                               "namespaces"),
                      ...){

  switch(type,
         "trees" = add_trees(x, nexml, ...),
         "characters" = add_characters(x, nexml, ...),
         "metadata" = add_meta(x, nexml, ...), 
         "namespaces" = add_namespaces(x, nexml, ...))

}

# get otus map
#
# @param nexml nexml object 
# @return a list showing the mapping between (internal) otu identifiers and labels (taxonomic names). List is named by the id of the otus block. 
# @details largely for internal use   
get_otu_maps <- function(nexml){
  otus <- as.list(nexml@otus)
  names(otus) <- name_by_id(otus)
  otu_maps <- 
    lapply(otus, function(otus){ # loop over all otus nodes  
    taxon <- sapply(otus@otu, function(otu){ # loop over each otu in the otus set
      if(length(otu@label) > 0) 
        label <- otu@label
      else
        label <- otu@id
      c(otu@id, label)
    })
    out <- taxon[2, ] #label 
    names(out) <- taxon[1, ] #id 
    out
  })
  otu_maps
}

get_char_maps <- function(nexml){
  map <- lapply(nexml@characters, function(characters)
         map_chars_to_label(characters@format))
  names(map) <- name_by_id(nexml@characters)
  map
}

get_state_maps <- function(nexml){
  map <- lapply(nexml@characters, function(characters){
    if(!isEmpty(characters@format@states))
      map_state_to_symbol(characters@format)
    else
     NULL
  })    
  names(map) <- name_by_id(nexml@characters)
  map
}


reverse_map <- function(map){
  out <- NULL
  if(is.list(map)){
    out <- lapply(map, 
                  function(x){
                    out <- names(x)
                    names(out) <- x
                    out})
   } else if(is.character(map)) {
    out <- names(map)
    names(out) <- map
    out
  }
  out   
}

#' Add namespaces
#' 
#' Add namespaces and their prefixes as a named vector of URIs, with the
#' names being the prefixes. Namespaces have most relevance for meta objects'
#' `rel` and `property`, and for embedded XML literals.
#'
#' The implementation attempts to avoid duplication, currently using the
#' prefix. I.e., namespaces with prefixes already defined will not get added.
#' Namespaces needed by the NeXML format, and for commonly used metadata
#' terms, are already included by default, see [get_namespaces()].
#' @note Often a user won't call this directly, but instead provide the
#'   namespace(s) through [add_meta()].
#' @param namespaces a named character vector of namespaces
#' @param nexml a nexml object. will create a new one if none is given.  
#' @return a nexml object with updated namespaces 
#' @examples
#' ## Write multiple metadata elements, including a new namespace:  
#' website <- meta(href = "http://carlboettiger.info", 
#'                 rel = "foaf:homepage")     # meta can be link-style metadata
#' modified <- meta(property = "prism:modificationDate",
#'                  content = "2013-10-04")
#' nex <- add_meta(list(modified,  website), 
#'                 namespaces = c(foaf = "http://xmlns.com/foaf/0.1/"))
#'                 # prism prefix already included by default
#'
#' ## Add namespace "by hand" before adding meta:
#' nex <- add_namespaces(c(skos = "http://www.w3.org/2004/02/skos/core#"),
#'                       nexml = nex)
#' history <- meta(property = "skos:historyNote",
#'                 content = "Mapped from the bird.orders data in the ape package using RNeXML")
#' nex <- add_meta(history, nexml = nex)
#' 
#' @seealso [meta()] [add_meta()] [get_namespaces()]
#' @export 
add_namespaces <- function(namespaces, nexml = new("nexml")){
  if(!is.null(namespaces)){
## check for duplicated abbreviation, not for duplicated URI. OKAY to have multiple abbrs for same URI...

## FIXME Make sure that cases where abbreviation match actually match the URI as well
    notdups <- match(names(namespaces), names(nexml@namespaces)) 
    notdups <- sapply(notdups, is.na)
    if(all(notdups)) # all are unique 
      nexml@namespaces <-  c(nexml@namespaces, namespaces)
    else {
      nexml@namespaces <-  c(nexml@namespaces, namespaces[notdups])
    } 
  }
  nexml
}

#################### Write character matices into S4 #####################


#' Add character data to a nexml object
#'
#' @param x character data, in which character traits labels are column names
#'        and taxon labels are row names.  x can be in matrix or data.frame
#'        format. 
#' @param nexml a nexml object, if appending character table to an existing
#'        nexml object.  If omitted will initiate a new nexml object.  
#' @param append_to_existing_otus logical. If TRUE, will add any new taxa
#' (taxa not matching any existing otus block) to the existing (first)
#' otus block.  Otherwise (default), a new otus block is created, even
#' though it may contain duplicate taxa to those already present.  While
#' FALSE is the safe option, TRUE may be appropriate when building nexml
#' files from scratch with both characters and trees.  
#' @export 
add_characters <- function(x, 
                           nexml = new("nexml"), 
                           append_to_existing_otus=FALSE){

  # FIXME does it make sense to take a phylo object here as an option?  
  # If so, perhaps don't call the argument 'nexml'.  
  # If not, then we don't really need this conversion. (maybe a type-check instead).   
  nexml <- as(nexml, "nexml")

  ## Check types  & row names ##
  x <- format_characters(x) 
  j <- length(nexml@characters) ## add after any existing character matrices

  nexml <- add_character_nodes(nexml, x)
  for(i in 1:length(x)){

    new_taxa <- rownames(x[[i]]) 
    nexml <- add_otu(nexml, new_taxa, append=append_to_existing_otus)
    ## Add the otus id to the characters node
    otus_id <- nexml@otus[[length(nexml@otus)]]@id
    nexml@characters[[i+j]]@otus <- get_by_id(nexml@otus, otus_id)@id

    nexml <- add_char(nexml, x, i, j)
    nexml <- add_states(nexml, x, i, j)
  }
  for(i in 1:length(x)){
    nexml <- add_rows(nexml, x, i, j)  
  }

  nexml
}

add_character_nodes <- function(nexml, x){
  n <- length(x)
  cs_list <- lapply(1:n, function(i){
         uid <- nexml_id("cs")
         characters <- nexml.characters(
             id = uid,
             about = paste0("#", uid))
         cls <- class(x[[i]][[1]])
         if(cls == "numeric")  ## Should be numeric but not integer!
           type <- "ContinuousCells"
         else ## Should be integer!
           type <- "StandardCells"
         slot(characters, "xsi:type") <- type  
         characters
      })
  cs_list <- c(nexml@characters, cs_list)
  nexml@characters <-  New("ListOfcharacters", cs_list)
  nexml
}



otu_list <- function(to_add, prefix="ou"){
  lapply(to_add, 
         function(label){
           uid <- nexml_id(prefix)
           nexml.otu(label=label, id =uid, about = paste0("#", uid))
         })
}


add_otu <- function(nexml, new_taxa, append=FALSE){

  current_taxa <- get_taxa_list(nexml)

  if(length(current_taxa) == 0) { # No otus exist, create a new node 
    otus <- new_otus_block(nexml, new_taxa)    
    nexml@otus <- New("ListOfotus", c(nexml@otus, otus))

  } else {

    otu_pos <- lapply(current_taxa, 
                      function(current) 
                        match(new_taxa, current))

    if(any(is.na(unlist(otu_pos)))){ # We have missing taxa
      if(append){
        ## append to otus block `otus_id` ##
        otus_id <- 1 # position that matches the id string
        to_add <- new_taxa[sapply(otu_pos, is.na)]  
        nexml@otus[[otus_id]]@otu <- New("ListOfotu",
                                   c(nexml@otus[[otus_id]]@otu, 
                                     otu_list(to_add, "ou_char"))) ## FIXME hack to make sure new ids are 'unique', 
      } else {
        ## Alternatively, do not append ## 
        otus <- new_otus_block(nexml, new_taxa)    
        nexml@otus <- New("ListOfotus", c(nexml@otus, otus))
      }
    } # else # all taxa matched, so we're all set
  }

  nexml
}

new_otus_block <- function(nexml, to_add){
    id <- nexml_id("os")
    New("ListOfotus",
        list(nexml.otus(
                 id = id,
                 about = paste0("#", id),
                 otu = New("ListOfotu", otu_list(to_add))
                 )))
}




# Turns char names into char nodes 
add_char <- function(nexml, x, i = 1, j = 0){
  char_labels <- colnames(x[[i]])
  char_list <- 
    lapply(char_labels, function(lab){
      id <- nexml_id("cr")
      char <- nexml.char(
                  id = id, 
                  about = paste0("#", id),
                  label = lab) 
      })
  nexml@characters[[i+j]]@format@char <- New("ListOfchar", char_list)
  nexml 
}



add_states <- function(nexml, x, i = 1, J = 0){
  # don't ctreate a states node if data is numeric
  if(all(sapply(x[[i]], is.numeric)))
    nexml
  else { 
    nchars <- length(x[[i]])
    char <- nexml@characters[[i+J]]@format@char
    states_list <- lapply(1:nchars, function(j){
      lab <- char[[j]]@label
      lvls <- levels(x[[i]][[lab]])
      id <- nexml_id("ss")
      states <- 
      nexml.states(
          id = id,
          about = paste0("#", id),
          state = New("ListOfstate",
          lapply(lvls, function(lvl){
            nexml.state(
                id=nexml_id("s"),
                symbol = as.integer(as.factor(lvl)))
          }))
      )
    })
    nexml@characters[[i+J]]@format@states <- New("ListOfstates", states_list)
    # Add the states's id to char
    for(j in 1:nchars)
      nexml@characters[[i+J]]@format@char[[j]]@states <- states_list[[j]]@id
  nexml
  }
  nexml
}      



## Assumes that otu ids have already been added to the nexml 
add_rows <- function(nexml, x, i = 1, j = 0){

  X <- x[[i]]
  taxa <- rownames(X)
  char_labels <- colnames(X)

  ## get the relevant characters block and otus block
  cs <- nexml@characters[[i+j]]@id   
  os <- nexml@characters[[i+j]]@otus 

  otu_map <- get_otu_maps(nexml)[[os]]
  char_map <- get_char_maps(nexml)[[cs]] 
  state_map <- get_state_maps(nexml)[[cs]]

  reverse_otu_map <- reverse_map(otu_map) 
  reverse_char_map <- reverse_map(char_map) 
  reverse_state_map <- reverse_map(state_map) 

  mat <- 
    nexml.matrix(
        row = New("ListOfrow",
    lapply(taxa, 
      function(taxon){
        id = nexml_id("rw")
        nexml.row(
            id = id,
            about = paste0("#", id),
            label = taxon,
            otu = reverse_otu_map[taxon],
            cell = New("ListOfcell",
             lapply(char_labels, function(char){
                    state <- X[taxon,char] # unmapped
                    char_id <- reverse_char_map[[char]]
                    if(!is.null(state_map))
                      state <- reverse_state_map[[char_id]][state]
                    nexml.cell(
                        char = char_id,
                        state = as.character(state))
                       }))
           )
        }))
    )
  nexml@characters[[i+j]]@matrix <- mat
  nexml
}




## divide matrix into discrete and continuous trait matrices, if necessary
## then write each as separate <characters> nodes: 
## x should now be a list of data.frames of common type
format_characters <- function(x){

  if(is(x, "numeric"))
    x <- as.data.frame(x)

  ## Actually useful conversions ##
  ## Matrices are either all-numeric or all-character class, so no risk of mixed discrete and continous states.  
  if(is(x, "matrix")){
    x <- list(as.data.frame(x))
    
  ## Data.frames can mix discrete and continous states, so we need to seperate them
  } else if(is(x, "data.frame") && dim(x)[2] > 1) {
    x <- split_by_class(x)

  } else if(is(x, "data.frame") && dim(x)[2] == 1) {
    x <- list(x) 

  #### Ugh, this next bit isn't pretty.  Maybe we should  just hope lists are formatted correctly, e.g. come from get_character_list. 

  ## If we're getting a list with matrices, coerce them into data.frames and hope for the best.  
  ## If the list has data.frames, check that each one has consistent class type.  
  ## Otherwise, panic.  
  } else if(is(x, "list")) {
    for(i in 1:length(x)){
      if(is(x[[i]], "matrix"))
        x[[i]] <- as.data.frame(x[[i]])  ## A list of matrices we can make into a list of data.frames...
    }
  ## Someone didn't even try to read the documentation...
  } else { 
    stop("x must be a named numeric, matrix, data.frame, or list thereof")
  }
  ## Let's just hope folks read the documentation and have 
  ## row names as taxa and column names as be character traits.  
  ## Kinda hard to check that for sure?  

  ## return the updated object: a list of data.frames
  x
}

## Helper function for the above, contains the primary functionality
# divide a data.frame into a list of data.frames, in which each has only a unique column class
split_by_class <- function(x){
    col.classes <- sapply(x, class)
    if (any(col.classes == "numeric")) {
      ## split into numerics and non-numerics 
      cts <- unname(which(col.classes=="numeric"))
      discrete <- unname(which(col.classes!="numeric"))
      x <- list(x[cts], x[discrete])
    } else
      x <- list(x)

  x
}

context("publish")

if(0){  # skip publishing tests.  These were all still passing at last check, but rfigshare configuration for testing is not ideal.  
  
  
  
# This loads the rOpenSci figshare sandbox credentials, so that the example 
# can run automatically during check and install.  Unlike normal figshare accounts,
# data loaded to this testing sandbox is periodically purged.  
library(rfigshare)
status <- try(fs_auth(token = "xdBjcKOiunwjiovwkfTF2QjGhROeLMw0y0nSCSgvg3YQxdBjcKOiunwjiovwkfTF2Q", token_secret = "4mdM3pfekNGO16X4hsvZdg"))
if(is(status, "try-error") || (is(status, "response") && status$status_code != 200)){
  warning("Could not authenticate figshare, skipping figshare tests")
} else {


## Create example file
  library(geiger)
  data(geospiza)

  geiger_nex <- add_trees(geospiza$phy)
  geiger_nex <- add_characters(geospiza$dat, geiger_nex)
  geiger_nex <- add_basic_meta(
    title = "Geospiza phylogeny with character data rendered as NeXML", 
    creator = "Carl Boettiger", 
    description = "This example NeXML file was created using the data originally provided in the geiger package for R to illustrate how this data can be stored, shared and distributed as NeXML.", 
    citation = citation("geiger"), 
    nexml = geiger_nex)


  test_that("We can publish to figshare", {

## Publish 
  id <- nexml_publish(geiger_nex, visibility="public", repo="figshare")


## Download and parse publication 
## Note that at present, only public files can be automatically downloaded from figshare
  library(rfigshare)
  test_nex <- nexml_read(fs_download(id))

## Extract and compare metadata from upload and download 
  m <- get_metadata(geiger_nex)
  test_m <- get_metadata(test_nex)
  expect_equal(m["dc:title"], test_m["dc:title"])
  expect_equal(m["dc:description"], test_m["dc:description"])

## Check that DOI resolves -- doesn't for the test account
#library(httr)
#page <- GET(test_m[["dc:identifier"]])
#expect_equal(page$status_code, 200)


# Check that we avoid repeated metadata entries

  expect_equal(sum(match(names(test_m), "dc:pubdate"), na.rm=TRUE), 1)
  expect_equal(sum(match(names(test_m), "cc:license"), na.rm=TRUE), 1)


  })



}

}

context("character matrices")


## All tests will use this data file
f <- system.file("examples", "comp_analysis.xml", package="RNeXML")

test_that("we can parse XML to S4 and serialize S4 to XML for the basic character classes", {

  doc <- xmlParse(f)
  root <- xmlRoot(doc)

  char <- as(root[["characters"]][["format"]][["char"]], "char")
  out <- as(char, "XMLInternalElementNode")
  expect_is(char, "char") # not as dumb as it looks, at least we're checking our own method here 
  expect_is(out, "XMLInternalElementNode") # dumb check, but provides a dot to show the code above executed successfully 

  format <- as(root[["characters"]][["format"]], "format")
  out <- as(format, "XMLInternalElementNode")
  expect_is(format, "format") 
  expect_is(out, "XMLInternalElementNode") 

  matrix <- as(root[["characters"]][["matrix"]], "obsmatrix")
  out <- as(matrix, "XMLInternalElementNode")
  expect_is(matrix, "obsmatrix") 
  expect_is(out, "XMLInternalElementNode") 

  characters <- as(root[["characters"]], "characters")
  out <- as(characters, "XMLInternalElementNode")
  expect_is(characters, "characters") 
  expect_is(out, "XMLInternalElementNode") 

})

test_that("we can actually parse NeXML files containing character data", {
  nex <- read.nexml(f)
  expect_is(nex, "nexml")
})

## Now that we tested this, store the result so we can use it in later tests
nex <- read.nexml(f)

### BEGIN *DO NOT MODIFY nex OBJECT* ###
###                                  ###

test_that("we can extract character matrix with get_characters", {
  x <- get_characters(nex)
  expect_is(x, "data.frame")
## FIXME add additional and more precise expect_ checks 
})

test_that("we can extract a list of character matrices with get_characters_list", {
  x <- get_characters_list(nex)
  expect_is(x, "list")
  expect_is(x[[1]], "data.frame")
## FIXME add additional and more precise expect_ checks 

})

## FIXME add_characters needs a method to add character names of states
## and then we need a test for that method

test_that("we can add characters to a nexml file using a data.frame", {
  x <- get_characters(nex)
  nexml <- add_characters(x)

  ##  Can we write it out and read it back? 
  nexml_write(nexml, file = "chartest.xml")
  tmp <- nexml_read("chartest.xml")
  tmp_x <- get_characters(tmp)

  expect_is(tmp_x, "data.frame")
  expect_is(tmp, "nexml")
  unlink("chartest.xml")
})

## based on bug on 2014-03-12 65ae459523c529452adb699c3d5d118c0a207402 
test_that("we can add multiple character matrices to a nexml file", {
          skip_if_not_installed("geiger")
          require(geiger)
          data(geospiza)
          data(primates)
          nexml <- add_characters(geospiza$dat)
          nexml <- add_characters(primates$dat, nexml)
          expect_is(nexml, "nexml")
})

###                                  ###
### END *DO NOT MODIFY nex OBJECT*   ###
###                                  ###
### Re-read nex object before using  ###
###                                  ###

test_that("add_otu can append only unmatched taxa to an existing otus block", {
  nex <- read.nexml(f)
  orig <- get_taxa(nex)
  x <- get_characters_list(nex)
  nex@otus[[1]]@otu <- new("ListOfotu", nex@otus[[1]]@otu[1:5]) # chop off some of the otu values

  new_taxa <- rownames(x[[1]])
  nex2 <- RNeXML:::add_otu(nex, new_taxa, append=TRUE) # add them back
  ## should have same contents as orig...
  get_taxa(nex2)
  expect_identical(sort(orig$label), sort(get_taxa(nex2)$label))

  ## Note that otu ids are not unique when we chop them off ...
})


test_that("we can serialize polymorphic states", {
  
  f <- system.file("examples", "ontotrace-result.xml", package="RNeXML")
  nex <- read.nexml(f)
  states <- nex@characters[[1]]@format@states[[1]]
  out <- as(states, "XMLInternalNode")
  testthat::expect_true("polymorphic_state_set" %in% names(out))
  
})

# simmap.R

# if(!is.null(phy$maps))
## FIXME write the characters/states block (but not matrix block) as well.  

## FIXME support writing multiphylos, list of multiphylos to nexml 


#' Convert phylo with attached simmap to nexml object
#' 
#' @param phy a phylo object containing simmap `phy$maps` element,
#'  from the phytools package
#' @param state_ids a named character vector giving the state 
#'  names corresponding to the ids used to refer to each state
#'  in nexml.  If null ids will be generated and states taken from 
#'  the phy$states names. 
#' @return a nexml representation of the simmap
#' @export 
#' @import XML
#' @examples
#' simmap_ex <- read.nexml(system.file("examples","simmap_ex.xml", package="RNeXML"))
#' phy <- nexml_to_simmap(simmap_ex)
#' nex <- simmap_to_nexml(phy) 
simmap_to_nexml <- function(phy, state_ids = NULL){

  
  ## Hack to deal with S3 class issues when coercing to S4
  
  #if("simmap" %in% class(phy))
    class(phy) <- "phylo"
  
  ## Create the NeXML object
  nexml <- as(phy, "nexml")

  if(!is.null(phy$states)){
    nexml <- add_characters(data.frame(states = as.integer(as.factor(phy$states))), nexml)
    
    ## FIXME doesn't have states
    chars_ids <- get_state_maps(nexml)[[1]]
    char_id <- names(chars_ids)
    state_ids <-  reverse_map(chars_ids[[1]]) 
    # can assume no other states added yet since works on a phy, not nexml  
  }


  if(!is.null(phy$maps))
    nexml <- simmap_edge_annotations(maps = phy$maps, 
                                     nexml = nexml, 
                                     state_ids = state_ids, 
                                     char_id = char_id)
  
 
  nexml
}


simmap_edge_annotations <- function(maps, nexml, state_ids = NULL, char_id = "simmapped_trait"){

  
  ## if state ids are not given
  if(is.null(state_ids)){
    state_ids <- levels(as.factor(names(unlist(maps))))
    names(state_ids) <- state_ids
  }


  # Loop over all edges, adding the simmap annotation to each: 
  for(i in 1:length(nexml@trees[[1]]@tree[[1]]@edge)){
   ## FIXME check to assure this is always the correct order??

    ## Read the mapping of the current edge 
    edge_map <- maps[[i]]

    ## Generate the list of XML "stateChange" nodes 
    mapping <- lapply(1:length(edge_map), function(j){
      ##  A node has an id, a length and a state 
      meta(rel = "simmap:stateChange",
           children = list(meta(property = "simmap:order",
                                content = j),
                           meta(property = "simmap:length", 
                                content = edge_map[[j]]),
                           meta(property = "simmap:state", 
                                content = state_ids[[names(edge_map[j])]] )
                           )
           )
    })

    reconstruction <-meta(rel = "simmap:reconstruction",
                          children =c(list(meta(property="simmap:char", 
                                              content = char_id)),
                                           mapping))
  
  ## Insert the reconstructions into a <meta> element in each nexml edge
    nexml@trees[[1]]@tree[[1]]@edge[[i]]@meta <- 
      c(meta(type = "ResourceMeta",
             rel = "simmap:reconstructions",
             children = list(reconstruction)))
  }

  ## Return the entire nexml object
 nexml <- add_namespaces(c(simmap = "https://github.com/ropensci/RNeXML/tree/master/inst/simmap.md"),
                        nexml)
 nexml 
}





## Returns list of multiPhylo ...

#' Convert nexml object with simmap to phylo
#'
#' @param nexml a nexml object
#' @return a simmap object (phylo object with a `$maps` element
#'  for use in phytools functions).
#' @export 
#' @describeIn simmap_to_nexml Convert nexml object with simmap to phylo
nexml_to_simmap <- function(nexml){

  ## Get the statemap, if available

  characters <- get_characters(nexml)

  ## loop over trees blocks
  out <- lapply(nexml@trees, function(trees){
    phys <- lapply(trees@tree, 
                   tree_to_simmap, 
                   get_otu_maps(nexml)[[trees@otus]],
                   get_state_maps(nexml)[[1]][[1]]
                  )
    phys <- lapply(phys, characters_to_simmap, characters)

    names(phys) <- NULL
    class(phys) <- "multiPhylo"
    phys 
  })


  if(length(out) == 1){
    if(length(out[[1]]) > 1){
      flatten_multiphylo(out)
    } else {
      out[[1]][[1]]
    }
  }

}

characters_to_simmap <- function(phy, characters){
  out <- as.character(characters[[1]]) # coerce factor to string
  names(out) <- rownames(characters) 
  phy$states <- out
  phy
}


# given the nexml tree:
tree_to_simmap <- function(tree, otus, state_maps = NULL){
  maps <- lapply(tree@edge, function(edge){

    reconstructions <- sapply(edge@meta,
                              function(x) .hasSlot(x, "rel") && x@rel == "simmap:reconstructions")
    if(any(reconstructions))
      reconstruction <- edge@meta[[which(reconstructions)]]@children 

    else {  # handle exceptions 
      warning("no simmap data found")
      return(toPhylo(tree, otus)) ## no simmap found
    }


#    lapply(reconstruction, function(reconstruction){ # for each reconstruction
          stateChange <- sapply(reconstruction[[1]]@children,
                                function(x) .hasSlot(x, "rel") && x@rel == "simmap:stateChange")

          values <- sapply(reconstruction[[1]]@children[which(stateChange)], function(stateChange){ 
                           # phytools only supports one reconstruction of one character per phy object
                        property <- sapply(stateChange@children, function(x) x@property)
                        names(stateChange@children) <- property # clean labels
                        sapply(stateChange@children, function(x) x@content)
          })
          out <- as.numeric(values["simmap:length", ])
          ordering <- as.numeric(values["simmap:order",])
          if(!is.null(state_maps))
            states <- state_maps[values["simmap:state", ]]
          else 
            states <- values["simmap:state", ]

          names(out) <- states
          out <- out[ordering] # sort according to explicit order

          out
 #   })
   
  })
  names(maps) <- NULL
  
  phy <- toPhylo(tree, otus)
  phy$maps <- maps

  ## create the rest of the maps elements


  ## Return phylo object
  phy
}



## Extend directly with XML representation instead of S4

#setClass("simmap:reconstructions",
#         slots = c(reconstruction = "ListOfreconstruction"))
#setClass("ListOfreconstruction", contains = "list")
#setClass("simmap:stateChange", contains = "IDTagged") 
#

context("simmap")


## Make a simmap tree 



test_that("we can coerce an ape::phylo tree with a 
          phytools:simmap extension into nexml", {
  skip_if_not_installed("phytools")
  require("phytools")
  set.seed(10) 
  tree <- rbdtree(b = log(50), d = 0, Tmax = .5)
  Q <- matrix(c(-2, 1, 1, 1, -2 ,1 ,1, 1, -2), 3, 3) 
  rownames(Q) <- colnames(Q) <- c("A", "B", "C") 
  ## Note that state symbols must be integers! factors will be converted
  mtree <- sim.history(tree, Q) 

  cols <- c("red", "blue", "green")
  names(cols) <- rownames(Q)
  nex <- simmap_to_nexml(mtree) 
  expect_is(nex, "nexml")

  phy <- nexml_to_simmap(nex) 

  orig <- plotSimmap(mtree,cols,ftype="off")
  roundtrip <- plotSimmap(phy,cols,ftype="off")

  # checks that the edge mappings are correct 
  expect_equal(mtree$maps, phy$maps)

  
  orig <- as.integer(as.factor(mtree$states[sort(names(mtree$states))]))
  converted <- as.integer(phy$states[sort(names(phy$states))])
  # checks that we got the states slot correct 
  expect_equal(converted, orig)
})

isEmpty <- function (obj) 
{
    if (!isS4(obj)) {
        if (length(obj) > 0) 
            FALSE
        else TRUE
    }
    else {
        if (identical(obj, new(class(obj)[1]))) 
            out <- TRUE
        else {
            empty <- sapply(slotNames(obj), function(s) {
                if (isS4(slot(obj, s))) 
                  isEmpty(slot(obj, s))
                else {
                  if (length(slot(obj, s)) == 0) 
                    TRUE
                  else if (length(slot(obj, s)) > 0) 
                    FALSE
                }
            })
            out <- !any(!empty)
        }
        out
    }
}

#' Concatenate nexml files 
#' 
#' Concatenate nexml files 
#' @param x,... nexml objects to be concatenated, e.g. from 
#'  \code{\link{write.nexml}} or \code{\link{read.nexml}}. 
#'  Must have unique ids on all elements
#' @param recursive  logical.  If 'recursive = TRUE', the function recursively
#'        descends through lists (and pairlists) combining all their
#'        elements into a vector. (Not implemented).  
#' @return a concatenated nexml file
#' @examples 
#' \dontrun{
#' f1 <- system.file("examples", "trees.xml", package="RNeXML")
#' f2 <- system.file("examples", "comp_analysis.xml", package="RNeXML")
#' nex1 <- read.nexml(f1)
#' nex2 <- read.nexml(f2)
#' nex <- c(nex1, nex2)
#' }
setMethod("c", 
          signature("nexml"), 
          function(x, ..., recursive = FALSE){
              elements = list(x, ...)
              nexml <- new("nexml")
  ## Check that ids are unique
  if(!do.call(unique_ids,elements))
    stop("ids are not unique across nexml files. 
          Consider regenerating ids")
  else {

  nexml@otus <- New("ListOfotus",
                    unlist(lapply(elements, 
                                  function(n) n@otus), 
                           recursive=FALSE))
  nexml@characters <- New("ListOfcharacters",
                    unlist(lapply(elements, 
                                  function(n) n@characters), 
                           recursive=FALSE))
  nexml@trees <- New("ListOftrees",
                    unlist(lapply(elements, 
                                  function(n) n@trees), 
                           recursive=FALSE))
  }
  nexml
})

get_ids <- function(nexml){
  doc <- xmlDoc(as(nexml, "XMLInternalNode"))
  out <- unname(xpathSApply(doc, "//@id"))
  free(doc)
  out
}

unique_ids <- function(...){
  set <- list(...)
  counts <- table(unlist(lapply(set, get_ids)))
  !any(counts > 1)
}

#' publish nexml files to the web and receive a DOI
#' 
#' publish nexml files to the web and receive a DOI
#' @param nexml a nexml object (or file path)
#' @param ... additional arguments, depending on repository. See examples.
#' @param repository destination repository
#' @return a digital object identifier to the published data
#' @export 
#' @examples \dontrun{
#' data(bird.orders)
#' birds <- add_trees(bird.orders)
#' doi <- nexml_publish(birds, visibility = "public", repository="figshare")
#' }
nexml_publish <- function(nexml, ..., repository="figshare"){
  repository = match.arg(repository)
  switch(repository, 
         figshare = nexml_figshare(nexml, ...))
}


#' publish nexml to figshare
#'
#' publish nexml to figshare
#' @param nexml a nexml object (or file path to a nexml file)
#' @param file The filename desired for the object, if nexml is not already a file.
#'  if the first argument is already a path, this value is ignored.  
#' @param categories The figshare categories, must match available set. see \code{fs_add_categories}
#' @param tags Any keyword tags you want to add to the data.  
#' @param visibility whether the results should be published (public), or kept private,
#'  or kept as a draft for further editing before publication.  (New versions can be updated,
#'  but any former versions that was once made public will always be archived and cannot be removed).  
#' @param id an existing figshare id (e.g. from fs_create), to which this file can be appended.  
#' @param ... additional arguments
#' @return the figshare id of the object  
#' @export
#' @examples \dontrun{
#' data(bird.orders)
#' birds <- add_trees(bird.orders)
#' doi <- nexml_figshare(birds, visibility = "public", repository="figshare")
#' }
nexml_figshare <- function(nexml,
                           file = "nexml.xml", 
                           categories = "Evolutionary Biology", 
                           tags = list("phylogeny", "NeXML"),
                           visibility = c("public", "private", "draft"),
                           id = NULL, 
                           ...){

  visibility = match.arg(visibility)

#  success <- require(rfigshare)
#  if(!success){
#    message("rfigshare package not found. Attempting to install")
#    install.packages("rfigshare")
#    success <- require(rfigshare)
#     if(!success)  
#      stop("The rfigshare package must be installed to publish data to figshare")
#  }



  # handle nexml as a file path or as an object
  if(!is(nexml, "nexml")){
    if(file.exists(nexml)){
      file <- nexml
      nexml <- nexml_read(nexml)
    } # else warning?
  }
  
  m <- get_metadata(nexml)

  
  if(is.null(id)){
    id <- rfigshare::fs_create(title = m[["dc:title"]],
                    description = m[["dc:description"]], 
                    type = "dataset")
  }


  doi <- paste("http://doi.org/10.6084/m9.figshare", id, sep=".")

  rfigshare::fs_add_authors(id, authors = m[["dc:creator"]])
  rfigshare::fs_add_categories(id, categories)
  rfigshare::fs_add_tags(id, tags)

  # Use object DOI instead of figshare id when available? 
  # Construct DOI from figshare id? 
  nexml <- add_meta(meta("dc:identifier", doi), nexml)

  nexml_write(nexml, file) 

  rfigshare::fs_upload(id, file)
  if (visibility == "private"){ 
      rfigshare::fs_make_private(id)
      message(paste("Your data has been uploaded to figshare privately.
           You may make further edits and publish the data from
           the online control panel at figshare.com or by using 
           the rfigshare package and the article_id:", id, ". Your 
           doi has been reserved but will not resolve until the article 
           is made public."))

  } else if (visibility == "public"){ 
      rfigshare::fs_make_public(id)
      message(paste("Your data is published and now accessible at", doi))
  } else {
  message(paste("Your data has been uploaded to figshare as a draft.
           You may make further edits and publish the data from
           the online control panel at figshare.com or by using 
           the rfigshare package and the article_id:", id, " Your 
           doi has been reserved but will not resolve until the article 
           is made public."))
  }

  # FIXME consider not returning the DOI as a link. 
  # Consider returning just the Figshare id
  # 

  id  
}

#' @include nexmlTree.R
NULL

#' add_trees
#' 
#' add_trees
#' @param phy a phylo object, multiPhylo object, or list of 
#'  multiPhylo to be added to the nexml
#' @param nexml a nexml object to which we should append this phylo.
#'  By default, a new nexml object will be created.  
#' @param append_to_existing_otus logical, indicating if we should 
#'  make a new OTU block (default) or append to the existing one. 
#' @return a nexml object containing the phy in nexml format. 
#' @export 
add_trees <- function(phy, 
                      nexml=new("nexml"), 
                      append_to_existing_otus=FALSE){
  nexml <- as(nexml, "nexml")

  phy <- standardize_phylo_list(phy)
  ## handle multiPhlyo cases
  new_taxa <- unlist(sapply(phy, function(y)
                      sapply(y, function(z) 
                        z$tip.label)))

  nexml <- add_otu(nexml, new_taxa, append=append_to_existing_otus)
  otus_id <- nexml@otus[[length(nexml@otus)]]@id
  nexml <- add_trees_block(nexml, phy, otus_id)
  nexml
}


##################### phylo -> nexml ###############

setAs("phylo", "nexml", function(from){
      add_trees(from)
})

setAs("multiPhylo", "nexml", function(from){
      add_trees(from)
})


standardize_phylo_list <- function(phy){
  if(is(phy, "list") && 
     (is(phy[[1]], "list") || is(phy[[1]], "multiPhylo")) && 
     is(phy[[1]][[1]], "phylo")){
    phy 
  } else if(is(phy, "multiPhylo") || (is(phy, "list") && is(phy[[1]], "phylo"))) { 
    list(phy)
  } else if(is(phy, "phylo")) {
    phy <- list(phy)
    class(phy) <- "multiPhylo"
    list(phy) 
  } else {
    # desperate
    phy <- list(as(phy, "phylo"))
    class(phy) <- "multiPhylo"
    list(phy) 
  }
}

add_trees_block <- function(nexml, phy, otus_id){
  phy <- standardize_phylo_list(phy)
  ## all trees will use the same  
  otu_map <- reverse_map(get_otu_maps(nexml))[[otus_id]]

  trees <- lapply(phy, function(trs){
         tree_id <- nexml_id("ts")
         nexml.trees(
             id = tree_id,
             about = paste0("#", tree_id),
             otus = otus_id,
             tree = New("ListOftree",
                        lapply(trs, function(tr)
                           fromPhylo(tr, otu_map)))
            )
  })

  ## Append to any existing trees nodes 
  nexml@trees <- New("ListOftrees", c(nexml@trees, trees))
  nexml
}


# Main routine to generate NeXML from ape:phylo
fromPhylo <- function(phy, otu_map){
  
  node_ids <- sapply(unique(as.numeric(phy$edge)), 
                     function(i) nexml_id("n")) 
  names(node_ids) <- as.character(unique(as.numeric(phy$edge)))

  ## Generate the "ListOfedge" made of "edge" objects
  edges <- 
    lapply(1:dim(phy$edge)[1], 
           function(i){
            edge_id <- nexml_id("e")
            source <- node_ids[as.character(phy$edge[i,1])]
            target <- node_ids[as.character(phy$edge[i,2])]
            e <- nexml.edge(
                     source = source, 
                     target = target, 
                     id = edge_id,
                     about = paste0("#", edge_id))
           if(!is.null(phy$edge.length))
             e@length <- as.numeric(phy$edge.length[i])
           e
           }
  )
  edges <- New("ListOfedge", edges)
  ## Generate the ListOfnode made of "node" objects
  ## In doing so, generate otu_id numbers for tip nodes
  nodes <- lapply(unique(as.numeric(phy$edge)), function(i){
    node_id <- node_ids[as.character(i)] 
    if(is.na(phy$tip.label[i]))
      nexml.node(id = node_id, about = paste0("#", node_id))
    else if(is.character(phy$tip.label[i])){
      otu_id <- otu_map[phy$tip.label[i]]
      nexml.node(
          id = node_id, 
          about = paste0("#", node_id), 
          otu = otu_id)  
    }
  })
  ## FIXME how about naming non-tip labels?  
  nodes <- New("ListOfnode", nodes)

 

  ## Create the "tree" S4 object
  tree_id <- nexml_id("tree") 
  tree <- nexml.tree(
      node = nodes, 
      edge = edges,
      'xsi:type' = 'FloatTree',
      id = tree_id,
      about = paste0("#", tree_id))  
}

context("taxonomy")


data(bird.orders)
birdorders_small <- drop.tip(bird.orders, tip = 1:10)
birds <- add_trees(birdorders_small)

data(chiroptera)
chiroptera_small <- drop.tip(chiroptera, tip = 1:906)
chir <- add_trees(chiroptera_small)
chiroptera_super_small <- drop.tip(chiroptera, tip = 1:911)
chir_super_small <- add_trees(chiroptera_super_small)





test_that("taxize_nexml correctly collects ncbi identifiers", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  testthat::skip_if_not_installed("taxalight")
  suppressWarnings({
    birds <- taxize_nexml(birds, "NCBI")
    chir <- taxize_nexml(chir, "NCBI")
    chir_super_small <- taxize_nexml(chir_super_small, "NCBI")
  })
  
  expect_is(birds@otus, "ListOfotus")
  
  expect_is(birds@otus@.Data[[1]]@otu, "ListOfotu")
  expect_is(birds@otus@.Data[[1]]@otu[[1]], "otu")
  expect_is(birds@otus@.Data[[1]]@otu[[1]]@meta, "ListOfmeta")

#  expect_is(birds@otus@.Data[[1]]@otu[[1]]@meta[[1]], "meta")
#  expect_equal(slot(birds@otus@.Data[[1]]@otu[[1]]@meta[[1]], "href"), 
#               "http://ncbi.nlm.nih.gov/taxonomy/56308")
#  expect_equal(slot(birds@otus@.Data[[1]]@otu[[1]]@meta[[1]], "rel"), 
#               "tc:toTaxon")
  
  expect_is(chir@otus, "ListOfotus")
  expect_is(chir@otus@.Data[[1]]@otu, "ListOfotu")
  expect_is(chir@otus@.Data[[1]]@otu[[1]], "otu")
})

test_that("we can extract taxonomy data from the object", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  expect_is(get_metadata(birds, "otus/otu"), "data.frame")
  expect_is(get_metadata(chir_super_small, 'otus/otu'), "data.frame")
})

### TODO: how to deal with missing meta slot elements???


test_that("taxize_nexml throws appropriate warnings", {
  
  testthat::skip_on_cran()
  testthat::skip_if_not_installed("taxadb")
  
  
  chir1 <- drop.tip(chiroptera, tip = 1:910)
  chir1 <- add_trees(chir1)
  expect_warning(taxize_nexml(chir1, "NCBI"))
  
  chiroptera_super_small <- drop.tip(chiroptera, tip = 1:912)
  chir_super_small <- add_trees(chiroptera_super_small)
  expect_is(taxize_nexml(chir_super_small, "NCBI"), "nexml")
  # note from Scott: above test used to test that a warning 
  # was not thrown, but not() is no longer exported from testthat
  # so instead testing that the object returned is of 
  # class nexml
})

library(rrdf)
library(phytools)
library(RNeXML)


nexml <- nexml_read(system.file("examples/primates.xml", package="RNeXML"))

# Extract the RDF graph from the nexml
rdf <- get_rdf(system.file("examples/primates.xml", package="RNeXML"))
saveXML(rdf, "rdf_meta.xml") # rrdf requires a file name, so we must write the XML out first
graph <- load.rdf("rdf_meta.xml")


# fetch the NCBI URI for the taxon that has rank 'Order', i.e. the root of the primates. The dot operator
# '.' between clauses implies a join, in this case
root <- sparql.rdf(graph, 
    "SELECT ?uri WHERE { 
        ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#rank> <http://rs.tdwg.org/ontology/voc/TaxonRank#Order> . 
        ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> ?uri    
    }"               
)

# Define a function to get the name 
get_name <- function(id) {
    max <- length(nexml@otus[[1]]@otu)
    for(i in 1:max) {
        if ( nexml@otus[[1]]@otu[[i]]@id == id ) {
            label <- nexml@otus[[1]]@otu[[i]]@label
            label <- gsub(" ","_",label)
            return(label)
        }
    }
}

# define a recursive function to build newick
recurse <- function(node){
  
    # fetch the taxonomic rank and id string
    rank_query <- paste0(
        "SELECT ?rank ?id WHERE {
            ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> <",node,"> .
            ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#rank> ?rank
          }")
    result <- sparql.rdf(graph, rank_query)
    
    # get the local ID, strip URI part
    id <- result[2]
    id <- gsub("^.+#", "", id, perl = TRUE)
    
    # if rank is terminal, return the name
    if (result[1] == "http://rs.tdwg.org/ontology/voc/TaxonRank#Species") {
        return(get_name(id))
    }
    
    # recurse deeper
    else {
        child_query <- paste0(
            "SELECT ?uri WHERE {
                ?id <http://www.w3.org/2000/01/rdf-schema#subClassOf> <",node,"> .
                ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> ?uri
            }")
        children <- sparql.rdf(graph, child_query)
        
        # the newick can be made to contain interior node labels by inserting get_name(id), before the sep="" argument
        return(paste("(", 
                     paste(sapply(children, recurse), 
                           sep = ",", collapse = "," ), 
                     ")",  sep = "", collapse = ""))
    }
}

# build the tree and visualize it
newick <- paste(recurse(root), ";", sep = "", collapse = "")
tree <- read.newick(text = newick)
collapsed <- collapse.singles(tree)
plot(collapsed, type = "cladogram")

#' Get character data.frame from nexml
#' 
#' @param nex a nexml object 
#' @param rownames_as_col option to return character matrix rownames (with taxon ids) as it's own column in the 
#' data.frame. Default is FALSE for compatibility with geiger and similar packages.
#' @param otu_id logical, default FALSE. return a column with the 
#'  otu id (for joining with otu metadata, etc)
#' @param otus_id logical, default FALSE. return a column with the 
#'  otus block id (for joining with otu metadata, etc)
#' @param include_state_types logical, default FALSE. whether to also return a 
#'  matrix of state types (with values standard, polymorphic, and uncertain)
#' @return the character matrix as a data.frame, or if `include_state_types` is
#'  TRUE a list of two elements, `characters` as the character matrix, and
#'  `state_types` as a matrix of state types. Both matrices will be in the same
#'  ordering of rows and columns.
#' @details RNeXML will attempt to return the matrix using the NeXML taxon (otu) labels to name the rows
#'  and the NeXML char labels to name the traits (columns).  If these are unavailable or not unique, the NeXML
#'  id values for the otus or traits will be used instead.
#' @importFrom tidyr spread
#' @importFrom dplyr left_join select mutate mutate_at matches any_of
#' @importFrom stringr str_replace
#' @importFrom stats setNames
#' @export
#' @examples
#' \dontrun{
#' # A simple example with a discrete and a continous trait
#' f <- system.file("examples", "comp_analysis.xml", package="RNeXML")
#' nex <- read.nexml(f)
#' get_characters(nex)
#' 
#' # A more complex example -- currently ignores sequence-type characters
#' f <- system.file("examples", "characters.xml", package="RNeXML")
#' nex <- read.nexml(f)
#' get_characters(nex)
#' 
#' # if polymorphic or uncertain states need special treatment, request state
#' # types to be returned as well:
#' f <- system.file("examples", "ontotrace-result.xml", package="RNeXML")
#' nex <- read.nexml(f)
#' res <- get_characters(nex, include_state_types = TRUE)
#' row.has.p <- apply(res$state_types, 1, 
#'                    function(x) any(x == "polymorphic", na.rm = TRUE))
#' col.has.p <- apply(res$state_types, 2, 
#'                    function(x) any(x == "polymorphic", na.rm = TRUE))
#' res$characters[row.has.p, col.has.p, drop=FALSE] # polymorphic rows and cols
#' res$characters[!row.has.p, drop=FALSE] # drop taxa with polymorphic states
#' # replace polymorphic state symbols in matrix with '?'
#' m1 <- mapply(function(s, s.t) ifelse(s.t == "standard", s, "?"), 
#'              res$characters, res$state_types)
#' row.names(m1) <- row.names(res$characters)
#' m1
#' }
get_characters <- function(nex, 
                           rownames_as_col=FALSE, otu_id=FALSE, otus_id=FALSE,
                           include_state_types=FALSE) {
  
  drop = c("about","xsi.type","format")
  
  otus <- get_level(nex, "otus/otu") %>% 
    dplyr::select(!any_of(drop)) %>%
    optional_labels(id_col = "otu")
  
  char <- get_level(nex, "characters/format/char") %>% 
    dplyr::select(!any_of(drop)) %>%
    optional_labels(id_col = "char")
  
  ## Rows have otu information
  rows <- get_level(nex, "characters/matrix/row") %>% 
    dplyr::select(c("otu", "row"))

  cellLevel <- get_level(nex, "characters/matrix/row/cell")
  cells <- cellLevel %>% 
    dplyr::select(c("char", "state", "row")) %>%
    dplyr::left_join(rows, by = "row")
    
  characters <- get_level(nex, "characters")
  
  ## States, not including polymorphic or uncertain states
  states <- get_level(nex, "characters/format/states/state") %>%
    dplyr::mutate(state.type = "standard")
  
  ## Include polymorphic and uncertain states.
  polymorph <- get_level(nex, "characters/format/states/polymorphic_state_set")
  uncertain <- get_level(nex, "characters/format/states/uncertain_state_set")
  if(dim(polymorph)[1] > 0) {
    ## For the result to work correctly for joining, the ID column needs to
    ## be uniformly named "state".
    polymorph <- polymorph %>%
      dplyr::mutate(state.type = "polymorphic") %>%
      dplyr::rename(state = "polymorphic_state_set")
    # StandardStates have integer symbols, but polymorphic state sets typically
    # do not. If states are DNA, then states will already be character type, and
    # then changing to character should presumably have no effect.
    states <- states %>% dplyr::mutate_at(c("symbol"), as.character) %>%
                         dplyr::bind_rows(polymorph)
  }
  if(dim(uncertain)[1] > 0) {
    ## Same issue for uncertain states.
    uncertain <- uncertain %>%
      dplyr::mutate(state.type = "uncertain") %>%
      dplyr::rename(state = "uncertain_state_set")
    states <- states %>% dplyr::mutate_at(c("symbol"), as.character) %>%
                         dplyr::bind_rows(uncertain)
  }
  states <- dplyr::select(states, !any_of(drop))


  if(dim(states)[1] > 0) 
    cells <- cells %>% 
    dplyr::left_join(states, by = c("state"))  %>% 
    dplyr::select(c("char", "symbol", "otu", "state", "state.type"))
  
  ## Join the matrices.  Note that we select unique column names after each join to avoid collisions
  cells <- cells %>% 
    dplyr::left_join(char, by = c("char")) %>% 
    dplyr::rename(c("trait" = "label")) %>% 
    dplyr::left_join(otus, by = c("otu")) %>%
    dplyr::rename(c("taxa" = "label"))
  # character state matrix with symbols:
  cells %>%
    na_symbol_to_state() %>% 
    dplyr::select(c("taxa", "symbol", "trait", "otu", "otus")) %>% 
    tidyr::spread("trait", "symbol") ->
    out
  # state type matrix
  if(dim(states)[1] == 0) {
    # this can be the case for example for continuous states
    cells <- dplyr::mutate(cells, state.type = "standard")
  }
  cells %>%
      dplyr::select(c("taxa", "state.type", "trait", "otu", "otus")) %>%
      dplyr::mutate_at("state.type", as.factor) %>%
      tidyr::spread("trait", "state.type") ->
      state_types

  ## Identify the class of each column and reset it appropriately
  cellclass <- function(x){
    x %>%
    stringr::str_replace(".*ContinuousCells", "numeric") %>%
    stringr::str_replace(".*StandardCells", "integer")
  }
  
  type <-
    cellLevel  %>% 
    dplyr::select(!any_of(drop)) %>%
    dplyr::left_join(characters, by = "characters") %>%
    dplyr::select(c( "xsi.type", "char", "characters")) %>%
    dplyr::left_join(char, by = c("char")) %>%
    dplyr::select(c("label", "xsi.type")) %>% 
    dplyr::distinct() %>%
    dplyr::mutate(class = cellclass(xsi.type))
  
  for(i in 1:dim(type)[1]) {
    if (all(state_types[[type$label[i]]] == "standard", na.rm = TRUE)) {
      class(out[[type$label[i]]]) <- type$class[i]
    }
  }
  
  ## drop unwanted columns if requested (default)
  if(!otu_id){
    out <- dplyr::select(out, !"otu")
    if (include_state_types)
      state_types <- dplyr::select(state_types, !"otu")
  }
  if(!otus_id){
    out <- dplyr::select(out, !"otus")
    if (include_state_types)
      state_types <- dplyr::select(state_types, !"otus")
  }
  if(!rownames_as_col){
    taxa <- out$taxa
    out <- as.data.frame(dplyr::select(out, !"taxa"))
    rownames(out) <- taxa
    if (include_state_types) {
      state_types <- as.data.frame(dplyr::select(state_types, !"taxa"))
      rownames(state_types) <- taxa
    }
  }
  if (include_state_types) {
    out <- list(characters = out, state_types = state_types)
  }
  
  out
}

## If 'label' column is missing, create it from 'id' column
## if label exists but has missing or non-unique values, also use ids instead
optional_labels <- function(df, id_col = "id"){
  who <- names(df)
  if(! "label" %in% who)
    df$label <- df[[id_col]]
  if(length(unique(df$label)) < length(df$label))
    df$label <- df[[id_col]]
  df
}

## Continuous traits have the values in "state" column, whereas 
## for discrete states we return the value of the "symbol" column 
na_symbol_to_state <- function(df){
  if(is.null(df$symbol))
    df$symbol <- NA
  df$symbol[is.na(df$symbol)] <- suppressWarnings(as.numeric(df$state[is.na(df$symbol)]))
  df
  }

## silence R CHECK warning
xsi.type <- NULL

context("get_characters")

test_that("Getting characters", {

  f <- system.file("examples", "comp_analysis.xml", package="RNeXML")
  nex <- read.nexml(f)
  out <- get_characters(nex)
  expect_is(out, "data.frame")
  
})

test_that("get_characters can deal with polymorphic character states", {
  f <- system.file("examples", "ontotrace-result.xml", package="RNeXML")
  nex <- read.nexml(f)
  m <- get_characters(nex,
                      rownames_as_col = TRUE,
                      otu_id = TRUE,
                      otus_id = TRUE)
  expect_is(m, "data.frame")
  expect_gt(ncol(m), 3)
  expect_true(any(is.na(m[, 4])))
  expect_false(all(is.na(m[, 4])))
  expect_is(m[, 4], "integer")
  expect_is(m[, ncol(m)], "character")
  expect_false(any(is.na(m[, ncol(m)])))
  expect_gt(max(nchar(m[, ncol(m)])), 1)
})

test_that("get_characters can return state type matrix", {
  f <- system.file("examples", "ontotrace-result.xml", package="RNeXML")
  nex <- read.nexml(f)
  ret <- get_characters(nex, include_state_types = TRUE)
  expect_is(ret, "list")
  expect_is(ret$characters, "data.frame")
  expect_is(ret$state_types, "data.frame")
  expect_equal(dim(ret$characters), dim(ret$state_types))
  expect_true(all(sapply(ret$state_types, is.factor)))
  expect_gt(sum(ret$state_types == "polymorphic", na.rm = TRUE), 1)
  expect_true(all(is.na(ret$characters) == is.na(ret$state_types)))
})

## Should be all element names (unless they are also attribute names!), since we only want attribute names 
## (alternately we should define to only grab possible attribute names..)
SKIP = c("meta", "children", "member", "row", "cell", "seq", "matrix", "format", "names")


#' get_level
#' 
#' get a data.frame of attribute values of a given node
#' 
#' @param nex a nexml object
#' @param level a character vector indicating the class of node, see details
#' 
#' @return Returns the attributes of specified class of nodes as a data.frame
#' 
#' @details level should be a character vector giving the path to the specified node
#' group.  For instance, `otus`, `characters`, and `trees` are top-level blocks (e.g.
#' child nodes of the root nexml block), and can be specified directly.  To get metadata
#' for all "char" elements from all characters blocks, you must specify that `char` nodes
#' are child nodes to `character` nodes: e.g. `get_level(nex, "characters/char")`,
#' or similarly for states: `get_level(nex, characters/states)`.  
#' 
#' The return object is a data frame whose columns are the attribute names of the elements
#' specified. The column names match the attribute names except for "id" attribute, for which the column
#' is renamed using the node itself. (Thus `<otus id="os2">` would be rendered in a data.frame with column
#' called "otus" instead of "id"). Additional columns are
#' added for each parent element in the path; e.g. `get_level(nex, "otus/otu")` would include a column
#' named "otus" with the id of each otus block.  Even though the method always returns the data frame 
#' for all matching nodes in all blocks, these ids let you see which otu values came from which 
#' otus block.  This is identical to the function call `get_taxa()`.  
#' Similarly, `get_level(nex, "otus/otu/meta")` would return additional columns 'otus' and 
#' also a column, 'otu', with the otu parent ids of each metadata block.  (This is identical to a 
#' function call to `get_metadata`).  This makes it easier to join data.frames as well, see examples
#' 
#' @export
#' @importFrom dplyr select
get_level <- function(nex, level){
  lvl <- strsplit(level, "/")[[1]]
  out <- recursion(1, lvl)(nex) %>% 
    dplyr::select(!"nexml")

  ## drop columns that are all-na?
#  all_na <- sapply(out, function(x) all(is.na(x)))
#  out <- out[!all_na]  
  out
}


## Trick to apply nodelist_to_df iteratively
closure <- function(level, fun) 
  function(node) nodelist_to_df(node, level, fun)

recursion <- function(i, level){
  if(i < length(level))
    closure(level[i], recursion(i+1, level))
  else
    closure(level[i], attributes_to_row)
}



## Assumes slot(node, element) is a list
#' @importFrom dplyr bind_rows coalesce mutate %>%
#' @importFrom rlang :=
#' @importFrom uuid UUIDgenerate
nodelist_to_df <- function(node, element, fn, nodeId=NA){
  idRefCol <- idRefColName(node)
  if (is.na(nodeId))
    nodeId <- node_id(node)
  nodelist <- slot(node, element)
  if(is.list(nodelist)){ ## node has a list of elements
    out <- suppressWarnings(lapply(nodelist, fn)) %>% dplyr::bind_rows()
    if (length(nodelist) > 0 && all(sapply(nodelist, is, "meta"))) {
      # meta elements may have nested meta elements, retrieve these here too
      ids <- sapply(nodelist,
                    function(n) if (length(n@children) > 0) uuid::UUIDgenerate() else NA)
      if (length(ids) > 0 && any(!is.na(ids))) {
        mout <- dplyr::mutate(out, "Meta" = coalesce_(out$LiteralMeta,
                                                      out$ResourceMeta,
                                                      ids))
        nested <- mapply(function(n, id) nodelist_to_df(n, "children", fn, id),
                         n = nodelist,
                         id = mout[,"Meta"])
        if (is.null(names(nested))) nested <- as.data.frame(nested[,1])
        out <- dplyr::bind_rows(mout, unname(nested))
      }
    }
    dplyr::mutate(out, "{idRefCol}" := {{ nodeId }}) -> out
  } else { ## handle case when node has only one element
    fn(nodelist) %>%
      dplyr::mutate("{idRefCol}" := {{ nodeId }})
  }
}

node_id <- function(node){
  if("id" %in% slotNames(node))
    slot(node, "id")
  else
    "root"
}

idRefColName <- function(node){
  clname <- class(node)
  super <- names(getClass(clname)@contains)
  # meta elements can be nested, avoid clobbering the ID column with the IDREF
  if (length(super) > 0 && (super[1] == "meta"))
    super[1]
  else
    clname
}

#' @importFrom rlang :=
#' @importFrom dplyr rename
attributes_to_row <- function(node){
  who <- slotNames(node)
  
  ## Avoid things that are not attributes:
  types <- getSlots(class(node)) # actual slot values may be derived types
  who <- who[ types %in% c("character", "integer", "numeric", "logical") ]
  # the "about" attribute is only needed for RDF extraction
  who <- who[!(who %in% c("names","about"))]
  
  ## Extract attributes, use NAs for numeric(0) / character(0) values
  tmp <- sapply(who, function(x) slot(node, x))
  tmp[sapply(tmp,length) < 1] <- NA
  
  ## Coerce into a row of a data.frame & rename id column to match class
  out <- data.frame(as.list(tmp), stringsAsFactors=FALSE) 
  if("id" %in% who)
    out <- dplyr::rename(out, "{class(node)}" := "id")
  
  out
}

#' new with namespaced class name
#'
#' Convenience function for [methods::new()] that ensures that the provided
#' class name is namespaced with a package name.
#'
#' If the provided class name is not already namespaced (see
#' `methods::packageSlot()`), it will be namespaced with this package. This
#' mechanism is used by `new()` to disambiguate if the class name clashes
#' with a class defined in another package.
#' @note This may not completely eliminate messages on standard error about
#'   classes with the same name having been found in different packages. If
#'   they appear, they will most likely have come from the call to the
#'   `methods::initialize()` generic that `new()` issues at the end.
#' @param Class the name of the S4 class to be instantiated
#' @param ... additional parameters for `methods::new()`
#' @importFrom methods new className packageSlot
New <- function(Class, ...) {
  if (is.null(packageSlot(Class))) Class <- className(Class, "RNeXML")
  methods::new(Class, ...)
}

#' Constructor for the respective class
#'
#' Creates an instance of the class corresponding to the respective NeXML
#' element, and initializes its slots with the provided parameters, if any.
#'
#' Usually, users won't need to invoke this directly.
#' @param ... optionally, parameters passed on to [`new()`][methods::new()]
#' @rdname constructors
#' @export
nexml.tree <- function(...) New("tree", ...)

#' @rdname constructors
#' @export
nexml.trees <- function(...) New("trees", ...)

#' @rdname constructors
#' @export
nexml.node <- function(...) New("node", ...)

#' @rdname constructors
#' @export
nexml.edge <- function(...) New("edge", ...)

#' @rdname constructors
#' @export
nexml.otu <- function(...) New("otu", ...)

#' @rdname constructors
#' @export
nexml.otus <- function(...) New("otus", ...)

#' @rdname constructors
#' @export
nexml.char <- function(...) New("char", ...)

#' @rdname constructors
#' @export
nexml.characters <- function(...) New("characters", ...)

#' @rdname constructors
#' @export
nexml.format <- function(...) New("format", ...)

#' @rdname constructors
#' @export
nexml.state <- function(...) New("state", ...)

#' @rdname constructors
#' @export
nexml.uncertain_state <- function(...) New("uncertain_state", ...)

#' @rdname constructors
#' @export
nexml.states <- function(...) New("states", ...)

#' @rdname constructors
#' @export
nexml.uncertain_states <- function(...) New("uncertain_state_set", ...)

#' @rdname constructors
#' @export
nexml.polymorphic_states <- function(...) New("polymorphic_state_set", ...)

#' @rdname constructors
#' @export
nexml.member <- function(...) New("member", ...)

#' @rdname constructors
#' @export
nexml.matrix <- function(...) New("obsmatrix", ...)

#' @rdname constructors
#' @export
nexml.row <- function(...) New("row", ...)

#' @rdname constructors
#' @export
nexml.seq <- function(...) New("seq", ...)

#' @rdname constructors
#' @export
nexml.cell <- function(...) New("cell", ...)

context("inheritance")

## FIXME
## Should include expect_that tests, rather than just running without errors.  
## ADD test to show that toggling xml->s4->xml returns IDENTICAL objects, 
## Add tests to check values on some nodes/attributes...

test_that("we can perform simple conversions between NeXML XML and S4", {
  # basic example
  node <- newXMLNode("meta", 
                     attrs = c('xsi:type'="nex:LiteralMeta",
                               id="dict1",
                               property="cdao:has_tag",
                               datatype="xsd:boolean",
                               content="true"), 
                     suppressNamespaceWarning=TRUE)
  n2 <- newXMLNode("node", 
                   attrs = c(about="#n4",
                             label="n4", 
                             id = "n4"), 
                   .children = node)
  
  # check conversions to/from NeXML
  s4 <- as(n2, "node")
  xmlfroms4 <- as(s4, "XMLInternalNode")
  
##  expect_identical(n2, xmlfroms4) #cannot compare two external pointers
  expect_identical(saveXML(n2), saveXML(xmlfroms4))

})

# test_that("We can parse a complete NeXML file and toggle back and forth between XML and S4", {
test_that("Parse a complete NeXML file to a single otu", {
  doc <- xmlParse(system.file("examples", "trees.xml", package="RNeXML"))
  root <- xmlRoot(doc)

  otu <- as(root[["otus"]][[1]], "otu")
  expect_that(otu, is_a("otu"))
  as(otu, "XMLInternalNode")
})


doc <- xmlParse(system.file("examples", "trees.xml", package="RNeXML"))
root <- xmlRoot(doc)



test_that("Parse a complete NeXML file to trees", {
  trees <- as(root[["trees"]], "trees")
  expect_that(trees, is_a("trees"))
  as(trees, "XMLInternalNode")
})

test_that("Parse a complete NeXML file to many otus", {
  otus <- as(root[["otus"]], "otus")
  expect_that(otus, is_a("otus"))
  tt <- as(otus, "XMLInternalNode")
  expect_that(tt, is_a("XMLInternalNode"))
})

test_that("Parse a complete NeXML file to xmlinternalnode", {
  parsed <- as(root, "nexml")
  expect_that(parsed, is_a("nexml"))
  serialized <- as(parsed, "XMLInternalNode")
  expect_that(serialized, is_a("XMLInternalNode"))
})

test_that("Check that values are correct in the otu class element", {
  otu <- as(root[["otus"]][[1]], "otu")
  expect_that(otu@id[[1]], equals("t1"))
  expect_that(otu@label[[1]], equals("species 1"))
  expect_that(otu@meta, is_a("list"))
  expect_that(otu@about, is_identical_to(character(0)))
})

test_that("Check that values are correct in the trees class element", {
  trees <- as(root[["trees"]], "trees")
  expect_that(trees@tree, is_a("ListOftree"))
  expect_that(trees@otus[[1]], equals("tax1"))
  expect_that(trees@id[[1]], equals("Trees"))
  expect_that(trees@label[[1]], equals("TreesBlockFromXML"))
  expect_that(trees@meta, is_a("list"))
  expect_that(trees@about, is_identical_to(character(0)))
})

test_that("Check that values are correct in the otus class element", {
  otus <- as(root[["otus"]], "otus")
  expect_that(otus@otu, is_a("ListOfotu"))
  expect_that(otus@id[[1]], equals("tax1"))
  expect_that(otus@label[[1]], equals("RootTaxaBlock"))
  expect_that(otus@meta, is_a(class=c("list","ListOfmeta")))
  expect_that(otus@about, is_identical_to(character(0)))
})

test_that("Checking whether an edge is a root edge is correct", {
  e <- nexml.edge()
  e.xml <- as(e, "XMLInternalNode")
  expect_is(e, "edge")
  expect_false(is(e, "rootEdge"))
  expect_identical(xmlName(e.xml), "edge")
  expect_named(xmlAttrs(e.xml), expected = c("source", "target"))

  e@target <- "foo"
  e.xml <- as(e, "XMLInternalNode")
  expect_true(is(e, "rootEdge"))
  expect_identical(xmlName(e.xml), "rootedge")
  expect_named(xmlAttrs(e.xml), expected = c("target"))

  e@source <- "bar"
  e.xml <- as(e, "XMLInternalNode")
  expect_false(is(e, "rootEdge"))
  expect_identical(xmlName(e.xml), "edge")
  expect_named(xmlAttrs(e.xml), expected = c("source", "target"))
})

## ----compile-settings, include=FALSE-------------------------------------
library("methods")
library("knitr")
opts_chunk$set(tidy = FALSE, warning = FALSE, message = FALSE, 
               cache = FALSE, comment = NA, verbose = TRUE)
basename <- gsub(".Rmd", "", knitr:::knit_concord$get('infile')) 
library("RNeXML")


## ------------------------------------------------------------------------
 m <- meta("simmap:reconstructions", children = c(
        meta("simmap:reconstruction", children = c(

          meta("simmap:char", "cr1"),
          meta("simmap:stateChange", children = c(
            meta("simmap:order", 1),
            meta("simmap:length", "0.2030"),
            meta("simmap:state", "s2"))),
          
          meta("simmap:char", "cr1"),
          meta("simmap:stateChange", children = c(
            meta("simmap:order", 2),
            meta("simmap:length", "0.0022"),
            meta("simmap:state", "s1")))
          ))))

## ------------------------------------------------------------------------
nex <- add_namespaces(c(simmap = "https://github.com/ropensci/RNeXML/tree/master/inst/simmap.md"))

## ------------------------------------------------------------------------
data(simmap_ex)

## ------------------------------------------------------------------------
phy <- nexml_to_simmap(simmap_ex)

## ----Figure1, fig.cap="Stochastic character mapping on a phylogeny, as generated by the phytools package after parsing the simmap-extended NeXML."----
library("phytools")
plotSimmap(phy)

## ------------------------------------------------------------------------
nex <- simmap_to_nexml(phy) 
nexml_write(nex, "simmap.xml")

## ----cleanup, include=FALSE----------------------------------------------
unlink("simmap.xml")

## utility functions, presumably used mostly or entirely internally

#' Compact list then lapply
#'
#' Compacts the list (i.e., removes NULL objects), then calls [`lapply()`][base::lapply()]
#' on the result with the remaining parameters.
#' @param X the list object
#' @param ... remaining arguments to `lapply()`
#' @importFrom plyr compact
lcapply <- function(X, ...) {
  X <- plyr::compact(X)
  lapply(X, ...)
}

#' Front-end to dplyr::coalesce to deal with NULL vectors
#'
#' Replaces any NULL argument with a vector of `NA`, and casts every vector
#' to the same type as the last vector. After that, calls [dplyr::coalesce()].
#' @param ... the vectors to coalesce on NA
#' @return a vector of the same type and length as the last argument
#' @seealso [dplyr::coalesce()]
#' @importFrom dplyr coalesce
coalesce_ <- function(...) {
  idvecs <- list(...)
  idvecs <- lapply(idvecs,
                   function(v, template) {
                     if (is.null(v))
                       as(rep(NA, times=length(template)), typeof(template))
                     else
                       as(v, typeof(template))
                   },
                   template = idvecs[[length(idvecs)]])
  dplyr::coalesce(!!!idvecs)
}

#' Treats zero-length character vectors as empty strings
#'
#' If the argument is a zero-length character vector (character(0)), returns
#' an empty string (which is a character vector of length 1). Otherwise passes
#' through the argument.
#' @param x the object to be tested for zero-length character vector
#' @return an empty string if `x` is a character vector of length zero, and `x`
#'   otherwise
charzero_as_empty <- function(x) {
  if (is.character(x) && length(x) == 0)
    ""
  else
    x
}

# rlang is used in classes.R to deal with the incorrect way in which
# the S4 system handles class collisions. 
# R check incorrectly fails to recognize it there and objects to rlang
# being in Imports. 
# So here's another cludgy hack
dummy <- function() {
  rlang::as_character(1)
}

#' Write nexml files
#' 
#' @param x a nexml object, or any phylogeny object (e.g. phylo, phylo4) 
#' that can be coerced into one. Can also be omitted, in which case a new 
#' nexml object will be constructed with the additional parameters specified.
#' @param file the name of the file to write out
#' @param trees phylogenetic trees to add to the nexml file (if not already given in x)
#' see \code{\link{add_trees}} for details.  
#' @param characters additional characters
#' @param meta A meta element or list of meta elements, see \code{\link{add_meta}}
#' @param ... additional arguments to add__basic_meta, such as the title.  See \code{\link{add_basic_meta}}.   
#' @return Writes out a nexml file
#' @import ape
#' @import XML 
#' @import methods
#' @aliases nexml_write write.nexml
#' @export nexml_write write.nexml
#' @seealso \code{\link{add_trees}} \code{\link{add_characters}} \code{\link{add_meta}} \code{\link{nexml_read}}

#' @examples
#'  ## Write an ape tree to nexml, analgous to write.nexus:
#'  library(ape); data(bird.orders)
#'  ex <- tempfile(fileext=".xml")
#'  write.nexml(bird.orders, file=ex)
nexml_write <- function(x = nexml(),
                        file = NULL,
                        trees = NULL,
                        characters = NULL,
                        meta = NULL, 
                        ...){
  
  nexml <- as(x, "nexml")
  if(!is.null(trees))
    nexml <- add_trees(trees, nexml = nexml)
  if(!is.null(characters))
    nexml <- add_characters(characters, nexml = nexml)
  if(!is.null(meta))
    nexml <- add_meta(meta, nexml = nexml)
  argList <- list(..., nexml = nexml)
  # set last modification time upon writing, if not set already
  if (is.null(argList$pubdate)) argList$pubdate <- Sys.time()
  nexml <- do.call(add_basic_meta, argList)
  
  out <- as(nexml, "XMLInternalNode")
  saveXML(out, file = file)
}

write.nexml <- nexml_write


############## Promotion methods ########
## FIXME -- Coercion is not the way to go about any of this


## want generator methods that can handle id creation better
# consider:
# setMethod("promote", 
#           signature("tree", "character"),
#           function(object, target_type)

setAs("tree", "nexml", function(from){
  trees = as(from, "trees")
  otus = as(from, "otus")
  otus@id = "tax1" #UUIDgenerate()
  trees@id = "Trees" #UUIDgenerate()
  trees@otus = otus@id
  nexml(
      trees = New("ListOftrees", list(trees)),
      otus = otus)
})


setAs("ListOfnode", "otus", function(from) nexml.otus(otu = from))

setAs("tree", "trees", function(from)
  nexml.trees(tree = New("ListOftree", list(from))))

context("concatenate method")


test_that("we can concatenate two files with unique ids", {
 f1 <- system.file("examples", "trees.xml", package="RNeXML")
 f2 <- system.file("examples", "comp_analysis.xml", package="RNeXML")
 nex1 <- read.nexml(f1)
 nex2 <- read.nexml(f2)
 expect_is(c(nex1, nex2), "nexml")

})


test_that("we get an error if the files to be concatenated have non-unique ids", {
 f1 <- system.file("examples", "trees.xml", package="RNeXML")
 nex1 <- read.nexml(f1)
 nex2 <- read.nexml(f1)
 expect_error(c(nex1, nex2),"ids are not unique")
})



test_that("we can concatenate meta elements", {
  out <-  c(meta(content="example", property="dc:title"),
            meta(content="Carl", property="dc:creator"))
  expect_is(out, "ListOfmeta")
  expect_is(out[[1]], "meta")

})


test_that("we can conatenate meta elements with empty ListOfmeta elements", {

  ## Doesn't trigger our method if x is not class `meta`
  out <- c(new("ListOfmeta"), 
           meta(content="example", property="dc:title"),
           meta(content="Carl", property="dc:creator"))

  expect_is(out, "ListOfmeta")
  expect_length(out, 2)
  expect_true(all(sapply(out, is, "meta")))
  
## in any order 
  out <-  c(meta(content="example", property="dc:title"),
            meta(content="Carl", property="dc:creator"),
            new("ListOfmeta"))

  expect_is(out, "ListOfmeta")
  expect_length(out, 2)
  expect_true(all(sapply(out, is, "meta")))

})


test_that("we can conatenate meta elements with ListOfmeta elements", {
out <- c(meta(content="example", property="dc:title"),
         meta(content="Carl", property="dc:creator"))

  out <- c(out, meta("skos:note", "an editorial note"))

  expect_is(out, "ListOfmeta")
  expect_length(out, 3)
  expect_true(all(sapply(out, is, "meta")))

  
  out <- c(meta("skos:note", "another editorial note"), out)

  expect_is(out, "ListOfmeta")
  expect_length(out, 4)
  expect_true(all(sapply(out, is, "meta")))

})


test_that("we can concatenate two ListOfmeta elements", {

  metalist <- c(meta(content="example", property="dc:title"),
                meta(content="Carl", property="dc:creator"))
  out <- c(metalist, metalist) 
  expect_is(out, "ListOfmeta")
  expect_length(out, 4)
  expect_true(all(sapply(out, is, "meta")))
})

test_that("we can concatenate a ListOfmeta and a meta", {

  metalist <- c(meta(content="example", property="dc:title"),
                meta(content="Carl", property="dc:creator"))

  out <- c(metalist, meta(content="a", property="b")) 
  expect_is(out, "ListOfmeta")
  expect_length(out, 3)
  expect_true(all(sapply(out, is, "meta")))

})

test_that("we can concatenate meta and ListOfmeta with ordinary lists", {

  m1 <- meta(content="example", property="dc:title")
  m2 <- meta(content="Carl", property="dc:creator")

  out <- c(meta(content="a", property="b"), list(m1, m2)) 
  expect_is(out, "ListOfmeta")
  expect_length(out, 3)
  expect_true(all(sapply(out, is, "meta")))

  out <- c(c(m1, m2), list(meta(content="a", property="b")))
  expect_is(out, "ListOfmeta")
  expect_length(out, 3)
  expect_true(all(sapply(out, is, "meta")))
})

test_that("NULLs are removed when concatenating meta", {

  m1 <- meta(content="example", property="dc:title")
  m2 <- meta(content="Carl", property="dc:creator")

  out <- c(meta(content="a", property="b"), NULL, list(m1, m2)) 
  expect_is(out, "ListOfmeta")
  expect_length(out, 3)
  expect_true(all(sapply(out, is, "meta")))

  out <- c(c(m1, m2), NULL, list(meta(content="a", property="b")))
  expect_is(out, "ListOfmeta")
  expect_length(out, 3)
  expect_true(all(sapply(out, is, "meta")))
})

test_that("must be list of lists, lists of meta, or meta when concatenating meta", {

  m1 <- meta(content="example", property="dc:title")
  m2 <- meta(content="Carl", property="dc:creator")

  expect_error(c(m1, "foo"))
  expect_error(c(c(m1, m2), "foo"))
  expect_error(c(m1, list("foo")))
  expect_error(c(c(m1, m2), list("foo")))
  expect_silent(out <- c(m1, list(list(list(m1), m2))))
  expect_is(out, "ListOfmeta")
  expect_length(out, 3)
  expect_true(all(sapply(out, is, "meta")))

})

test_that("we can read in a file with existing meta and append without overwriting", 
          {
            f <- system.file("examples", "biophylo.xml", package="RNeXML")
            nex <- nexml_read(f)
            g <- tempfile()
            nexml_write(nex, g)
            nex2 <- nexml_read(g)
            expect_gt(length(get_metadata(nex2)), length(get_metadata(nex)))
          })

## Note: these tests do not run in the typical test suite process as the file name
## doesn't start with "test-"

# nms <- c("S100","S1000","S10000","S10001","S10005","S10006","S10007","S10009","S10014","S10018","S10301","S1044","S10500","S10636","S1064","S1073","S10774","S1207","S13135","S1452","S938","S9981","S999")
# nexml_files <- lapply(nms, function(x) content(GET(sprintf("https://raw.github.com/rvosa/supertreebase/master/data/treebase/%s.xml",x)), as="text"))
# save(nexml_files, file="~/github/ropensci/RNeXML_testfiles/nexml_files.rda")
load("~/github/ropensci/RNeXML_testfiles/nexml_files.rda")

context("nexml files parse correctly")
test_that("nexml files parse correctly", {
  expect_is(nexml_read(nexml_files[[1]]), "multiPhylo")
  expect_is(nexml_read(nexml_files[[2]]), "phylo")
  expect_is(nexml_read(nexml_files[[3]]), "list")
  expect_is(nexml_read(nexml_files[[4]]), "phylo")
  expect_is(nexml_read(nexml_files[[5]]), "list")
  expect_is(nexml_read(nexml_files[[6]]), "list")
  expect_is(nexml_read(nexml_files[[7]]), "phylo")
  expect_is(nexml_read(nexml_files[[8]]), "list")
  expect_is(nexml_read(nexml_files[[9]]), "list")
  expect_is(nexml_read(nexml_files[[10]]), "phylo")
  expect_is(nexml_read(nexml_files[[11]]), "list")
  expect_is(nexml_read(nexml_files[[12]]), "list")
  expect_is(nexml_read(nexml_files[[13]]), "phylo")
  expect_is(nexml_read(nexml_files[[14]]), "phylo")
  expect_is(nexml_read(nexml_files[[15]]), "phylo")
  expect_is(nexml_read(nexml_files[[16]]), "list")
  expect_is(nexml_read(nexml_files[[17]]), "list")
  expect_is(nexml_read(nexml_files[[18]]), "phylo")
  expect_is(nexml_read(nexml_files[[19]]), "phylo")
  expect_is(nexml_read(nexml_files[[20]]), "list")
  expect_is(nexml_read(nexml_files[[21]]), "phylo")
  expect_is(nexml_read(nexml_files[[22]]), "list")
  expect_is(nexml_read(nexml_files[[23]]), "multiPhylo")
})

# puma <- search_treebase('"Puma"', by="taxon")
# save(puma, file="~/github/ropensci/RNeXML_testfiles/puma.rda")
# ursus <- search_treebase('"Ursus"', by="taxon")
# save(ursus, file="~/github/ropensci/RNeXML_testfiles/ursus.rda")
# quercus <- search_treebase('"Quercus"', by="taxon")
# save(quercus, file="~/github/ropensci/RNeXML_testfiles/quercus.rda")

load(file="~/github/ropensci/RNeXML_testfiles/puma.rda")
load(file="~/github/ropensci/RNeXML_testfiles/ursus.rda")
load(file="~/github/ropensci/RNeXML_testfiles/quercus.rda")

context("ape files convert to class nexml correctly")
test_that("ape files convert to class nexml correctly", {
  expect_that(as(puma[[1]], "nexml"), is_a("nexml"))
  expect_that(as(puma[[2]], "nexml"), is_a("nexml"))
  expect_that(as(puma[[3]], "nexml"), is_a("nexml"))
  expect_that(as(puma[[5]], "nexml"), is_a("nexml"))
  expect_that(as(puma[[7]], "nexml"), is_a("nexml"))
  expect_that(as(puma[[9]], "nexml"), is_a("nexml"))
  expect_that(as(puma[[11]], "nexml"), is_a("nexml"))
  
  expect_that(as(ursus[[1]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[2]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[3]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[5]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[7]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[9]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[11]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[13]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[15]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[18]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[22]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[27]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[30]], "nexml"), is_a("nexml"))
  expect_that(as(ursus[[33]], "nexml"), is_a("nexml"))
  
  expect_that(as(quercus[[1]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[2]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[3]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[5]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[7]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[9]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[11]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[20]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[30]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[40]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[50]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[70]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[90]], "nexml"), is_a("nexml"))
  expect_that(as(quercus[[92]], "nexml"), is_a("nexml"))
})

context("ape files write to nexml files correctly")
test_that("ape files write to nexml files correctly, set 1", {
  nexml_write(puma[[1]], "one.xml")
  nexml_write(puma[[5]], "two.xml")
  nexml_write(puma[[9]], "three.xml")
  
  expect_is(nexml_read("~/one.xml"), "phylo")
  expect_is(nexml_read("~/two.xml"), "phylo")
  expect_is(nexml_read("~/three.xml"), "phylo")
})

test_that("ape files write to nexml files correctly, set 2", {
  nexml_write(ursus[[1]], "one_u.xml")
  nexml_write(ursus[[5]], "two_u.xml")
  nexml_write(ursus[[9]], "three_u.xml")
  nexml_write(ursus[[1]], "four_u.xml")
  nexml_write(ursus[[5]], "five_u.xml")
  nexml_write(ursus[[9]], "six_u.xml")
  
  expect_is(nexml_read("~/one_u.xml"), "phylo")
  expect_is(nexml_read("~/two_u.xml"), "phylo")
  expect_is(nexml_read("~/three_u.xml"), "phylo")
  expect_is(nexml_read("~/four_u.xml"), "phylo")
  expect_is(nexml_read("~/five_u.xml"), "phylo")
  expect_is(nexml_read("~/six_u.xml"), "phylo")
})

test_that("ape files write to nexml files correctly, set 3", {
  nexml_write(quercus[[1]], "one_q.xml")
  nexml_write(quercus[[5]], "two_q.xml")
  nexml_write(quercus[[9]], "three_q.xml")
  nexml_write(quercus[[1]], "four_q.xml")
  nexml_write(quercus[[5]], "five_q.xml")
  nexml_write(quercus[[9]], "six_q.xml")
  
  expect_is(as(nexml_read("~/one_q.xml"), "phylo"), "phylo")
  expect_is(as(nexml_read("~/two_q.xml"), "phylo"), "phylo")
  expect_is(as(nexml_read("~/three_q.xml"), "phylo"), "phylo")
  expect_is(as(nexml_read("~/four_q.xml"), "phylo"), "phylo")
  expect_is(as(nexml_read("~/five_q.xml"), "phylo"), "phylo")
  expect_is(as(nexml_read("~/six_q.xml"), "phylo"), "phylo")
})

#' Read NeXML files into various R formats
#' 
#' @param x Path to the file to be read in. An `XML::XMLDocument-class`
#' or \code{\link[XML]{XMLNode-class}} 
#' @param ... Further arguments passed on to \code{\link[XML]{xmlTreeParse}}
#' @import XML
#' @import httr
#' @aliases nexml_read read.nexml 
#' @export nexml_read read.nexml 
#' @examples
#' # file
#' f <- system.file("examples", "trees.xml", package="RNeXML")
#' nexml_read(f)
#' \dontrun{ # may take > 5 s
#' # url
#' url <- "https://raw.githubusercontent.com/ropensci/RNeXML/master/inst/examples/trees.xml"
#' nexml_read(url)
#' # character string of XML
#' str <- paste0(readLines(f), collapse = "")
#' nexml_read(str)
#' # XMLInternalDocument
#' library("httr")
#' library("XML")
#' x <- xmlParse(content(GET(url)))
#' nexml_read(x)
#' # XMLInternalNode
#' nexml_read(xmlRoot(x))
#' }
nexml_read <- function(x, ...) {
  UseMethod("nexml_read")    
}

#' @export
#' @rdname nexml_read
nexml_read.character <- function(x, ...) {
  if (!any(grepl("^https?://", x), 
                XML::isXMLString(x), 
                file.exists(x))) {
    stop("character input must be a URL, xml string or file path", call. = FALSE)
  }
  # handle remote paths using httr::GET
  if (grepl("^https?://", x)) {
    tmp <- GET(x)
    stop_for_status(tmp)
    x <- content(tmp, as = "text")
  }
  doc <- xmlParse(x, ...)
  output <- as(xmlRoot(doc), "nexml")
  free(doc) # explicitly free the pointers after conversion into S4
  return(output)
}

#' @export
#' @rdname nexml_read
nexml_read.XMLInternalDocument <- function(x, ...) {
  as(xmlRoot(x), "nexml")
}

#' @export
#' @rdname nexml_read
nexml_read.XMLInternalNode <- function(x, ...) {
  as(x, "nexml")
}


setAs("XMLInternalNode", "phylo", function(from)
  as(as(from, "nexml"), "phylo")
)

read.nexml <- nexml_read

testthat::context("summary for nexml objects")

test_that("summary() doesn't error on various inputs", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  files <- list.files(path = system.file("examples", package = "RNeXML"),
                      pattern = ".*\\.xml$", full.names = TRUE)
  for (f in files) {
    doc <- XML::xmlParse(f)
    docType <- XML::xmlName(XML::xmlChildren(doc)[[1]])
    if (docType == "nexml") {
      nex <- nexml_read(f)
      s <- summary(nex)
      testthat::expect_is(s, "list")
      testthat::expect_equal(names(s$nblocks), c("trees", "otus", "characters"))
      for (elem in names(s)) {
        if (elem == "nmeta") {
          testthat::expect_is(s[[elem]], "list")
          testthat::expect_length(s[[elem]], 4)
        } else {
          testthat::expect_true(is.integer(s[[elem]]) || is.matrix(s[[elem]]))
        }
      }
      testthat::expect_output(show(nex), regexp = "NeXML")
    }
  }
})

test_that("summary() gives the correct counts", {
  nex <- nexml_read(system.file("examples", "comp_analysis.xml", package = "RNeXML"))
  s <- summary(nex)
  testthat::expect_equivalent(s$nblocks, c(1, 1, 2))
  testthat::expect_true(all(is.na(s$nstates[,1])))
  testthat::expect_true(all(s$nstates[,2] == 2))
  testthat::expect_equivalent(s$nmatrixrows, c(10, 10))
  testthat::expect_equivalent(s$notus, c(10))

  nex <- nexml_read(system.file("examples", "treebase-record.xml", package = "RNeXML"))
  s <- summary(nex)
  testthat::expect_equivalent(s$nblocks, c(1, 1, 1))
  testthat::expect_equal(ncol(s$nstates), 1)
  testthat::expect_true(all(s$nstates[,1] == 4))
  testthat::expect_equivalent(s$nnonstdstatedefs, c(0, 14))
  testthat::expect_equivalent(s$nmatrixrows, c(52))
  testthat::expect_equivalent(s$notus, c(52))

  nex <- nexml_read(system.file("examples", "ontotrace-result.xml", package = "RNeXML"))
  s <- summary(nex)
  testthat::expect_equivalent(s$nblocks, c(1, 1, 1))
  testthat::expect_equivalent(s$ncharacters, c(3))
  testthat::expect_equal(ncol(s$nstates), 1)
  testthat::expect_true(all(s$nstates[,1] == 2))
  testthat::expect_equivalent(s$nnonstdstatedefs, c(2, 0))
  testthat::expect_equivalent(s$nmatrixrows, c(9))
  testthat::expect_equivalent(s$nmeta$otu, c(10))
  testthat::expect_equivalent(s$nmeta$char, c(3))
})

context("read nexml")

f <- system.file("examples", "trees.xml", package = "RNeXML")
url <- "https://raw.githubusercontent.com/ropensci/RNeXML/master/inst/examples/trees.xml"

test_that("we can read nexml from a file path", {
  nex <- nexml_read(f)
  
  expect_is(nex, "nexml")
  expect_equal(nex@trees@names, "trees")
})

test_that("we can read nexml from a url", {
  nex <- read.nexml(url)
  
  expect_is(nex, "nexml")
  expect_equal(nex@trees@names, "trees")
})

test_that("we can read nexml from a character string of xml", {
  str <- paste0(readLines(f), collapse = "")
  nex <- nexml_read(str)
  
  expect_is(nex, "nexml")
  expect_equal(nex@trees@names, "trees")
})

test_that("we can read nexml from a XMLInternalDocument object", {
  library("XML")
  x <- xmlParse(f)
  nex <- nexml_read(x)

  expect_is(nex, "nexml")
  expect_equal(nex@trees@names, "trees")
})

test_that("we can read nexml from a XMLInternalNode object", {
  library("XML")
  x <- xmlParse(f)
  nex <- nexml_read(xmlRoot(x))

  expect_is(nex, "nexml")
  expect_equal(nex@trees@names, "trees")
})

test_that("alias for nexml_read works", {
  nex <- read.nexml(f)
  
  expect_is(nex, "nexml")
  expect_equal(nex@trees@names, "trees")
})

## utilities to better deal with S4 classes and generics

#' Finds the method that callNextMethod() should chain to
#'
#' Attempts to find the "next" method in the inheritance chain. This would
#' (ideally) be the method that `methods::callNextMethod()` would chain to,
#' as a result of the method `methods::addNextMethod()` would find (and return
#' in the `nextMethod` slot of the `MethodWithNext`
#' object). Hence, in theory one shouldn't ever need this, but unfortunately
#' `addNextMethod()` is broken (and errors out) if one of the classes in the
#' signature name-clashes with an S4 class defined in another package that is
#' loaded.
#'
#' The next method will be determined by the S4 inheritance chain. However,
#' this function will walk only the inheritance chain of those arguments in
#' the signature that are defined in the package of the generic method from
#' which this function was invoked (directly or indirectly). If there are
#' no such parameters in the signature, or if there is more than one,
#' finding the next method is handed off to `methods::addNextMethod()`.
#' @note In theory a class name clash between packages shouldn't be a problem
#'   because class names can be namespaced, and the `MethodDefinition`
#'   object passed to `addNextMethod()` has all the necessary namespace
#'   information. Hopefully, at some point this gets fixed in R, and then we
#'   don't need this anymore.
#' @param method `MethodDefinition`, the method for which to find
#'   the next method
#' @param f `standardGeneric`, the standard generic for which to find
#'   the next method. By default this will be obtained from `method`.
#' @param envir the environment in which to find the method
#' @return a `MethodDefinition` object that is the next method in the
#'   chain by inheritance
#' @importFrom methods getClassDef selectMethod addNextMethod
#' @importFrom utils packageName
findNextMethod <- function(method, f = NULL, envir = topenv()) {
  if (is(method, "MethodWithNext"))
    method@nextMethod
  else {
    if (is.null(f)) f <- method@generic
    sigDefined <- method@defined
    nextMethod <- NULL
    ourPkg <- packageName(env = envir)
    if (is.null(ourPkg)) {
      warning("failed to determine package where findNextMethod() was called")
      ourPkg <- "RNeXML"
    }
    isOurs <- sigDefined@package == ourPkg
    if (sum(isOurs) == 1) {
      clnames <- mapply(className,
                        sigDefined@.Data,
                        sigDefined@package,
                        SIMPLIFY = FALSE)
      names(clnames) <- sigDefined@names
      ourClass <- clnames[isOurs][[1]]
      super <- getClassDef(ourClass)@contains
      for (clExt in super) {
        superClassName <- className(clExt@superClass, clExt@package)
        sig <- clnames
        sig[[sigDefined@names[isOurs]]] <- superClassName
        nextMethod <- selectMethod(f, signature = sig, optional = TRUE)
        if (! is.null(nextMethod)) {
          nextMethod@target <- method@defined
          break
        }
      }
    }
    if (is.null(nextMethod)) {
      method <- addNextMethod(method, f, envir = envir)
      nextMethod <- method@nextMethod
    }
    nextMethod
  }
}

#' Saves the next method in the method meta data
#'
#' Promotes the given method definition to an instance of
#' `MethodWithNext`, thereby recording the next
#' method in the `nextMethod` slot.
#' @note `MethodWithNext` objects are normally returned by
#'   `methods::addNextMethod()`, but a constructor function for the class
#'   seems missing (or is undocumented?). This provides one.
#' @param method the `MethodDefinition` object to promote
#' @param nextMethod the `MethodDefinition`
#'   object to record as the next method
#' @param .cache whether to cache the promoted method definition object
#'   (using `methods::cacheMethod()`)
#' @return an instance of `MethodWithNext`,
#'   which has the next method in the `nextMethod` slot
#' @importFrom methods getGeneric cacheMethod
#' @importClassesFrom methods MethodWithNext
.methodWithNext <- function(method, nextMethod, .cache = FALSE) {
  methodWithNext <- new("MethodWithNext",
                        method,
                        nextMethod = nextMethod,
                        excluded = list(.sigLabel(method@defined)))
  if (.cache) {
    cacheMethod(method@generic,
                method@target,
                methodWithNext,
                fdef = getGeneric(method@generic),
                inherited = TRUE)
  }
  methodWithNext
}

#' Create a label for a method signature
#'
#' Creates a label for a signature mirroring the result of `.sigLabel()`
#' in the `methods` package, which unfortunately does not export the function.
#' This is needed, for example, for the `excluded` slot in the
#' `MethodWithNext` class.
#' @param signature the signature for which to create a label, as a vector
#'   or list of strings, or as an instance of `signature`.
#' @return a character string
.sigLabel <- function(signature) {
  if (is(signature, "signature")) signature <- signature@.Data
  paste(signature, collapse = "#")
}

#' Caches next method in the calling environment
#'
#' If the calling environment does not have the next method to be invoked
#' in the inheritance chain cached yet, this will find the next method
#' (using `findNextMethod()`, and cache it in the calling environment such
#' that a subsequent call to `methods::callNextMethod()` will find and use
#' it.
#'
#' As per the description, what this function does would normally already
#' be done by invoking `methods::callNextMethod()`, so in theory this should
#' be entirely redundant at best. However, `methods::addNextMethod()`, which
#' is invoked by `callNextMethod()` if a next method isn't cached yet, is
#' broken (errors out) if one of the classes in the signature name-clashes
#' with a class defined in another package. Calling this function prior to
#' `callNextMethod()` is meant to work around that.
.cacheNextMethod <- function() {
  callEnv <- parent.frame(1L)
  nextMethod <- callEnv$.nextMethod
  if (is.null(nextMethod)) {
    parent <- sys.parent(1)
    methodFun <- sys.function(parent)
    if (! is(methodFun, "MethodDefinition"))
      stop(".cacheNextMethod() not invoked from class method")
    method <- callEnv$.Method
    if (is.null(method)) method <- methodFun
    f <- callEnv$.Generic
    if (is.null(f)) f <- method@generic
    nextMethod <- findNextMethod(method, f = f, envir = callEnv)
    if (is.null(nextMethod)) stop("No next method available")
    assign(".nextMethod", nextMethod, envir = callEnv)
    assign(".Generic", f, envir = callEnv)
    if (is.null(callEnv$.Method)) {
      assign(".Method", 
             .methodWithNext(method = method, nextMethod = nextMethod),
             method,
             envir = callEnv)
    }
  }
}

#' Calls the given generic with the given arguments
#'
#' Calls the given generic with the given arguments, using the method
#' whose signature matches the arguments.
#'
#' Uses `methods::selectMethod()` to find the matching method. In theory,
#' this is at best wholly redundant with what standard S4 generics already
#' do by themselves. However, the generics dispatch for S4 seems (at least
#' currently) broken at least if the first argument in the signature is
#' a class that name-clashes with a class defined in another package. In
#' that case, whether the standard dispatch works correctly or not can depend
#' on `search()` order, and can change within a session
#' depending on the order in which packages are loaded.
#' @param f the generic, as a character string or a `standardGeneric`
#'   object
#' @param ... the arguments (named and/or unnamed) with which to call the
#'   matching method
#' @param .package the package name for finding the generic (if `f` is a character
#'   string); by default the package is determined from the calling environment
#' @return the value returned by the method
#' @importFrom methods getGeneric selectMethod
#' @importFrom utils packageName
.callGeneric <- function(f, ..., .package = NULL) {
  if (is.null(.package)) {
    .package <- packageName()
    if (is.null(.package)) {
      warning("failed to determine package where .callGeneric() was called")
      .package <- "RNeXML"
    }
  }
  where <- asNamespace(.package)
  if (!is(f, "standardGeneric")) f <- getGeneric(f, where = where)
  .Args <- list(...)
  sigArgs <- .Args[seq_along(f@signature)]
  sigClasses <- sapply(sigArgs, .classForSig, f)
  method <- selectMethod(f, sigClasses)
  method(...)
}

.classForSig <- function(sigArg, f = NULL) {
  clname <- class(sigArg)
  # multiple classes?
  if (length(clname) > 1) {
    # if the generic has a value class, try and match it
    if ((! is.null(f)) && length(f@valueClass) > 0) {
      isValueClass <- clname == f@valueClass
      if (any(isValueClass)) clname <- clname[isValueClass]
    }
    # if none match, or if there's no value class, take the first one
  }
  clname[1]
}

context("internal utility functions")

test_that("lcapply() works correctly", {
  l <- list(a = "foo", b = "bar", c = NULL, NULL, "baz", d = "foobar")
  l.ans1 <- lapply(l, function(x) x)
  l.ans2 <- lcapply(l, function(x) x)
  testthat::expect_length(l.ans1, length(l))
  testthat::expect_length(l.ans2, length(l) - 2)
  testthat::expect_equal(names(l.ans1), names(l))
  testthat::expect_equal(names(l.ans2), names(l[c(1,2,5,6)]))
  testthat::expect_false("c" %in% names(l.ans2))
  testthat::expect_true("" %in% names(l.ans2))
  testthat::expect_equal(l.ans2[names(l.ans2) == ""][[1]], "baz")
})

test_that("findNextMethod() works correctly", {
  pkgEnv <- asNamespace("RNeXML")
  mdef <- selectMethod("toNeXML",
                       c("trees", "XMLInternalElementNode"),
                       optional = TRUE)
  testthat::expect_is(mdef, "MethodDefinition")
  mdef.next <- findNextMethod(mdef, envir = pkgEnv)
  testthat::expect_is(mdef.next, "MethodDefinition")
  testthat::expect_equal(mdef@defined, mdef.next@target)
  testthat::expect_equal(mdef@generic, mdef.next@generic)
  o.mm <- new(className(mdef@defined@.Data[1], mdef@defined@package[1]))
  testthat::expect_true(is(o.mm, mdef.next@defined@.Data[1]))
  mwn1 <- .methodWithNext(method = mdef, nextMethod = mdef.next)
  mwn2 <- addNextMethod(mdef, f = mdef@generic, envir = pkgEnv)
  testthat::expect_is(mwn1, "MethodWithNext")
  testthat::expect_is(mwn1@nextMethod, "MethodDefinition")
  testthat::expect_equal(mwn1@generic, mwn2@generic)
  testthat::expect_equal(mwn1@target, mwn2@target)
  testthat::expect_equal(mwn1@defined, mwn2@defined)
  testthat::expect_equal(mwn1@excluded, mwn2@excluded)
  testthat::expect_equal(mwn1@nextMethod@generic, mwn2@nextMethod@generic)
  testthat::expect_equal(mwn1@nextMethod@.Data, mwn2@nextMethod@.Data)
  # ignore attributes for the following two - package is in its own slot
  # of the signature object anyway, which will be part of the test
  testthat::expect_equivalent(mwn1@nextMethod@target, mwn2@nextMethod@target)
  testthat::expect_equivalent(mwn1@nextMethod@defined, mwn2@nextMethod@defined)
})

test_that(".classForSig() works correctly", {
  f1 <- getGeneric("toNeXML", package = "RNeXML")
  f2 <- getGeneric("fromNeXML", package = "RNeXML")
  obj <- nexml.tree()
  testthat::expect_equal(.classForSig(obj, f1), class(obj)[1])
  testthat::expect_equal(.classForSig(obj, f2), class(obj)[1])
  obj <- XML::newXMLNode("foo")
  testthat::expect_equal(.classForSig(obj, f1), "XMLInternalElementNode")
  testthat::expect_equal(.classForSig(obj, f2), "XMLInternalElementNode")
  class(obj) <- c("bogus", class(obj))
  testthat::expect_equal(.classForSig(obj, f1), class(obj)[2])
  testthat::expect_equal(.classForSig(obj, f2), class(obj)[1])
})

test_that(".callGeneric works correctly", {
  tr1 <- nexml.tree(id = "tr1",
                    node = New("ListOfnode", list(New("node", id = "n1"))))
  trblock <- New("trees", id = "trees1", tree = New("ListOftree", list(tr1)))
  xml1 <- as(trblock, "XMLInternalNode")
  xml2 <- .callGeneric("toNeXML", trblock, XML::newXMLNode("trees"), .package = "RNeXML")
  xml3 <- toNeXML(trblock, XML::newXMLNode("trees"))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml2, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml3, indent = FALSE))
  nexml1 <- as(xml1, "trees")
  nexml2 <- .callGeneric("fromNeXML", New("trees"), xml1, .package = "RNeXML")
  nexml3 <- fromNeXML(New("trees"), xml1)
  # roundtrip back to XML, using "as"
  xml1.1 <- as(nexml1, "XMLInternalNode")
  xml1.2 <- as(nexml2, "XMLInternalNode")
  xml1.3 <- as(nexml3, "XMLInternalNode")
  testthat::expect_equal(XML::saveXML(xml1.1, indent = FALSE),
                         XML::saveXML(xml1.2, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1.1, indent = FALSE),
                         XML::saveXML(xml1.3, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml1.1, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml1.2, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml1.3, indent = FALSE))
  # roundtrip back to XML, using .callGeneric
  xml1.1 <- .callGeneric("toNeXML", nexml1, XML::newXMLNode("trees"), .package = "RNeXML")
  xml1.2 <- .callGeneric("toNeXML", nexml2, XML::newXMLNode("trees"), .package = "RNeXML")
  xml1.3 <- .callGeneric("toNeXML", nexml3, XML::newXMLNode("trees"), .package = "RNeXML")
  testthat::expect_equal(XML::saveXML(xml1.1, indent = FALSE),
                         XML::saveXML(xml1.2, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1.1, indent = FALSE),
                         XML::saveXML(xml1.3, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml1.1, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml1.2, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml1.3, indent = FALSE))
  # roundtrip back to XML, using toNeXML generic dispatch
  xml1.1 <- toNeXML(nexml1, XML::newXMLNode("trees"))
  xml1.2 <- toNeXML(nexml2, XML::newXMLNode("trees"))
  xml1.3 <- toNeXML(nexml3, XML::newXMLNode("trees"))
  testthat::expect_equal(XML::saveXML(xml1.1, indent = FALSE),
                         XML::saveXML(xml1.2, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1.1, indent = FALSE),
                         XML::saveXML(xml1.3, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml1.1, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml1.2, indent = FALSE))
  testthat::expect_equal(XML::saveXML(xml1, indent = FALSE),
                         XML::saveXML(xml1.3, indent = FALSE))
})

test_that("coalesce_() works correctly", {
  dta <- data.frame(col1 = rep(NA, times = 5),
                    col2 = 1:5,
                    col3 = as.factor(letters[1:5]))
  dta[3:4, "col2"] <- NA
  dta[2:3, "col3"] <- NA
  last <- letters[6:10]
  # let's start off with some tests for the presence of the problems that
  # coalesce_ is trying to solve, because in their absence we won't need it
  #
  # (1) dplyr::coalesce is strict about type, and for some reason vectors
  # that are all NAs default to type logical
  testthat::expect_is(dta[, "col1"], "logical")
  
  # EDIT: I believe these ungraceful behaviors in dplyr are resolved
  # in the most recent version, and so these no longer throw errors:
  
  ## testthat::expect_error(dplyr::coalesce(dta$col1, dta$col3, last))
  ## testthat::expect_error(dplyr::coalesce(dta$col3, last)) # col3 is a factor
  # (2) dplyr::coalesce doesn't gracefully deal with NULL arguments, which
  # can result from referencing column names that aren't there, requiring
  # a conditional testing for presence of the column
  testthat::expect_error(dplyr::coalesce(dta$col2, dta$foo, last))

  # now test that coalesce_ deals with these issues and gives correct results
  res <- ifelse(is.na(dta[,"col3"]), last, as.character(dta[,"col3"]))
  testthat::expect_is(coalesce_(dta$col1, dta$col3, last), "character")
  testthat::expect_false(any(is.na(coalesce_(dta$col1, dta$col3, last))))
  testthat::expect_equal(coalesce_(dta$col1, dta$col3, last), res)
  testthat::expect_equal(coalesce_(dta$col1, dta$col3, dta$foo, last), res)
  testthat::expect_equal(coalesce_(dta$col1, dta$col3, NULL, last), res)
  testthat::expect_is(coalesce_(dta$col1, dta$col2), "integer")
  testthat::expect_equal(sum(is.na(coalesce_(dta$col1, dta$col2))), 2)
  testthat::expect_false(any(is.na(coalesce_(dta$col1, dta$col2, 1:5))))
})

## Utilities for adding additional metadata

#' @name nexml.meta_
#' @aliases nexml.meta_
#' @seealso [nexml.meta()][meta()] for documentation of `nexml.meta()`
#' @rdname constructors
NULL

#' Constructor function for metadata nodes
#' 
#' @param property specify the ontological definition together with it's namespace, e.g. dc:title
#' @param content content of the metadata field
#' @param rel Ontological definition of the reference provided in href 
#' @param href A link to some reference
#' @param datatype optional RDFa field
#' @param id optional id element (otherwise id will be automatically generated).  
#' @param type optional xsi:type.  If not given, will use either "LiteralMeta" or "ResourceMeta" as 
#'  determined by the presence of either a property or a href value. 
#' @param children Optional element containing any valid XML block (XMLInternalElementNode class, see the XML package for details).  
#' @details User must either provide property+content or rel+href.  Mixing these will result in potential garbage. 
#' The datatype attribute will be detected automatically from the class of the content argument.  Maps from R class
#' to schema datatypes are as follows: 
#' character - xs:string, 
#' Date - xs:date,
#' integer - xs:integer,
#' numeric - xs:decimal,
#' logical - xs:boolean
#' 
#' @examples
#' meta(content="example", property="dc:title")
#' @export 
#' @seealso \code{\link{nexml_write}}
#' @aliases nexml.meta
#' @include classes.R
#' @importFrom plyr compact
meta <- function(property = NULL,
                 content = NULL,
                 rel = NULL,
                 href = NULL,
                 datatype = NULL,
                 id = NULL,
                 type = NULL,
                 children = list()){
  # if datatype isn't provided, try to infer it from content
  if (is.null(datatype) && !is.null(content)) {
    if(is.logical(content))
      datatype <- "xsd:boolean"
    else if(is(content, "Date"))
      datatype <- "xsd:date"
    else if(is.numeric(content))
      datatype <- "xsd:decimal"
    else if(is.character(content) && length(content) > 0)
      datatype <- "xsd:string"
    else if(is.integer(content))
      "xsd:integer"
  }
  # the content text must be written as a string
  if (length(content) > 0)
    content <- as.character(content)

  if(length(id) == 0)
    id <- nexml_id("m")
   
  if(is(children, "XMLAbstractNode") || is(children, "XMLInternalNode"))
    children <- list(children)

  # if type is not provided, try to determine it from the parameters
  if (is.null(type)) {
    if (length(rel) > 0)
      type <- "ResourceMeta"
    else if (length(href) > 0 ||
             (length(children) > 0 && all(sapply(children, is, "meta")))) {
      type <- "ResourceMeta"
      rel <- property
      property <- NULL
    } else if (length(property) > 0)
      type <- "LiteralMeta"
  }
  # if null is provided for value, don't create any object
  if (length(children) == 0 &&
      ((is.null(content) && type == "LiteralMeta") ||
       (is.null(href) && type == "ResourceMeta")))
    NULL
  else {
    # determine the meta class to instantiate
    clname <- type
    if (is.null(type)) {
      clname <- "meta"
      # we have to default to some xsi:type that makes sense
      type <- "LiteralMeta"
    }
    # create the meta instance
    args <- plyr::compact(list(property = property,
                               content = content,
                               datatype = datatype,
                               rel = rel,
                               href = href,
                               id = id,
                               'xsi:type' = type,
                               children = children))
    do.call(New, c(clname, args))
  }
}

nexml.meta <- meta

## Common helper functions 

#' @importFrom plyr compact
nexml_citation <- function(obj){
  if(is(obj, "BibEntry"))
    class(obj) <- "bibentry"
  if(is(obj, "bibentry")){
    out <- lapply(obj, function(obj){
      if(length(grep("--", obj$pages)) > 0){
        pgs <- strsplit(obj$pages, "--")[[1]]
        start_page <- pgs[[1]]
        end_page <- if(length(pgs)>1) pgs[[2]] else " "
      } else if(length(grep("-", obj$pages)) > 0){
        pgs <- strsplit(obj$pages, "-")[[1]]
        start_page <- pgs[[1]]
        end_page <- if(length(pgs)>1) pgs[[2]] else " "
      } else {
        start_page <- NULL
        end_page <- NULL
      }
      list_of_metadata_nodes <- plyr::compact(c(list(
        meta(content=obj$volume, 
            property="prism:volume"),
        meta(content=obj$journal, 
            property="dc:publisher"),
        meta(content=obj$journal, 
            property="prism:publicationName"),
        meta(content = end_page, 
            property="prism:endingPage"),
        meta(content=start_page, 
            property="prism:startingPage"),
        meta(content=obj$year, 
            property="prism:publicationDate"),
        meta(content=obj$title,
            property="dc:title")),
        lapply(obj$author, function(x){
          meta(content = format(x, c("given", "family")),
               property="dc:contributor")
          }),
        meta(content=format(obj, "text"), 
             property="dcterms:bibliographicCitation")
      ))
      citation_elements = New("ListOfmeta", list_of_metadata_nodes)
      meta(rel = "dcterms:references",
           children = citation_elements)
    })
    out 
  }
}



#' Concatenate meta elements into a ListOfmeta
#' 
#' Concatenate meta elements into a ListOfmeta
#' @param x,... `meta` and `ListOfmeta` elements to be concatenated, see \code{\link{meta}}
#' @param recursive  logical, if 'recursive=TRUE', the function recursively
#'   descends through lists and combines their elements into a flat vector.
#'   This method does not support `recursive=FALSE`, use [list][base::list()]
#'   instead.
#' @return a ListOfmeta object containing a flat list of meta elements.
#' @examples 
#' c(meta(content="example", property="dc:title"),
#'   meta(content="Carl", property="dc:creator"))
#' @rdname c-meta
#' @aliases c-meta
#' @include classes.R
#' @importFrom plyr compact
setMethod("c", 
          signature("meta"),
          function(x, ..., recursive = TRUE){
            elements <- list(x, ...)
            if (identical(recursive, FALSE))
              stop("Use list() to concatenate 'meta' and 'ListOfmeta' non-recursively")
            new("ListOfmeta", unlist(elements))

          })


#' Concatenate ListOfmeta elements into a ListOfmeta
#' 
#' Concatenate ListOfmeta elements into a flat ListOfmeta
#' @examples 
#' metalist <- c(meta(content="example", property="dc:title"),
#'               meta(content="Carl", property="dc:creator"))
#' out <- c(metalist, metalist) 
#' out <- c(metalist, meta(content="a", property="b")) 
#' @rdname c-meta
#' @aliases c-ListOfmeta
#' @importFrom plyr compact
setMethod("c", 
          signature("ListOfmeta"),
          function(x, ..., recursive = TRUE){
            elements <- list(x, ...)
            if (identical(recursive, FALSE))
              stop("Use list() to concatenate 'meta' and 'ListOfmeta' non-recursively")
            new("ListOfmeta", plyr::compact(unlist(elements)))

          })

setGeneric("slot")
setGeneric("slot<-")

#' Access or set slot of S4 object
#'
#' See [methods::slot()]. This version allows using "property" consistently
#' for both LiteralMeta and ResourceMeta (which internally uses "rel" because
#' RDFa does), which is easier to program. It also allows using "meta"
#' as an alias for "children" for ResourceMeta, to be consistent with the
#' corresponding slot for instances of `Annotated`.
#' @param object the object
#' @param name name of the slot
#' @aliases slot-ResourceMeta
#' @seealso [methods::slot()]
#' @export
setMethod("slot",
          signature("ResourceMeta", "ANY"),
          function(object, name) {
            if (name == "property")
              object@rel
            else if (name == "meta")
              object@children
            else
              callNextMethod()
          })

#' @param value the new value
#' @rdname slot-ResourceMeta-method
#' @export
setMethod("slot<-",
          signature("ResourceMeta", "ANY"),
          function(object, name, value) {
            if (name == "property")
              object@rel <- value
            else if (name == "meta")
              object@children <- value
            else
              object <- callNextMethod()
            object
          })

context("meta")


data(bird.orders)

test_that("We can add additional metadata", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  ## The short version using an RNeXML API
  nex <- add_basic_meta(
              title = "My test title",
              description = "A description of my test",
              creator = "Carl Boettiger <cboettig@gmail.com>",
              publisher = "unpublished data",
              pubdate = "2012-04-01")
  write.nexml(nex, file = "meta_example.xml")

  expect_true_or_null(nexml_validate("meta_example.xml"))
  expect_is(nexml_read("meta_example.xml"), "nexml")

  unlink("meta_example.xml") # cleanup

})


test_that("We can add R bibentry type metadata", {
  
  
  ## tests are too slow for CRAN
  skip_on_cran()
  ## The short version using an RNeXML API

  nex <- add_trees(bird.orders)
  nex <- add_basic_meta(nexml=nex, citation=citation("ape")) 
  write.nexml(nex, file = "meta_example.xml")

  expect_true_or_null(nexml_validate("meta_example.xml"))
  expect_is(nexml_read("meta_example.xml"), "nexml")
  unlink("meta_example.xml") # cleanup

})

test_that("Adding meta data has some basic error checking", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  testthat::expect_error(add_meta(NULL, level = "nexml"))
  m <- meta("foo-rel", "a test")
  testthat::expect_error(add_meta(list(m, "not a meta")))
  testthat::expect_error(add_meta(m, level = "foo"))
})

test_that("ResourceMeta maps rel to property for simplicity of API", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  m <- meta(rel = "foo-rel", href="http://example.com/")
  testthat::expect_is(m, "ResourceMeta")
  testthat::expect_error(m@property)
  testthat::expect_equal(slot(m, "rel"), "foo-rel")
  testthat::expect_equal(slot(m, "property"), "foo-rel")
  testthat::expect_error(m@property <- "bar-rel")
  testthat::expect_silent(slot(m, "property") <- "bar-rel")
  testthat::expect_equal(slot(m, "rel"), "bar-rel")
  testthat::expect_equal(slot(m, "property"), "bar-rel")
})

test_that("ResourceMeta maps meta to children for simplicity of API", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  nested <- c(meta(property = "foo-prop", content = "foo"),
              meta(property = "bar-prop", content = "bar"))
  m <- meta(rel = "foo-rel", children = nested)
  testthat::expect_is(m, "ResourceMeta")
  testthat::expect_error(m@meta)
  testthat::expect_length(m@children, 2)
  testthat::expect_length(slot(m, "meta"), 2)
  testthat::expect_equal(slot(m, "meta"), m@children)
  testthat::expect_equivalent(sapply(slot(m, "meta"), slot, "content"),
                              c("foo", "bar"))
  m <- new("ResourceMeta", rel = "baz-rel") # meta() returns NULL if no "content"
  testthat::expect_length(m@children, 0)
  testthat::expect_error(m@meta <- nested)
  testthat::expect_silent(slot(m, "meta") <- nested)
  testthat::expect_length(slot(m, "meta"), 2)
  testthat::expect_equal(slot(m, "meta"), m@children)
  testthat::expect_equivalent(sapply(slot(m, "meta"), slot, "content"),
                              c("foo", "bar"))
})

test_that("Citation BibEntry objects are transformed to structured metadata", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  nexml_cit <- nexml_citation(citation("RNeXML"))
  testthat::expect_is(nexml_cit, "list")
  testthat::expect_length(nexml_cit, 1)
  citrec <- nexml_cit[[1]]
  testthat::expect_is(citrec, "meta")
  testthat::expect_true(.hasSlot(citrec, "children"))
  testthat::expect_gt(length(citrec@children), 10)
  citxml <- as(citrec, "XMLInternalNode")
  testthat::expect_length(XML::xmlChildren(citxml), length(citrec@children))
  testthat::expect_equal(slot(citrec, "xsi:type"), "ResourceMeta")
  testthat::expect_false(grepl("bibliographicCitation", slot(citrec, "rel")))
  propvals <- sapply(citrec@children, 
                     function(x) if (.hasSlot(x, "property")) x@property else NULL)
  testthat::expect_true(any(grepl("bibliographicCitation", propvals)))
})

test_that("We can add additional metadata", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  ## The short version using an RNeXML API
  nex <- add_trees(bird.orders)
  nex <- add_basic_meta(nexml = nex, citation=citation("ape")) 

  history <- meta(property = "skos:historyNote",
                  content = "Mapped from the bird.orders data in the ape package using RNeXML",
                  id = "meta5144")
  modified <- meta(property = "prism:modificationDate",
                  content = "2013-10-04")
  website <- meta(href = "http://carlboettiger.info", 
                 rel = "foaf:homepage")

  nex <- add_meta(list(history, modified, website), 
                  nex, 
                  namespaces = c(skos = "http://www.w3.org/2004/02/skos/core#",
                                 prism = "http://prismstandard.org/namespaces/1.2/basic/", # check and remove duplicates
                                 foaf = "http://xmlns.com/foaf/0.1/"))

  nexml_write(nex, file = "meta_example.xml")  

  expect_true_or_null(nexml_validate("meta_example.xml"))
  expect_is(nexml_read("meta_example.xml"), "nexml")
  unlink("meta_example.xml") # cleanup

})


test_that("We can directly add additional metadata at arbitrary level", {
  ## tests are too slow for CRAN
  skip_on_cran()
  
    nex <- add_trees(bird.orders)
  modified <- meta(property = "prism:modificationDate",
                  content = "2013-10-04")

  # For trees block, using API function
  nex <- add_meta(modified, nexml = nex, level = "trees")
  get_metadata(nex, "trees") %>%
    dplyr::filter(property == "prism:modificationDate") %>% 
    dplyr::select(content) -> 
    tmp
  expect_identical(tmp[[1]], modified@content)

  # For trees block, accessing object hierarchy directly
  nex@trees[[1]]@meta <- c(modified)
  get_metadata(nex, "trees") %>%
    dplyr::filter(property == "prism:modificationDate") %>%
    dplyr::select(content) ->
    tmp
  expect_identical(tmp[[1]], modified@content)

  # For otus block, using API function
  nex <- add_meta(modified, nexml = nex, level = "otus")
  get_metadata(nex, "otus") %>%
    dplyr::filter(property == "prism:modificationDate") %>%
    dplyr::select(content) ->
    tmp
  expect_identical(tmp[[1]], modified@content)

  # For otus block, accessing object hierarchy directly
  nex@otus[[1]]@meta <- c(modified)
  get_metadata(nex, "otus") %>%
    dplyr::filter(property == "prism:modificationDate") %>%
    dplyr::select(content) ->
    tmp
  expect_identical(tmp[[1]], modified@content)

  # For characters block, using API function
  testthat::expect_error(add_meta(modified, nexml = nex, level = "characters"))
  # one needs to have called add_characters() first, as otherwise there won't
  # be a characters block
  cdata <- data.frame(trait1 = c(1, 1, 0),
                      row.names = c("Aus bus", "Aus foo", "Aus bar"))
  nex <- add_characters(cdata, nexml = nex)
  # now this should work
  testthat::expect_silent(nex <- add_meta(modified, nexml = nex, level = "characters"))
  get_metadata(nex, "characters") %>%
    dplyr::filter(property == "prism:modificationDate") %>%
    dplyr::select(content) ->
    tmp
  expect_identical(tmp[[1]], modified@content)

  # For otus block, accessing object hierarchy directly
  nex@characters[[1]]@meta <- c(modified)
  get_metadata(nex, "characters") %>%
    dplyr::filter(property == "prism:modificationDate") %>%
    dplyr::select(content) ->
    tmp
  expect_identical(tmp[[1]], modified@content)

})



test_that("We can directly add additional metadata using concatenation notation", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  nex <- add_trees(bird.orders)
  modified <- meta(property = "prism:modificationDate",
                  content = "2013-10-04")
  website <- meta(href = "http://carlboettiger.info", 
                 rel = "foaf:homepage")

  nex@trees[[1]]@meta <- c(modified)         # we can add just one element 
  nex@trees[[1]]@meta <- c(modified,website) # or more than one element

  get_metadata(nex, "trees") %>%
    dplyr::filter(property == "prism:modificationDate") %>% 
    dplyr::select(content) -> 
    tmp
  
  expect_identical(tmp[[1]], modified@content)
})





test_that("We can add arbitrary metadata", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  rdfa <- '<xhtml:div typeof="foaf:Person" about="http://carlboettiger.info#me">
             <a rel="foaf:account" href="https://twitter.com/cboettig">twitter</a> 
             <a rel="foaf:account" href="https://github.com/cboettig">github</a>
           </xhtml:div>'
  parsed <- xmlRoot(xmlParse(rdfa))
  arbitrary_rdfa <- meta(property="eml:additionalMetadata", content="additional metadata", children = parsed)

  nex <- add_meta(arbitrary_rdfa, 
                  namespaces = c(foaf = "http://xmlns.com/foaf/0.1/", 
                                 eml = "eml://ecoinformatics.org/eml-2.1.1", 
                                 xhtml = "http://www.w3.org/1999/xhtml"))

  nexml_write(nex, file = "example.xml")


  
  expect_is(nexml_read("example.xml", "nexml"), "nexml")
  unlink("example.xml") # cleanup

})


test_that("we can write numeric types of meta elements and get correct datatype", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
          m <- meta(property="numericTest", content = 3.141)
          expect_is(m@content, "character")
          expect_match(m@datatype, ".*:decimal")
})

test_that("we can serialize nested meta elements", {
  
  ## tests are too slow for CRAN
  skip_on_cran()

  f <- system.file("examples", "meta_example.xml", package="RNeXML")
  nex <- read.nexml(f)
  s <- nex@meta[sapply(nex@meta,
                       function(x)
                         ("rel" %in% slotNames(x)) &&
                         (x@rel == "dc:source"))]
  out <- as(s$meta, "XMLInternalNode")
  out_m <- sapply(xmlChildren(out), xmlAttrs)
  testthat::expect_equal(dim(out_m), c(3, 3))
})

#' @include classes.R

####################################################


setClass("char",
         slots = c(states = "character"),
         contains = "IDTagged")
setMethod("fromNeXML", 
          signature("char", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            if(!is.na(xmlAttrs(from)["states"]))
              obj@states <- xmlAttrs(from)["states"]
            obj
          })
setMethod("toNeXML", 
          signature("char", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            if(length(object@states) > 0)
              addAttributes(parent, "states" = object@states)
            parent
          })
setAs("char", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("char")))
setAs("char", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("char")))
setAs("XMLInternalElementNode", "char",
      function(from) fromNeXML(nexml.char(), from))




###############################################

setClass("ListOfrow", slots = c(names="character"), contains="list")
setClass("obsmatrix",
         slots = c(row="ListOfrow"),
         contains = "Annotated")
setMethod("fromNeXML", 
          signature("obsmatrix", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            kids <- xmlChildren(from)
            if(length(kids) > 0)
              obj@row <- New("ListOfrow",
                              lapply(kids[names(kids) == "row"], 
                                     as, "row"))
            obj
          })
setMethod("toNeXML", 
          signature("obsmatrix", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addChildren(parent, kids = lcapply(object@row, as, "XMLInternalNode"))
            parent
          })
setAs("obsmatrix", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("matrix")))
setAs("obsmatrix", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("matrix")))
setAs("XMLInternalElementNode", "obsmatrix",
      function(from) fromNeXML(nexml.matrix(), from))





######################################################

setClass("ListOfcell", slots = c(names="character"), contains="list")
setClass("ListOfseq", slots = c(names="character"), contains="list")

setClass("row",
         slots = c(cell = "ListOfcell",
                        seq = "ListOfseq"),
         contains = "OptionalTaxonLinked")
setMethod("fromNeXML", 
          signature("row", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            kids <- xmlChildren(from)
            if(length(kids) > 0){
              if("cell" %in% names(kids))
              obj@cell <- New("ListOfcell",
                lapply(kids[names(kids) == "cell"], as, "cell"))
              if("seq" %in% names(kids))
              obj@seq <- New("ListOfseq",
                lapply(kids[names(kids) == "seq"], as, "seq"))
            }
            obj
          })
setMethod("toNeXML", 
          signature("row", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addChildren(parent, kids = lcapply(object@cell, as, "XMLInternalNode"))
            addChildren(parent, kids = lcapply(object@seq, as, "XMLInternalNode"))
            parent
          })
setAs("row", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("row")))
setAs("row", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("row")))
setAs("XMLInternalElementNode", "row",
      function(from) fromNeXML(nexml.row(), from))

#######################################################
setClass("ListOfstate", slots = c(names="character"), contains="list")
setClass("ListOfpolymorphic_state_set", slots = c(names="character"), contains="list")
setClass("ListOfuncertain_state_set", slots = c(names="character"), contains="list")

setClass("states",
         slots = c(state="ListOfstate",
                   polymorphic_state_set="ListOfpolymorphic_state_set",
                   uncertain_state_set="ListOfuncertain_state_set"),
         contains = "IDTagged")
setMethod("fromNeXML", 
          signature("states", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            kids <- xmlChildren(from)
            if(length(kids) > 0){
              obj@state <- New("ListOfstate",
                              lapply(kids[names(kids) == "state"], 
                                     as, "state"))
              obj@polymorphic_state_set <- New("ListOfpolymorphic_state_set", 
                             lapply(kids[names(kids) == "polymorphic_state_set"], 
                                    as, "polymorphic_state_set"))
              obj@uncertain_state_set <- New("ListOfuncertain_state_set", 
                                               lapply(kids[names(kids) == "uncertain_state_set"], 
                                                      as, "uncertain_state_set"))
            }
            obj
          })
setMethod("toNeXML", 
          signature("states", "XMLInternalElementNode"),
          function(object, parent){
            suppressWarnings({ # avoid arcane XML warning message
            parent <- callNextMethod()
            addChildren(parent, kids = lcapply(object@state, as, "XMLInternalNode"))
            addChildren(parent, kids = lcapply(object@uncertain_state_set,
                                              as, "XMLInternalNode"))
            addChildren(parent, kids = lcapply(object@polymorphic_state_set,
                                              as, "XMLInternalNode"))
            })
            parent
          })
setAs("states", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("states")))
setAs("states", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("states")))
setAs("XMLInternalElementNode", "states",
      function(from) fromNeXML(nexml.states(), from))




####################################################### 
## technically symbol is positive integer http://nexml.org/doc/schema-1/characters/standard/#StandardToken
setClass("state",
         slots = c(symbol = "integer"), 
         contains = "IDTagged")
setMethod("fromNeXML", 
          signature("state", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            obj@symbol <- as.integer(xmlAttrs(from)["symbol"])
            obj
          })
setMethod("toNeXML", 
          signature("state", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addAttributes(parent, "symbol" = object@symbol)
            parent
          })
setAs("state", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("state")))
setAs("state", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("state")))
setAs("XMLInternalElementNode", "state",
      function(from) suppressWarnings(fromNeXML(nexml.state(), from)))

################################################

##  a symbol for an uncertain stat is a character string
setClass("uncertain_state",
         slots = c(symbol = "character"), 
         contains = "IDTagged")
setMethod("fromNeXML", 
          signature("uncertain_state", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            obj@symbol <- xmlAttrs(from)["symbol"]
            obj
          })
setMethod("toNeXML", 
          signature("uncertain_state", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addAttributes(parent, "symbol" = object@symbol)
            parent
          })
setAs("uncertain_state", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("uncertain_state")))
setAs("uncertain_state", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("uncertain_state")))
setAs("XMLInternalElementNode", "uncertain_state",
      function(from) suppressWarnings(fromNeXML(nexml.uncertain_state(), from)))

################################################


setClass("ListOfmember", slots = c(names="character"), contains="list")

setClass("uncertain_state_set", 
         slots = c(member = "ListOfmember"),
         contains="uncertain_state")
setMethod("fromNeXML", 
          signature("uncertain_state_set", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            kids <- xmlChildren(from)
            if(length(kids) > 0)
              obj@member <- New("ListOfmember",
                              lapply(kids[names(kids) == "member"], 
                                     as, "member"))
            obj
          })
setMethod("toNeXML", 
          signature("uncertain_state_set", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addChildren(parent, kids = lcapply(object@member, as, "XMLInternalNode"))
            parent
          })
setAs("uncertain_state_set", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("uncertain_state_set")))
setAs("uncertain_state_set", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("uncertain_state_set")))
setAs("XMLInternalElementNode", "uncertain_state_set",
      function(from) fromNeXML(nexml.uncertain_states(), from))

################################################

setClass("polymorphic_state_set", contains="uncertain_state_set")
setMethod("fromNeXML", 
          signature("polymorphic_state_set", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            kids <- xmlChildren(from)
            if(length(kids) > 0)
              obj@member <- New("ListOfmember",
                              lapply(kids[names(kids) == "member"], 
                                     as, "member"))
            obj
          })
setMethod("toNeXML", 
          signature("polymorphic_state_set", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addChildren(parent, kids = lcapply(object@member, as, "XMLInternalNode"))
            parent
          })
setAs("polymorphic_state_set", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("polymorphic_state_set")))
setAs("polymorphic_state_set", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("polymorphic_state_set")))
setAs("XMLInternalElementNode", "polymorphic_state_set",
      function(from) fromNeXML(nexml.polymorphic_states(), from))


#####################

setClass("cell",
         slots = c(char="character", 
                        state= "character"),
         contains="Base")
setMethod("fromNeXML", 
          signature("cell", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            obj@char <- xmlAttrs(from)["char"]
            obj@state <- xmlAttrs(from)["state"]
            obj
          })
setMethod("toNeXML", 
          signature("cell", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addAttributes(parent, "char" = object@char)
            addAttributes(parent, "state" = object@state)
            parent
          })
setAs("cell", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("cell")))
setAs("cell", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("cell")))
setAs("XMLInternalElementNode", "cell",
      function(from) fromNeXML(nexml.cell(), from))

#########################

setClass("member", 
         slots = c(state="character"),
         contains="Base")
setMethod("fromNeXML", 
          signature("member", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            obj@state <- xmlAttrs(from)["state"]
            obj
          })
setMethod("toNeXML", 
          signature("member", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addAttributes(parent, "state" = object@state)
            parent
          })
setAs("member", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("member")))
setAs("member", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("member")))
setAs("XMLInternalElementNode", "member",
      function(from) fromNeXML(nexml.member(), from))


########################

setClass("seq", 
         slots = c(seq = "character"),
         contains="Base")
setMethod("fromNeXML", 
          signature("seq", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            obj@seq <- xmlValue(from)
            obj
          }
)
setMethod("toNeXML", 
          signature("seq", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addChildren(parent, object@seq)
            parent
          })
setAs("seq", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("seq")))
setAs("seq", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("seq")))
setAs("XMLInternalElementNode", "seq",
      function(from) fromNeXML(nexml.seq(), from))

#########################################

setClass("ListOfchar", slots = c(names="character"), contains="list")
setClass("ListOfstates", slots = c(names="character"), contains="list")

setClass("format", 
         slots = c(states = "ListOfstates", ## FIXME Should be ListOfstates
                        char = "ListOfchar"),
         contains = "Annotated")
setMethod("fromNeXML", 
          signature("format", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            kids <- xmlChildren(from)
            if(length(kids) > 0){
              if("char" %in% names(kids))
                obj@char <- New("ListOfchar",
                                lapply(kids[names(kids) == "char"], 
                                       as, "char"))
              if("states" %in% names(kids))
                obj@states <- New("ListOfstates",
                                lapply(kids[names(kids) == "states"], 
                                       as, "states"))
            }
            obj
          })
setMethod("toNeXML", 
          signature("format", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            if(!isEmpty(object@char))
              addChildren(parent, kids = lcapply(object@char, as, "XMLInternalNode"))
            if(length(object@states) > 0)
              addChildren(parent, kids = lcapply(object@states, as, "XMLInternalNode"))
            parent
          })
setAs("format", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("format")))
setAs("format", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("format")))
setAs("XMLInternalElementNode", "format",
      function(from) fromNeXML(nexml.format(), from))





####################################################
setClass("characters",
         slots = c(format = "format",
                        matrix = "obsmatrix"),
        contains = "TaxaLinked")
setMethod("fromNeXML", 
          signature("characters", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            obj@format <- as(from[["format"]], "format")
            obj@matrix <- as(from[["matrix"]], "obsmatrix")
            obj
          })
setMethod("toNeXML", 
          signature("characters", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            parent <- addChildren(parent, as(object@format, "XMLInternalNode"))
            parent <- addChildren(parent, as(object@matrix, "XMLInternalNode"))
            parent
          })
setAs("characters", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("characters")))
setAs("characters", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("characters")))
setAs("XMLInternalElementNode", "characters",
      function(from) fromNeXML(nexml.characters(), from))

#' taxize nexml
#'
#' Check taxonomic names against the specified service and 
#' add appropriate semantic metadata to the nexml OTU unit 
#' containing the corresponding identifier. 
#' @param nexml a nexml object
#' @param type the name of the identifier to use
#' @param warnings should we show warning messages if no match can be found?
#' @param ... additional arguments to `[taxadb::get_ids()]`
#' @export 
#' @examples \dontrun{
#' data(bird.orders)
#' birds <- add_trees(bird.orders)
#' birds <- taxize_nexml(birds, "NCBI")
#' }
# @importFrom taxize get_uid
taxize_nexml <- function(nexml, 
                         type = c("ncbi", "itis", "col", "tpl",
                                  "gbif", "wd"), 
                         warnings = TRUE,
                         ...){
  
  ## Soft dependency on taxadb
  if (!requireNamespace("taxalight", quietly = TRUE)) {
    stop("taxadb package required to convert look up taxonomic ids",
         call. = FALSE)
  }
  get_ids <- getExportedValue("taxalight", "get_ids")
  
  type <- tolower(type)
  type <- match.arg(type)

  for(j in 1:length(nexml@otus)){
    
    # Resolve all ids at once
    labels <- unname(vapply(nexml@otus[[j]]@otu, slot, character(1L), "label"))
    clean_labels <- gsub("_", " ", labels)
    taxa_ids <- get_ids(clean_labels, type, ...)
    
    for(i in 1:length(taxa_ids)){
      id <- taxa_ids[[i]]
      if(is.na(id) & warnings)
        warning(paste("ID for otu", 
                      nexml@otus[[j]]@otu[[i]]@label, 
                      "not found. Consider checking the spelling
                      or alternate classification"))
      else 
        nexml@otus[[j]]@otu[[i]]@meta <- 
          c(nexml@otus[[j]]@otu[[i]]@meta, 
                       meta(href = taxa_ids[[i]],
                            rel = "tc:toTaxon"))

    }
  }
        
  nexml
}

setClass("phyloS4", 
         slots = c(edge = "matrix",
                        Nnode = "integer",
                        tip.label = "character",
                        edge.length = "numeric"))
setOldClass("phylo", S4Class="phyloS4")

## multiPhylo and multiPhyloList are used in other files, but perhaps it is
## best to keep the definitions all here
setOldClass("multiPhylo")
setOldClass("multiPhyloList")

setClass("nexmlTree", slots = c(nexml = "nexml"), contains="phylo")
setMethod("show", "nexmlTree", function(object) print.phylo(object))
# callNextMethod(object) ## callNextMethod might have been an option, but it looks for 'show' method, not print method?? 

## constructor function
nexmlTree <- function(object){
  if(is(object, "nexml")){
    phylo <- as(object, "phylo")
  }
  new("nexmlTree", nexml = object, phylo)
}

## Coercions between classes
setAs("XMLInternalElementNode", "nexmlTree", function(from)
      nexmlTree(as(from, "nexml")))
setAs("nexmlTree", "XMLInternalElementNode", function(from)
      as(from@nexml, "XMLInternalElementNode"))
setAs("XMLInternalNode", "nexmlTree", function(from)
      nexmlTree(as(from, "nexml")))
setAs("nexmlTree", "XMLInternalNode", function(from)
      as(from@nexml, "XMLInternalNode"))
setAs("phylo", "nexmlTree", function(from)
      nexmlTree(as(from, "nexml")))
setAs("nexmlTree", "phylo", function(from)
      as(from@nexml, "phylo"))
setAs("nexml", "nexmlTree", function(from)
      nexmlTree(from))
setAs("nexmlTree", "nexml", function(from)
      from@nexml)



### Testing 
# a <- new("phylo", bird.orders)
# expect_is(a, "phylo")
# a 
# plot(a)
# 
# b <- new("nexmlTree", bird.orders, nexml = as(bird.orders, "nexml"))
# expect_is(b, "phylo")
# b
# plot(b) 
# 
# Some ape functions don't check class properly.  i.e. class(b) == "phylo" is FALSE, but is(b, "phylo") is TRUE.  

# Don't really need these, but here they are mapping between S3 and S4 
setAs("phyloS4", "phylo", function(from){
      out <- list(edge = from@edge,
                  Nnode = from@Nnode,
                  tip.label = from@tip.label,
                  edge.length = from@edge.length)
      class(out) <- "phylo"
      out
})
setAs("phylo", "phyloS4", function(from)
      new("phyloS4", 
          edge = from$edge,
          Nnode = from$Nnode,
          tip.label = from$tip.label,
          edge.length = from$edge.length))

## Some of these methods are still in use by simmap and various add_ functions.
## Ideally all would be replaced by the equivalent get_level() version



## Conversions between matrix and characters node

#' Extract the character matrix
#'
#' @param nexml nexml object (e.g. from read.nexml)
#' @param rownames_as_col option to return character matrix rownames
#'  (with taxon ids) as it's own column in the data.frame. Default is FALSE
#'  for compatibility with geiger and similar packages.
#' @return the list of taxa
#' @examples
#' comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
#' nex <- nexml_read(comp_analysis)
#' get_characters_list(nex)
#' @export
get_characters_list <- function(nexml, rownames_as_col=FALSE){
  # extract mapping between otus and taxon ids 
  maps <- get_otu_maps(nexml)
  # loop over all character matrices 
  out <- lapply(nexml@characters, function(characters){
    
    # Make numeric data class numeric, discrete data class discrete
    type <- slot(characters, 'xsi:type') # check without namespace?
    if (grepl("ContinuousCells", type)) {
      dat <- extract_character_matrix(characters@matrix)
      dat <- otu_to_label(dat, maps[[characters@otus]])
      dat <- character_to_label(dat, characters@format)
      for(i in length(dat)) ## FIXME something more elegant, no?
        dat[[i]] <- as.numeric(dat[[i]])
    } else if (grepl("StandardCells", type)) {
      dat <- extract_character_matrix(characters@matrix)
      dat <- state_to_symbol(dat, characters@format)
      dat <- otu_to_label(dat, maps[[characters@otus]])
      dat <- character_to_label(dat, characters@format)
      for(i in length(dat))
        dat[[i]] <- factor(dat[[i]])
    } else {
      dat <- NULL 
    }
    if(rownames_as_col){
      dat <- cbind(taxa = rownames(dat), dat)
      rownames(dat) <- NULL
    }
    dat
  })
  # name the character matrices by their labels,
  # if available, otherwise, by id.    
  names(out) <- name_by_id_or_label(nexml@characters) 
  out
} 


# for lists only 
# identical_rownames <- function(x) all(sapply(lapply(x, rownames), identical, rownames(x[[1]])))


####  Subroutines (not exported)  ########### 


### The subroutines of "get_characters_list# function
## Fixme these could be adapted to use the get_*_maps functions


otu_to_label <- function(dat, otu_map){
  rownames(dat) <- otu_map[rownames(dat)]
  dat
}

character_to_label <- function(dat, format){ 
  ## Compute the mapping
  map <- map_chars_to_label(format)
  ## replace colnames with matching labels
  colnames(dat) <- map[colnames(dat)]
  dat
}

state_to_symbol <- function(dat, format){
  if(!isEmpty(format@states)){
    map_by_char <- map_state_to_symbol(format)
    for(n in names(dat)){
      symbol <- map_by_char[[n]] 
      dat[[n]] <- symbol[dat[[n]]]
    }
    dat 
  } else {
    dat ## Nothing to do if we don't have a states list
  }
}



### Subroutine for characters_to_label, 
###  Also subroutine for get_char_map . 

map_chars_to_label <- function(format){
  map <- sapply(format@char, function(char){
    if(length(char@label) > 0)
      label <- char@label
    else
      label <- char@id
    c(char@id, label)
  })
  out <- map[2,]
  names(out) <- map[1,]
  out
}




# Subroutine of the get_state_maps and state_to_symbol functions
#
# For each character, find the matching `states` set.
# For that set, map each state id to the state symbol 
map_state_to_symbol <- function(format){
  # loop over characters
  map <- lapply(format@char, function(char){ 
    # name the list with elements as `states` sets by their ids 
    states <- format@states 
    ids <- sapply(states, function(states) states@id)
    names(states) <- ids
    # Get the relevant states set matching the current character 
    map_states_to_symbols( states[[char@states]] )
  })  
  names(map) <- name_by_id(format@char)
  map
}
## Subroutine of the map_state_to_symbol function above  
map_states_to_symbols <- function(states){
  map <- sapply(states@state,
                function(state){
                  if(length(state@symbol) > 0)
                    symbol <- state@symbol
                  else
                    symbol <- state@id
                  c(state@id, symbol)
                })
  out <- map[2, ]
  names(out) <- map[1, ]
  out
}





#' @import reshape2
extract_character_matrix <- function(matrix){ 
  otu <- sapply(matrix@row, function(row) row@otu)
  # charnames <- unname(sapply(matrix@row[[1]]@cell, function(cell) cell@char))
  names(matrix@row) <- otu
  mat <- lapply(matrix@row, function(row){
    names(row@cell) <- unname(sapply(row@cell, function(b) b@char))
    # names(row@cell) <- charnames
    lapply(row@cell, function(cell) cell@state)
  })
  mat <- melt(mat)
  colnames(mat) <- c("state", "character", "otu")
  mat <- dcast(mat, otu ~ character, value.var = "state")
  
  # Move otus into rownames and drop the column 
  rownames(mat) <- mat[["otu"]]
  # mat <- mat[-1]
  mat 
}

#' get_taxa
#'
#' Retrieve names of all species/otus otus (operational taxonomic units) included in the nexml 
#' @aliases get_taxa get_otu
#' @param nexml a nexml object
#' @return the list of taxa
#' @export get_taxa get_otu
#' @examples
#' comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
#' nex <- nexml_read(comp_analysis)
#' get_taxa(nex)
#' @seealso  \code{\link{get_item}}
get_taxa <- function(nexml){
  get_level(nexml, "otus/otu")
}
get_otu <- get_taxa

#' get_taxa_list
#' 
#' Retrieve names of all species/otus otus (operational taxonomic units) included in the nexml 
#' @aliases get_taxa_list get_otus_list
#' @param nexml a nexml object
#' @return the list of taxa
#' @export get_taxa_list get_otus_list
#' @seealso  \code{\link{get_item}}
get_taxa_list <-
          function(nexml){
            out <- lapply(nexml@otus, function(otus){
              out <- sapply(otus@otu, function(otu) otu@label)
              names(out) <- name_by_id(otus@otu)
              out
              })
            names(out) <- name_by_id(nexml@otus)
            out
          }
get_otus_list <- get_taxa_list

# filename does not begin with `test` so not run by `testthat::test_dir()`

# This test assumes the working directory contains all the XML files provided 
# here: https://github.com/rvosa/supertreebase/tree/master/data/treebase

files <- system("ls *.xml", intern=TRUE)

print("testing parsing only")

parses <- sapply(files, function(x){
                out <- try(xmlParse(x))
                if(is(out, "try-error"))
                  out <- x
                else {
                  free(out)
                  out = "success"
                }
                out
                })

fails <- parses[parses!="success"]
works <- files[parses == "success"]

writeLines(fails, "unparseable.txt")
print("testing parsing only")


treebase <- sapply(works, 
                function(x){
                  print(x)
                  tree <- try(nexml_read(x, "nexml"))
                  if(is(tree, "try-error"))
                    out = "read failed:"
                  else {
                    tree <- try(as(tree, "phylo"))
                    if(is(tree, "try-error"))
                      out = "conversion failed:"
                    else 
                      out = "success"
                  }
                  rm(tree)
                  out
                })
save(list=ls(), file = "RNeXML_test_results.rda")

table(treebase)

#' Get citation from metadata
#' 
#' Extracts the citation annotation from the metadata annotation of the`nexml`
#' object, and returns its value.
#'
#' Currently the implementation looks for `dcterms:bibliographicCitation`
#' annotations. (Note that these may be given with any prefix in the metadata
#' so long as they expand to the same full property URIs.)
#' @seealso [get_metadata_values()]
#' @param nexml a nexml object
#' @return the citation if the metadata provides one that is non-empty, and
#'   NA otherwise. If multiple non-empty annotations are found, only the first
#'   one is returned.
#' @export
get_citation <- function(nexml){
  cit <- get_metadata_values(nexml, props = c("dcterms:bibliographicCitation"))
  if (length(cit) > 0) {
    # remove empty values
    cit[sapply(cit, length) > 0][1]
  } else
    NA
}

#' Get license from metadata
#'
#' Extracts the license annotation from the metadata annotation of the`nexml`
#' object, and returns its value.
#'
#' Currently the implementation looks for `cc:license` and `dc:rights`
#' annotations. (Note that these may be given with any prefix in the metadata
#' so long as they expand to the same full property URIs.)
#' @seealso [get_metadata_values()]
#' @param nexml a nexml object
#' @return the license if the metadata asserts one that is non-empty, and
#'   NA otherwise.If multiple non-empty annotations are found, only the first
#'   one is returned.
#' @export 
get_license <- function(nexml){
  lic <- get_metadata_values(nexml, props = c("dc:rights", "cc:license"))
  if (length(lic) > 0) {
    # remove empty values
    lic[sapply(lic, length) > 0][1]
  } else
    NA
}

#' Add basic metadata
#' 
#' adds Dublin Core metadata elements to (top-level) nexml 

#' @param title A title for the dataset
#' @param description a description of the dataset
#' @param creator name of the data creator.  Can be a string or R person object
#' @param pubdate publication date.  Default is current date.  
#' @param rights the intellectual property rights associated with the data.  
#'   The default is Creative Commons Zero (CC0) public domain declaration, 
#'   compatible with all other licenses and appropriate for deposition 
#'   into the Dryad or figshare repositories.  CC0 is also recommended by the Panton Principles.
#'   Alternatively, any other plain text string can be added and will be provided as the content
#'   attribute to the dc:rights property.  
#' @param publisher the publisher of the dataset. Usually where a user may go to find the canonical
#'   copy of the dataset: could be a repository, journal, or academic institution.  
#' @param citation a citation associated with the data.  Usually an academic journal 
#'    article that indicates how the data should be cited in an academic context.  Multiple citations
#'    can be included here.  
#'    citation can be a plain text object, but is preferably an R `citation` or `bibentry` object (which
#'    can include multiple citations.  See examples
#' @param nexml a nexml object to which metadata should be added.  A new 
#'   nexml object will be created if none exists. 
#' @return an updated nexml object
#' @details \code{add_basic_meta()} is just a wrapper for \code{\link{add_meta}} to make it easy to 
#'    provide generic metadata without explicitly providing the namespace.  For instance, 
#'    \code{add_basic_meta(title="My title", description="a description")} is identical to:
#'    \code{add_meta(list(meta("dc:title", "My title"), meta("dc:description", "a description")))}

#'    Most function arguments are mapped directly to the Dublin Core terms
#'    of the same name, with the exception of `rights`, which by default maps
#'    to the Creative Commons namespace when using CC0 license.
#'    
#' @seealso \code{\link{add_trees}} \code{\link{add_characters}} \code{\link{add_meta}}
#' @export 
#' @importFrom stats na.omit
#' @importFrom utils capture.output head object.size
#' @examples
#' nex <- add_basic_meta(title = "My test title",
#'              description = "A description of my test",
#'              creator = "Carl Boettiger <cboettig@@gmail.com>",
#'              publisher = "unpublished data",
#'              pubdate = "2012-04-01")
#' 
#'  ## Adding citation to an R package:
#'  nexml <- add_basic_meta(citation=citation("ape"))
#' \dontrun{
#'  ## Use knitcitations package to add a citation by DOI:
#'  library(knitcitations)
#'  nexml <- add_basic_meta(citation = bib_metadata("10.2307/2408428"))
#'  }
#' @include classes.R
add_basic_meta <- function(title = NULL, 
                           description = NULL,
                           creator = Sys.getenv("USER"),
                           pubdate = NULL,
                           rights = "CC0",
                           publisher = NULL,
                           citation = NULL,
                           nexml = new("nexml")
                           ){

  if(!is.null(title)) 
    nexml <- add_meta(meta("dc:title", title), nexml)
  if(!is.null(creator) || creator == "") {
    creators <- get_metadata_values(nexml, props = c("dc:creator"))
    if (! (creator %in% creators))
      nexml <- add_meta(meta("dc:creator", format(creator)), nexml)
  }
  if(!is.null(pubdate))
    nexml <- add_meta(meta("dcterms:modified",
                           format(pubdate, tz = "GMT", usetz = TRUE)),
                      nexml)
  if(!is.null(description)) 
    nexml <- add_meta(meta("dc:description", description), nexml)
  if (!is.null(publisher))
    nexml <- add_meta(meta("dcterms:publisher", publisher), nexml)
  if(!is.null(rights)){
    if(is.na(get_license(nexml))) {
      if(rights == "CC0")
        nexml <- add_meta(meta(rel="cc:license",
                               href="http://creativecommons.org/publicdomain/zero/1.0/"),
                          nexml)
      else
        nexml <- add_meta(meta("dc:rights", rights), nexml)
    }
  }
  if(!is.null(citation)) {
    if(is(citation, "BibEntry"))
      class(citation) = "bibentry"
    if(is(citation, "bibentry"))
      nexml <- add_meta(nexml_citation(citation), nexml)
    else
      nexml <- add_meta(meta("dcterms:bibliographicCitation", citation), nexml)
  }
  nexml
}

#' Get the desired element from the nexml object
#' 
#' Get the desired element from the nexml object
#' @aliases nexml_get get_item
#' @param nexml a nexml object (from read_nexml)
#' @param element the kind of object desired, see details.  
#' @param ... additional arguments, if applicable to certain elements 
#' @details
#'  
#' \itemize{
#'  \item{"tree"}{ an ape::phylo tree, if only one tree is represented.  Otherwise returns a list of lists of multiphylo trees.  To consistently receive the list of lists format (preserving the hierarchical nature of the nexml), use \code{trees} instead.}
#'  \item{"trees"}{ returns a list of lists of multiphylo trees, even if all trees are in the same `trees` node (and hence the outer list will be of length 1) or if there is only a single tree (and hence the inner list will also be of length 1.  This ensures a consistent return type regardless of the number of trees present in the nexml file, and also preserves any hierarchy/grouping of trees.  }
#'  \item{"flat_trees"}{ a multiPhylo object (list of ape::phylo objects) Note that this method collapses any hierarchical structure that may have been present as multiple `trees` nodes in the original nexml (though such a feature is rarely used).  To preserve that structure, use `trees` instead.}
#'  \item{"metadata"}{Get metadata from the specified level (default is top/nexml level) }
#'  \item{"otu"}{ returns a named character vector containing all available metadata.  names indicate \code{property} (or \code{rel} in the case of links/resourceMeta), while values indicate the \code{content} (or \code{href} for links). }
#'  \item{"taxa"}{ alias for otu }
#' }
#' For a slightly cleaner interface, each of these elements is also defined as an S4 method
#' for a nexml object.  So in place of `get_item(nexml, "tree")`, one could use `get_tree(nexml)`,
#' and so forth for each element type.  
#' @return return type depends on the element requested.  See details.  
#' @export
#' @seealso \code{\link{get_trees}}
#' @include classes.R
#' @examples
#' comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
#' nex <- nexml_read(comp_analysis)
#' nexml_get(nex, "trees")
#' nexml_get(nex, "characters_list")
nexml_get <- function(nexml, 
                     element = c("trees", 
                                 "trees_list", 
                                 "flat_trees", 
                                 "metadata", 
                                 "otu",
                                 "taxa",
                                 "characters", 
                                 "characters_list",
                                 "namespaces"), 
                     ...){
  element <- match.arg(element)

  switch(element,
         trees = get_trees(nexml), # will warn if more than one tree is available
         trees_list = get_trees_list(nexml),
         flat_trees = get_flat_trees(nexml),
         metadata = get_metadata(nexml, ...),
         otu = get_taxa(nexml),
         taxa = get_taxa(nexml),
         characters = get_characters(nexml),
         characters_list = get_characters_list(nexml),
         namespaces = get_namespaces(nexml))
}

get_item <- nexml_get

### TODO: how to deal with missing meta slot elements???
### Right now the functions break when an otu doesn't have meta slot


#' get_taxa_meta
#'
#' Retrieve metadata of all species/otus otus (operational taxonomic units) included in the nexml
#' @param nexml a nexml object
#' @param what One of href, rel, id, or xsi:type
#' @return the list of metadata for each taxon
#' @seealso  \code{\link{get_item}}
#' @keywords internal 
#' @examples 
#' \dontrun{
#' data(bird.orders)
#' birds <- add_trees(bird.orders)
#' birds <- taxize_nexml(birds, "NCBI")
#' RNeXML:::get_taxa_meta(birds)
#' RNeXML:::get_taxa_meta(birds, 'rel')
#' RNeXML:::get_taxa_meta(birds, 'id')
#' RNeXML:::get_taxa_meta(birds, 'xsi:type')
#'  }
get_taxa_meta <-  
  function(nexml, what='href'){
    out <- lapply(nexml@otus, function(otus)
#       sapply(otus@otu, function(otu) slot(otu@meta, what)))
    sapply(otus@otu, function(otu) slot(otu@meta[[1]], what)))
    unname(unlist(out, recursive = FALSE))
  }
get_otu_meta <- get_taxa_meta

#' get_taxa_meta_list
#'
#' Retrieve metadata of all species/otus otus (operational taxonomic units) included in the nexml
#' @param nexml a nexml object
#' @param what One of href, rel, id, or xsi:type
#' @return the list of metadata for each taxon
#' @seealso  \code{\link{get_item}}
#' @keywords internal
#' @examples \dontrun{
#' data(bird.orders)
#' birds <- add_trees(bird.orders)
#' birds <- taxize_nexml(birds, "NCBI")
#' RNeXML:::get_taxa_meta_list(birds)
#' RNeXML:::get_taxa_meta_list(birds, 'rel')
#' RNeXML:::get_taxa_meta_list(birds, 'id')
#' RNeXML:::get_taxa_meta_list(birds, 'xsi:type')
#' }
get_taxa_meta_list <-
  function(nexml, what='href'){
    out <- lapply(nexml@otus, function(otus){
      out <- sapply(otus@otu, function(otu) slot(otu@meta[[1]], what))
      names(out) <- name_by_id(otus@otu)
      out
    })
    names(out) <- name_by_id(nexml@otus)
    out
  }
get_otu_meta_list <- get_taxa_meta_list

## code for better dealing with namespace-prefixed strings (generally URIs)

#' Expand namespace-prefixed string
#'
#' Substitutes the namespace prefix in the input vector of strings with
#' the corresponding namespaces.
#'
#' Namespace prefixes are expected to be separated by one or more semicolons.
#' Prefixes that cannot be matched to the vector of namespaces will be left
#' as is. For strings that do not have a namespace prefix, the vector of
#' namespaces can contain a base namespace, identified as not having a name,
#' with which these strings will be expanded.
#' @param x a character vector of potentially namespace-prefixed strings
#' @param namespaces a named vector of namespaces, with namespace prefixes
#'   being the names. A "base" namespace with an empty name can be included.
#'   If not provided, or if empty, the input vector is returned as is.
#' @return a character vector, of the same length as the input vector
#' @examples
#' uris <- c("cc:license", "dc:title")
#' ns <- c(dc = "http://purl.org/dc/elements/1.1/",
#'         dcterms = "http://purl.org/dc/terms/",
#'         dct = "http://purl.org/dc/terms/",
#'         cc = "http://creativecommons.org/ns#")
#' # expansion is vectorized
#' expand_prefix(uris, ns)
#'
#' # strings with non-matching or no prefix are left as is
#' uris <- c(uris, "my:title", "title")
#' expand_prefix(uris, ns)
#'
#' # NAs in the input list become NA in the output
#' uris <- c(uris, NA)
#' expand_prefix(uris, ns)
#'
#' # can include a "base" (unnamed) namespace for expanding unprefixed strings
#' ns <- c(ns, "//local/")
#' xuris <- expand_prefix(uris, ns)
#' xuris
#' xuris[uris == "title"] == paste0("//local/", uris[uris == "title"])
#'
#' # different prefixes may expand to the same result
#' expand_prefix("dcterms:modified", ns) == expand_prefix("dct:modified", ns)
#'
#' # or they may result in different expansions
#' expand_prefix("dc:title", ns) != expand_prefix("dcterms:title", ns)
#' @export
#' @importFrom stringi stri_match_first_regex stri_replace_first_regex
expand_prefix <- function(x, namespaces = NULL) {
  if (is.null(namespaces) || length(namespaces) == 0)
    x
  else {
    nsNames <- names(namespaces)
    if (is.null(nsNames))
      # single non-named element (or all elements unnamed)
      isBase <- rep(TRUE, length(namespaces))
    else
      isBase <- nsNames == ""
    # We're being clever here: if the prefix pattern doesn't match, we assign
    # the pattern for matching at the beginning of the string, and to have that
    # matched as all other namespaces we'll make it the name of the base
    # namespace, using an empty string for expansion if none was provided.
    if (any(isBase))
      names(namespaces)[isBase] <- "^"
    else
      namespaces <- c(namespaces, "^" = "")
    prefixes <- stringi::stri_match_first_regex(x, "^([^:]*):", cg_missing = "^")
    ns <- namespaces[match(prefixes[,2], names(namespaces))]
    if (all(is.na(ns)))
      x
    else {
      expanded <- stringi::stri_replace_first_regex(x, prefixes[,1], ns)
      ifelse(is.na(expanded), x, expanded)
    }
  }
}

context("expored utility functions")

test_that("expand_prefix() works correctly", {
  uris <- c("license", "title")
  # returns unchanged if no namespaces
  testthat::expect_equal(expand_prefix(uris), uris)
  # returns unchanged if no matching namespace
  testthat::expect_equal(expand_prefix(uris, c(ns = "//foo/")), uris)
  uris2 <- paste0("foo:", uris)
  testthat::expect_equal(expand_prefix(uris2, c(ns = "//foo/")), uris2)
  # if none are prefixed and we give a base namespace, it's the same as paste0
  testthat::expect_equal(expand_prefix(uris, c("//foo/")), paste0("//foo/", uris))
  # should be able to handle NAs in the input
  uris <- c(NA, uris, NA)
  testthat::expect_equal(expand_prefix(uris), uris)
  testthat::expect_equal(expand_prefix(uris, c(ns = "//foo/")), uris)
  # not quite the same as paste0 if NAs are to be allowed
  testthat::expect_equal(expand_prefix(uris, c("//foo/")),
                         ifelse(is.na(uris), uris, paste0("//foo/", uris)))
  # namespaces get expanded 
  uris <- c("cc:license", "dc:title", "/nexml", "foo:bar", NA)
  nex <- new("nexml")
  expanded <- expand_prefix(uris, namespaces = nex@namespaces)
  testthat::expect_equal(expanded[1],
                         sub("^cc:", nex@namespaces["cc"], expanded[1]))
  testthat::expect_equal(expanded[2],
                         sub("^dc:", nex@namespaces["dc"], expanded[2]))
  testthat::expect_equal(expanded[3],
                         paste0(nex@namespaces["nex"], uris[3]))
  testthat::expect_equal(expanded[4], "foo:bar")
  testthat::expect_length(expanded, length(uris))
  testthat::expect_true(is.na(expanded[5]))
  # exact prefix doesn't matter so long as they expand to the same URI
  testthat::expect_equivalent(expand_prefix("ns1:foo", c(ns1 = "http://example.org/")),
                              expand_prefix("ns2:foo", c(ns2 = "http://example.org/")))
})

## FIXME might want to define this for sub-nodes.  e.g. so we can get all metadata on "nodes" in tree2...

#' get_metadata
#' 
#' get_metadata 
#' @param nexml a nexml object
#' @param level the name of the level of element desired, see details
#' @param simplify logical, see Details
#' @return the requested metadata as a data.frame. Additional columns
#' indicate the parent element of the return value.
#' @details 'level' should be either the name of a child element of a NeXML document 
#' (e.g. "otu", "characters"), or a path to the desired element, e.g. 'trees/tree'
#' will return the metadata for all phylogenies in all trees blocks.
#' 
#' If a metadata element has other metadata elements nested within it, the
#' nested metadata are returned as well. A column "Meta" will contain the
#' IDs consolidated from the type-specific LiteralMeta and ResourceMeta
#' columns, and IDs are generated for meta elements that have nested elements
#' but do not have an ID ("blank nodes"). A column "meta" contains the
#' IDs of the parent meta elements for nested ones. This means that the
#' resulting table can be self-joined on those columns.
#' 
#' If `simplify` is `FALSE`, the type-specific "LiteralMeta" and "ResourceMeta"
#' columns will be retained even if a consolidated "Meta" column is present.
#' Otherwise, only the consolidated column will be included in the result.
#' Also, if `simplify` is `TRUE` the values for "property" (LiteralMeta) and
#' "rel" (ResourceMeta) will be consolidated to "property", and "rel" will be
#' removed from the result.
#' 
#' @import XML
#' @examples \dontrun{
#' comp_analysis <- system.file("examples", "primates.xml", package="RNeXML")
#' nex <- nexml_read(comp_analysis)
#' get_metadata(nex)
#' get_metadata(nex, "otus/otu")
#' }
#' @export
get_metadata <- function(nexml, level = "nexml", simplify = TRUE){
  
#  level = c("nexml", "otus", "trees", "characters", 
#            "otus/otu", "trees/tree", "characters/format", "characters/matrix",
#            "characters/format/states")
#  level <- match.arg(level)

  ## Handle deprecated formats
  if(level =="otu")
    level <- "otus/otu"
  if(level =="tree")
    level <- "trees/tree"

  
  
  if(level == "nexml")
    level <- "meta"
  else
    level <- paste(level, "meta", sep="/") 
 
  out <- get_level(nexml, level)
  if (simplify) {
    cnames = colnames(out)
    if (! ("Meta" %in% cnames)) {
      out <- dplyr::mutate(out,
                           "Meta" = coalesce_(out$LiteralMeta,
                                              out$ResourceMeta,
                                              as.character(rep(NA, times=nrow(out)))))
    }
    out <- dplyr::mutate(out, LiteralMeta = NULL, ResourceMeta = NULL)
    if (all(c("rel","property") %in% cnames))
      out <- dplyr::mutate(out,
                           property = dplyr::if_else(is.na(out$property),
                                                     out$rel,
                                                     out$property),
                           rel = NULL)
    else if ("rel" %in% cnames)
      out <- dplyr::rename(out, property = "rel")
  }

  out
}

#' Get the value(s) for metadata
#'
#' Extracts the values from the metadata annotations for the given property
#' or properties, and returns the result.
#'
#' For matching property identifiers (i.e., URIs), prefixes in the input list
#' as well as in the `annotated` object will be expanded using the namespaces
#' of the `nexml` object. Names in the returned vector are mapped to the
#' (possibly prefixed) form in the input list.
#' @param nexml a nexml object
#' @param annotated the nexml component object from which to obtain metadata
#'   annotations, defaults to the nexml object itself
#' @param props a character vector of property names for which to extract
#'   metadata annotations
#' @return a named character vector, giving the values and names being the
#'   property names
#' @export
get_metadata_values <- function(nexml, annotated = NULL, props){
  metaList <- get_meta(nexml, annotated = annotated, props = props)
  if (length(metaList) > 0) {
    sapply(metaList,
           function(m)
             if (is(m, "LiteralMeta")) charzero_as_empty(m@content) else m@href)
  } else
    c()
}

#' Extracts meta objects matching properties
#'
#' Extracts the metadata annotations for the given property or properties,
#' and returns the result as a list of `meta` objects.
#'
#' For matching property identifiers (i.e., URIs), prefixes in the input list
#' as well as in the `annotated` object will be expanded using the namespaces
#' of the `nexml` object. Names in the returned list are mapped to the
#' (possibly prefixed) form in the input list. The resulting list is flat,
#' and hence does not retain the nesting hierarchy in the object's annotation. 
#' @param nexml a nexml object
#' @param annotated the nexml component object from which to obtain metadata
#'   annotations, or a list of such objects. Defaults to the nexml object itself.
#' @param props a character vector of property names for which to extract
#'   metadata annotations
#' @return a named list of the matching meta objects
#' @export
get_meta <- function(nexml, annotated = NULL, props){
  if (is.null(annotated)) annotated <- nexml
  if (is.null(props) || length(props) == 0)
    stop("Parameter 'props' must be a non-empty vector")
  uris <- expand_prefix(props, nexml@namespaces)
  if (is(annotated, "Annotated"))
    metaList <- get_all_meta(annotated)
  else if (is(annotated, "list"))
    metaList <- unlist(lapply(annotated, get_all_meta))
  else
    stop("'annotated' must be either an instance of 'Annotated', or a list")
  metaProps <- expand_prefix(sapply(metaList, slot, "property"), nexml@namespaces)
  isMatch <- metaProps %in% uris
  if (any(isMatch)) {
    values <- metaList[isMatch]
    mapToURIs <- match(metaProps, uris)
    names(values) <- props[mapToURIs[! is.na(mapToURIs)]]
    New("ListOfmeta", values)
  } else
    New("ListOfmeta")
}

#' Get flattened list of meta annotations
#'
#' Collects recursively (in the case of nested meta annotations) all meta
#' object annotations for the given object, and returns the result as a flat
#' list.
#'
#' Does not check that the input object can actually have meta annotations.
#' An invalid slot error will be generated if it can't.
#' @param annotated the object from which to extract meta object annotations
#' @return a flat list of `meta` objects
#' @export
get_all_meta <- function(annotated) {
  metaList <- slot(annotated, "meta")
  if (length(metaList) > 0) {
    containsNested <- sapply(metaList, is, "ResourceMeta")
    if (any(containsNested)) {
      nested <- lapply(metaList[containsNested], get_all_meta)
      if (length(nested) > 0)
        metaList <- c(metaList, nested)
    }
  }
  metaList
}

context("serializing")


## More tests at lower-level serializing from S4 to XML in inheritance.R

test_that("We can serialize ape to S4 RNeXML into valid NeXML",{
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  data(bird.orders)


  nexml <- as(bird.orders, "nexml") 

  saveXML(as(nexml, "XMLInternalNode"), file = "test.xml")
  expect_true_or_null(nexml_validate("test.xml"))

  ###  Higher level API tests
  nexml_write(bird.orders, file="test.xml")
  expect_true_or_null(nexml_validate("test.xml"))

  ## also can write with providing multiple metadata (see README.Rmd)
  expect_silent(nexml_write(bird.orders, file="test.xml",
                            title = "My test title",
                            description = "A description of my test",
                            creator = "Carl Boettiger <cboettig@gmail.com>",
                            publisher = "unpublished data",
                            pubdate = "2012-04-01"))
  nex <- nexml_read("test.xml")
  metavals <- get_metadata_values(nex, props = c("dc:creator", "dcterms:modified"))
  expect_equivalent(metavals["dc:creator"], "Carl Boettiger <cboettig@gmail.com>")
  expect_equivalent(metavals["dcterms:modified"], "2012-04-01")

  ##  Clean up
  unlink("test.xml")

  })


test_that("We can serialize parsed NeXML to S4 RNeXML into valid NeXML",{
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  root <- xmlRoot(xmlParse(system.file("examples", "trees.xml", package="RNeXML")))
  tree <- as(root, "nexml")
  nexml_write(tree, file="test.xml")

  ## validate
  expect_true_or_null(nexml_validate("test.xml"))

  ##  Clean up
  unlink("test.xml")

  })

test_that("We can correctly serialize XML literals as metadata", {
  
  ## tests are too slow for CRAN
  skip_on_cran()
  
  nex1 <- read.nexml(system.file("examples", "phenex.xml", package="RNeXML"))
  m_xml1 <- get_metadata(nex1, "characters/format/states/state") %>%
    dplyr::filter(property == "ps:describesPhenotype")
  # write it out and then read it back in
  write.nexml(nex1, file="test.xml")
  
  # not sure why XML is considered invalid
  #expect_true_or_null(nexml_validate("test.xml"))
  
  
  nex2 <- read.nexml("test.xml")
  m_xml2 <- get_metadata(nex2, "characters/format/states/state") %>%
    dplyr::filter(property == "ps:describesPhenotype")
  # the XML literals in the form of string values should be the same
  testthat::expect_true(all(dplyr::arrange(m_xml1, state)[,"content"] ==
                              dplyr::arrange(m_xml2, state)[,"content"]))

  ## clean up
  unlink("test.xml")
})

test_that("we can serialize NeXML with rootedge to a valid NeXML file", {
  
  skip("rootedge is not valid by XSD schema?")
  ## tests are too slow for CRAN
  skip_on_cran()
  
  f <- system.file("examples", "coal.xml", package = "RNeXML")
  nex <- read.nexml(f)
  write.nexml(nex, file = "test.xml")
  valid <- nexml_validate("test.xml")
  expect_true_or_null()

  unlink("test.xml")
})

context("extract_metadata")

nex <- add_basic_meta(
            title = "My test title",
            description = "A description of my test",
            creator = "Carl Boettiger <cboettig@gmail.com>",
            publisher = "unpublished data",
            pubdate = "2012-04-01",
            citation = citation("ape"))

test_that("get_meta() and friends have some basic parameter checking", {
  testthat::expect_error(get_meta(nex, props = c()))
  testthat::expect_error(get_meta(nex, props = NULL))
  testthat::expect_error(get_meta(nex, "foo", props = "foo"))
})

test_that("get_meta() and friends return the correct types", {
  mlist <- get_meta(nex, props = "dcterms:modified")
  testthat::expect_is(mlist, "ListOfmeta")
  testthat::expect_equal(names(mlist), "dcterms:modified")
  mlist <- get_meta(nex, props = "foo")
  testthat::expect_is(mlist, "ListOfmeta")
  testthat::expect_length(mlist, 0)
  mlist <- get_meta(nex, props = c("dcterms:modified", "dc:description"))
  testthat::expect_is(mlist, "ListOfmeta")
  testthat::expect_equal(names(mlist), c("dcterms:modified", "dc:description"))

  testthat::expect_is(get_all_meta(nex), "ListOfmeta")
  citMeta <- get_meta(nex, props = "dcterms:references")
  testthat::expect_gt(length(citMeta), 0)
  testthat::expect_is(citMeta, "ListOfmeta")
  nested <- get_all_meta(citMeta[[1]])
  testthat::expect_gt(length(nested), 0)
  testthat::expect_is(nested, "ListOfmeta")

})

test_that("we can extract metadata using the dedicated functions", {

  testthat::expect_equivalent(get_citation(nex), format(citation("ape"), "text"))
  testthat::expect_equivalent(get_license(nex), "http://creativecommons.org/publicdomain/zero/1.0/")
  m <- get_metadata(nex)
  mlist <- get_all_meta(nex)
  testthat::expect_gte(length(m[,1]), 6)
  testthat::expect_length(m[,1], length(mlist))
  mvalues <- get_metadata_values(nex, props = c("dc:creator",
                                                "dc:title",
                                                "dc:description",
                                                "dcterms:publisher",
                                                "dcterms:modified"))
  testthat::expect_equivalent(mvalues["dc:creator"], "Carl Boettiger <cboettig@gmail.com>")
  testthat::expect_equivalent(mvalues["dc:title"], "My test title")
  testthat::expect_equivalent(mvalues["dc:description"], "A description of my test")
  testthat::expect_equivalent(mvalues["dcterms:publisher"], "unpublished data")
  testthat::expect_equivalent(mvalues["dcterms:modified"], "2012-04-01")
  summary(nex)
})



test_that("we can parse literal meta nodes with literal node content", {
  
  f <- system.file("examples", "ontotrace-result.xml", package="RNeXML")
  nex <- read.nexml(f)
  out <- get_metadata(nex, level = "nexml")
  matches <- sum(grepl("Phenoscape Knowledgebase", out$content))
  testthat::expect_true(matches > 0)

  # empty literal node content
  mlist <- get_meta(nex, props = "dc:creator")
  testthat::expect_is(mlist, "ListOfmeta")
  testthat::expect_length(mlist, 1)
  testthat::expect_is(mlist[[1]], "LiteralMeta")
  testthat::expect_is(mlist[[1]]@content, "character")
  testthat::expect_length(mlist[[1]]@content, 0)
  vals <- get_metadata_values(nex, props = "dc:creator")
  testthat::expect_is(vals, "character")
  testthat::expect_identical(names(vals), c("dc:creator"))
  testthat::expect_length(vals, 1)
  testthat::expect_is(vals[1], "character")
  testthat::expect_equivalent(vals[1], "")
  vals <- get_metadata_values(nex, props = c("dc:creator", "dc:description"))
  testthat::expect_is(vals, "character")
  testthat::expect_identical(names(vals), c("dc:creator", "dc:description"))
  testthat::expect_length(vals, 2)
  testthat::expect_is(vals["dc:creator"], "character")
  testthat::expect_equivalent(vals["dc:creator"], "")
  
  # test correct roundtrip
  doc <- as(add_meta(mlist[[1]]), "XMLInternalNode")
  nex1 <- as(doc, "nexml")
  mlist1 <- get_meta(nex1, props = "dc:creator")
  testthat::expect_is(mlist1, "ListOfmeta")
  testthat::expect_length(mlist1, 1)
  testthat::expect_is(mlist1[[1]]@content, "character")
  testthat::expect_length(mlist1[[1]]@content, 0)
})

test_that("we can extract all available metadata at a specified level of the DOM", {
  f <- system.file("examples", "ontotrace-result.xml", package = "RNeXML")
  nex <- read.nexml(f)

  # otu metadata
  m.otu <- get_metadata(nex, "otus/otu")
  m.taxonId <- dplyr::filter(m.otu, property == "dwc:taxonID")
  testthat::expect_equal(nrow(m.taxonId), length(nex@otus[[1]]@otu))
  testthat::expect_gt(nrow(m.otu), nrow(m.taxonId))
  taxonIDs <- get_metadata_values(nex, nex@otus[[1]]@otu, "dwc:taxonID")
  testthat::expect_length(taxonIDs, nrow(m.taxonId))
  testthat::expect_equivalent(taxonIDs, m.taxonId[,"href"])

  # character metadata
  m.c <- get_metadata(nex, "characters/format/char")
  denotes <- expand_prefix(m.c[1,"property"], nex@namespaces)
  m.denotes <- dplyr::filter(m.c,
                             expand_prefix(property, nex@namespaces) == denotes)
  testthat::expect_equal(nrow(m.c), nrow(m.denotes))
  testthat::expect_equal(nrow(m.c), length(nex@characters[[1]]@format@char))
  entityIDs <- get_metadata_values(nex, nex@characters[[1]]@format@char, denotes)
  testthat::expect_length(entityIDs, nrow(m.denotes))
  testthat::expect_equivalent(entityIDs, m.denotes[,"href"])
})


test_that("we can correctly parse nested ResourceMeta annotations", {
  f <- system.file("examples", "meta_example.xml", package="RNeXML")
  nex <- read.nexml(f)
  meta <- get_metadata(nex)
  lic <- dplyr::filter(meta, property == "cc:license")$href
  testthat::expect_equal(lic, "http://creativecommons.org/publicdomain/zero/1.0/")

  # remove rows from nested meta elements (which there should be!)
  testthat::expect_true("meta" %in% colnames(meta))
  topMeta <- meta[is.na(meta[,"meta"]),]
  testthat::expect_lt(nrow(topMeta), nrow(meta))
  # there should be one dc:title at top level, and two if we include nested
  testthat::expect_gt(nrow(dplyr::filter(meta, property == "dc:title")),
                      nrow(dplyr::filter(topMeta, property == "dc:title")))
  # test that the ID referencing for self-joining is correct
  testthat::expect_true(all(meta[! is.na(meta[, "meta"]), "meta"] %in%
                              meta[! is.na(meta[, "Meta"]), "Meta"]))
  # ResourceMetas should _either_ have an href _or_ have nested meta elements
  topMeta <- cbind(topMeta, nkids = sapply(nex@meta, function(x) length(x@children)))
  rmeta <- dplyr::filter(topMeta, xsi.type == "ResourceMeta")
  testthat::expect_true(all(xor(is.na(rmeta[, "href"]), rmeta[, "nkids"] == 0)))
  testthat::expect_gt(max(rmeta[,"nkids"]), 0)
  # the sum of the children should equal the number of nested meta elements
  testthat::expect_equal(sum(rmeta[, "nkids"]),
                         nrow(meta) - nrow(topMeta))
})

test_that("metadata tables can be requested in simplified form", {
  f <- system.file("examples", "meta_example.xml", package="RNeXML")
  nex <- read.nexml(f)
  meta1 <- get_metadata(nex, simplify = FALSE)
  meta2 <- get_metadata(nex, simplify = TRUE)
  
  testthat::expect_equal(dplyr::filter(meta1, rel == "cc:license")$href,
                         dplyr::filter(meta2, property == "cc:license")$href)
  removedCols <- c("ResourceMeta", "LiteralMeta", "rel")
  testthat::expect_true(all(removedCols %in% colnames(meta1)))
  testthat::expect_false(any(removedCols %in% colnames(meta2)))
})

test_that("ID assignments are correct and complete when meta are nested", {
  f <- system.file("examples", "phenex.xml", package="RNeXML")
  nex <- read.nexml(f)
  m.otu <- get_metadata(nex, level = "otus/otu")
  # the nested meta elements are actually there
  testthat::expect_equal(sum(m.otu[,"property"] == "dwc:catalogNumber",
                             na.rm = TRUE),
                         5)
  # the ID column for the containing element (here: otu) is populated in full
  testthat::expect_false(any(is.na(m.otu[,"otu"])))
  # the IDs for the containing element are not all the same
  testthat::expect_gt(length(unique(m.otu[,"otu"])), 1)
  # expect 5 groups if we group by otu ID
  testthat::expect_length(tapply(m.otu[,"xsi.type"], m.otu[,"otu"], length), 5)
  # expect 4 rows for each otu ID
  testthat::expect_true(all(tapply(m.otu[,"xsi.type"], m.otu[,"otu"], length) == 4))
  # expect 1 LiteralMeta and 3 ResourceMeta for each otu ID
  meta.grouped <- tapply(m.otu[,"xsi.type"],
                         list(m.otu[,"otu"], m.otu[,"xsi.type"]),
                         length)
  testthat::expect_true(all(meta.grouped[,"LiteralMeta"] == 1))
  testthat::expect_true(all(meta.grouped[,"ResourceMeta"] == 3))
  # for each otu ID, two meta are nested (have IDREF to a containing meta)
  meta.nested <- m.otu[!is.na(m.otu[,"meta"]),]
  testthat::expect_true(all(tapply(meta.nested[,"meta"], meta.nested[,"otu"],
                                   length) == 2))
  # but the two nested meta for each otu have the same IDREF
  testthat::expect_true(all(tapply(meta.nested[,"meta"], meta.nested[,"otu"],
                                   function(x) length(unique(x))) == 1))
  # the IDREFs of the nested meta match up with ID of containing meta
  xsi.type <- NULL # silence R check
  meta.cont <- dplyr::filter(m.otu, 
                             xsi.type == "ResourceMeta",
                             is.na(m.otu[,"href"]))
  testthat::expect_true(all(meta.nested[,"meta"] %in% meta.cont[,"Meta"]))
  # they are all different
  testthat::expect_length(unique(meta.nested[,"meta"]), 5)
  testthat::expect_length(unique(meta.cont[,"Meta"]), 5)
  # and every containing ID is referenced by exactly one of the nested IDREFs
  testthat::expect_true(all(sort(meta.cont[,"Meta"]) == sort(unique(meta.nested[,"meta"]))))
})

test_that("we can parse LiteralMeta annotations with XML literals as values", {
  f <- system.file("examples", "phenex.xml", package="RNeXML")
  nex <- read.nexml(f)

  # the XML annotations for state elements should have been parsed
  states <- nex@characters[[1]]@format@states[[1]]@state # list of state objects
  testthat::expect_true(all(sapply(states, function(s) length(s@meta) >= 1)))
  testthat::expect_setequal(sapply(states, function(s) class(s@meta[[1]])),
                            "LiteralMeta")
  testthat::expect_true(all(sapply(states, 
                                   function(s) length(s@meta[[1]]@content)) > 0))
  testthat::expect_setequal(sapply(states, function(s) class(s@meta[[1]]@content)),
                            "XMLString")
  # the correct namespace definition should have been retained
  testthat::expect_false(any(sapply(states, 
                                   function(s) is.null(attr(s@meta[[1]]@content,
                                                            "namespaces")))))
  nsPrefix <- names(attr(states[[1]]@meta[[1]]@content, "namespaces"))
  testthat::expect_setequal(sapply(states,
                                   function(s) attr(s@meta[[1]]@content, "namespaces")),
                            get_namespaces(nex)[nsPrefix])

  # the XML literals are returned as meta content
  m <- get_metadata(nex, "characters/format/states/state")
  m_xml <- dplyr::filter(m, property == "ps:describesPhenotype")
  # should have one row for each state
  n_states <- sum(sapply(nex@characters[[1]]@format@states,
                         function(x) length(x@state)))
  testthat::expect_gte(n_states, 6) # 3 characters, at least 2 states each
  testthat::expect_length(m_xml[,1], n_states)
  # they are of type LiteralMeta
  testthat::expect_setequal(m_xml[,"xsi.type"], "LiteralMeta")
  # their content is non-empty, and a string that is valid XML
  testthat::expect_false(any(sapply(m_xml[,"content"], is.na)))
  testthat::expect_true(all(sapply(m_xml[,"content"], is.character)))
  testthat::expect_true(all(sapply(m_xml[,"content"], nchar) > 0))
  testthat::expect_true(all(sapply(m_xml[,"content"], XML::isXMLString)))
})

setGeneric("fromNeXML", function(obj, from) standardGeneric("fromNeXML")) 


setGeneric("toNeXML", 
           valueClass="XMLInternalElementNode",
           function(object, parent) 
             standardGeneric("toNeXML"))

# Rather verbose methods definitions manually reading and writing each class...
# Rather than just slot matching names, we explicitly map each attribute...

##############################

setClass("Base",
         slots = c('xsi:type' = "character"))
setMethod("toNeXML", 
          signature("Base", "XMLInternalElementNode"), 
          function(object, parent){
            type <- slot(object, "xsi:type")
            if(length(type) > 0){
              #if(is.na(pmatch("nex:", type)))  # nex or relevant namespace should come from default anyway
              #  type <- paste0("nex:", type)
              addAttributes(parent, 
                           "xsi:type" = type,  
                            suppressNamespaceWarning=TRUE) # We always define xsi namespace in the header... 
              }
            parent
          })
setMethod("fromNeXML", 
          signature("Base", "XMLInternalElementNode"),
          function(obj, from){
            if(!is.null(xmlAttrs(from))){
              if(!is.na(xmlAttrs(from)["type"]))  ## FIXME use [["type"]] or ["type"]
                slot(obj, "xsi:type") <- as.character(xmlAttrs(from)["type"])
              if(!is.na(xmlAttrs(from)["xsi:type"])) ## Shouldn't be necessary but seems to be for first test in test_inheritance.R...
                slot(obj, "xsi:type") <- as.character(xmlAttrs(from)["xsi:type"])
            }
            obj
          }
)

#########################

setClass("meta",
         slots = c(children = "list"), 
         contains = "Base")
setAs("XMLInternalNode", "meta", function(from){
  type <- xmlAttrs(from)["type"]
  if(is.na(type)) ## FIXME This is CRUDE
    type <- xmlAttrs(from)["xsi:type"]
  if(is.na(type)) # if still not defined...
    type = "meta"
  else
    type <- sub(".*:", "", type)[1] ## FIXME This is CRUDE
  fromNeXML(New(type), from)
})


#########################

setClass("LiteralMeta", 
         slots = c(id = "character",
                        property = "character", 
                        datatype = "character",
                        content = "character"),
         contains="meta")
setMethod("fromNeXML", 
          signature("LiteralMeta", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            literalContent <- xmlValue(from)
            if (literalContent != "")
              obj@content <- literalContent
            attrs <- xmlAttrs(from)
            obj@property <- attrs[["property"]]
            if(!is.na(attrs["datatype"]))
                 obj@datatype <- attrs[["datatype"]]
            if(is.na(attrs["content"])) {
                 kids <- xmlChildren(from)
                 if (length(kids) == 1 && names(kids) != "text") {
                   cnt <- saveXML(kids[[1]], indent = FALSE)
                   # need to capture the XML namespaces for later serialization
                   nsList <- xmlNamespace(kids[[1]])
                   attr(cnt, "namespaces") <- nsList
                   if (length(nsList) > 0)
                     # now that namespaces are captured, their declarations
                     # are redundant and can get in the way at the string level
                     cnt <- gsub(" xmlns:[A-Za-z0-9=]*\"[^\"]+\"", "", cnt)
                   obj@content <- xml(cnt)
                 }
            } else
                 obj@content <- attrs[["content"]]
            if(!is.na(attrs["id"]))
                 obj@id <- attrs[["id"]]
            obj
          }
)
setMethod("toNeXML", 
          signature("LiteralMeta", "XMLInternalElementNode"), 
          function(object, parent){
            parent <- callNextMethod()
            cnt <- object@content
            if (length(cnt) > 0 && is(cnt, "XMLString")) {
              # need to add back in the namespace definitions used by the
              # XML value to avoid a stream of warnings, even if this will
              # probably be redundant with the doc root
              parseXMLAndAdd(cnt, parent = parent, nsDefs = attr(cnt, "namespaces"))
              cnt <- character(0)
            }
            attrs <- c(id = unname(object@id),
                       property = unname(object@property),  # required
                       datatype = unname(object@datatype),  # optional
                       content = unname(cnt))               # required
            attrs <- plyr::compact(attrs)
            addAttributes(parent, .attrs = attrs)
})
setAs("XMLInternalElementNode", "LiteralMeta", function(from) fromNeXML(New("LiteralMeta"), from))
setAs("LiteralMeta", "XMLInternalElementNode", function(from) toNeXML(from, newXMLNode("meta")))
setAs("LiteralMeta", "XMLInternalNode", function(from) toNeXML(from, newXMLNode("meta")))


##############################################

setClass("ResourceMeta", 
         slots = c(id = "character",
                        rel = "character", 
                        href = "character"),
         contains="meta")
setMethod("fromNeXML", 
          signature("ResourceMeta", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            attrs <- xmlAttrs(from)
            if(!is.na(attrs["href"]))
              obj@href <- attrs[["href"]]
            if(!is.na(attrs["id"]))
              obj@id <- attrs[["id"]]
            if(!is.na(attrs[["rel"]]))
                 obj@rel <- attrs[["rel"]]
            kids <- xmlChildren(from)
            if(length(kids) > 0)
              obj@children <- lapply(kids[names(kids) == "meta"], as, "meta")
            obj
          }
)
setMethod("toNeXML", 
          signature("ResourceMeta", "XMLInternalElementNode"), 
          function(object, parent){
            parent <- callNextMethod()
            attrs <- c(id = unname(object@id),
                       href = unname(object@href),
                       rel = unname(object@rel))  
            attrs <- plyr::compact(attrs)
            addAttributes(parent, .attrs = attrs)
            if (length(object@children) > 0)
              addChildren(parent, kids = lcapply(object@children, as, "XMLInternalNode"))
            parent
})
setAs("XMLInternalElementNode", "ResourceMeta", function(from) fromNeXML(New("ResourceMeta"), from))
setAs("ResourceMeta", "XMLInternalNode", function(from) toNeXML(from, newXMLNode("meta")))
setAs("ResourceMeta", "XMLInternalElementNode", function(from) toNeXML(from, newXMLNode("meta")))



##############################################


setClass("ListOfmeta",
         slots = c(names="character"),
         contains = "list",
         validity = function(object)
           if(!all(sapply(object, is, "meta")))
             "not all elements are meta objects"
           else
             TRUE
         )


###############################################

#' Class of objects that have metadata as lists of meta elements
#'
#' @slot meta list of `meta` objects
#' @slot about for RDF extraction, the identifier for the resource that this
#'   object is about
setClass("Annotated",
         slots = c(meta = "ListOfmeta",
                        about = "character"),
         contains = "Base")
setMethod("fromNeXML",
          signature("Annotated", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            kids <- xmlChildren(from)
            if(length(kids) > 0)
              obj@meta <- New("ListOfmeta",
                              lapply(kids[names(kids) == "meta"], 
                                     as, "meta"))
            if(!is.null(xmlAttrs(from)))
              if(!is.na(xmlAttrs(from)["about"]))
                 obj@about <- xmlAttrs(from)["about"]
            obj
})
setMethod("toNeXML", 
          signature("Annotated", "XMLInternalElementNode"), 
           function(object, parent){
             parent <- callNextMethod()
             addChildren(parent, kids = lcapply(object@meta, as, "XMLInternalNode"))
             if(length(object@about) > 0)
               addAttributes(parent, "about" = object@about)
             parent
          })


######################################################

setClass("Labelled",
         slots = c(label = "character"),
         contains = "Annotated")
setMethod("fromNeXML", 
          signature("Labelled", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            if(!is.na(xmlAttrs(from)["label"]))
                 obj@label <- xmlAttrs(from)["label"]
               obj
          }
)
setMethod("toNeXML", 
          signature("Labelled", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            if(length(object@label) > 0)
               addAttributes(parent, "label" = object@label)
            parent
          })

##############################

setClass("IDTagged",
         slots = c(id = "character"),
         contains = "Labelled")
setMethod("fromNeXML", 
          signature("IDTagged", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            if(!is.na(xmlAttrs(from)["id"]))
                 obj@id <- as.character(xmlAttrs(from)["id"])
               obj
          }
)
setMethod("toNeXML", 
          signature("IDTagged", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            if(length(object@id) > 0)
               addAttributes(parent, "id" = object@id)
            parent
          })


##############################


setClass("OptionalTaxonLinked", 
         slots = c(otu = "character"),
         contains = "IDTagged")
setMethod("fromNeXML", 
          signature("OptionalTaxonLinked", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
             if(!is.na(xmlAttrs(from)["otu"]))
               obj@otu <- as.character(xmlAttrs(from)["otu"])
             obj
          }
)
setMethod("toNeXML", 
          signature("OptionalTaxonLinked", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            if(length(object@otu) > 0)
               addAttributes(parent, "otu" = object@otu)
            parent
          })

##############################


setClass("TaxaLinked", 
         slots = c(otus = "character"),
         contains = "IDTagged")
setMethod("fromNeXML", 
          signature("TaxaLinked", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
             if(!is.na(xmlAttrs(from)["otus"]))
               obj@otus <- as.character(xmlAttrs(from)["otus"])
             obj
          }
)
setMethod("toNeXML", 
          signature("TaxaLinked", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            if(length(object@otus) > 0)
               addAttributes(parent, "otus" = object@otus)
            parent
          })


############################## Really AbstractNode

setClass("node", 
         slots = c(root = "logical"),
         contains = "OptionalTaxonLinked")
setMethod("fromNeXML", signature("node", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod() 
             if(!is.na(xmlAttrs(from)["root"]))
               obj@root <- as.logical(xmlAttrs(from)["root"])
             obj
          }
)
setMethod("toNeXML", 
          signature("node", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            if(length(object@root) > 0)
               addAttributes(parent, "root" = tolower(object@root))
            parent
          })
setAs("node", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("node")))
setAs("node", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("node")))
setAs("XMLInternalElementNode", "node",
      function(from) fromNeXML(nexml.node(), from))



################################ Really AbstractEdge

setClass("edge", 
         slots = c(source = "character",
                        target = "character", 
                        length = "numeric"), 
         contains="IDTagged")
setMethod("fromNeXML", 
          signature("edge", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            attrs <- xmlAttrs(from)
            if (!is.na(attrs["source"]))
              obj@source <- attrs["source"]
            obj@target <- attrs["target"]
             if(!is.na(attrs["length"]))
               obj@length <- as.numeric(attrs["length"])
             obj
          }
)
setMethod("toNeXML", 
          signature("edge", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            if(is(object, "rootEdge"))
              xmlName(parent) <- "rootedge"
            else
              addAttributes(parent, "source" = object@source)
            addAttributes(parent, "target" = object@target)
            if(length(object@length) > 0)
               addAttributes(parent, "length" = object@length)
            parent
          })
setAs("edge", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("edge")))
setAs("edge", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("edge")))
setAs("XMLInternalElementNode", "edge",
      function(from) fromNeXML(nexml.edge(), from))

# virtual class, allows asking is(edge, "rootEdge") to determine whether
# a given edge object is a root edge or not
setClass("rootEdge") # creates a virtual class
setIs("edge", "rootEdge",
      test = function(Object) {
        length(Object@target) > 0 && length(Object@source) == 0
      },
      replace = function(Object, value) {
        stop("rootEdge is virtual, cannot replace")
      })

################################ alternatively called "Taxon" by the schema


setClass("otu", contains = "IDTagged")
setMethod("fromNeXML", 
          signature("otu", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            obj
          })
setMethod("toNeXML", 
          signature("otu", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            parent
          })
setAs("otu", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("otu")))
setAs("otu", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("otu")))
setAs("XMLInternalElementNode", "otu",
      function(from) fromNeXML(nexml.otu(), from))

################################ alternatively called Taxa by the schema

setClass("ListOfotu", slots = c(names="character"),  
         contains = "list",
         validity = function(object)
                       if(!all(sapply(object, is, "otu")))
                          "not all elements are otu objects"
                       else
                         TRUE)

###############################

setClass("otus", 
         slots = c(otu = "ListOfotu", names="character"), 
         contains = "IDTagged")
setMethod("fromNeXML", 
          signature("otus", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            kids <- xmlChildren(from)
            if(length(kids) > 0)
              obj@otu <- New("ListOfotu",
                              lapply(kids[names(kids) == "otu"], 
                                     as, "otu"))
            obj
          })
setMethod("toNeXML", 
          signature("otus", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addChildren(parent, kids = lcapply(object@otu, as, "XMLInternalNode"))
            parent
          })
setAs("otus", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("otus")))
setAs("otus", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("otus")))
setAs("XMLInternalElementNode", "otus",
      function(from) fromNeXML(nexml.otus(), from))

################################


setClass("ListOfedge", slots = c(names="character"), 
         contains = "list",
         validity = function(object)
                       if(!all(sapply(object, is, "edge")))
                          "not all elements are edge objects"
                       else
                         TRUE)



setClass("ListOfnode", slots = c(names="character"), 
         contains = "list",
         validity = function(object)
                       if(!all(sapply(object, is, "node")))
                          "not all elements are node objects"
                       else
                         TRUE)

################################## actually AbstractTree

## Thanks Gabor for work-around from S4 namespaces collision issue
# https://github.com/ropensci/RNeXML/issues/251#issuecomment-1031996431
tryCatch({
  rlang::env_unlock(asNamespace("cli"))
  removeClass("tree", asNamespace("cli"))
  rlang::env_lock(asNamespace("cli"))
}, error = function(err) NULL)

setClass("tree", 
         slots = c(node = "ListOfnode", 
                        edge = "ListOfedge",
                        rootedge = "edge"),
         contains = "IDTagged")
setMethod("fromNeXML", 
          signature("tree", "XMLInternalElementNode"),
          function(obj, from){
            .cacheNextMethod()
            obj <- callNextMethod()
            kids <- xmlChildren(from)
            obj@node <- New("ListOfnode",
                            lapply(kids[names(kids) == "node"], 
                                   as, "node"))
            obj@edge <- New("ListOfedge",
                            lapply(kids[names(kids) == "edge"], 
                                   as, "edge"))
            rootEdge <- kids[names(kids) == "rootedge"]
            if (length(rootEdge) > 0)
              obj@rootedge <- as(rootEdge[[1]], "edge")
            obj
          })
setMethod("toNeXML", 
          signature("tree", "XMLInternalElementNode"),
          function(object, parent){
            .cacheNextMethod()
            parent <- callNextMethod()
            addChildren(parent, kids = lcapply(object@node, as, "XMLInternalNode"))
            addChildren(parent, kids = lcapply(object@edge, as, "XMLInternalNode"))
            if (is(object@rootedge, "rootEdge"))
              addChildren(parent, as(object@rootedge, "XMLInternalNode"))
            parent
          })
setAs("tree", "XMLInternalNode",
      function(from) .callGeneric("toNeXML", from, newXMLNode("tree")))
setAs("tree", "XMLInternalElementNode",
      function(from) .callGeneric("toNeXML", from, newXMLNode("tree")))
setAs("XMLInternalElementNode", "tree",
      function(from) .callGeneric("fromNeXML", nexml.tree(), from))



################################################

setClass("ListOftree", slots = c(names="character"), contains = "list") # validity can contain tree or network nodes?

setClass("trees", 
         slots = c(tree = "ListOftree"), # Can contain networks...
         contains = "TaxaLinked")
setMethod("fromNeXML", 
          signature("trees", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()
            kids <- xmlChildren(from)
            obj@tree <- New("ListOftree",
                            lapply(kids[names(kids) == "tree"], 
                                   as, "tree"))
            obj
          })
setMethod("toNeXML", 
          signature("trees", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addChildren(parent, kids = lapply(object@tree, as, "XMLInternalNode"))
#            addChildren(parent, kids = object@network)
            parent
          })
setAs("trees", "XMLInternalNode",
      function(from) toNeXML(from, newXMLNode("trees")))
setAs("trees", "XMLInternalElementNode",
      function(from) toNeXML(from, newXMLNode("trees")))
setAs("XMLInternalElementNode", "trees",
      function(from) fromNeXML(nexml.trees(), from))


####################################################

setClass("ListOfotus", slots = c(names="character"), contains = "list")
setClass("ListOftrees", slots = c(names="character"), contains = "list")
setClass("ListOfcharacters", slots = c(names="character"), contains = "list")

####################################################



nexml_namespaces <- 
  c("nex"   = "http://www.nexml.org/2009",
    "xsi"   = "http://www.w3.org/2001/XMLSchema-instance",
    "xml"   = "http://www.w3.org/XML/1998/namespace",
    "cdao"  = "http://purl.obolibrary.org/obo/",
    "xsd"   = "http://www.w3.org/2001/XMLSchema#",
    "dc"    = "http://purl.org/dc/elements/1.1/",
    "dcterms" = "http://purl.org/dc/terms/",
    "prism" = "http://prismstandard.org/namespaces/1.2/basic/",
    "cc"    = "http://creativecommons.org/ns#",
    "ncbi"  = "http://www.ncbi.nlm.nih.gov/taxonomy#",
    "tc"    = "http://rs.tdwg.org/ontology/voc/TaxonConcept#")

#' Class representing a NeXML document
#'
#' The `nexml` class represents a NeXML document, and is the top of the
#' class hierarchy defined in this package, corresponding to the root node
#' of the corresponding XML document.
#'
#' Normally objects of this type are created by the package as a result of
#' reading a NeXML file, or of converting from another type, such as
#' `ape::phylo`. Also, interacting directly with the slots of the class is
#' normally not necessary. Instead, use the `get_XXX()` and `add_XXX()`
#' functions in the API. 
#' @slot trees list, corresponding to the list of `<trees/>` elements in
#'   NeXML. Elements will be of class `trees`.
#' @slot characters list, corresponding to the list of `<characters/>`
#'   elements in NeXML. Elements will be of class `characters`.
#' @slot otus list, corresponding to the list of `<otus/>` elements in NeXML.
#'   Elements will be of class `otus`.
#' @slot about inherited, see [Annotated][Annotated-class]
#' @slot meta inherited, see [Annotated][Annotated-class]
#' @slot xsi:type for internal use
#' @slot version NeXML schema version, do not change
#' @slot generator name of software generating the XML
#' @slot xsi:schemaLocation for internal use, do not change
#' @slot namespaces named character vector giving the XML namespaces
#' @seealso [read.nexml()]
#' @examples
#' nex <- nexml() # a nexml object with no further content
#' nex <- new("nexml") # accomplishes the same thing
#' nex@generator
#' length(nex@trees)
#'
#' data(bird.orders)
#' nex <- as(bird.orders, "nexml")
#' summary(nex)
#' length(nex@trees)
#' @export nexml
#' @exportClass nexml
nexml <- setClass("nexml",
         slots = c(version = "character",
                        generator = "character",
                        "xsi:schemaLocation" = "character", # part of base?
                        namespaces = "character",           # part of base? 
                        otus = "ListOfotus",             
                        trees = "ListOftrees",
                        characters="ListOfcharacters"),
         prototype = prototype(version = "0.9",
                   generator = "RNeXML",
                   "xsi:schemaLocation" = "http://www.nexml.org/2009/nexml.xsd",
                   namespaces = c(nexml_namespaces, 
                                  "http://www.nexml.org/2009")),
         contains = "Annotated")

setMethod("fromNeXML", 
          signature("nexml", "XMLInternalElementNode"),
          function(obj, from){
            obj <- callNextMethod()

            # handle attributes
            attrs <- xmlAttrs(from)
            obj@version <- attrs["version"]     # required attribute
            if(!is.na(attrs["generator"]))       # optional attribute
               obj@generator <- attrs["generator"]

            if(!is.na(attrs["xsi:schemaLocation"]))
              slot(obj, "xsi:schemaLocation") <- attrs["xsi:schemaLocation"]
            if(!is.na(attrs["schemaLocation"]))
              slot(obj, "xsi:schemaLocation") <- attrs["schemaLocation"]

            if(!is.na(attrs["xsi:type"]))
              slot(obj, "xsi:type") <- attrs["xsi:type"]
            if(!is.na(attrs["type"]))
              slot(obj, "xsi:type") <- attrs["type"]

            if(!is.na(attrs["about"]))
              obj@about <- attrs["about"]


            ns_defs <- xmlNamespaceDefinitions(from)
            ns <- sapply(ns_defs, `[[`, "uri")
            obj <- add_namespaces(ns, obj)

            # add our own namespaces, which will skip already present prefixes;
            # also add a base namespace if there isn't one already
            ourNS <- nexml_namespaces;
            if (! any(names(obj@namespaces) == ""))
              ourNS <- c(ourNS, unname(nexml_namespaces["nex"]))
            obj <- add_namespaces(ourNS, nexml = obj)

            # Handle children
            kids <- xmlChildren(from)
            # at least 1 OTU block is required 
            obj@otus <- New("ListOfotus",
                            lapply(kids[names(kids) == "otus"], 
                                   as, "otus"))
            if("characters" %in% names(kids))
              obj@characters <- New("ListOfcharacters",
                            lapply(kids[names(kids) == "characters"], 
                                   as, "characters"))
            if("trees" %in% names(kids))
              obj@trees <- New("ListOftrees",
                            lapply(kids[names(kids) == "trees"], 
                                   as, "trees"))
            obj
          })

setMethod("toNeXML", 
          signature("nexml", "XMLInternalElementNode"),
          function(object, parent){
            parent <- callNextMethod()
            addAttributes(parent, "version" = object@version)
            if(length(object@generator)>0)
              addAttributes(parent, "generator" = object@generator)

            # Coercion of object to XML happens automatically
            addChildren(parent, kids = object@otus) # a list of "otus" objects
            addChildren(parent, kids = object@trees) # a list of "trees" objects
            addChildren(parent, kids = object@characters) # a list of "characters" objects
            parent
          })

## NOTE: The root nexml element must have it's namespace
setAs("nexml", "XMLInternalNode",
      function(from) suppressWarnings(toNeXML(from, newXMLNode("nex:nexml", namespaceDefinitions = from@namespaces))))
setAs("nexml", "XMLInternalElementNode",
      function(from) suppressWarnings(toNeXML(from, newXMLNode("nex:nexml", namespaceDefinitions = from@namespaces))))
setAs("XMLInternalElementNode", "nexml",
      function(from) fromNeXML(nexml(namespaces = character(0)), from))



#######################################################

#' @importFrom rlang enquo
expect_true_or_null <- function(o){
  val <- rlang::enquo(o)
  if(!is.null(o)){
    testthat::expect_true(!!val)
  } else {
    testthat::expect_null(!!val)
  }
}

library("XML")

---
title: "A Brief Introduction to RNeXML"
author:
  - Carl Boettiger
  - Scott Chamberlain
  - Rutger Vos
  - Hilmar Lapp

output: html_vignette
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{A Brief Introduction to RNeXMLL}
-->




Read in a `nexml` file:


```r
f <- system.file("examples", "comp_analysis.xml", package="RNeXML")
nexml <- nexml_read(f)
```

Pretty-print an overview of the components and metadata that make up the nexml object:

```r
nexml  # this is the same as show(nexml)
```

```
## A nexml object representing:
##  	 1 phylogenetic tree block(s), where:
## 	   block 1 contains 1 phylogenetic tree(s) 
##  	 2 character block(s), where:
## 	   block 1 defines 1 continuous character(s) 
## 	   matrix 1 has 10 row(s)
## 	   block 2 defines 1 standard/discrete character(s), with 2 states each 
## 	    and  0 polymorphic or uncertain state(s) defined 
## 	   matrix 2 has 10 row(s) 
##  	 10 taxonomic units in 1 block(s) 
##   Taxa:	 taxon_1, taxon_2, taxon_3, taxon_4, taxon_5, taxon_6 ...
##   Metadata annotations: 
## 	2 at top level 
## 	0 in block 1 at otu level 
## 	0 in block 1 at char level
## 	0 in block 2 at char level 
## 	0 in block 1 at state level
## 	0 in block 2 at state level
## 
## Author(s): rvosa
## 
## NeXML generated by Bio::Phylo::Project v.0.56 using schema version: 0.9 
## Size: 289.3 Kb
```

Create a summary object of various component and metadata counts (the `show()` method uses this):

```r
summary(nexml)
```

```
## $nblocks
##      trees       otus characters 
##          1          1          2 
## 
## $ncharacters
## block.1 block.2 
##       1       1 
## 
## $nstates
##         block.1 block.2
## Min.         NA       2
## 1st Qu.      NA       2
## Median       NA       2
## Mean         NA       2
## 3rd Qu.      NA       2
## Max.         NA       2
## 
## $nnonstdstatedefs
##         polymorphic uncertain
## block.1          NA        NA
## block.2           0         0
## 
## $nmatrixrows
## block.1 block.2 
##      10      10 
## 
## $ntrees
## block.1 
##       1 
## 
## $notus
## block.1 
##      10 
## 
## $nmeta
## $nmeta$nexml
## [1] 2
## 
## $nmeta$otu
## block.1 
##       0 
## 
## $nmeta$char
## block.1 block.2 
##       0       0 
## 
## $nmeta$state
## block.1 block.2 
##       0       0
```

Extract trees from nexml into the `ape::phylo` format:

```r
tr <- get_trees(nexml) # or: as(nexml, "phylo")
plot(tr)
```

![plot of chunk unnamed-chunk-5](intro-unnamed-chunk-5-1.png)

Write an `ape::phylo` tree into the `nexml` format:


```r
data(bird.orders)
nexml_write(bird.orders, "test.xml", creator = "Carl Boettiger")
```

```
## [1] "test.xml"
```

A key feature of NeXML is the ability to formally validate the construction of the data file against the standard (the lack of such a feature in nexus files had lead to inconsistencies across different software platforms, and some files that cannot be read at all).  While it is difficult to make an invalid NeXML file from `RNeXML`, it never hurts to validate just to be sure:


```r
nexml_validate("test.xml")
```

```
## [1] TRUE
```



Extract metadata from the NeXML file: 


```r
birds <- nexml_read("test.xml")
get_taxa(birds)
```

```
##     otu            label xsi.type otus
## 1  ou37 Struthioniformes       NA  os3
## 2  ou38     Tinamiformes       NA  os3
## 3  ou39      Craciformes       NA  os3
## 4  ou40      Galliformes       NA  os3
## 5  ou41     Anseriformes       NA  os3
## 6  ou42    Turniciformes       NA  os3
## 7  ou43       Piciformes       NA  os3
## 8  ou44    Galbuliformes       NA  os3
## 9  ou45   Bucerotiformes       NA  os3
## 10 ou46      Upupiformes       NA  os3
## 11 ou47    Trogoniformes       NA  os3
## 12 ou48    Coraciiformes       NA  os3
## 13 ou49      Coliiformes       NA  os3
## 14 ou50     Cuculiformes       NA  os3
## 15 ou51   Psittaciformes       NA  os3
## 16 ou52      Apodiformes       NA  os3
## 17 ou53   Trochiliformes       NA  os3
## 18 ou54  Musophagiformes       NA  os3
## 19 ou55     Strigiformes       NA  os3
## 20 ou56    Columbiformes       NA  os3
## 21 ou57       Gruiformes       NA  os3
## 22 ou58    Ciconiiformes       NA  os3
## 23 ou59    Passeriformes       NA  os3
```

```r
get_metadata(birds) 
```

```
##           property   datatype                 content     xsi.type                                              href Meta
## 1       dc:creator xsd:string          Carl Boettiger  LiteralMeta                                              <NA> m278
## 2 dcterms:modified xsd:string 2020-01-28 23:24:19 GMT  LiteralMeta                                              <NA> m279
## 3       cc:license       <NA>                    <NA> ResourceMeta http://creativecommons.org/publicdomain/zero/1.0/ m280
```

--------------------------------------------


Add basic additional metadata:  


```r
  nexml_write(bird.orders, file="meta_example.xml",
              title = "My test title",
              description = "A description of my test",
              creator = "Carl Boettiger <cboettig@gmail.com>",
              publisher = "unpublished data",
              pubdate = "2012-04-01")
```

```
## [1] "meta_example.xml"
```
By default, `RNeXML` adds certain metadata, including the NCBI taxon id numbers for all named taxa.  This acts a check on the spelling and definitions of the taxa as well as providing a link to additional metadata about each taxonomic unit described in the dataset.  


### Advanced annotation


We can also add arbitrary metadata to a NeXML tree by define `meta` objects:


```r
modified <- meta(property = "prism:modificationDate",
                 content = "2013-10-04")
```

Advanced use requires specifying the namespace used.  Metadata follows the RDFa conventions.  Here we indicate the modification date using the prism vocabulary. This namespace is included by default, as it is used for some of the basic metadata shown in the previous example.  We can see from this list:


```r
RNeXML:::nexml_namespaces
```

```
##                                              nex                                              xsi 
##                      "http://www.nexml.org/2009"      "http://www.w3.org/2001/XMLSchema-instance" 
##                                              xml                                             cdao 
##           "http://www.w3.org/XML/1998/namespace"                "http://purl.obolibrary.org/obo/" 
##                                              xsd                                               dc 
##              "http://www.w3.org/2001/XMLSchema#"               "http://purl.org/dc/elements/1.1/" 
##                                          dcterms                                            prism 
##                      "http://purl.org/dc/terms/" "http://prismstandard.org/namespaces/1.2/basic/" 
##                                               cc                                             ncbi 
##                 "http://creativecommons.org/ns#"          "http://www.ncbi.nlm.nih.gov/taxonomy#" 
##                                               tc 
##  "http://rs.tdwg.org/ontology/voc/TaxonConcept#"
```

This next block defines a resource (link), described by the `rel` attribute as a homepage, a term in the `foaf` vocabulary.  Because `foaf` is not a default namespace, we will have to provide its URL in the full definition below. 


```r
website <- meta(href = "http://carlboettiger.info", 
                rel = "foaf:homepage")
```

Here we create a history node using the `skos` namespace.  We can also add id values to any metadata element to make the element easier to reference externally: 


```r
  history <- meta(property = "skos:historyNote", 
                  content = "Mapped from the bird.orders data in the ape package using RNeXML",
                  id = "meta123")
```

For this kind of richer annotation, it is best to build up our NeXML object sequentially. First
we will add `bird.orders` phylogeny to a new phylogenetic object, and then we will add the metadata
elements created above to this object. Finally, we will write the object out as an XML file:


```r
  birds <- add_trees(bird.orders)
  birds <- add_meta(meta = list(history, modified, website),
                    namespaces = c(skos = "http://www.w3.org/2004/02/skos/core#",
                                   foaf = "http://xmlns.com/foaf/0.1/"),
                    nexml=birds)
  nexml_write(birds, 
              file = "example.xml")
```

```
## [1] "example.xml"
```

### Taxonomic identifiers

Add taxonomic identifier metadata to the OTU elements:
<!-- This block relies on a robust internet connection that can occassionally fail.  Also it's a bit slow, so don't run it. After all, this command is tested in the unit tests.-->


```r
nex <- add_trees(bird.orders)
nex <- taxize_nexml(nex)
```



## Working with character data

NeXML also provides a standard exchange format for handling character data.  The R platform is particularly popular in the context of phylogenetic comparative methods, which consider both a given phylogeny and a set of traits.  NeXML provides an ideal tool for handling this metadata.  

### Extracting character data

We can load the library, parse the NeXML file and extract both the characters and the phylogeny.  


```r
library(RNeXML)
nexml <- read.nexml(system.file("examples", "comp_analysis.xml", package="RNeXML"))
traits <- get_characters(nexml)
tree <- get_trees(nexml)
```

(Note that `get_characters` would return both discrete and continuous characters together in the same data.frame, but we use `get_characters_list` to get separate data.frames for the continuous `characters` block and the discrete `characters` block).  

We can then fire up `geiger` and fit, say, a Brownian motion model the continuous data and a Markov transition matrix to the discrete states:  

```r
library(geiger)
fitContinuous(tree, traits[1], ncores=1)
fitDiscrete(tree, traits[2], ncores=1)
```

## simmap NeXML definitions 

- Author: Carl Boettiger
- Initial version: 2014-03-21



Definitions of the `simmap` namespace, as defined for the use in `RNeXML`. The prefix `nex:` refers to the [NeXML schema](http://www.nexml.org/2009).


  term               | definition
 ------------------- | -------------
 `simmap:reconstructions`   | A container of one or more stochastic character map reconstructions, as a `meta` child of a the `nex:edge` element to which the contained stochastic character map reconstructions are being assigned.
 `simmap:reconstruction`    | A single stochastic character map reconstruction for a given `nex:edge`. Normally nested within a `simmap:reconstructions` element.
 `simmap:char`              | The id of a character trait, as defined by the `nex:char` element with this value as its `id`. This is a property of a `simmap:reconstruction`.
 `simmap:stateChange`       | A character state assignment to the given `nex:edge` during a specified interval, as a property of a `simmap:reconstruction`. Must have children `simmap:order`, `simmap:length`, and `simmap:state`.
`simmap:order`              | The chronological order (from the root) in which the state is assigned to the edge.  An edge that does not change states still has `simmap:order` 1.   This is a property of a `simmap:stateChange`.  
`simmap:length`             | The duration for which the edge occupies the assigned state, in the same units as the `nex:length` attribute defined on the `nex:edge` being annotated. This is a property of a `simmap:stateChange`.  
 `simmap:state`             | The id of a `nex:state` of the `nex:char` identified by the `simmap:char` property of the `simmap:reconstruction`. This is a property of a `simmap:stateChange`.  

---
title: "SPARQL with RNeXML"
author:
  - Carl Boettiger
  - Scott Chamberlain
  - Rutger Vos
  - Hilmar Lapp
output: html_vignette
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{SPARQL with RNeXML}
-->









## SPARQL Queries

Rich, semantically meaningful metadata lies at the heart of the NeXML
standard.  R provides a rich environment to unlock this information.
While our previous examples have relied on the user knowing exactly
what metadata they intend to extract (title, publication date, citation
information, and so forth), _semantic_ metadata has meaning that a
computer can make use of, allowing us to make much more conceptually
rich queries than those simple examples.  The SPARQL query language
is a powerful way to make use of such semantic information in making 
complex queries. 

While users should consult a formal introduction to SPARQL for further
background, here we illustrate how SPARQL can be used in combination
with R functions in ways that would be much more tedious to assemble
with only traditional/non-semantic queries.  The SPARQL query language
is provided for the R environment through the `rrdf` package [@Willighagen_2014],
so we start by loading that package.  We will also make use of functions
from `phytools` and `RNeXML`.  


```r
library("rdflib")
library("xml2")
library("phytools")
library("RNeXML")
```

We read in an example file that contains semantic metadata annotations
describing the taxonomic units (OTUs) used in the tree.  


```r
nexml <- nexml_read(system.file("examples/primates.xml", package="RNeXML"))
```

In particular, this example declares the taxon rank, NCBI identifier and parent taxon
for each OTU, such as:

```xml
<otu about="#ou541" id="ou541" label="Alouatta guariba">
      <meta href="http://ncbi.nlm.nih.gov/taxonomy/182256" 
            id="ma20" 
            rel="concept:toTaxon" 
            xsi:type="nex:ResourceMeta"/>
      <meta href="http://rs.tdwg.org/ontology/voc/TaxonRank#Species" 
            id="ma21" 
            rel="concept:rank" 
            xsi:type="nex:ResourceMeta"/>
      <meta href="http://ncbi.nlm.nih.gov/taxonomy/9499" 
            id="ma22" 
            rel="rdfs:subClassOf" 
            xsi:type="nex:ResourceMeta"/>
    </otu>

```

In this example, we will construct a cladogram by using this information
to identify the taxonomic rank of each OTU, and its shared parent
taxonomic rank.  (If this example looks complex, try writing down the
steps to do this without the aid of the SPARQL queries).  These examples
show the manipulation of semantic triples, Unique Resource Identifiers
(URIs) and use of the SPARQL "Join" operator.

Note that this example can be run using `demo("sparql", "RNeXML")` to see the 
code displayed in the R terminal and to avoid character errors that
can occur in having to copy and paste from PDF files.  

We begin by extracting the RDF graph from the NeXML,


```r
rdf <- get_rdf(system.file("examples/primates.xml", package="RNeXML"))
tmp <- tempfile()  # so we must write the XML out first
xml2::write_xml(rdf, tmp) 

graph <- rdf_parse(tmp)
```

We then fetch the NCBI URI for the taxon that has rank 'Order', i.e. the
root of the primates phylogeny. The dot operator `.` between clauses
implies a join, in this case


```r
root <- rdf_query(graph, 
"SELECT ?uri WHERE { 
    ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#rank> <http://rs.tdwg.org/ontology/voc/TaxonRank#Order> . 
    ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> ?uri    
}")
```


This makes use of the SPARQL query language provided by the `rrdf`
package.  We will also define some helper functions that use SPARQL
queries.  Here we define a function to get the name


```r
get_name <- function(id) {
  max <- length(nexml@otus[[1]]@otu)
  for(i in 1:max) {
    if ( nexml@otus[[1]]@otu[[i]]@id == id ) {
      label <- nexml@otus[[1]]@otu[[i]]@label
      label <- gsub(" ","_",label)
      return(label)
    }
  }
}
```


Next, we define a recursive function to build a newick tree from the taxonomic rank information.  


```r
recurse <- function(node){
  
    # fetch the taxonomic rank and id string
    rank_query <- paste0(
        "SELECT ?rank ?id WHERE {
            ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> <",node,"> .
            ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#rank> ?rank
          }")
    result <- rdf_query(graph, rank_query)
    
    # get the local ID, strip URI part
    id <- result$id
    id <- gsub("^.+#", "", id, perl = TRUE)
    
    # if rank is terminal, return the name
    if (result$rank == "http://rs.tdwg.org/ontology/voc/TaxonRank#Species") {
        return(get_name(id))
    }
    
    # recurse deeper
    else {
        child_query <- paste0(
            "SELECT ?uri WHERE {
                ?id <http://www.w3.org/2000/01/rdf-schema#subClassOf> <",node,"> .
                ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> ?uri
            }")
        children <- rdf_query(graph, child_query)
        
        return(paste("(", 
                     paste(sapply(children$uri, recurse), 
                           sep = ",", collapse = "," ), 
                     ")",  
                     get_name(id), # label interior nodes
                     sep = "", collapse = ""))
    }
}
```


With these functions in place, it is straight forward to build the tree
from the semantic RDFa data and then visualize it



```r
newick <- paste(recurse(root), ";", sep = "", collapse = "")
tree <- read.newick(text = newick)
collapsed <- collapse.singles(tree)
plot(collapsed, 
     type='cladogram', 
     show.tip.label=FALSE, 
     show.node.label=TRUE, 
     cex=0.75, 
     edge.color='grey60', 
     label.offset=-9)
```

![plot of chunk unnamed-chunk-8](sparql-unnamed-chunk-8-1.png)

---
title: "The nexml S4 Object"
author:
  - Carl Boettiger
  - Scott Chamberlain
  - Rutger Vos
  - Hilmar Lapp
output: html_vignette
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{The nexml S4 Object}
-->







## Understanding the `nexml` S4 object

The `RNeXML` package provides many convenient functions to add and extract
information from `nexml` objects in the R environment without requiring
the reader to understand the details of the NeXML data structure and
making it less likely that a user will generate invalid NeXML syntax
that could not be read by other parsers. The `nexml` object we have been using
in all of the examples is built on R's S4 mechanism. Advanced users may
sometimes prefer to interact with the data structure more directly using 
R's S4 class mechanism and subsetting methods. Many R users are more familiar
with the S3 class mechanism (such as in the `ape` package phylo objects)
rather than the S4 class mechanism used in phylogenetics packages such as 
`ouch` and `phylobase`. The `phylobase` vignette provides an excellent introduction
to these data structures.  Users already familiar with subsetting lists and other
S3 objects in R are likely familiar with the use of the `$` operator, such as
`phy$edge`. S4 objects simply use an `@` operator instead (but cannot be subset
using numeric arguments such as `phy[[1]]` or named arguments such as phy[["edge"]]).  


The `nexml` object is an S4 object, as are all of its components (slots).  Its 
hierarchical structure corresponds exactly with the XML tree of a NeXML file, with 
the single exception that both XML attributes and children are represented as slots.  
S4 objects have constructor functions to initialize them.  We create a new `nexml` 
object with the command:


```r
nex <- new("nexml")
```

We can see a list of slots contained in this object with


```r
slotNames(nex)
```

```
 [1] "version"            "generator"          "xsi:schemaLocation" "namespaces"         "otus"              
 [6] "trees"              "characters"         "meta"               "about"              "xsi:type"          
```

Some of these slots have already been populated for us, for instance, the schema version and default namespaces:


```r
nex@version
```

```
[1] "0.9"
```

```r
nex@namespaces
```

```
                                             nex                                              xsi 
                     "http://www.nexml.org/2009"      "http://www.w3.org/2001/XMLSchema-instance" 
                                             xml                                             cdao 
          "http://www.w3.org/XML/1998/namespace"                "http://purl.obolibrary.org/obo/" 
                                             xsd                                               dc 
             "http://www.w3.org/2001/XMLSchema#"               "http://purl.org/dc/elements/1.1/" 
                                         dcterms                                            prism 
                     "http://purl.org/dc/terms/" "http://prismstandard.org/namespaces/1.2/basic/" 
                                              cc                                             ncbi 
                "http://creativecommons.org/ns#"          "http://www.ncbi.nlm.nih.gov/taxonomy#" 
                                              tc                                                  
 "http://rs.tdwg.org/ontology/voc/TaxonConcept#"                      "http://www.nexml.org/2009" 
```

Recognize that `nex@namespaces` serves the same role as `get_namespaces`
function, but provides direct access to the slot data.  For instance,
with this syntax we could also overwrite the existing namespaces with
`nex@namespaces <- NULL`.  Changing the namespace in this way is not
advised.

Some slots can contain multiple elements of the same type, such as
`trees`, `characters`, and `otus`.  For instance, we see that


```r
class(nex@characters)
```

```
[1] "ListOfcharacters"
attr(,"package")
[1] "RNeXML"
```

is an object of class `ListOfcharacters`, and is currently empty,


```r
length(nex@characters)
```

```
[1] 0
```

In order to assign an object to a slot, it must match the class definition
of the slot.  We can create a new element of any given class with the
`new` function,


```r
nex@characters <- new("ListOfcharacters", list(new("characters")))
```

and now we have a length-1 list of character matrices,


```r
length(nex@characters)
```

```
[1] 1
```

and we access the first character matrix using the list notation,
`[[1]]`. Here we check the class is a `characters` object.


```r
class(nex@characters[[1]])
```

```
[1] "characters"
attr(,"package")
[1] "RNeXML"
```

Direct subsetting has two primary use cases: (a) useful in looking up
(and possibly editing) a specific value of an element, or (b) when adding
metadata annotations to specific elements. Consider the example file



```r
f <- system.file("examples", "trees.xml", package="RNeXML")
nex <- nexml_read(f)
```

We can look up the species label of the first `otu` in the first `otus` block:


```r
nex@otus[[1]]@otu[[1]]@label
```

```
      label 
"species 1" 
```

We can add metadata to this particular OTU using this subsetting format


```r
nex@otus[[1]]@otu[[1]]@meta <- 
  c(meta("skos:note", 
          "This species was incorrectly identified"),
         nex@otus[[1]]@otu[[1]]@meta)
```

Here we use the `c` operator to append this element to any existing meta annotations to this otu.  

---
title: "Extending NeXML: an example based on simmap"
author:
  - Carl Boettiger
  - Scott Chamberlain
  - Rutger Vos
  - Hilmar Lapp
output: html_vignette
bibliography: references.bib
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{Extending NeXML to simmap}
-->





## Extending the NeXML standard through metadata annotation.

Here we illustrate this process using the example of stochastic character
mapping [@Huelsenbeck_2003]. A stochastic character map is simply
an annotation of the branches on a phylogeny, assigning each section
of each branch to a particular "state" (typically of a morphological
characteristic).

@Bollback_2006 provides a widely used stand-alone software implementation
of this method in the software `simmap`, which modified the standard
Newick tree format to express this additional information. This can
break compatibility with other software, and creates a format that
cannot be interpreted without additional information describing this
convention.  By contrast, the NeXML extension is not only backwards
compatible but contains a precise and machine-readable description of
what it is encoding.


In this example, we illustrate how the additional information required
to define a stochastic character mapping (a `simmap` mapping) in NeXML. 

@Revell_2012 describes the `phytools` package for R, which includes
utilities for reading, manipulating, and writing `simmap` files in R.
In this example, we also show how to define `RNeXML` functions that 
map the R representation used by Revell (an extension of the `ape` class)
into the NeXML extension we have defined by using `RNeXML` functions.

Since a stochastic character map simply assigns different states to
parts of a branch (or edge) on the phylogenetic tree, we can create
a NeXML representation by annotating the `edge` elements with appropriate
`meta` elements.  These elements need to describe the character state
being assigned and the duration (in terms of branch-length) that the edge 
spends in that state (Stochastic character maps are specific to time-calibrated
or ultrametric trees).  

NeXML already defines the `characters` element to handle discrete character traits (`nex:char`)
and the states they can assume (`nex:state`).  We will thus reuse the `characters` element for
this purpose, referring to both the character trait and the states by the ids assigned to them
in that element.  (NeXML's convention of referring to everything by id permits a single canonical
definition of each term, making it clear where additional annotation belongs).  For each edge, we 
need to indicate:

- That our annotation contains a stochastic character mapping reconstruction
- Since many reconstructions are possible for a single edge, we give each reconstruction an id
- We indicate for which character trait we are defining the reconstruction 
- We then indicate which states the character assumes on that edge. 
  For each state realized on the edge, that involves stating: 
    + the state assignment
    + the duration (length of time) for which the edge spends in the given state
    + the order in which the state changes happen (Though we could just assume 
      state transitions are listed chronologically, NeXML suggests making all 
      data explicit, rather than relying on the structure of the data file to
      convey information).  

Thus the annotation for an edge that switches from state `s2` to state 
`s1` of character `cr1` would be constructed like this:


```r
 m <- meta("simmap:reconstructions", children = c(
        meta("simmap:reconstruction", children = c(

          meta("simmap:char", "cr1"),
          meta("simmap:stateChange", children = c(
            meta("simmap:order", 1),
            meta("simmap:length", "0.2030"),
            meta("simmap:state", "s2"))),
          
          meta("simmap:char", "cr1"),
          meta("simmap:stateChange", children = c(
            meta("simmap:order", 2),
            meta("simmap:length", "0.0022"),
            meta("simmap:state", "s1")))
          ))))
```

Of course writing out such a definition manually becomes tedious quickly. Because
these are just R commands, we can easily define a function that can loop over an
assignment like this for each edge, extracting the appropriate order, length and
state from an existing R object such as that provided in the `phytools` package.  
Likewise, it is straightforward to define a function that reads this data using
the `RNeXML` utilities and converts it back to the `phytools` package. The full
implementation of this mapping can be seen in the `simmap_to_nexml()` and the
`nexml_to_simmap()` functions provided in the `RNeXML` package.  

As the code indicates, the key step is simply to define the data in meta elements. In 
so doing, we have defined a custom namespace, `simmap`, to hold our variables.  This
allows us to provide a URL with more detailed descriptions of what each of these 
elements mean:


```r
nex <- add_namespaces(c(simmap = "https://github.com/ropensci/RNeXML/tree/master/inst/simmap.md"))
```

At that URL we have posted a simple description of each term. 

Using this convention we can generate NeXML files containing `simmap`
data, read those files into R, and convert them back into the `phytools`
package format. These simple functions serve as further illustration of
how `RNeXML` can be used to extend the NeXML standard.  We illustrate
their use briefly here, starting with  loading a `nexml` object containing
a `simmap` reconstruction into R:



```r
f <- system.file("examples", "simmap_ex.xml", package = "RNeXML")
simmap_ex <- read.nexml(f)
```

The `get_trees()` function can be used to return an `ape::phylo` tree as
usual.  `RNeXML` automatically detects the `simmap` reconstruction data
and returns includes this in a `maps` element of the `ape::phylo` object,
for use with other `phytools` functions.


```r
phy <- nexml_to_simmap(simmap_ex)
```

We can then use various functions from `phytools` designed for `simmap`
objects [@Revell_2012], such as the plotting function:


```r
library("phytools")
plotSimmap(phy)
```

```
no colors provided. using the following legend:
       A        B        C 
 "black"    "red" "green3" 
```

![Stochastic character mapping on a phylogeny, as generated by the phytools package after parsing the simmap-extended NeXML.](simmap-Figure1-1.png)

Likewise, we can convert the object back in the NeXML format and write
it out to file to be read by other users. 


```r
nex <- simmap_to_nexml(phy) 
nexml_write(nex, "simmap.xml")
```

```
[1] "simmap.xml"
```

Though other NeXML parsers (for instance, for Perl or Python) have
not been written explicitly to express `simmap` data, those parsers will
nonetheless be able to successfully parse this file and expose the `simmap`
data to the user.

---
title: "Extending NeXML: an example based on simmap"
author:
  - Carl Boettiger
  - Scott Chamberlain
  - Rutger Vos
  - Hilmar Lapp
output: html_vignette
bibliography: references.bib
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{Extending NeXML to simmap}
-->


```{r compile-settings, include=FALSE}
library("methods")
library("knitr")
opts_chunk$set(tidy = FALSE, warning = FALSE, message = FALSE, 
               cache = FALSE, comment = NA, verbose = TRUE)
basename <- gsub(".Rmd", "", knitr:::knit_concord$get('infile')) 
library("RNeXML")

```

## Extending the NeXML standard through metadata annotation.

Here we illustrate this process using the example of stochastic character
mapping [@Huelsenbeck_2003]. A stochastic character map is simply
an annotation of the branches on a phylogeny, assigning each section
of each branch to a particular "state" (typically of a morphological
characteristic).

@Bollback_2006 provides a widely used stand-alone software implementation
of this method in the software `simmap`, which modified the standard
Newick tree format to express this additional information. This can
break compatibility with other software, and creates a format that
cannot be interpreted without additional information describing this
convention.  By contrast, the NeXML extension is not only backwards
compatible but contains a precise and machine-readable description of
what it is encoding.


In this example, we illustrate how the additional information required
to define a stochastic character mapping (a `simmap` mapping) in NeXML. 

@Revell_2012 describes the `phytools` package for R, which includes
utilities for reading, manipulating, and writing `simmap` files in R.
In this example, we also show how to define `RNeXML` functions that 
map the R representation used by Revell (an extension of the `ape` class)
into the NeXML extension we have defined by using `RNeXML` functions.

Since a stochastic character map simply assigns different states to
parts of a branch (or edge) on the phylogenetic tree, we can create
a NeXML representation by annotating the `edge` elements with appropriate
`meta` elements.  These elements need to describe the character state
being assigned and the duration (in terms of branch-length) that the edge 
spends in that state (Stochastic character maps are specific to time-calibrated
or ultrametric trees).  

NeXML already defines the `characters` element to handle discrete character traits (`nex:char`)
and the states they can assume (`nex:state`).  We will thus reuse the `characters` element for
this purpose, referring to both the character trait and the states by the ids assigned to them
in that element.  (NeXML's convention of referring to everything by id permits a single canonical
definition of each term, making it clear where additional annotation belongs).  For each edge, we 
need to indicate:

- That our annotation contains a stochastic character mapping reconstruction
- Since many reconstructions are possible for a single edge, we give each reconstruction an id
- We indicate for which character trait we are defining the reconstruction 
- We then indicate which states the character assumes on that edge. 
  For each state realized on the edge, that involves stating: 
    + the state assignment
    + the duration (length of time) for which the edge spends in the given state
    + the order in which the state changes happen (Though we could just assume 
      state transitions are listed chronologically, NeXML suggests making all 
      data explicit, rather than relying on the structure of the data file to
      convey information).  

Thus the annotation for an edge that switches from state `s2` to state 
`s1` of character `cr1` would be constructed like this:

```{r}
 m <- meta("simmap:reconstructions", children = c(
        meta("simmap:reconstruction", children = c(

          meta("simmap:char", "cr1"),
          meta("simmap:stateChange", children = c(
            meta("simmap:order", 1),
            meta("simmap:length", "0.2030"),
            meta("simmap:state", "s2"))),
          
          meta("simmap:char", "cr1"),
          meta("simmap:stateChange", children = c(
            meta("simmap:order", 2),
            meta("simmap:length", "0.0022"),
            meta("simmap:state", "s1")))
          ))))
```

Of course writing out such a definition manually becomes tedious quickly. Because
these are just R commands, we can easily define a function that can loop over an
assignment like this for each edge, extracting the appropriate order, length and
state from an existing R object such as that provided in the `phytools` package.  
Likewise, it is straightforward to define a function that reads this data using
the `RNeXML` utilities and converts it back to the `phytools` package. The full
implementation of this mapping can be seen in the `simmap_to_nexml()` and the
`nexml_to_simmap()` functions provided in the `RNeXML` package.  

As the code indicates, the key step is simply to define the data in meta elements. In 
so doing, we have defined a custom namespace, `simmap`, to hold our variables.  This
allows us to provide a URL with more detailed descriptions of what each of these 
elements mean:

```{r}
nex <- add_namespaces(c(simmap = "https://github.com/ropensci/RNeXML/tree/master/inst/simmap.md"))
```

At that URL we have posted a simple description of each term. 

Using this convention we can generate NeXML files containing `simmap`
data, read those files into R, and convert them back into the `phytools`
package format. These simple functions serve as further illustration of
how `RNeXML` can be used to extend the NeXML standard.  We illustrate
their use briefly here, starting with  loading a `nexml` object containing
a `simmap` reconstruction into R:


```{r}
data(simmap_ex)
```

The `get_trees()` function can be used to return an `ape::phylo` tree as
usual.  `RNeXML` automatically detects the `simmap` reconstruction data
and returns includes this in a `maps` element of the `ape::phylo` object,
for use with other `phytools` functions.

```{r}
phy <- nexml_to_simmap(simmap_ex)
```

We can then use various functions from `phytools` designed for `simmap`
objects [@Revell_2012], such as the plotting function:

```{r Figure1, fig.cap="Stochastic character mapping on a phylogeny, as generated by the phytools package after parsing the simmap-extended NeXML."}
library("phytools")
plotSimmap(phy)
```

Likewise, we can convert the object back in the NeXML format and write
it out to file to be read by other users. 

```{r}
nex <- simmap_to_nexml(phy) 
nexml_write(nex, "simmap.xml")
```

Though other NeXML parsers (for instance, for Perl or Python) have
not been written explicitly to express `simmap` data, those parsers will
nonetheless be able to successfully parse this file and expose the `simmap`
data to the user.

```{r cleanup, include=FALSE}
unlink("simmap.xml")
```

---
title: "Handling Metadata in RNeXML"
author:
  - Carl Boettiger
  - Scott Chamberlain
  - Rutger Vos
  - Hilmar Lapp

output: html_vignette
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{Handling Metadata in RNeXML}
-->




## Writing NeXML metadata

The `add_basic_meta()` function takes as input an existing `nexml` object
(like the other `add_` functions, if none is provided it will create one), and at the time
of this writing any of the following
parameters: `title`, `description`, `creator`, `pubdate`, `rights`, `publisher`,
`citation`.  Other metadata elements and corresponding parameters may
be added in the future.

Load the packages and data:


```r
library('RNeXML')
data(bird.orders)
```

Create an `nexml` object for the phylogeny `bird.orders` and add appropriate metadata:


```r
birds <- add_trees(bird.orders)
birds <- add_basic_meta(
  title = "Phylogeny of the Orders of Birds From Sibley and Ahlquist",

  description = "This data set describes the phylogenetic relationships of the
     orders of birds as reported by Sibley and Ahlquist (1990). Sibley
     and Ahlquist inferred this phylogeny from an extensive number of
     DNA/DNA hybridization experiments. The ``tapestry'' reported by
     these two authors (more than 1000 species out of the ca. 9000
     extant bird species) generated a lot of debates.

     The present tree is based on the relationships among orders. The
     branch lengths were calculated from the values of Delta T50H as
     found in Sibley and Ahlquist (1990, fig. 353).",

  citation = "Sibley, C. G. and Ahlquist, J. E. (1990) Phylogeny and
     classification of birds: a study in molecular evolution. New
     Haven: Yale University Press.",

  creator = "Sibley, C. G. and Ahlquist, J. E.",
	nexml=birds)
```

Instead of a literal string, citations can also be provided in R's
`bibentry` type, which is the one in which R package citations are obtained:


```r
birds <- add_basic_meta(citation = citation("ape"), nexml = birds)
```

## Taxonomic identifiers

RNeXML now uses `taxald` for fast look-up of taxonomic information.
If a unique match is found, a metadata annotation is added to the taxon
providing the unique URI to the taxonomic unit.



```r
birds <- taxize_nexml(birds, "NCBI")
```

If no perfect match is found, the user is warned to check for possible typographic
errors in the taxonomic labels provided.


## Custom metadata extensions

We can get a list of namespaces along with their prefixes from the `nexml` object: 


```r
prefixes <- get_namespaces(birds)
prefixes["dc"]
```

```
                                dc 
"http://purl.org/dc/elements/1.1/" 
```

We create a `meta` element containing this annotation using the `meta` function:


```r
modified <- meta(property = "prism:modificationDate", content = "2013-10-04")
```

We can add this annotation to our existing `birds` NeXML file using the
`add_meta()` function.  Because we do not specify a level, it is added to
the root node, referring to the NeXML file as a whole.


```r
birds <- add_meta(modified, birds) 
```

The built-in vocabularies are just the tip of the iceberg of established
vocabularies. Here we add an annotation from the `skos` namespace which
describes the history of where the data comes from:


```r
history <- meta(property = "skos:historyNote",
  content = "Mapped from the bird.orders data in the ape package using RNeXML")
```

Because `skos` is not in the current namespace list, we add it with a
url when adding this meta element.  We also specify that this annotation
be placed at the level of the `trees` sub-node in the NeXML file.


```r
birds <- add_meta(history, 
                birds, 
                level = "trees",
                namespaces = c(skos = "http://www.w3.org/2004/02/skos/core#"))
```


For finer control of the level at which a `meta` element is added,
we will manipulate the `nexml` R object directly using S4 sub-setting,
as shown in the supplement.


Much richer metadata annotation is possible. Later we illustrate how
metadata annotation can be used to extend the base NeXML format to
represent new forms of data while maintaining compatibility with any
NeXML parser. The `RNeXML` package can be easily extended to support
helper functions such as `taxize_nexml` to add additional metadata
without imposing a large burden on the user.


## Reading NeXML metadata

A call to the `nexml` object prints some metadata summarizing the data structure: 


```r
birds
```

```
A nexml object representing:
 	 1 phylogenetic tree block(s), where:
	   block 1 contains 1 phylogenetic tree(s) 
 	 0 character block(s),  
 	 23 taxonomic units in 1 block(s) 
  Taxa:	 Struthioniformes, Tinamiformes, Craciformes, Galliformes, Anseriformes, Turniciformes ...
  Metadata annotations: 
	18 at top level 
	0 in block 1 at otu level  

This data set describes the phylogenetic relationships of the
     orders of birds as reported by Sibley and Ahlquist (1990). Sibley
     and Ahlquist inferred this phylogeny from an extensive number of
     DNA/DNA hybridization experiments. The ``tapestry'' reported by
     these two authors (more than 1000 species out of the ca. 9000
     extant bird species) generated a lot of debates.

     The present tree is based on the relationships among orders. The
     branch lengths were calculated from the values of Delta T50H as
     found in Sibley and Ahlquist (1990, fig. 353).

Author(s): Sibley, C. G. and Ahlquist, J. E., cboettig

License: http://creativecommons.org/publicdomain/zero/1.0/

NeXML generated by RNeXML using schema version: 0.9 
Size: 358.3 Kb 
```

We can extract all metadata pertaining to the NeXML document as a whole
(annotations of the XML root node, `<nexml>`) with the command


```r
meta <- get_metadata(birds) 
```

This returns a data.frame of available metadata. We can see the kinds
of metadata recorded from the names:


```r
meta
```

```
                        property   datatype
1                       dc:title xsd:string
2                     dc:creator xsd:string
3                 dc:description xsd:string
4                     cc:license       <NA>
5  dcterms:bibliographicCitation xsd:string
6                     dc:creator xsd:string
7             dcterms:references       <NA>
8         prism:modificationDate xsd:string
9                   prism:volume xsd:string
10                  dc:publisher xsd:string
11         prism:publicationName xsd:string
12              prism:endingPage xsd:string
13            prism:startingPage xsd:string
14         prism:publicationDate xsd:string
15                      dc:title xsd:string
16                dc:contributor xsd:string
17                dc:contributor xsd:string
18 dcterms:bibliographicCitation xsd:string
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             content
1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          Phylogeny of the Orders of Birds From Sibley and Ahlquist
2                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  Sibley, C. G. and Ahlquist, J. E.
3  This data set describes the phylogenetic relationships of the\n     orders of birds as reported by Sibley and Ahlquist (1990). Sibley\n     and Ahlquist inferred this phylogeny from an extensive number of\n     DNA/DNA hybridization experiments. The ``tapestry'' reported by\n     these two authors (more than 1000 species out of the ca. 9000\n     extant bird species) generated a lot of debates.\n\n     The present tree is based on the relationships among orders. The\n     branch lengths were calculated from the values of Delta T50H as\n     found in Sibley and Ahlquist (1990, fig. 353).
4                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               <NA>
5                                                                                                                                                                                                                                                                                                                                                                                                                                                      Sibley, C. G. and Ahlquist, J. E. (1990) Phylogeny and\n     classification of birds: a study in molecular evolution. New\n     Haven: Yale University Press.
6                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           cboettig
7                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               <NA>
8                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         2013-10-04
9                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 35
10                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    Bioinformatics
11                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    Bioinformatics
12                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               528
13                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               526
14                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              2018
15                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 ape 5.0: an environment for modern phylogenetics and evolutionary analyses in {R}
16                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        E. Paradis
17                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        K. Schliep
18                                                                                                                                                                                                                                                                                                                                                                                                                                                                 Paradis E, Schliep K (2018). "ape 5.0: an environment for modern phylogenetics and evolutionary analyses in R."\n_Bioinformatics_, *35*, 526-528.
       xsi.type                                              href Meta meta
1   LiteralMeta                                              <NA> m544 <NA>
2   LiteralMeta                                              <NA> m545 <NA>
3   LiteralMeta                                              <NA> m546 <NA>
4  ResourceMeta http://creativecommons.org/publicdomain/zero/1.0/ m547 <NA>
5   LiteralMeta                                              <NA> m548 <NA>
6   LiteralMeta                                              <NA> m549 <NA>
7  ResourceMeta                                              <NA> m560 <NA>
8   LiteralMeta                                              <NA> m561 <NA>
9   LiteralMeta                                              <NA> <NA> m560
10  LiteralMeta                                              <NA> <NA> m560
11  LiteralMeta                                              <NA> <NA> m560
12  LiteralMeta                                              <NA> <NA> m560
13  LiteralMeta                                              <NA> <NA> m560
14  LiteralMeta                                              <NA> <NA> m560
15  LiteralMeta                                              <NA> <NA> m560
16  LiteralMeta                                              <NA> <NA> m560
17  LiteralMeta                                              <NA> <NA> m560
18  LiteralMeta                                              <NA> <NA> m560
```

We can also access a table of taxonomic metadata:



```r
get_taxa(birds)
```

```
     otu            label xsi.type otus
1  ou362 Struthioniformes       NA  os8
2  ou363     Tinamiformes       NA  os8
3  ou364      Craciformes       NA  os8
4  ou365      Galliformes       NA  os8
5  ou366     Anseriformes       NA  os8
6  ou367    Turniciformes       NA  os8
7  ou368       Piciformes       NA  os8
8  ou369    Galbuliformes       NA  os8
9  ou370   Bucerotiformes       NA  os8
10 ou371      Upupiformes       NA  os8
11 ou372    Trogoniformes       NA  os8
12 ou373    Coraciiformes       NA  os8
13 ou374      Coliiformes       NA  os8
14 ou375     Cuculiformes       NA  os8
15 ou376   Psittaciformes       NA  os8
16 ou377      Apodiformes       NA  os8
17 ou378   Trochiliformes       NA  os8
18 ou379  Musophagiformes       NA  os8
19 ou380     Strigiformes       NA  os8
20 ou381    Columbiformes       NA  os8
21 ou382       Gruiformes       NA  os8
22 ou383    Ciconiiformes       NA  os8
23 ou384    Passeriformes       NA  os8
```

Which returns text from the otu element labels, typically used to define
taxonomic names, rather than text from explicit meta elements.

We can also access metadata at a specific level (or use `level=all`
to extract all meta elements in a list).  Here we show only the first
few results:


```r
otu_meta <- get_metadata(birds, level="otus/otu")
otu_meta
```

```
# A tibble: 0 x 3
# … with 3 variables: otu <chr>, otus <chr>, Meta <chr>
```


## Merging metadata tables

We often want to combine metadata from multiple tables.  For instance, in this exercise we want to include the taxonomic identifier and id value for each species returned in the character table.  This helps us more precisely identify the species whose traits are described by the table.   



```r
library("RNeXML")
library("dplyr")
library("geiger")
```

To begin, let's generate a `NeXML` file using the tree and trait data from the `geiger` package's "primates" data:


```r
data("primates")
add_trees(primates$phy) %>% 
  add_characters(primates$dat, ., append=TRUE) %>% 
  taxize_nexml("itis") -> nex 
```

(Note that we've used `dplyr`'s cute pipe syntax, but unfortunately our `add_` methods take the `nexml` object as the _second_
argument instead of the first, so this isn't as elegant since we need the stupid `.` to show where the piped output should go...)

We now read in the three tables of interest.  Note that we tell `get_characters` to give us species labels as there own column, rather than as rownames.  The latter is the default only because this plays more nicely with the default format for character matrices that is expected by `geiger` and other phylogenetics packages, but is in general a silly choice for data manipulation. 


```r
otu_meta <- get_metadata(nex, "otus/otu")
taxa <- get_taxa(nex)
char <- get_characters(nex, rownames_as_col = TRUE)
```


We can take a peek at what the tables look like, just to orient ourselves:


```r
otu_meta
```

```
      property                                                                                  href     xsi.type   otu otus
1   tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572992 ResourceMeta ou385  os9
2   tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572857 ResourceMeta ou386  os9
3   tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572935 ResourceMeta ou387  os9
4   tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572936 ResourceMeta ou388  os9
5   tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945168 ResourceMeta ou389  os9
6   tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945164 ResourceMeta ou390  os9
7   tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572939 ResourceMeta ou391  os9
8   tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572940 ResourceMeta ou392  os9
9   tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572941 ResourceMeta ou393  os9
10  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572942 ResourceMeta ou394  os9
11  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944172 ResourceMeta ou395  os9
12  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572944 ResourceMeta ou396  os9
13  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572949 ResourceMeta ou397  os9
14  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945149 ResourceMeta ou398  os9
15  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572949 ResourceMeta ou399  os9
16  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572947 ResourceMeta ou400  os9
17  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572945 ResourceMeta ou401  os9
18  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572946 ResourceMeta ou402  os9
19  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572951 ResourceMeta ou403  os9
20  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572952 ResourceMeta ou404  os9
21  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572887 ResourceMeta ou405  os9
22  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572888 ResourceMeta ou406  os9
23  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572954 ResourceMeta ou407  os9
24  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572955 ResourceMeta ou408  os9
25  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572957 ResourceMeta ou409  os9
26  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572958 ResourceMeta ou410  os9
27  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572956 ResourceMeta ou411  os9
28  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572953 ResourceMeta ou412  os9
29  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572881 ResourceMeta ou413  os9
30  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572959 ResourceMeta ou414  os9
31  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572983 ResourceMeta ou415  os9
32  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572984 ResourceMeta ou416  os9
33  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025131 ResourceMeta ou417  os9
34  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025134 ResourceMeta ou418  os9
35  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025122 ResourceMeta ou419  os9
36  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025127 ResourceMeta ou420  os9
37  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025112 ResourceMeta ou421  os9
38  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025134 ResourceMeta ou422  os9
39  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025125 ResourceMeta ou423  os9
40  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025116 ResourceMeta ou424  os9
41  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025119 ResourceMeta ou425  os9
42  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025114 ResourceMeta ou426  os9
43  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025115 ResourceMeta ou427  os9
44  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572962 ResourceMeta ou428  os9
45  tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025106 ResourceMeta ou429  os9
46  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572909 ResourceMeta ou430  os9
47  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944125 ResourceMeta ou431  os9
48  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572911 ResourceMeta ou432  os9
49  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572912 ResourceMeta ou433  os9
50  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572913 ResourceMeta ou434  os9
51  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944138 ResourceMeta ou435  os9
52  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572915 ResourceMeta ou436  os9
53  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572916 ResourceMeta ou437  os9
54  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572918 ResourceMeta ou438  os9
55  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944123 ResourceMeta ou439  os9
56  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572975 ResourceMeta ou440  os9
57  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944156 ResourceMeta ou441  os9
58  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572977 ResourceMeta ou442  os9
59  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572978 ResourceMeta ou443  os9
60  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572993 ResourceMeta ou444  os9
61  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572994 ResourceMeta ou445  os9
62  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572995 ResourceMeta ou446  os9
63  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572996 ResourceMeta ou447  os9
64  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572997 ResourceMeta ou448  os9
65  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572998 ResourceMeta ou449  os9
66  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572999 ResourceMeta ou450  os9
67  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573000 ResourceMeta ou451  os9
68  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573001 ResourceMeta ou452  os9
69  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573002 ResourceMeta ou453  os9
70  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573003 ResourceMeta ou454  os9
71  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944219 ResourceMeta ou455  os9
72  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573005 ResourceMeta ou456  os9
73  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573006 ResourceMeta ou457  os9
74  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573007 ResourceMeta ou458  os9
75  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573008 ResourceMeta ou459  os9
76  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573009 ResourceMeta ou460  os9
77  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573010 ResourceMeta ou461  os9
78  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945227 ResourceMeta ou462  os9
79  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573014 ResourceMeta ou463  os9
80  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944221 ResourceMeta ou464  os9
81  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573011 ResourceMeta ou465  os9
82  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572858 ResourceMeta ou466  os9
83  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572859 ResourceMeta ou467  os9
84  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572985 ResourceMeta ou468  os9
85  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572986 ResourceMeta ou469  os9
86  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=552515 ResourceMeta ou470  os9
87  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573035 ResourceMeta ou471  os9
88  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573036 ResourceMeta ou472  os9
89  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573037 ResourceMeta ou473  os9
90  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573038 ResourceMeta ou474  os9
91  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572886 ResourceMeta ou475  os9
92  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573015 ResourceMeta ou476  os9
93  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572865 ResourceMeta ou477  os9
94  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572864 ResourceMeta ou478  os9
95  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572866 ResourceMeta ou479  os9
96  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572867 ResourceMeta ou480  os9
97  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572868 ResourceMeta ou481  os9
98  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572893 ResourceMeta ou482  os9
99  tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572894 ResourceMeta ou483  os9
100 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944100 ResourceMeta ou484  os9
101 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572896 ResourceMeta ou485  os9
102 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572897 ResourceMeta ou486  os9
103 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572898 ResourceMeta ou487  os9
104 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572899 ResourceMeta ou488  os9
105 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572900 ResourceMeta ou489  os9
106 tc:toTaxon http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025177 ResourceMeta ou490  os9
107 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573080 ResourceMeta ou491  os9
108 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572869 ResourceMeta ou492  os9
109 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572870 ResourceMeta ou493  os9
110 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944072 ResourceMeta ou494  os9
111 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=180092 ResourceMeta ou495  os9
112 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573069 ResourceMeta ou496  os9
113 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944286 ResourceMeta ou497  os9
114 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944292 ResourceMeta ou498  os9
115 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945338 ResourceMeta ou499  os9
116 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573073 ResourceMeta ou500  os9
117 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573074 ResourceMeta ou501  os9
118 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944290 ResourceMeta ou502  os9
119 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573076 ResourceMeta ou503  os9
120 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573077 ResourceMeta ou504  os9
121 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573078 ResourceMeta ou505  os9
122 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944293 ResourceMeta ou506  os9
123 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572882 ResourceMeta ou507  os9
124 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572961 ResourceMeta ou508  os9
125 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944187 ResourceMeta ou509  os9
126 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572872 ResourceMeta ou510  os9
127 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572921 ResourceMeta ou511  os9
128 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572920 ResourceMeta ou513  os9
129 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572922 ResourceMeta ou514  os9
130 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572874 ResourceMeta ou515  os9
131 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572875 ResourceMeta ou516  os9
132 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572876 ResourceMeta ou517  os9
133 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572877 ResourceMeta ou518  os9
134 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572878 ResourceMeta ou519  os9
135 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572879 ResourceMeta ou520  os9
136 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572880 ResourceMeta ou521  os9
137 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573016 ResourceMeta ou522  os9
138 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572889 ResourceMeta ou523  os9
139 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573017 ResourceMeta ou524  os9
140 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573018 ResourceMeta ou525  os9
141 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573019 ResourceMeta ou526  os9
142 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=180098 ResourceMeta ou527  os9
143 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=555659 ResourceMeta ou528  os9
144 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573020 ResourceMeta ou529  os9
145 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=180099 ResourceMeta ou530  os9
146 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573021 ResourceMeta ou531  os9
147 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573022 ResourceMeta ou532  os9
148 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573023 ResourceMeta ou533  os9
149 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573024 ResourceMeta ou534  os9
150 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573025 ResourceMeta ou535  os9
151 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573026 ResourceMeta ou536  os9
152 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573027 ResourceMeta ou537  os9
153 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573028 ResourceMeta ou538  os9
154 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573029 ResourceMeta ou539  os9
155 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573030 ResourceMeta ou540  os9
156 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573031 ResourceMeta ou541  os9
157 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944041 ResourceMeta ou542  os9
158 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572861 ResourceMeta ou543  os9
159 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572862 ResourceMeta ou544  os9
160 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573032 ResourceMeta ou545  os9
161 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944263 ResourceMeta ou546  os9
162 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573040 ResourceMeta ou547  os9
163 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572890 ResourceMeta ou548  os9
164 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572891 ResourceMeta ou549  os9
165 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572902 ResourceMeta ou550  os9
166 tc:toTaxon  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572903 ResourceMeta ou551  os9
    Meta
1   m563
2   m564
3   m565
4   m566
5   m567
6   m568
7   m569
8   m570
9   m571
10  m572
11  m573
12  m574
13  m575
14  m576
15  m577
16  m578
17  m579
18  m580
19  m581
20  m582
21  m583
22  m584
23  m585
24  m586
25  m587
26  m588
27  m589
28  m590
29  m591
30  m592
31  m593
32  m594
33  m595
34  m596
35  m597
36  m598
37  m599
38  m600
39  m601
40  m602
41  m603
42  m604
43  m605
44  m606
45  m607
46  m608
47  m609
48  m610
49  m611
50  m612
51  m613
52  m614
53  m615
54  m616
55  m617
56  m618
57  m619
58  m620
59  m621
60  m622
61  m623
62  m624
63  m625
64  m626
65  m627
66  m628
67  m629
68  m630
69  m631
70  m632
71  m633
72  m634
73  m635
74  m636
75  m637
76  m638
77  m639
78  m640
79  m641
80  m642
81  m643
82  m644
83  m645
84  m646
85  m647
86  m648
87  m649
88  m650
89  m651
90  m652
91  m653
92  m654
93  m655
94  m656
95  m657
96  m658
97  m659
98  m660
99  m661
100 m662
101 m663
102 m664
103 m665
104 m666
105 m667
106 m668
107 m669
108 m670
109 m671
110 m672
111 m673
112 m674
113 m675
114 m676
115 m677
116 m678
117 m679
118 m680
119 m681
120 m682
121 m683
122 m684
123 m685
124 m686
125 m687
126 m688
127 m689
128 m690
129 m691
130 m692
131 m693
132 m694
133 m695
134 m696
135 m697
136 m698
137 m699
138 m700
139 m701
140 m702
141 m703
142 m704
143 m705
144 m706
145 m707
146 m708
147 m709
148 m710
149 m711
150 m712
151 m713
152 m714
153 m715
154 m716
155 m717
156 m718
157 m719
158 m720
159 m721
160 m722
161 m723
162 m724
163 m725
164 m726
165 m727
166 m728
 [ reached 'max' / getOption("max.print") -- omitted 66 rows ]
```

```r
taxa
```

```
      otu                        label xsi.type otus
1   ou385  Allenopithecus_nigroviridis       NA  os9
2   ou386          Allocebus_trichotis       NA  os9
3   ou387            Alouatta_belzebul       NA  os9
4   ou388              Alouatta_caraya       NA  os9
5   ou389           Alouatta_coibensis       NA  os9
6   ou390               Alouatta_fusca       NA  os9
7   ou391            Alouatta_palliata       NA  os9
8   ou392               Alouatta_pigra       NA  os9
9   ou393                Alouatta_sara       NA  os9
10  ou394           Alouatta_seniculus       NA  os9
11  ou395                 Aotus_azarai       NA  os9
12  ou396              Aotus_brumbacki       NA  os9
13  ou397           Aotus_hershkovitzi       NA  os9
14  ou398              Aotus_infulatus       NA  os9
15  ou399              Aotus_lemurinus       NA  os9
16  ou400                Aotus_miconax       NA  os9
17  ou401              Aotus_nancymaae       NA  os9
18  ou402              Aotus_nigriceps       NA  os9
19  ou403            Aotus_trivirgatus       NA  os9
20  ou404             Aotus_vociferans       NA  os9
21  ou405            Arctocebus_aureus       NA  os9
22  ou406      Arctocebus_calabarensis       NA  os9
23  ou407             Ateles_belzebuth       NA  os9
24  ou408                Ateles_chamek       NA  os9
25  ou409             Ateles_fusciceps       NA  os9
26  ou410             Ateles_geoffroyi       NA  os9
27  ou411            Ateles_marginatus       NA  os9
28  ou412              Ateles_paniscus       NA  os9
29  ou413                Avahi_laniger       NA  os9
30  ou414      Brachyteles_arachnoides       NA  os9
31  ou415               Cacajao_calvus       NA  os9
32  ou416       Cacajao_melanocephalus       NA  os9
33  ou417          Callicebus_brunneus       NA  os9
34  ou418         Callicebus_caligatus       NA  os9
35  ou419       Callicebus_cinerascens       NA  os9
36  ou420           Callicebus_cupreus       NA  os9
37  ou421      Callicebus_donacophilus       NA  os9
38  ou422            Callicebus_dubius       NA  os9
39  ou423        Callicebus_hoffmannsi       NA  os9
40  ou424          Callicebus_modestus       NA  os9
41  ou425            Callicebus_moloch       NA  os9
42  ou426          Callicebus_oenanthe       NA  os9
43  ou427           Callicebus_olallae       NA  os9
44  ou428        Callicebus_personatus       NA  os9
45  ou429         Callicebus_torquatus       NA  os9
46  ou430            Callimico_goeldii       NA  os9
47  ou431         Callithrix_argentata       NA  os9
48  ou432            Callithrix_aurita       NA  os9
49  ou433         Callithrix_flaviceps       NA  os9
50  ou434         Callithrix_geoffroyi       NA  os9
51  ou435      Callithrix_humeralifera       NA  os9
52  ou436           Callithrix_jacchus       NA  os9
53  ou437            Callithrix_kuhlii       NA  os9
54  ou438       Callithrix_penicillata       NA  os9
55  ou439           Callithrix_pygmaea       NA  os9
56  ou440              Cebus_albifrons       NA  os9
57  ou441                 Cebus_apella       NA  os9
58  ou442              Cebus_capucinus       NA  os9
59  ou443              Cebus_olivaceus       NA  os9
60  ou444            Cercocebus_agilis       NA  os9
61  ou445         Cercocebus_galeritus       NA  os9
62  ou446         Cercocebus_torquatus       NA  os9
63  ou447       Cercopithecus_ascanius       NA  os9
64  ou448      Cercopithecus_campbelli       NA  os9
65  ou449         Cercopithecus_cephus       NA  os9
66  ou450          Cercopithecus_diana       NA  os9
67  ou451          Cercopithecus_dryas       NA  os9
68  ou452  Cercopithecus_erythrogaster       NA  os9
69  ou453     Cercopithecus_erythrotis       NA  os9
70  ou454        Cercopithecus_hamlyni       NA  os9
71  ou455        Cercopithecus_lhoesti       NA  os9
72  ou456          Cercopithecus_mitis       NA  os9
73  ou457           Cercopithecus_mona       NA  os9
74  ou458      Cercopithecus_neglectus       NA  os9
75  ou459      Cercopithecus_nictitans       NA  os9
76  ou460     Cercopithecus_petaurista       NA  os9
77  ou461       Cercopithecus_pogonias       NA  os9
78  ou462        Cercopithecus_preussi       NA  os9
79  ou463       Cercopithecus_sclateri       NA  os9
80  ou464        Cercopithecus_solatus       NA  os9
81  ou465          Cercopithecus_wolfi       NA  os9
82  ou466           Cheirogaleus_major       NA  os9
83  ou467          Cheirogaleus_medius       NA  os9
84  ou468         Chiropotes_albinasus       NA  os9
85  ou469           Chiropotes_satanas       NA  os9
86  ou470         Chlorocebus_aethiops       NA  os9
87  ou471           Colobus_angolensis       NA  os9
88  ou472              Colobus_guereza       NA  os9
89  ou473            Colobus_polykomos       NA  os9
90  ou474              Colobus_satanas       NA  os9
91  ou475 Daubentonia_madagascariensis       NA  os9
92  ou476           Erythrocebus_patas       NA  os9
93  ou477            Eulemur_coronatus       NA  os9
94  ou478               Eulemur_fulvus       NA  os9
95  ou479               Eulemur_macaco       NA  os9
96  ou480               Eulemur_mongoz       NA  os9
97  ou481          Eulemur_rubriventer       NA  os9
98  ou482         Euoticus_elegantulus       NA  os9
99  ou483            Euoticus_pallidus       NA  os9
100 ou484                Galago_alleni       NA  os9
101 ou485              Galago_gallarum       NA  os9
102 ou486             Galago_matschiei       NA  os9
103 ou487                Galago_moholi       NA  os9
104 ou488          Galago_senegalensis       NA  os9
105 ou489          Galagoides_demidoff       NA  os9
106 ou490      Galagoides_zanzibaricus       NA  os9
107 ou491              Gorilla_gorilla       NA  os9
108 ou492             Hapalemur_aureus       NA  os9
109 ou493            Hapalemur_griseus       NA  os9
110 ou494              Hapalemur_simus       NA  os9
111 ou495                 Homo_sapiens       NA  os9
112 ou496             Hylobates_agilis       NA  os9
113 ou497           Hylobates_concolor       NA  os9
114 ou498         Hylobates_gabriellae       NA  os9
115 ou499            Hylobates_hoolock       NA  os9
116 ou500            Hylobates_klossii       NA  os9
117 ou501                Hylobates_lar       NA  os9
118 ou502         Hylobates_leucogenys       NA  os9
119 ou503             Hylobates_moloch       NA  os9
120 ou504           Hylobates_muelleri       NA  os9
121 ou505           Hylobates_pileatus       NA  os9
122 ou506        Hylobates_syndactylus       NA  os9
123 ou507                  Indri_indri       NA  os9
124 ou508         Lagothrix_flavicauda       NA  os9
125 ou509         Lagothrix_lagotricha       NA  os9
126 ou510                  Lemur_catta       NA  os9
127 ou511      Leontopithecus_caissara       NA  os9
128 ou512    Leontopithecus_chrysomela       NA  os9
129 ou513   Leontopithecus_chrysopygus       NA  os9
130 ou514       Leontopithecus_rosalia       NA  os9
131 ou515           Lepilemur_dorsalis       NA  os9
132 ou516           Lepilemur_edwardsi       NA  os9
133 ou517           Lepilemur_leucopus       NA  os9
134 ou518           Lepilemur_microdon       NA  os9
135 ou519         Lepilemur_mustelinus       NA  os9
136 ou520       Lepilemur_ruficaudatus       NA  os9
137 ou521    Lepilemur_septentrionalis       NA  os9
138 ou522          Lophocebus_albigena       NA  os9
139 ou523            Loris_tardigradus       NA  os9
140 ou524             Macaca_arctoides       NA  os9
141 ou525            Macaca_assamensis       NA  os9
142 ou526              Macaca_cyclopis       NA  os9
143 ou527          Macaca_fascicularis       NA  os9
144 ou528               Macaca_fuscata       NA  os9
145 ou529                 Macaca_maura       NA  os9
146 ou530               Macaca_mulatta       NA  os9
147 ou531            Macaca_nemestrina       NA  os9
148 ou532                 Macaca_nigra       NA  os9
149 ou533              Macaca_ochreata       NA  os9
150 ou534               Macaca_radiata       NA  os9
151 ou535               Macaca_silenus       NA  os9
152 ou536                Macaca_sinica       NA  os9
153 ou537              Macaca_sylvanus       NA  os9
154 ou538             Macaca_thibetana       NA  os9
155 ou539              Macaca_tonkeana       NA  os9
156 ou540       Mandrillus_leucophaeus       NA  os9
157 ou541            Mandrillus_sphinx       NA  os9
158 ou542         Microcebus_coquereli       NA  os9
159 ou543           Microcebus_murinus       NA  os9
160 ou544             Microcebus_rufus       NA  os9
161 ou545         Miopithecus_talapoin       NA  os9
162 ou546             Nasalis_concolor       NA  os9
163 ou547             Nasalis_larvatus       NA  os9
164 ou548           Nycticebus_coucang       NA  os9
165 ou549          Nycticebus_pygmaeus       NA  os9
166 ou550      Otolemur_crassicaudatus       NA  os9
167 ou551           Otolemur_garnettii       NA  os9
168 ou552                 Pan_paniscus       NA  os9
169 ou553              Pan_troglodytes       NA  os9
170 ou554              Papio_hamadryas       NA  os9
171 ou555           Perodicticus_potto       NA  os9
172 ou556              Phaner_furcifer       NA  os9
173 ou557       Pithecia_aequatorialis       NA  os9
174 ou558            Pithecia_albicans       NA  os9
175 ou559            Pithecia_irrorata       NA  os9
176 ou560            Pithecia_monachus       NA  os9
177 ou561            Pithecia_pithecia       NA  os9
178 ou562               Pongo_pygmaeus       NA  os9
179 ou563             Presbytis_comata       NA  os9
180 ou564          Presbytis_femoralis       NA  os9
181 ou565           Presbytis_frontata       NA  os9
182 ou566              Presbytis_hosei       NA  os9
183 ou567         Presbytis_melalophos       NA  os9
184 ou568         Presbytis_potenziani       NA  os9
185 ou569          Presbytis_rubicunda       NA  os9
186 ou570            Presbytis_thomasi       NA  os9
187 ou571            Procolobus_badius       NA  os9
188 ou572         Procolobus_pennantii       NA  os9
189 ou573           Procolobus_preussi       NA  os9
190 ou574      Procolobus_rufomitratus       NA  os9
191 ou575             Procolobus_verus       NA  os9
192 ou576          Propithecus_diadema       NA  os9
193 ou577      Propithecus_tattersalli       NA  os9
194 ou578        Propithecus_verreauxi       NA  os9
195 ou579          Pygathrix_avunculus       NA  os9
196 ou580              Pygathrix_bieti       NA  os9
197 ou581           Pygathrix_brelichi       NA  os9
198 ou582            Pygathrix_nemaeus       NA  os9
199 ou583          Pygathrix_roxellana       NA  os9
200 ou584             Saguinus_bicolor       NA  os9
201 ou585         Saguinus_fuscicollis       NA  os9
202 ou586           Saguinus_geoffroyi       NA  os9
203 ou587           Saguinus_imperator       NA  os9
204 ou588             Saguinus_inustus       NA  os9
205 ou589            Saguinus_labiatus       NA  os9
206 ou590            Saguinus_leucopus       NA  os9
207 ou591               Saguinus_midas       NA  os9
208 ou592              Saguinus_mystax       NA  os9
209 ou593         Saguinus_nigricollis       NA  os9
210 ou594             Saguinus_oedipus       NA  os9
211 ou595         Saguinus_tripartitus       NA  os9
212 ou596          Saimiri_boliviensis       NA  os9
213 ou597            Saimiri_oerstedii       NA  os9
214 ou598             Saimiri_sciureus       NA  os9
215 ou599                Saimiri_ustus       NA  os9
216 ou600           Saimiri_vanzolinii       NA  os9
217 ou601       Semnopithecus_entellus       NA  os9
218 ou602             Tarsius_bancanus       NA  os9
219 ou603               Tarsius_dianae       NA  os9
220 ou604              Tarsius_pumilus       NA  os9
221 ou605             Tarsius_spectrum       NA  os9
222 ou606             Tarsius_syrichta       NA  os9
223 ou607         Theropithecus_gelada       NA  os9
224 ou608       Trachypithecus_auratus       NA  os9
225 ou609     Trachypithecus_cristatus       NA  os9
226 ou610     Trachypithecus_francoisi       NA  os9
227 ou611          Trachypithecus_geei       NA  os9
228 ou612        Trachypithecus_johnii       NA  os9
229 ou613      Trachypithecus_obscurus       NA  os9
230 ou614       Trachypithecus_phayrei       NA  os9
231 ou615      Trachypithecus_pileatus       NA  os9
232 ou616       Trachypithecus_vetulus       NA  os9
233 ou617            Varecia_variegata       NA  os9
```

```r
head(char)
```

```
                         taxa        x
1 Allenopithecus_nigroviridis 8.465900
2         Allocebus_trichotis 4.368181
3           Alouatta_belzebul 8.729074
4             Alouatta_caraya 8.628735
5          Alouatta_coibensis 8.764053
6              Alouatta_fusca 8.554489
```

Now that we have nice `data.frame` objects for all our data, it's easy to join them into the desired table with a few obvious `dplyr` commands:


```r
taxa %>% 
  left_join(char, by = c("label" = "taxa")) %>% 
  left_join(otu_meta, by = "otu") %>%
  select(otu, label, x, href)
```

```
      otu                        label         x
1   ou385  Allenopithecus_nigroviridis  8.465900
2   ou386          Allocebus_trichotis  4.368181
3   ou387            Alouatta_belzebul  8.729074
4   ou388              Alouatta_caraya  8.628735
5   ou389           Alouatta_coibensis  8.764053
6   ou390               Alouatta_fusca  8.554489
7   ou391            Alouatta_palliata  8.791790
8   ou392               Alouatta_pigra  8.881836
9   ou393                Alouatta_sara  8.796339
10  ou394           Alouatta_seniculus  8.767173
11  ou395                 Aotus_azarai  7.127694
12  ou396              Aotus_brumbacki  6.401917
13  ou397           Aotus_hershkovitzi  6.684612
14  ou398              Aotus_infulatus  7.122867
15  ou399              Aotus_lemurinus  6.781058
16  ou400                Aotus_miconax  6.684612
17  ou401              Aotus_nancymaae  6.678342
18  ou402              Aotus_nigriceps  6.966024
19  ou403            Aotus_trivirgatus  6.815640
20  ou404             Aotus_vociferans  6.771936
21  ou405            Arctocebus_aureus  5.459586
22  ou406      Arctocebus_calabarensis  5.552960
23  ou407             Ateles_belzebuth  8.812843
24  ou408                Ateles_chamek  8.872067
25  ou409             Ateles_fusciceps  9.112728
26  ou410             Ateles_geoffroyi  8.937218
27  ou411            Ateles_marginatus  8.738735
28  ou412              Ateles_paniscus  9.072227
29  ou413                Avahi_laniger  6.927558
30  ou414      Brachyteles_arachnoides  9.268609
31  ou415               Cacajao_calvus  8.137396
32  ou416       Cacajao_melanocephalus  8.051978
33  ou417          Callicebus_brunneus  6.747587
34  ou418         Callicebus_caligatus  6.899723
35  ou419       Callicebus_cinerascens  6.899723
36  ou420           Callicebus_cupreus  7.021084
37  ou421      Callicebus_donacophilus  6.801283
38  ou422            Callicebus_dubius  6.899723
39  ou423        Callicebus_hoffmannsi  6.975414
40  ou424          Callicebus_modestus  6.899723
41  ou425            Callicebus_moloch  6.872128
42  ou426          Callicebus_oenanthe  6.899723
43  ou427           Callicebus_olallae  6.899723
44  ou428        Callicebus_personatus  7.237059
45  ou429         Callicebus_torquatus  7.106606
46  ou430            Callimico_goeldii  6.324359
47  ou431         Callithrix_argentata  5.910797
48  ou432            Callithrix_aurita  5.961005
49  ou433         Callithrix_flaviceps  5.926926
50  ou434         Callithrix_geoffroyi  5.834811
51  ou435      Callithrix_humeralifera  5.926926
52  ou436           Callithrix_jacchus  5.673323
53  ou437            Callithrix_kuhlii  5.926926
54  ou438       Callithrix_penicillata  5.828946
55  ou439           Callithrix_pygmaea  4.828314
56  ou440              Cebus_albifrons  7.832014
57  ou441                 Cebus_apella  7.922986
58  ou442              Cebus_capucinus  8.009695
59  ou443              Cebus_olivaceus  7.937375
60  ou444            Cercocebus_agilis  8.870663
61  ou445         Cercocebus_galeritus  8.865029
62  ou446         Cercocebus_torquatus  8.894259
63  ou447       Cercopithecus_ascanius  8.171882
64  ou448      Cercopithecus_campbelli  8.199739
65  ou449         Cercopithecus_cephus  8.143227
66  ou450          Cercopithecus_diana  8.382518
67  ou451          Cercopithecus_dryas  7.930206
68  ou452  Cercopithecus_erythrogaster  8.143227
69  ou453     Cercopithecus_erythrotis  8.086410
70  ou454        Cercopithecus_hamlyni  8.438150
71  ou455        Cercopithecus_lhoesti  8.579229
72  ou456          Cercopithecus_mitis  8.525161
73  ou457           Cercopithecus_mona  8.289037
74  ou458      Cercopithecus_neglectus  8.579229
75  ou459      Cercopithecus_nictitans  8.567886
76  ou460     Cercopithecus_petaurista  8.083329
77  ou461       Cercopithecus_pogonias  8.183118
78  ou462        Cercopithecus_preussi  8.544808
79  ou463       Cercopithecus_sclateri  8.029433
80  ou464        Cercopithecus_solatus  8.567886
81  ou465          Cercopithecus_wolfi  8.089482
82  ou466           Cheirogaleus_major  6.104793
83  ou467          Cheirogaleus_medius  5.283204
84  ou468         Chiropotes_albinasus  7.937375
85  ou469           Chiropotes_satanas  7.996317
86  ou470         Chlorocebus_aethiops  8.301522
87  ou471           Colobus_angolensis  9.103868
88  ou472              Colobus_guereza  9.210340
89  ou473            Colobus_polykomos  9.060680
90  ou474              Colobus_satanas  9.121509
91  ou475 Daubentonia_madagascariensis  7.919356
92  ou476           Erythrocebus_patas  8.988446
93  ou477            Eulemur_coronatus  7.438384
94  ou478               Eulemur_fulvus  7.779049
95  ou479               Eulemur_macaco  7.816014
96  ou480               Eulemur_mongoz  7.478735
97  ou481          Eulemur_rubriventer  7.620705
98  ou482         Euoticus_elegantulus  5.690359
99  ou483            Euoticus_pallidus  5.627621
100 ou484                Galago_alleni  5.521461
101 ou485              Galago_gallarum  5.365976
102 ou486             Galago_matschiei  5.267858
103 ou487                Galago_moholi  5.375278
104 ou488          Galago_senegalensis  5.594711
105 ou489          Galagoides_demidoff  4.206184
106 ou490      Galagoides_zanzibaricus  4.997212
107 ou491              Gorilla_gorilla 11.643954
108 ou492             Hapalemur_aureus  7.358831
109 ou493            Hapalemur_griseus  6.862758
110 ou494              Hapalemur_simus  7.620705
111 ou495                 Homo_sapiens 10.980195
112 ou496             Hylobates_agilis  8.675905
113 ou497           Hylobates_concolor  8.768730
114 ou498         Hylobates_gabriellae  9.047821
115 ou499            Hylobates_hoolock  8.809863
116 ou500            Hylobates_klossii  8.674197
117 ou501                Hylobates_lar  8.635865
118 ou502         Hylobates_leucogenys  8.898366
119 ou503             Hylobates_moloch  8.677610
120 ou504           Hylobates_muelleri  8.687779
121 ou505           Hylobates_pileatus  8.630522
122 ou506        Hylobates_syndactylus  9.296518
123 ou507                  Indri_indri  9.065315
124 ou508         Lagothrix_flavicauda  9.020390
125 ou509         Lagothrix_lagotricha  8.743532
126 ou510                  Lemur_catta  7.878534
127 ou511      Leontopithecus_caissara  6.405228
128 ou512    Leontopithecus_chrysomela  6.352629
129 ou513   Leontopithecus_chrysopygus  6.486161
130 ou514       Leontopithecus_rosalia  6.385194
131 ou515           Lepilemur_dorsalis  6.232448
132 ou516           Lepilemur_edwardsi  6.711740
133 ou517           Lepilemur_leucopus  6.396930
134 ou518           Lepilemur_microdon  6.863803
135 ou519         Lepilemur_mustelinus  6.508769
136 ou520       Lepilemur_ruficaudatus  6.637258
137 ou521    Lepilemur_septentrionalis  6.638568
138 ou522          Lophocebus_albigena  8.903815
139 ou523            Loris_tardigradus  5.517453
140 ou524             Macaca_arctoides  9.148465
141 ou525            Macaca_assamensis  9.054855
142 ou526              Macaca_cyclopis  8.658693
143 ou527          Macaca_fascicularis  8.431635
144 ou528               Macaca_fuscata  9.220291
145 ou529                 Macaca_maura  8.894259
146 ou530               Macaca_mulatta  8.771835
147 ou531            Macaca_nemestrina  8.970813
148 ou532                 Macaca_nigra  8.906529
149 ou533              Macaca_ochreata  7.919356
150 ou534               Macaca_radiata  8.517193
151 ou535               Macaca_silenus  8.699515
152 ou536                Macaca_sinica  8.446771
153 ou537              Macaca_sylvanus  9.350102
154 ou538             Macaca_thibetana  9.268609
155 ou539              Macaca_tonkeana  9.220291
156 ou540       Mandrillus_leucophaeus  9.560997
157 ou541            Mandrillus_sphinx  9.729134
158 ou542         Microcebus_coquereli  5.793014
159 ou543           Microcebus_murinus  4.198705
160 ou544             Microcebus_rufus  3.881564
161 ou545         Miopithecus_talapoin  7.130899
162 ou546             Nasalis_concolor  8.979354
163 ou547             Nasalis_larvatus  9.417355
164 ou548           Nycticebus_coucang  6.878326
165 ou549          Nycticebus_pygmaeus  5.840642
166 ou550      Otolemur_crassicaudatus  7.130899
167 ou551           Otolemur_garnettii  6.701960
168 ou552                 Pan_paniscus 10.468801
169 ou553              Pan_troglodytes 10.716638
170 ou554              Papio_hamadryas  9.735069
171 ou555           Perodicticus_potto  6.984716
172 ou556              Phaner_furcifer  6.016157
173 ou557       Pithecia_aequatorialis  7.774856
174 ou558            Pithecia_albicans  7.937375
175 ou559            Pithecia_irrorata  7.749322
176 ou560            Pithecia_monachus  7.654443
177 ou561            Pithecia_pithecia  7.420579
178 ou562               Pongo_pygmaeus 10.860920
179 ou563             Presbytis_comata  9.039552
180 ou564          Presbytis_femoralis  8.797851
181 ou565           Presbytis_frontata  8.853665
182 ou566              Presbytis_hosei  8.725832
183 ou567         Presbytis_melalophos  8.748305
184 ou568         Presbytis_potenziani  8.774931
185 ou569          Presbytis_rubicunda  8.773385
186 ou570            Presbytis_thomasi  8.765615
187 ou571            Procolobus_badius  8.808369
188 ou572         Procolobus_pennantii  9.122601
189 ou573           Procolobus_preussi  9.094930
190 ou574      Procolobus_rufomitratus  8.995909
191 ou575             Procolobus_verus  8.289037
192 ou576          Propithecus_diadema  8.794825
193 ou577      Propithecus_tattersalli  8.171882
194 ou578        Propithecus_verreauxi  8.202482
195 ou579          Pygathrix_avunculus  9.164296
196 ou580              Pygathrix_bieti  9.118225
197 ou581           Pygathrix_brelichi  9.305651
198 ou582            Pygathrix_nemaeus  9.417355
199 ou583          Pygathrix_roxellana  9.510445
200 ou584             Saguinus_bicolor  6.142037
201 ou585         Saguinus_fuscicollis  5.976351
202 ou586           Saguinus_geoffroyi  6.200509
203 ou587           Saguinus_imperator  6.016157
204 ou588             Saguinus_inustus  6.688355
205 ou589            Saguinus_labiatus  6.234411
206 ou590            Saguinus_leucopus  6.124683
207 ou591               Saguinus_midas  6.293419
208 ou592              Saguinus_mystax  6.326149
209 ou593         Saguinus_nigricollis  6.109248
210 ou594             Saguinus_oedipus  6.139885
211 ou595         Saguinus_tripartitus  5.953243
212 ou596          Saimiri_boliviensis  6.690842
213 ou597            Saimiri_oerstedii  6.570883
214 ou598             Saimiri_sciureus  6.620073
215 ou599                Saimiri_ustus  6.791221
216 ou600           Saimiri_vanzolinii  6.664409
217 ou601       Semnopithecus_entellus  9.441452
218 ou602             Tarsius_bancanus  4.736198
219 ou603               Tarsius_dianae  4.709530
220 ou604              Tarsius_pumilus  4.808111
221 ou605             Tarsius_spectrum  5.111988
222 ou606             Tarsius_syrichta  4.753590
223 ou607         Theropithecus_gelada  9.680344
224 ou608       Trachypithecus_auratus  9.181941
225 ou609     Trachypithecus_cristatus  8.872067
226 ou610     Trachypithecus_francoisi  9.008224
227 ou611          Trachypithecus_geei  9.031214
228 ou612        Trachypithecus_johnii  9.268609
229 ou613      Trachypithecus_obscurus  8.870663
230 ou614       Trachypithecus_phayrei  8.948976
231 ou615      Trachypithecus_pileatus  9.323669
232 ou616       Trachypithecus_vetulus  8.969542
233 ou617            Varecia_variegata  8.261010
                                                                                     href
1    http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572992
2    http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572857
3    http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572935
4    http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572936
5    http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945168
6    http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945164
7    http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572939
8    http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572940
9    http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572941
10   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572942
11   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944172
12   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572944
13   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572949
14   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945149
15   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572949
16   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572947
17   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572945
18   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572946
19   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572951
20   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572952
21   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572887
22   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572888
23   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572954
24   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572955
25   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572957
26   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572958
27   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572956
28   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572953
29   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572881
30   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572959
31   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572983
32   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572984
33  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025131
34  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025134
35  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025122
36  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025127
37  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025112
38  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025134
39  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025125
40  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025116
41  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025119
42  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025114
43  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025115
44   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572962
45  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025106
46   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572909
47   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944125
48   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572911
49   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572912
50   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572913
51   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944138
52   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572915
53   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572916
54   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572918
55   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944123
56   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572975
57   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944156
58   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572977
59   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572978
60   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572993
61   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572994
62   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572995
63   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572996
64   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572997
65   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572998
66   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572999
67   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573000
68   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573001
69   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573002
70   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573003
71   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944219
72   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573005
73   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573006
74   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573007
75   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573008
76   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573009
77   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573010
78   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945227
79   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573014
80   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944221
81   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573011
82   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572858
83   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572859
84   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572985
85   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572986
86   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=552515
87   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573035
88   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573036
89   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573037
90   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573038
91   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572886
92   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573015
93   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572865
94   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572864
95   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572866
96   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572867
97   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572868
98   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572893
99   http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572894
100  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944100
101  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572896
102  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572897
103  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572898
104  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572899
105  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572900
106 http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025177
107  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573080
108  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572869
109  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572870
110  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944072
111  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=180092
112  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573069
113  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944286
114  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944292
115  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945338
116  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573073
117  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573074
118  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944290
119  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573076
120  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573077
121  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573078
122  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944293
123  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572882
124  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572961
125  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944187
126  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572872
127  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572921
128                                                                                  <NA>
129  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572920
130  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572922
131  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572874
132  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572875
133  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572876
134  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572877
135  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572878
136  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572879
137  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572880
138  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573016
139  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572889
140  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573017
141  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573018
142  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573019
143  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=180098
144  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=555659
145  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573020
146  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=180099
147  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573021
148  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573022
149  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573023
150  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573024
151  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573025
152  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573026
153  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573027
154  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573028
155  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573029
156  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573030
157  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573031
158  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944041
159  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572861
160  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572862
161  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573032
162  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944263
163  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573040
164  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572890
165  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572891
166  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572902
167  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572903
168  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573081
169  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573082
170  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573033
171  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572892
172  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572863
173  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572987
174  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572988
175  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572989
176  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572990
177  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572991
178  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573083
179  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573041
180  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573042
181  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573043
182  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573044
183  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573045
184  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573046
185  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573047
186  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573048
187  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944230
188  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944235
189  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944238
190  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944239
191  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573053
192  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572883
193  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572884
194  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572885
195  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944258
196  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944259
197  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944260
198  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573057
199  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945308
200  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572923
201  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=998616
202  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572925
203  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572926
204  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572927
205  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572928
206  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572929
207  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572930
208  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572931
209  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=998617
210  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572933
211 http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=1025098
212  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572979
213  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572980
214  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=180095
215  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572981
216  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572982
217  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573059
218  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945103
219  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944117
220  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572906
221  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944115
222  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=945107
223  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573034
224  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573060
225  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573061
226  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573062
227  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573063
228  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944270
229  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573065
230  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573066
231  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=573067
232  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=944269
233  http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=TSN&search_value=572873
```

Because these are all from the same otus block anyway, we haven't selected that column, but were it of interest it is also available in the taxa table.

---
output:
 md_document:
   variant: markdown_github
---




```{r}
library("RNeXML")
library("dplyr")
library("geiger")
knitr::opts_chunk$set(message = FALSE, warning=FALSE, comment = NA)
```

Let's generate a `NeXML` file using the tree and trait data from the `geiger` package's "primates" data:

```{r}
data("primates")
add_trees(primates$phy) %>% 
  add_characters(primates$dat, ., append=TRUE) %>% 
  taxize_nexml() -> nex 
```

(Note that we've used `dplyr`'s cute pipe syntax, but unfortunately our `add_` methods take the `nexml` object as the _second_
argument instead of the first, so this isn't as elegant since we need the stupid `.` to show where the piped output should go...)

We now read in the three tables of interest.  Note that we tell `get_characters` to give us species labels as there own column, rather than as rownames.  The latter is the default only because this plays more nicely with the default format for character matrices that is expected by `geiger` and other phylogenetics packages, but is in general a silly choice for data manipulation. 

```{r}
otu_meta <- get_metadata(nex, "otus/otu")
taxa <- get_taxa(nex)
char <- get_characters(nex, rownames_as_col = TRUE)
```


We can take a peek at what the tables look like, just to orient ourselves:

```{r}
otu_meta
taxa
head(char)
```

Now that we have nice `data.frame` objects for all our data, it's easy to join them into the desired table with a few obvious `dplyr` commands:

```{r}
taxa %>% 
  left_join(char, by = c("label" = "taxa")) %>% 
  left_join(otu_meta, by = c("id" = "otu")) %>%
  select(id, label, x, href)
```

Because these are all from the same otus block anyway, we haven't selected that column, but were it of interest it is also available in the taxa table.

``` r
library("RNeXML")
```

    ## Loading required package: ape

``` r
library("dplyr")
```

    ## 
    ## Attaching package: 'dplyr'
    ## 
    ## The following objects are masked from 'package:stats':
    ## 
    ##     filter, lag
    ## 
    ## The following objects are masked from 'package:base':
    ## 
    ##     intersect, setdiff, setequal, union

``` r
library("geiger")
knitr::opts_chunk$set(message = FALSE, comment = NA)
```

Let's generate a `NeXML` file using the tree and trait data from the `geiger` package's "primates" data:

``` r
data("primates")
add_trees(primates$phy) %>% 
  add_characters(primates$dat, ., append=TRUE) %>% 
  taxize_nexml() -> nex 
```

    Warning in taxize_nexml(.): ID for otu Alouatta_coibensis not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Aotus_hershkovitzi not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Aotus_miconax not found. Consider
    checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Callicebus_cinerascens not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Callicebus_dubius not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Callicebus_modestus not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Callicebus_oenanthe not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Callicebus_olallae not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Euoticus_pallidus not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Lagothrix_flavicauda not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Leontopithecus_caissara not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Leontopithecus_chrysomela not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Pithecia_aequatorialis not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Pithecia_albicans not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Procolobus_pennantii not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Procolobus_preussi not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Procolobus_rufomitratus not found.
    Consider checking the spelling or alternate classification

    Warning in taxize_nexml(.): ID for otu Tarsius_pumilus not found. Consider
    checking the spelling or alternate classification

(Note that we've used `dplyr`'s cute pipe syntax, but unfortunately our `add_` methods take the `nexml` object as the *second* argument instead of the first, so this isn't as elegant since we need the stupid `.` to show where the piped output should go...)

We now read in the three tables of interest. Note that we tell `get_characters` to give us species labels as there own column, rather than as rownames. The latter is the default only because this plays more nicely with the default format for character matrices that is expected by `geiger` and other phylogenetics packages, but is in general a silly choice for data manipulation.

``` r
otu_meta <- get_metadata(nex, "otus/otu")
taxa <- get_taxa(nex)
char <- get_characters(nex, rownames_as_col = TRUE)
```

We can take a peek at what the tables look like, just to orient ourselves:

``` r
otu_meta
```

    Source: local data frame [215 x 9]

          id property datatype content     xsi.type        rel
       (chr)    (lgl)    (lgl)   (lgl)        (chr)      (chr)
    1     m1       NA       NA      NA ResourceMeta tc:toTaxon
    2     m2       NA       NA      NA ResourceMeta tc:toTaxon
    3     m3       NA       NA      NA ResourceMeta tc:toTaxon
    4     m4       NA       NA      NA ResourceMeta tc:toTaxon
    5     m5       NA       NA      NA ResourceMeta tc:toTaxon
    6     m6       NA       NA      NA ResourceMeta tc:toTaxon
    7     m7       NA       NA      NA ResourceMeta tc:toTaxon
    8     m8       NA       NA      NA ResourceMeta tc:toTaxon
    9     m9       NA       NA      NA ResourceMeta tc:toTaxon
    10   m10       NA       NA      NA ResourceMeta tc:toTaxon
    ..   ...      ...      ...     ...          ...        ...
    Variables not shown: href (chr), otu (chr), otus (chr)

``` r
taxa
```

    Source: local data frame [233 x 5]

          id                       label about xsi.type  otus
       (chr)                       (chr) (chr)    (lgl) (chr)
    1    ou1 Allenopithecus_nigroviridis  #ou1       NA   os1
    2    ou2         Allocebus_trichotis  #ou2       NA   os1
    3    ou3           Alouatta_belzebul  #ou3       NA   os1
    4    ou4             Alouatta_caraya  #ou4       NA   os1
    5    ou5          Alouatta_coibensis  #ou5       NA   os1
    6    ou6              Alouatta_fusca  #ou6       NA   os1
    7    ou7           Alouatta_palliata  #ou7       NA   os1
    8    ou8              Alouatta_pigra  #ou8       NA   os1
    9    ou9               Alouatta_sara  #ou9       NA   os1
    10  ou10          Alouatta_seniculus #ou10       NA   os1
    ..   ...                         ...   ...      ...   ...

``` r
head(char)
```

                             taxa        x
    1 Allenopithecus_nigroviridis 8.465900
    2          Alouatta_seniculus 8.767173
    3               Galago_alleni 5.521461
    4             Galago_gallarum 5.365976
    5            Galago_matschiei 5.267858
    6               Galago_moholi 5.375278

Now that we have nice `data.frame` objects for all our data, it's easy to join them into the desired table with a few obvious `dplyr` commands:

``` r
taxa %>% 
  left_join(char, by = c("label" = "taxa")) %>% 
  left_join(otu_meta, by = c("id" = "otu")) %>%
  select(id, label, x, href)
```

    Warning in left_join_impl(x, y, by$x, by$y): joining factor and character
    vector, coercing into character vector

    Source: local data frame [233 x 4]

          id                       label        x
       (chr)                       (chr)    (dbl)
    1    ou1 Allenopithecus_nigroviridis 8.465900
    2    ou2         Allocebus_trichotis 4.368181
    3    ou3           Alouatta_belzebul 8.729074
    4    ou4             Alouatta_caraya 8.628735
    5    ou5          Alouatta_coibensis 8.764053
    6    ou6              Alouatta_fusca 8.554489
    7    ou7           Alouatta_palliata 8.791790
    8    ou8              Alouatta_pigra 8.881836
    9    ou9               Alouatta_sara 8.796339
    10  ou10          Alouatta_seniculus 8.767173
    ..   ...                         ...      ...
    Variables not shown: href (chr)

Because these are all from the same otus block anyway, we haven't selected that column, but were it of interest it is also available in the taxa table.

---
title: "Handling Metadata in RNeXML"
author:
  - Carl Boettiger
  - Scott Chamberlain
  - Rutger Vos
  - Hilmar Lapp

output: html_vignette
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{Handling Metadata in RNeXML}
-->


```{r compile-settings, include=FALSE}
library("methods")
library("knitr")
opts_chunk$set(tidy = FALSE, warning = FALSE, message = FALSE, 
               cache = FALSE, comment = NA, verbose = TRUE)
basename <- gsub(".Rmd", "", knitr:::knit_concord$get('infile')) 

```

## Writing NeXML metadata

The `add_basic_meta()` function takes as input an existing `nexml` object
(like the other `add_` functions, if none is provided it will create one), and at the time
of this writing any of the following
parameters: `title`, `description`, `creator`, `pubdate`, `rights`, `publisher`,
`citation`.  Other metadata elements and corresponding parameters may
be added in the future.

Load the packages and data:

```{r}
library('RNeXML')
data(bird.orders)
```

Create an `nexml` object for the phylogeny `bird.orders` and add appropriate metadata:

```{r}
birds <- add_trees(bird.orders)
birds <- add_basic_meta(
  title = "Phylogeny of the Orders of Birds From Sibley and Ahlquist",

  description = "This data set describes the phylogenetic relationships of the
     orders of birds as reported by Sibley and Ahlquist (1990). Sibley
     and Ahlquist inferred this phylogeny from an extensive number of
     DNA/DNA hybridization experiments. The ``tapestry'' reported by
     these two authors (more than 1000 species out of the ca. 9000
     extant bird species) generated a lot of debates.

     The present tree is based on the relationships among orders. The
     branch lengths were calculated from the values of Delta T50H as
     found in Sibley and Ahlquist (1990, fig. 353).",

  citation = "Sibley, C. G. and Ahlquist, J. E. (1990) Phylogeny and
     classification of birds: a study in molecular evolution. New
     Haven: Yale University Press.",

  creator = "Sibley, C. G. and Ahlquist, J. E.",
	nexml=birds)
```

Instead of a literal string, citations can also be provided in R's
`bibentry` type, which is the one in which R package citations are obtained:

```{r}
birds <- add_basic_meta(citation = citation("ape"), nexml = birds)
```

## Taxonomic identifiers

The `taxize_nexml()` function uses the R package `taxize` 
[@Chamberlain_2013] to check each taxon label against the NCBI database.
If a unique match is found, a metadata annotation is added to the taxon
providing the NCBI identification number to the taxonomic unit.


```{r message=FALSE, results='hide'}
birds <- taxize_nexml(birds, "NCBI")
```

If no match is found, the user is warned to check for possible typographic
errors in the taxonomic labels provided. If multiple matches are found,
the user will be prompted to choose between them.


## Custom metadata extensions

We can get a list of namespaces along with their prefixes from the `nexml` object: 

```{r}
prefixes <- get_namespaces(birds)
prefixes["dc"]
```

We create a `meta` element containing this annotation using the `meta` function:

```{r}
modified <- meta(property = "prism:modificationDate", content = "2013-10-04")
```

We can add this annotation to our existing `birds` NeXML file using the
`add_meta()` function.  Because we do not specify a level, it is added to
the root node, referring to the NeXML file as a whole.

```{r}
birds <- add_meta(modified, birds) 
```

The built-in vocabularies are just the tip of the iceberg of established
vocabularies. Here we add an annotation from the `skos` namespace which
describes the history of where the data comes from:

```{r}
history <- meta(property = "skos:historyNote",
  content = "Mapped from the bird.orders data in the ape package using RNeXML")
```

Because `skos` is not in the current namespace list, we add it with a
url when adding this meta element.  We also specify that this annotation
be placed at the level of the `trees` sub-node in the NeXML file.

```{r}
birds <- add_meta(history, 
                birds, 
                level = "trees",
                namespaces = c(skos = "http://www.w3.org/2004/02/skos/core#"))
```


For finer control of the level at which a `meta` element is added,
we will manipulate the `nexml` R object directly using S4 sub-setting,
as shown in the supplement.


Much richer metadata annotation is possible. Later we illustrate how
metadata annotation can be used to extend the base NeXML format to
represent new forms of data while maintaining compatibility with any
NeXML parser. The `RNeXML` package can be easily extended to support
helper functions such as `taxize_nexml` to add additional metadata
without imposing a large burden on the user.


## Reading NeXML metadata

A call to the `nexml` object prints some metadata summarizing the data structure: 

```{r }
birds
```

We can extract all metadata pertaining to the NeXML document as a whole
(annotations of the XML root node, `<nexml>`) with the command

```{r}
meta <- get_metadata(birds) 
```

This returns a named list of available metadata. We can see the kinds
of metadata recorded from the names (showing the first 4):

```{r}
names(meta)[1:4]
```

and can ask for a particular element using the standard list sub-setting
mechanism (i.e. either the name of an element or its numeric position),

```{r}
meta[["dc:title"]]
```

All metadata terms must belong to an explicit *namespace* or vocabulary
that allows a computer to interpret the term precisely. The prefix (before
the `:`) indicates to which vocabulary the term belongs, e.g. `dc` in
this case.  The `get_namespaces` function tells us the definition of
the vocabulary using a link:

```{r}
prefixes <- get_namespaces(birds)
prefixes["dc"]
```

Common metadata can be accessed with a few dedicated functions:

```{r get_citation}
get_citation(birds)
```


```{r get_taxa}
get_taxa(birds)
```

Which returns text from the otu element labels, typically used to define
taxonomic names, rather than text from explicit meta elements.

We can also access metadata at a specific level (or use `level=all`
to extract all meta elements in a list).  Here we show only the first
few results:

```{r} 
otu_meta <- get_metadata(birds, level="otu")
otu_meta[1:4]
```

---
title: "The nexml S4 Object"
author:
  - Carl Boettiger
  - Scott Chamberlain
  - Rutger Vos
  - Hilmar Lapp
output: html_vignette
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{The nexml S4 Object}
-->


```{r supplement-compile-settings, include=FALSE}
library("methods")
library("knitr")
opts_chunk$set(tidy = FALSE, warning = FALSE, message = FALSE, 
               cache = FALSE, comment = NA, verbose = TRUE)
basename <- 'S4' 

```

```{r include=FALSE}
library("RNeXML")
```


## Understanding the `nexml` S4 object

The `RNeXML` package provides many convenient functions to add and extract
information from `nexml` objects in the R environment without requiring
the reader to understand the details of the NeXML data structure and
making it less likely that a user will generate invalid NeXML syntax
that could not be read by other parsers. The `nexml` object we have been using
in all of the examples is built on R's S4 mechanism. Advanced users may
sometimes prefer to interact with the data structure more directly using 
R's S4 class mechanism and subsetting methods. Many R users are more familiar
with the S3 class mechanism (such as in the `ape` package phylo objects)
rather than the S4 class mechanism used in phylogenetics packages such as 
`ouch` and `phylobase`. The `phylobase` vignette provides an excellent introduction
to these data structures.  Users already familiar with subsetting lists and other
S3 objects in R are likely familar with the use of the `$` operator, such as
`phy$edge`. S4 objects simply use an `@` operator instead (but cannot be subset
using numeric arguments such as `phy[[1]]` or named arguments such as phy[["edge"]]).  


The `nexml` object is an S4 object, as are all of its components (slots).  Its 
hierarchical structure corresponds exactly with the XML tree of a NeXML file, with 
the single exception that both XML attributes and children are represented as slots.  
S4 objects have constructor functions to initialize them.  We create a new `nexml` 
object with the command:

```{r}
nex <- new("nexml")
```

We can see a list of slots contained in this object with

```{r}
slotNames(nex)
```

Some of these slots have already been populated for us, for instance, the schema version and default namespaces:

```{r}
nex@version
nex@namespaces
```

Recognize that `nex@namespaces` serves the same role as `get_namespaces`
function, but provides direct access to the slot data.  For instance,
with this syntax we could also overwrite the existing namespaces with
`nex@namespaces <- NULL`.  Changing the namespace in this way is not
advised.

Some slots can contain multiple elements of the same type, such as
`trees`, `characters`, and `otus`.  For instance, we see that

```{r}
class(nex@characters)
```

is an object of class `ListOfcharacters`, and is currently empty,

```{r}
length(nex@characters)
```

In order to assign an object to a slot, it must match the class definition
of the slot.  We can create a new element of any given class with the
`new` function,

```{r}
nex@characters <- new("ListOfcharacters", list(new("characters")))
```

and now we have a length-1 list of character matrices,

```{r}
length(nex@characters)
```

and we access the first character matrix using the list notation,
`[[1]]`. Here we check the class is a `characters` object.

```{r}
class(nex@characters[[1]])
```

Direct subsetting has two primary use cases: (a) useful in looking up
(and possibly editing) a specific value of an element, or (b) when adding
metadata annotations to specific elements. Consider the example file


```{r}
f <- system.file("examples", "trees.xml", package="RNeXML")
nex <- nexml_read(f)
```

We can look up the species label of the first `otu` in the first `otus` block:

```{r}
nex@otus[[1]]@otu[[1]]@label
```

We can add metadata to this particular OTU using this subsetting format

```{r}
nex@otus[[1]]@otu[[1]]@meta <- 
  c(meta("skos:note", 
          "This species was incorrectly identified"),
         nex@otus[[1]]@otu[[1]]@meta)
```

Here we use the `c` operator to append this element to any existing meta annotations to this otu.  

---
title: "SPARQL with RNeXML"
author:
  - Carl Boettiger
  - Scott Chamberlain
  - Rutger Vos
  - Hilmar Lapp
output: html_vignette
---

<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{SPARQL with RNeXML}
-->


```{r supplement-compile-settings, include=FALSE}
library("methods")
library("knitr")
opts_chunk$set(tidy = FALSE, warning = FALSE, message = FALSE, 
               cache = FALSE, comment = NA, verbose = TRUE, eval=require("rrdf"))
basename <- 'sparql' 


```

```{r include=FALSE}
library("RNeXML")
```




## SPARQL Queries

Rich, semantically meaningful metadata lies at the heart of the NeXML
standard.  R provides a rich environment to unlock this information.
While our previous examples have relied on the user knowing exactly
what metadata they intend to extract (title, publication date, citation
information, and so forth), _semantic_ metadata has meaning that a
computer can make use of, allowing us to make much more conceptually
rich queries than those simple examples.  The SPARQL query language
is a powerful way to make use of such semantic information in making 
complex queries. 

While users should consult a formal introduction to SPARQL for further
background, here we illustrate how SPARQL can be used in combination
with R functions in ways that would be much more tedious to assemble
with only traditional/non-semantic queries.  The SPARQL query language
is provided for the R environment through the `rrdf` package [@Willighagen_2014],
so we start by loading that package.  We will also make use of functions
from `phytools` and `RNeXML`.  

```{r}
library("rrdf")
library("XML")
library("phytools")
library("RNeXML")
```

We read in an example file that contains semantic metadata annotations
describing the taxonomic units (OTUs) used in the tree.  

```{r}
nexml <- nexml_read(system.file("examples/primates.xml", package="RNeXML"))
```

In particular, this example declares the taxon rank, NCBI identifier and parent taxon
for each OTU, such as:

```xml
<otu about="#ou541" id="ou541" label="Alouatta guariba">
      <meta href="http://ncbi.nlm.nih.gov/taxonomy/182256" 
            id="ma20" 
            rel="concept:toTaxon" 
            xsi:type="nex:ResourceMeta"/>
      <meta href="http://rs.tdwg.org/ontology/voc/TaxonRank#Species" 
            id="ma21" 
            rel="concept:rank" 
            xsi:type="nex:ResourceMeta"/>
      <meta href="http://ncbi.nlm.nih.gov/taxonomy/9499" 
            id="ma22" 
            rel="rdfs:subClassOf" 
            xsi:type="nex:ResourceMeta"/>
    </otu>

```

In this example, we will construct a cladogram by using this information
to identify the taxonomic rank of each OTU, and its shared parent
taxonomic rank.  (If this example looks complex, try writing down the
steps to do this without the aid of the SPARQL queries).  These examples
show the manipulation of semantic triples, Unique Resource Identifiers
(URIs) and use of the SPARQL "Join" operator.

Note that this example can be run using `demo("sparql", "RNeXML")` to see the 
code displayed in the R terminal and to avoid character errors that
can occur in having to copy and paste from PDF files.  

We begin by extracting the RDF graph from the NeXML,

```{r}
rdf <- get_rdf(system.file("examples/primates.xml", package="RNeXML"))
tmp <- tempfile()  # so we must write the XML out first
saveXML(rdf, tmp) 
graph <- load.rdf(tmp)
```

We then fetch the NCBI URI for the taxon that has rank 'Order', i.e. the
root of the primates phylogeny. The dot operator `.` between clauses
implies a join, in this case

```{r}
root <- sparql.rdf(graph, 
"SELECT ?uri WHERE { 
    ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#rank> <http://rs.tdwg.org/ontology/voc/TaxonRank#Order> . 
    ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> ?uri    
}")
```


This makes use of the SPARQL query language provided by the `rrdf`
package.  We will also define some helper functions that use SPARQL
queries.  Here we define a function to get the name

```{r}
get_name <- function(id) {
  max <- length(nexml@otus[[1]]@otu)
  for(i in 1:max) {
    if ( nexml@otus[[1]]@otu[[i]]@id == id ) {
      label <- nexml@otus[[1]]@otu[[i]]@label
      label <- gsub(" ","_",label)
      return(label)
    }
  }
}
```


Next, we define a recursive function to build a newick tree from the taxonomic rank information.  

```{r}
recurse <- function(node){
  
    # fetch the taxonomic rank and id string
    rank_query <- paste0(
        "SELECT ?rank ?id WHERE {
            ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> <",node,"> .
            ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#rank> ?rank
          }")
    result <- sparql.rdf(graph, rank_query)
    
    # get the local ID, strip URI part
    id <- result[2]
    id <- gsub("^.+#", "", id, perl = TRUE)
    
    # if rank is terminal, return the name
    if (result[1] == "http://rs.tdwg.org/ontology/voc/TaxonRank#Species") {
        return(get_name(id))
    }
    
    # recurse deeper
    else {
        child_query <- paste0(
            "SELECT ?uri WHERE {
                ?id <http://www.w3.org/2000/01/rdf-schema#subClassOf> <",node,"> .
                ?id <http://rs.tdwg.org/ontology/voc/TaxonConcept#toTaxon> ?uri
            }")
        children <- sparql.rdf(graph, child_query)
        
        return(paste("(", 
                     paste(sapply(children, recurse), 
                           sep = ",", collapse = "," ), 
                     ")",  
                     get_name(id), # label interior nodes
                     sep = "", collapse = ""))
    }
}

```


With these functions in place, it is straight forward to build the tree
from the semantic RDFa data and then visualize it


```{r}
newick <- paste(recurse(root), ";", sep = "", collapse = "")
tree <- read.newick(text = newick)
collapsed <- collapse.singles(tree)
plot(collapsed, 
     type='cladogram', 
     show.tip.label=FALSE, 
     show.node.label=TRUE, 
     cex=0.75, 
     edge.color='grey60', 
     label.offset=-9)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_metadata.R
\name{get_metadata}
\alias{get_metadata}
\title{get_metadata}
\usage{
get_metadata(nexml, level = "nexml", simplify = TRUE)
}
\arguments{
\item{nexml}{a nexml object}

\item{level}{the name of the level of element desired, see details}

\item{simplify}{logical, see Details}
}
\value{
the requested metadata as a data.frame. Additional columns
indicate the parent element of the return value.
}
\description{
get_metadata
}
\details{
'level' should be either the name of a child element of a NeXML document
(e.g. "otu", "characters"), or a path to the desired element, e.g. 'trees/tree'
will return the metadata for all phylogenies in all trees blocks.

If a metadata element has other metadata elements nested within it, the
nested metadata are returned as well. A column "Meta" will contain the
IDs consolidated from the type-specific LiteralMeta and ResourceMeta
columns, and IDs are generated for meta elements that have nested elements
but do not have an ID ("blank nodes"). A column "meta" contains the
IDs of the parent meta elements for nested ones. This means that the
resulting table can be self-joined on those columns.

If \code{simplify} is \code{FALSE}, the type-specific "LiteralMeta" and "ResourceMeta"
columns will be retained even if a consolidated "Meta" column is present.
Otherwise, only the consolidated column will be included in the result.
Also, if \code{simplify} is \code{TRUE} the values for "property" (LiteralMeta) and
"rel" (ResourceMeta) will be consolidated to "property", and "rel" will be
removed from the result.
}
\examples{
\dontrun{
comp_analysis <- system.file("examples", "primates.xml", package="RNeXML")
nex <- nexml_read(comp_analysis)
get_metadata(nex)
get_metadata(nex, "otus/otu")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nexml_publish.R
\name{nexml_publish}
\alias{nexml_publish}
\title{publish nexml files to the web and receive a DOI}
\usage{
nexml_publish(nexml, ..., repository = "figshare")
}
\arguments{
\item{nexml}{a nexml object (or file path)}

\item{...}{additional arguments, depending on repository. See examples.}

\item{repository}{destination repository}
}
\value{
a digital object identifier to the published data
}
\description{
publish nexml files to the web and receive a DOI
}
\examples{
\dontrun{
data(bird.orders)
birds <- add_trees(bird.orders)
doi <- nexml_publish(birds, visibility = "public", repository="figshare")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_metadata.R
\name{get_meta}
\alias{get_meta}
\title{Extracts meta objects matching properties}
\usage{
get_meta(nexml, annotated = NULL, props)
}
\arguments{
\item{nexml}{a nexml object}

\item{annotated}{the nexml component object from which to obtain metadata
annotations, or a list of such objects. Defaults to the nexml object itself.}

\item{props}{a character vector of property names for which to extract
metadata annotations}
}
\value{
a named list of the matching meta objects
}
\description{
Extracts the metadata annotations for the given property or properties,
and returns the result as a list of \code{meta} objects.
}
\details{
For matching property identifiers (i.e., URIs), prefixes in the input list
as well as in the \code{annotated} object will be expanded using the namespaces
of the \code{nexml} object. Names in the returned list are mapped to the
(possibly prefixed) form in the input list. The resulting list is flat,
and hence does not retain the nesting hierarchy in the object's annotation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{charzero_as_empty}
\alias{charzero_as_empty}
\title{Treats zero-length character vectors as empty strings}
\usage{
charzero_as_empty(x)
}
\arguments{
\item{x}{the object to be tested for zero-length character vector}
}
\value{
an empty string if \code{x} is a character vector of length zero, and \code{x}
otherwise
}
\description{
If the argument is a zero-length character vector (character(0)), returns
an empty string (which is a character vector of length 1). Otherwise passes
through the argument.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_basic_metadata.R
\name{get_license}
\alias{get_license}
\title{get_license}
\usage{
get_license(nexml)
}
\arguments{
\item{nexml}{a nexml object}
}
\value{
the list of taxa
}
\description{
get_license
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simmap.R
\name{simmap_to_nexml}
\alias{simmap_to_nexml}
\alias{nexml_to_simmap}
\title{Convert phylo with attached simmap to nexml object}
\usage{
simmap_to_nexml(phy, state_ids = NULL)

nexml_to_simmap(nexml)
}
\arguments{
\item{phy}{a phylo object containing simmap \code{phy$maps} element,
from the phytools package}

\item{state_ids}{a named character vector giving the state
names corresponding to the ids used to refer to each state
in nexml.  If null ids will be generated and states taken from
the phy$states names.}

\item{nexml}{a nexml object}
}
\value{
a nexml representation of the simmap

a simmap object (phylo object with a \verb{$maps} element
for use in phytools functions).
}
\description{
Convert phylo with attached simmap to nexml object

Convert nexml object with simmap to phylo
}
\section{Functions}{
\itemize{
\item \code{nexml_to_simmap()}: Convert nexml object with simmap to phylo

}}
\examples{
simmap_ex <- read.nexml(system.file("examples","simmap_ex.xml", package="RNeXML"))
phy <- nexml_to_simmap(simmap_ex)
nex <- simmap_to_nexml(phy) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_metadata.R
\name{get_metadata_values}
\alias{get_metadata_values}
\title{Get the value(s) for metadata}
\usage{
get_metadata_values(nexml, annotated = NULL, props)
}
\arguments{
\item{nexml}{a nexml object}

\item{annotated}{the nexml component object from which to obtain metadata
annotations, defaults to the nexml object itself}

\item{props}{a character vector of property names for which to extract
metadata annotations}
}
\value{
a named character vector, giving the values and names being the
property names
}
\description{
Extracts the values from the metadata annotations for the given property
or properties, and returns the result.
}
\details{
For matching property identifiers (i.e., URIs), prefixes in the input list
as well as in the \code{annotated} object will be expanded using the namespaces
of the \code{nexml} object. Names in the returned vector are mapped to the
(possibly prefixed) form in the input list.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4-utils.R
\name{.methodWithNext}
\alias{.methodWithNext}
\title{Saves the next method in the method meta data}
\usage{
.methodWithNext(method, nextMethod, .cache = FALSE)
}
\arguments{
\item{method}{the \code{MethodDefinition} object to promote}

\item{nextMethod}{the \code{MethodDefinition}
object to record as the next method}

\item{.cache}{whether to cache the promoted method definition object
(using \code{methods::cacheMethod()})}
}
\value{
an instance of \code{MethodWithNext},
which has the next method in the \code{nextMethod} slot
}
\description{
Promotes the given method definition to an instance of
\code{MethodWithNext}, thereby recording the next
method in the \code{nextMethod} slot.
}
\note{
\code{MethodWithNext} objects are normally returned by
\code{methods::addNextMethod()}, but a constructor function for the class
seems missing (or is undocumented?). This provides one.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prefixed-uris.R
\name{expand_prefix}
\alias{expand_prefix}
\title{Expand namespace-prefixed string}
\usage{
expand_prefix(x, namespaces = NULL)
}
\arguments{
\item{x}{a character vector of potentially namespace-prefixed strings}

\item{namespaces}{a named vector of namespaces, with namespace prefixes
being the names. A "base" namespace with an empty name can be included.
If not provided, or if empty, the input vector is returned as is.}
}
\value{
a character vector, of the same length as the input vector
}
\description{
Substitutes the namespace prefix in the input vector of strings with
the corresponding namespaces.
}
\details{
Namespace prefixes are expected to be separated by one or more semicolons.
Prefixes that cannot be matched to the vector of namespaces will be left
as is. For strings that do not have a namespace prefix, the vector of
namespaces can contain a base namespace, identified as not having a name,
with which these strings will be expanded.
}
\examples{
uris <- c("cc:license", "dc:title")
ns <- c(dc = "http://purl.org/dc/elements/1.1/",
        dcterms = "http://purl.org/dc/terms/",
        dct = "http://purl.org/dc/terms/",
        cc = "http://creativecommons.org/ns#")
# expansion is vectorized
expand_prefix(uris, ns)

# strings with non-matching or no prefix are left as is
uris <- c(uris, "my:title", "title")
expand_prefix(uris, ns)

# NAs in the input list become NA in the output
uris <- c(uris, NA)
expand_prefix(uris, ns)

# can include a "base" (unnamed) namespace for expanding unprefixed strings
ns <- c(ns, "//local/")
xuris <- expand_prefix(uris, ns)
xuris
xuris[uris == "title"] == paste0("//local/", uris[uris == "title"])

# different prefixes may expand to the same result
expand_prefix("dcterms:modified", ns) == expand_prefix("dct:modified", ns)

# or they may result in different expansions
expand_prefix("dc:title", ns) != expand_prefix("dcterms:title", ns)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4-utils.R
\name{findNextMethod}
\alias{findNextMethod}
\title{Finds the method that callNextMethod() should chain to}
\usage{
findNextMethod(method, f = NULL, envir = topenv())
}
\arguments{
\item{method}{\code{MethodDefinition}, the method for which to find
the next method}

\item{f}{\code{standardGeneric}, the standard generic for which to find
the next method. By default this will be obtained from \code{method}.}

\item{envir}{the environment in which to find the method}
}
\value{
a \code{MethodDefinition} object that is the next method in the
chain by inheritance
}
\description{
Attempts to find the "next" method in the inheritance chain. This would
(ideally) be the method that \code{methods::callNextMethod()} would chain to,
as a result of the method \code{methods::addNextMethod()} would find (and return
in the \code{nextMethod} slot of the \code{MethodWithNext}
object). Hence, in theory one shouldn't ever need this, but unfortunately
\code{addNextMethod()} is broken (and errors out) if one of the classes in the
signature name-clashes with an S4 class defined in another package that is
loaded.
}
\details{
The next method will be determined by the S4 inheritance chain. However,
this function will walk only the inheritance chain of those arguments in
the signature that are defined in the package of the generic method from
which this function was invoked (directly or indirectly). If there are
no such parameters in the signature, or if there is more than one,
finding the next method is handed off to \code{methods::addNextMethod()}.
}
\note{
In theory a class name clash between packages shouldn't be a problem
because class names can be namespaced, and the \code{MethodDefinition}
object passed to \code{addNextMethod()} has all the necessary namespace
information. Hopefully, at some point this gets fixed in R, and then we
don't need this anymore.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_taxa.R
\name{get_taxa_list}
\alias{get_otus_list}
\alias{get_taxa_list}
\title{get_taxa_list}
\usage{
get_taxa_list(nexml)
}
\arguments{
\item{nexml}{a nexml object}
}
\value{
the list of taxa
}
\description{
Retrieve names of all species/otus otus (operational taxonomic units) included in the nexml
}
\seealso{
\code{\link{get_item}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_trees.R
\name{get_trees}
\alias{get_trees}
\title{extract a phylogenetic tree from the nexml}
\usage{
get_trees(nexml)
}
\arguments{
\item{nexml}{a representation of the nexml object from
which the data is to be retrieved}
}
\value{
an ape::phylo tree, if only one tree is represented.
Otherwise returns a list of lists of multiphylo trees.
To consistently receive the list of lists format (preserving
the hierarchical nature of the nexml), use \code{\link{get_trees_list}} instead.
}
\description{
extract a phylogenetic tree from the nexml
}
\examples{
comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
nex <- nexml_read(comp_analysis)
get_trees(nex)
}
\seealso{
\code{\link{get_trees}} \code{\link{get_flat_trees}} \code{\link{get_item}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_namespaces.R
\name{get_namespaces}
\alias{get_namespaces}
\title{get namespaces}
\usage{
get_namespaces(nexml)
}
\arguments{
\item{nexml}{a nexml object}
}
\value{
a named character vector providing the URLs defining each
of the namespaces used in the nexml file.  Names correspond to
the prefix abbreviations of the namespaces.
}
\description{
get namespaces
}
\examples{
comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
nex <- nexml_read(comp_analysis)
get_namespaces(nex)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_namespaces.R
\name{add_namespaces}
\alias{add_namespaces}
\title{Add namespaces}
\usage{
add_namespaces(namespaces, nexml = new("nexml"))
}
\arguments{
\item{namespaces}{a named character vector of namespaces}

\item{nexml}{a nexml object. will create a new one if none is given.}
}
\value{
a nexml object with updated namespaces
}
\description{
Add namespaces and their prefixes as a named vector of URIs, with the
names being the prefixes. Namespaces have most relevance for meta objects'
\code{rel} and \code{property}, and for embedded XML literals.
}
\details{
The implementation attempts to avoid duplication, currently using the
prefix. I.e., namespaces with prefixes already defined will not get added.
Namespaces needed by the NeXML format, and for commonly used metadata
terms, are already included by default, see \code{\link[=get_namespaces]{get_namespaces()}}.
}
\note{
Often a user won't call this directly, but instead provide the
namespace(s) through \code{\link[=add_meta]{add_meta()}}.
}
\examples{
## Write multiple metadata elements, including a new namespace:  
website <- meta(href = "http://carlboettiger.info", 
                rel = "foaf:homepage")     # meta can be link-style metadata
modified <- meta(property = "prism:modificationDate",
                 content = "2013-10-04")
nex <- add_meta(list(modified,  website), 
                namespaces = c(foaf = "http://xmlns.com/foaf/0.1/"))
                # prism prefix already included by default

## Add namespace "by hand" before adding meta:
nex <- add_namespaces(c(skos = "http://www.w3.org/2004/02/skos/core#"),
                      nexml = nex)
history <- meta(property = "skos:historyNote",
                content = "Mapped from the bird.orders data in the ape package using RNeXML")
nex <- add_meta(history, nexml = nex)

}
\seealso{
\code{\link[=meta]{meta()}} \code{\link[=add_meta]{add_meta()}} \code{\link[=get_namespaces]{get_namespaces()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_characters.R
\name{get_characters}
\alias{get_characters}
\title{Get character data.frame from nexml}
\usage{
get_characters(
  nex,
  rownames_as_col = FALSE,
  otu_id = FALSE,
  otus_id = FALSE,
  include_state_types = FALSE
)
}
\arguments{
\item{nex}{a nexml object}

\item{rownames_as_col}{option to return character matrix rownames (with taxon ids) as it's own column in the
data.frame. Default is FALSE for compatibility with geiger and similar packages.}

\item{otu_id}{logical, default FALSE. return a column with the
otu id (for joining with otu metadata, etc)}

\item{otus_id}{logical, default FALSE. return a column with the
otus block id (for joining with otu metadata, etc)}

\item{include_state_types}{logical, default FALSE. whether to also return a
matrix of state types (with values standard, polymorphic, and uncertain)}
}
\value{
the character matrix as a data.frame, or if \code{include_state_types} is
TRUE a list of two elements, \code{characters} as the character matrix, and
\code{state_types} as a matrix of state types. Both matrices will be in the same
ordering of rows and columns.
}
\description{
Get character data.frame from nexml
}
\details{
RNeXML will attempt to return the matrix using the NeXML taxon (otu) labels to name the rows
and the NeXML char labels to name the traits (columns).  If these are unavailable or not unique, the NeXML
id values for the otus or traits will be used instead.
}
\examples{
\dontrun{
# A simple example with a discrete and a continous trait
f <- system.file("examples", "comp_analysis.xml", package="RNeXML")
nex <- read.nexml(f)
get_characters(nex)

# A more complex example -- currently ignores sequence-type characters
f <- system.file("examples", "characters.xml", package="RNeXML")
nex <- read.nexml(f)
get_characters(nex)

# if polymorphic or uncertain states need special treatment, request state
# types to be returned as well:
f <- system.file("examples", "ontotrace-result.xml", package="RNeXML")
nex <- read.nexml(f)
res <- get_characters(nex, include_state_types = TRUE)
row.has.p <- apply(res$state_types, 1, 
                   function(x) any(x == "polymorphic", na.rm = TRUE))
col.has.p <- apply(res$state_types, 2, 
                   function(x) any(x == "polymorphic", na.rm = TRUE))
res$characters[row.has.p, col.has.p, drop=FALSE] # polymorphic rows and cols
res$characters[!row.has.p, drop=FALSE] # drop taxa with polymorphic states
# replace polymorphic state symbols in matrix with '?'
m1 <- mapply(function(s, s.t) ifelse(s.t == "standard", s, "?"), 
             res$characters, res$state_types)
row.names(m1) <- row.names(res$characters)
m1
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nexml_methods.R
\name{summary,nexml-method}
\alias{summary,nexml-method}
\alias{summary.nexml}
\title{Summary method for nexml objects}
\usage{
\S4method{summary}{nexml}(object)
}
\arguments{
\item{object}{the \link[=nexml-class]{nexml} object}
}
\value{
A list with the following elements:
\itemize{
\item \code{nblocks} the number of trees, otus, and characters blocks
\item \code{ncharacters} the number of characters in each characters block
\item \code{nstates} summary statistics of the number of character states per state set
defined for each characters block
\item \code{nnonstdstatedefs} the number of polymorphic and uncertain states defined
for each character block
\item \code{nmatrixrows} the number of rows in the matrix for each character block
\item \code{ntrees} the number of trees contained in each trees block
\item \code{notus} the number of OTUs defined in each OTUs block
\item \code{nmeta} a list of the number of the number of metadata annotations at
several levels, specifically:
\itemize{
\item \code{nexml} at the top (nexml) level
\item \code{otu} at the OTU level, for each OTUs block
\item \code{char} at the character level, for each characters block
\item \code{state} at the character state level, for each characters block
}
}
}
\description{
Generates a list of various counts of the major elements that comprise a
\link[=nexml-class]{nexml} object, such as number of different kinds of blocks,
characters, states, OTUs (taxa), etc.
}
\details{
The \link[methods:show]{show} method uses this summary for pretty-printing a
summary of the NeXML object, but it can be used on its own as well, in
particular for quick inspection of key properties of a NeXML file.
}
\examples{
nex <- nexml_read(system.file("examples", "comp_analysis.xml", package = "RNeXML"))
s <- summary(nex)
# number of major blocks:
s$nblocks

# each characters block defines 1 character:
s$ncharacters

# summary stats of states per character (for morphological matrices there is
# typically one state set per character)
s$nstates # note that first block is of continuous type, so no stats there

# pretty-printed summary:
nex # this is the same as show(nex)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concatenate_nexml.R
\name{c,nexml-method}
\alias{c,nexml-method}
\title{Concatenate nexml files}
\usage{
\S4method{c}{nexml}(x, ..., recursive = FALSE)
}
\arguments{
\item{x, ...}{nexml objects to be concatenated, e.g. from
\code{\link{write.nexml}} or \code{\link{read.nexml}}.
Must have unique ids on all elements}

\item{recursive}{logical.  If 'recursive = TRUE', the function recursively
descends through lists (and pairlists) combining all their
elements into a vector. (Not implemented).}
}
\value{
a concatenated nexml file
}
\description{
Concatenate nexml files
}
\examples{
\dontrun{
f1 <- system.file("examples", "trees.xml", package="RNeXML")
f2 <- system.file("examples", "comp_analysis.xml", package="RNeXML")
nex1 <- read.nexml(f1)
nex2 <- read.nexml(f2)
nex <- c(nex1, nex2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_rdf.R
\name{get_rdf}
\alias{get_rdf}
\title{Extract rdf-xml from a NeXML file}
\usage{
get_rdf(file)
}
\arguments{
\item{file}{the name of a nexml file, or otherwise a nexml object.}
}
\value{
an RDF-XML object (XMLInternalDocument).  This can be manipulated with
tools from the XML R package, or converted into a triplestore for use with
SPARQL queries from the rdflib R package.
}
\description{
Extract rdf-xml from a NeXML file
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Annotated-class}
\alias{Annotated-class}
\title{Class of objects that have metadata as lists of meta elements}
\description{
Class of objects that have metadata as lists of meta elements
}
\section{Slots}{

\describe{
\item{\code{meta}}{list of \code{meta} objects}

\item{\code{about}}{for RDF extraction, the identifier for the resource that this
object is about}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_basic_metadata.R
\name{get_citation}
\alias{get_citation}
\title{Get citation from metadata}
\usage{
get_citation(nexml)
}
\arguments{
\item{nexml}{a nexml object}
}
\value{
the citation if the metadata provides one that is non-empty, and
NA otherwise. If multiple non-empty annotations are found, only the first
one is returned.
}
\description{
Extracts the citation annotation from the metadata annotation of the\code{nexml}
object, and returns its value.
}
\details{
Currently the implementation looks for \code{dcterms:bibliographicCitation}
annotations. (Note that these may be given with any prefix in the metadata
so long as they expand to the same full property URIs.)
}
\seealso{
\code{\link[=get_metadata_values]{get_metadata_values()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_taxa_meta.R
\name{get_taxa_meta}
\alias{get_taxa_meta}
\title{get_taxa_meta}
\usage{
get_taxa_meta(nexml, what = "href")
}
\arguments{
\item{nexml}{a nexml object}

\item{what}{One of href, rel, id, or xsi:type}
}
\value{
the list of metadata for each taxon
}
\description{
Retrieve metadata of all species/otus otus (operational taxonomic units) included in the nexml
}
\examples{
\dontrun{
data(bird.orders)
birds <- add_trees(bird.orders)
birds <- taxize_nexml(birds, "NCBI")
RNeXML:::get_taxa_meta(birds)
RNeXML:::get_taxa_meta(birds, 'rel')
RNeXML:::get_taxa_meta(birds, 'id')
RNeXML:::get_taxa_meta(birds, 'xsi:type')
 }
}
\seealso{
\code{\link{get_item}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta.R
\docType{methods}
\name{c,ListOfmeta-method}
\alias{c,ListOfmeta-method}
\title{Concatenate ListOfmeta elements into a ListOfmeta}
\usage{
\S4method{c}{ListOfmeta}(x, ..., recursive = FALSE)
}
\arguments{
\item{x, ...}{meta or ListOfmeta elements to be concatenated, e.g. see \code{\link{meta}}}

\item{recursive}{logical, if 'recursive=TRUE', the function 
descends through lists and combines their elements into a vector.}
}
\value{
a listOfmeta object containing multiple meta elements.
}
\description{
Concatenate ListOfmeta elements into a ListOfmeta
}
\examples{
metalist <- c(meta(content="example", property="dc:title"),
              meta(content="Carl", property="dc:creator"))
out <- c(metalist, metalist) 
out <- c(metalist, meta(content="a", property="b")) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nexml_read.R
\name{nexml_read}
\alias{nexml_read}
\alias{read.nexml}
\alias{nexml_read.character}
\alias{nexml_read.XMLInternalDocument}
\alias{nexml_read.XMLInternalNode}
\title{Read NeXML files into various R formats}
\usage{
nexml_read(x, ...)

\method{nexml_read}{character}(x, ...)

\method{nexml_read}{XMLInternalDocument}(x, ...)

\method{nexml_read}{XMLInternalNode}(x, ...)
}
\arguments{
\item{x}{Path to the file to be read in. An \code{XML::XMLDocument-class}
or \code{\link[XML]{XMLNode-class}}}

\item{...}{Further arguments passed on to \code{\link[XML]{xmlTreeParse}}}
}
\description{
Read NeXML files into various R formats
}
\examples{
# file
f <- system.file("examples", "trees.xml", package="RNeXML")
nexml_read(f)
\dontrun{ # may take > 5 s
# url
url <- "https://raw.githubusercontent.com/ropensci/RNeXML/master/inst/examples/trees.xml"
nexml_read(url)
# character string of XML
str <- paste0(readLines(f), collapse = "")
nexml_read(str)
# XMLInternalDocument
library("httr")
library("XML")
x <- xmlParse(content(GET(url)))
nexml_read(x)
# XMLInternalNode
nexml_read(xmlRoot(x))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nexml_validate.R
\name{nexml_validate}
\alias{nexml_validate}
\title{validate nexml using the online validator tool}
\usage{
nexml_validate(
  file,
  schema = system.file("xsd/nexml.xsd", package = "RNeXML"),
  local = TRUE
)
}
\arguments{
\item{file}{path to the nexml file to validate}

\item{schema}{URL of schema (for fallback method only, set by default).}

\item{local}{logical, if TRUE we skip the online validator and rely only on pure XML-schema validation.  This may fail to detect invalid use of some semantic elements.}
}
\value{
TRUE if the file is valid, FALSE or error message otherwise
}
\description{
validate nexml using the online validator tool
}
\details{
Requires an internet connection if local=FALSE.  see http://www.nexml.org/nexml/phylows/validator for more information in debugging invalid files
}
\examples{
\dontrun{
data(bird.orders)
birds <- nexml_write(bird.orders, "birds_orders.xml")
nexml_validate("birds_orders.xml")
unlink("birds_orders.xml") # delete file to clean up
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simmap.R
\docType{data}
\name{simmap_ex}
\alias{simmap_ex}
\title{A nexml class R object that includes simmap annotations}
\format{a \code{nexml} instance}
\source{
Simulated tree and stochastic character mapping based on 
Revell 2011 (doi:10.1111/j.2041-210X.2011.00169.x)
}
\usage{
simmap_ex
}
\description{
A nexml object with simmap stochastic character mapping
 annotations added to the edges, for use with the RNeXML package
 parsing and serializing NeXML into formats that work with the ape and
 phytools packages.
}
\author{
Carl Boettiger
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_taxa.R
\name{get_taxa}
\alias{get_taxa}
\alias{get_otu}
\title{get_taxa}
\usage{
get_taxa(nexml)
}
\arguments{
\item{nexml}{a nexml object}
}
\value{
the list of taxa
}
\description{
Retrieve names of all species/otus otus (operational taxonomic units) included in the nexml
}
\examples{
comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
nex <- nexml_read(comp_analysis)
get_taxa(nex)
}
\seealso{
\code{\link{get_item}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_metadata.R
\name{get_all_meta}
\alias{get_all_meta}
\title{Get flattened list of meta annotations}
\usage{
get_all_meta(annotated)
}
\arguments{
\item{annotated}{the object from which to extract meta object annotations}
}
\value{
a flat list of \code{meta} objects
}
\description{
Collects recursively (in the case of nested meta annotations) all meta
object annotations for the given object, and returns the result as a flat
list.
}
\details{
Does not check that the input object can actually have meta annotations.
An invalid slot error will be generated if it can't.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constructors.R
\name{New}
\alias{New}
\title{new with namespaced class name}
\usage{
New(Class, ...)
}
\arguments{
\item{Class}{the name of the S4 class to be instantiated}

\item{...}{additional parameters for \code{methods::new()}}
}
\description{
Convenience function for \code{\link[methods:new]{methods::new()}} that ensures that the provided
class name is namespaced with a package name.
}
\details{
If the provided class name is not already namespaced (see
\code{methods::packageSlot()}), it will be namespaced with this package. This
mechanism is used by \code{new()} to disambiguate if the class name clashes
with a class defined in another package.
}
\note{
This may not completely eliminate messages on standard error about
classes with the same name having been found in different packages. If
they appear, they will most likely have come from the call to the
\code{methods::initialize()} generic that \code{new()} issues at the end.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{nexml-class}
\alias{nexml-class}
\alias{nexml}
\title{Class representing a NeXML document}
\description{
The \code{nexml} class represents a NeXML document, and is the top of the
class hierarchy defined in this package, corresponding to the root node
of the corresponding XML document.
}
\details{
Normally objects of this type are created by the package as a result of
reading a NeXML file, or of converting from another type, such as
\code{ape::phylo}. Also, interacting directly with the slots of the class is
normally not necessary. Instead, use the \code{get_XXX()} and \code{add_XXX()}
functions in the API.
}
\section{Slots}{

\describe{
\item{\code{trees}}{list, corresponding to the list of \verb{<trees/>} elements in
NeXML. Elements will be of class \code{trees}.}

\item{\code{characters}}{list, corresponding to the list of \verb{<characters/>}
elements in NeXML. Elements will be of class \code{characters}.}

\item{\code{otus}}{list, corresponding to the list of \verb{<otus/>} elements in NeXML.
Elements will be of class \code{otus}.}

\item{\code{about}}{inherited, see \link[=Annotated-class]{Annotated}}

\item{\code{meta}}{inherited, see \link[=Annotated-class]{Annotated}}

\item{\code{xsi:type}}{for internal use}

\item{\code{version}}{NeXML schema version, do not change}

\item{\code{generator}}{name of software generating the XML}

\item{\code{xsi:schemaLocation}}{for internal use, do not change}

\item{\code{namespaces}}{named character vector giving the XML namespaces}
}}

\examples{
nex <- nexml() # a nexml object with no further content
nex <- new("nexml") # accomplishes the same thing
nex@generator
length(nex@trees)

data(bird.orders)
nex <- as(bird.orders, "nexml")
summary(nex)
length(nex@trees)
}
\seealso{
\code{\link[=read.nexml]{read.nexml()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nexml_add.R
\name{nexml_add}
\alias{nexml_add}
\title{add elements to a new or existing nexml object}
\usage{
nexml_add(
  x,
  nexml = new("nexml"),
  type = c("trees", "characters", "meta", "namespaces"),
  ...
)
}
\arguments{
\item{x}{the object to be added}

\item{nexml}{an existing nexml object onto which the object should be appended}

\item{type}{the type of object being provided.}

\item{...}{additional optional arguments to the add functions}
}
\value{
a nexml object with the additional data
}
\description{
add elements to a new or existing nexml object
}
\seealso{
\code{\link{add_trees}} \code{\link{add_characters}} \code{\link{add_meta}} \code{\link{add_namespaces}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nexml_write.R
\name{nexml_write}
\alias{nexml_write}
\alias{write.nexml}
\title{Write nexml files}
\usage{
nexml_write(
  x = nexml(),
  file = NULL,
  trees = NULL,
  characters = NULL,
  meta = NULL,
  ...
)
}
\arguments{
\item{x}{a nexml object, or any phylogeny object (e.g. phylo, phylo4)
that can be coerced into one. Can also be omitted, in which case a new
nexml object will be constructed with the additional parameters specified.}

\item{file}{the name of the file to write out}

\item{trees}{phylogenetic trees to add to the nexml file (if not already given in x)
see \code{\link{add_trees}} for details.}

\item{characters}{additional characters}

\item{meta}{A meta element or list of meta elements, see \code{\link{add_meta}}}

\item{...}{additional arguments to add__basic_meta, such as the title.  See \code{\link{add_basic_meta}}.}
}
\value{
Writes out a nexml file
}
\description{
Write nexml files
}
\examples{
 ## Write an ape tree to nexml, analgous to write.nexus:
 library(ape); data(bird.orders)
 ex <- tempfile(fileext=".xml")
 write.nexml(bird.orders, file=ex)
}
\seealso{
\code{\link{add_trees}} \code{\link{add_characters}} \code{\link{add_meta}} \code{\link{nexml_read}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simmap.R
\name{nexml_to_simmap}
\alias{nexml_to_simmap}
\title{nexml_to_simmap}
\usage{
nexml_to_simmap(nexml)
}
\arguments{
\item{nexml}{a nexml object}
}
\value{
a simmap object (phylo object with a $maps element 
 for use in phytools functions).
}
\description{
nexml_to_simmap
}
\examples{
data(simmap_ex)
phy <- nexml_to_simmap(simmap_ex)
nex <- simmap_to_nexml(phy) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4-utils.R
\name{.callGeneric}
\alias{.callGeneric}
\title{Calls the given generic with the given arguments}
\usage{
.callGeneric(f, ..., .package = NULL)
}
\arguments{
\item{f}{the generic, as a character string or a \code{standardGeneric}
object}

\item{...}{the arguments (named and/or unnamed) with which to call the
matching method}

\item{.package}{the package name for finding the generic (if \code{f} is a character
string); by default the package is determined from the calling environment}
}
\value{
the value returned by the method
}
\description{
Calls the given generic with the given arguments, using the method
whose signature matches the arguments.
}
\details{
Uses \code{methods::selectMethod()} to find the matching method. In theory,
this is at best wholly redundant with what standard S4 generics already
do by themselves. However, the generics dispatch for S4 seems (at least
currently) broken at least if the first argument in the signature is
a class that name-clashes with a class defined in another package. In
that case, whether the standard dispatch works correctly or not can depend
on \code{search()} order, and can change within a session
depending on the order in which packages are loaded.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constructors.R, R/meta.R
\name{nexml.tree}
\alias{nexml.tree}
\alias{nexml.trees}
\alias{nexml.node}
\alias{nexml.edge}
\alias{nexml.otu}
\alias{nexml.otus}
\alias{nexml.char}
\alias{nexml.characters}
\alias{nexml.format}
\alias{nexml.state}
\alias{nexml.uncertain_state}
\alias{nexml.states}
\alias{nexml.uncertain_states}
\alias{nexml.polymorphic_states}
\alias{nexml.member}
\alias{nexml.matrix}
\alias{nexml.row}
\alias{nexml.seq}
\alias{nexml.cell}
\alias{nexml.meta_}
\title{Constructor for the respective class}
\usage{
nexml.tree(...)

nexml.trees(...)

nexml.node(...)

nexml.edge(...)

nexml.otu(...)

nexml.otus(...)

nexml.char(...)

nexml.characters(...)

nexml.format(...)

nexml.state(...)

nexml.uncertain_state(...)

nexml.states(...)

nexml.uncertain_states(...)

nexml.polymorphic_states(...)

nexml.member(...)

nexml.matrix(...)

nexml.row(...)

nexml.seq(...)

nexml.cell(...)
}
\arguments{
\item{...}{optionally, parameters passed on to \code{\link[methods:new]{new()}}}
}
\description{
Creates an instance of the class corresponding to the respective NeXML
element, and initializes its slots with the provided parameters, if any.
}
\details{
Usually, users won't need to invoke this directly.
}
\seealso{
\link[=meta]{nexml.meta()} for documentation of \code{nexml.meta()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_trees.R
\name{toPhylo}
\alias{toPhylo}
\title{nexml to phylo}
\usage{
toPhylo(tree, otus)
}
\arguments{
\item{tree}{an nexml tree element}

\item{otus}{a character string of taxonomic labels, named by the otu ids.
e.g. (from get_otu_maps for the otus set matching the relevant trees node.}
}
\value{
phylo object.  If a "reconstructions" annotation is found on the
edges, return simmap maps slot as well.
}
\description{
nexml to phylo coercion
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta.R
\docType{methods}
\name{c,meta-method}
\alias{c,meta-method}
\title{Concatenate meta elements into a ListOfmeta}
\usage{
\S4method{c}{meta}(x, ..., recursive = FALSE)
}
\arguments{
\item{x, ...}{meta elements to be concatenated, e.g. see \code{\link{meta}}}

\item{recursive}{logical, if 'recursive=TRUE', the function 
descends through lists and combines their elements into a vector.}
}
\value{
a listOfmeta object containing multiple meta elements.
}
\description{
Concatenate meta elements into a ListOfmeta
}
\examples{
c(meta(content="example", property="dc:title"),
  meta(content="Carl", property="dc:creator"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal_nexml_id.R
\name{reset_id_counter}
\alias{reset_id_counter}
\title{reset id counter}
\usage{
reset_id_counter()
}
\description{
reset the id counter
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta.R
\name{meta}
\alias{meta}
\alias{nexml.meta}
\title{Constructor function for metadata nodes}
\usage{
meta(
  property = NULL,
  content = NULL,
  rel = NULL,
  href = NULL,
  datatype = NULL,
  id = NULL,
  type = NULL,
  children = list()
)
}
\arguments{
\item{property}{specify the ontological definition together with it's namespace, e.g. dc:title}

\item{content}{content of the metadata field}

\item{rel}{Ontological definition of the reference provided in href}

\item{href}{A link to some reference}

\item{datatype}{optional RDFa field}

\item{id}{optional id element (otherwise id will be automatically generated).}

\item{type}{optional xsi:type.  If not given, will use either "LiteralMeta" or "ResourceMeta" as
determined by the presence of either a property or a href value.}

\item{children}{Optional element containing any valid XML block (XMLInternalElementNode class, see the XML package for details).}
}
\description{
Constructor function for metadata nodes
}
\details{
User must either provide property+content or rel+href.  Mixing these will result in potential garbage.
The datatype attribute will be detected automatically from the class of the content argument.  Maps from R class
to schema datatypes are as follows:
character - xs:string,
Date - xs:date,
integer - xs:integer,
numeric - xs:decimal,
logical - xs:boolean
}
\examples{
meta(content="example", property="dc:title")
}
\seealso{
\code{\link{nexml_write}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_meta.R
\name{add_meta}
\alias{add_meta}
\title{Add metadata to a nexml file}
\usage{
add_meta(
  meta,
  nexml = new("nexml"),
  level = c("nexml", "otus", "trees", "characters"),
  namespaces = NULL,
  i = 1,
  at_id = NULL
)
}
\arguments{
\item{meta}{a meta S4 object, e.g. ouput of the function \code{\link{meta}}, or a list of these meta objects}

\item{nexml}{(S4) object}

\item{level}{the level at which the metadata annotation should be added.}

\item{namespaces}{named character string for any additional namespaces that should be defined.}

\item{i}{for otus, trees, characters: if there are multiple such blocks, which one should be annotated?  Default is first/only block.}

\item{at_id}{the id of the element to be annotated.  Optional, advanced use only.}
}
\value{
the updated nexml object
}
\description{
Add metadata to a nexml file
}
\examples{
## Create a new nexml object with a single metadata element: 
modified <- meta(property = "prism:modificationDate", content = "2013-10-04")
nex <- add_meta(modified) # Note: 'prism' is defined in nexml_namespaces by default.  

## Write multiple metadata elements, including a new namespace:  
website <- meta(href = "http://carlboettiger.info", 
                rel = "foaf:homepage")              # meta can be link-style metadata
nex <- add_meta(list(modified,  website), 
                namespaces = c(foaf = "http://xmlns.com/foaf/0.1/"))

## Append more metadata, and specify a level: 
history <- meta(property = "skos:historyNote",
                 content = "Mapped from the bird.orders data in the ape package using RNeXML")
data(bird.orders)
nex <- add_trees(bird.orders) # need to have created a trees block first
nex <- add_meta(history, 
                nexml = nex,
                level = "trees",
                namespaces = c(skos = "http://www.w3.org/2004/02/skos/core#"))

}
\seealso{
\code{\link{meta}} \code{\link{add_trees}} \code{\link{add_characters}} \code{\link{add_basic_meta}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxize_nexml.R
\name{taxize_nexml}
\alias{taxize_nexml}
\title{taxize nexml}
\usage{
taxize_nexml(
  nexml,
  type = c("ncbi", "itis", "col", "tpl", "gbif", "wd"),
  warnings = TRUE,
  ...
)
}
\arguments{
\item{nexml}{a nexml object}

\item{type}{the name of the identifier to use}

\item{warnings}{should we show warning messages if no match can be found?}

\item{...}{additional arguments to \verb{[taxadb::get_ids()]}}
}
\description{
Check taxonomic names against the specified service and
add appropriate semantic metadata to the nexml OTU unit
containing the corresponding identifier.
}
\examples{
\dontrun{
data(bird.orders)
birds <- add_trees(bird.orders)
birds <- taxize_nexml(birds, "NCBI")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{lcapply}
\alias{lcapply}
\title{Compact list then lapply}
\usage{
lcapply(X, ...)
}
\arguments{
\item{X}{the list object}

\item{...}{remaining arguments to \code{lapply()}}
}
\description{
Compacts the list (i.e., removes NULL objects), then calls \code{\link[base:lapply]{lapply()}}
on the result with the remaining parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_basic_meta.R
\name{add_basic_meta}
\alias{add_basic_meta}
\title{Add basic metadata}
\usage{
add_basic_meta(
  title = NULL,
  description = NULL,
  creator = Sys.getenv("USER"),
  pubdate = NULL,
  rights = "CC0",
  publisher = NULL,
  citation = NULL,
  nexml = new("nexml")
)
}
\arguments{
\item{title}{A title for the dataset}

\item{description}{a description of the dataset}

\item{creator}{name of the data creator.  Can be a string or R person object}

\item{pubdate}{publication date.  Default is current date.}

\item{rights}{the intellectual property rights associated with the data.
The default is Creative Commons Zero (CC0) public domain declaration,
compatible with all other licenses and appropriate for deposition
into the Dryad or figshare repositories.  CC0 is also recommended by the Panton Principles.
Alternatively, any other plain text string can be added and will be provided as the content
attribute to the dc:rights property.}

\item{publisher}{the publisher of the dataset. Usually where a user may go to find the canonical
copy of the dataset: could be a repository, journal, or academic institution.}

\item{citation}{a citation associated with the data.  Usually an academic journal
article that indicates how the data should be cited in an academic context.  Multiple citations
can be included here.
citation can be a plain text object, but is preferably an R \code{citation} or \code{bibentry} object (which
can include multiple citations.  See examples}

\item{nexml}{a nexml object to which metadata should be added.  A new
nexml object will be created if none exists.}
}
\value{
an updated nexml object
}
\description{
adds Dublin Core metadata elements to (top-level) nexml
}
\details{
\code{add_basic_meta()} is just a wrapper for \code{\link{add_meta}} to make it easy to
provide generic metadata without explicitly providing the namespace.  For instance,
\code{add_basic_meta(title="My title", description="a description")} is identical to:
\code{add_meta(list(meta("dc:title", "My title"), meta("dc:description", "a description")))}
Most function arguments are mapped directly to the Dublin Core terms
of the same name, with the exception of \code{rights}, which by default maps
to the Creative Commons namespace when using CC0 license.
}
\examples{
nex <- add_basic_meta(title = "My test title",
             description = "A description of my test",
             creator = "Carl Boettiger <cboettig@gmail.com>",
             publisher = "unpublished data",
             pubdate = "2012-04-01")

 ## Adding citation to an R package:
 nexml <- add_basic_meta(citation=citation("ape"))
\dontrun{
 ## Use knitcitations package to add a citation by DOI:
 library(knitcitations)
 nexml <- add_basic_meta(citation = bib_metadata("10.2307/2408428"))
 }
}
\seealso{
\code{\link{add_trees}} \code{\link{add_characters}} \code{\link{add_meta}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nexml_get.R
\name{nexml_get}
\alias{nexml_get}
\alias{get_item}
\title{Get the desired element from the nexml object}
\usage{
nexml_get(
  nexml,
  element = c("trees", "trees_list", "flat_trees", "metadata", "otu", "taxa",
    "characters", "characters_list", "namespaces"),
  ...
)
}
\arguments{
\item{nexml}{a nexml object (from read_nexml)}

\item{element}{the kind of object desired, see details.}

\item{...}{additional arguments, if applicable to certain elements}
}
\value{
return type depends on the element requested.  See details.
}
\description{
Get the desired element from the nexml object
}
\details{
\itemize{
\item{"tree"}{ an ape::phylo tree, if only one tree is represented.  Otherwise returns a list of lists of multiphylo trees.  To consistently receive the list of lists format (preserving the hierarchical nature of the nexml), use \code{trees} instead.}
\item{"trees"}{ returns a list of lists of multiphylo trees, even if all trees are in the same \code{trees} node (and hence the outer list will be of length 1) or if there is only a single tree (and hence the inner list will also be of length 1.  This ensures a consistent return type regardless of the number of trees present in the nexml file, and also preserves any hierarchy/grouping of trees.  }
\item{"flat_trees"}{ a multiPhylo object (list of ape::phylo objects) Note that this method collapses any hierarchical structure that may have been present as multiple \code{trees} nodes in the original nexml (though such a feature is rarely used).  To preserve that structure, use \code{trees} instead.}
\item{"metadata"}{Get metadata from the specified level (default is top/nexml level) }
\item{"otu"}{ returns a named character vector containing all available metadata.  names indicate \code{property} (or \code{rel} in the case of links/resourceMeta), while values indicate the \code{content} (or \code{href} for links). }
\item{"taxa"}{ alias for otu }
}
For a slightly cleaner interface, each of these elements is also defined as an S4 method
for a nexml object.  So in place of \code{get_item(nexml, "tree")}, one could use \code{get_tree(nexml)},
and so forth for each element type.
}
\examples{
comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
nex <- nexml_read(comp_analysis)
nexml_get(nex, "trees")
nexml_get(nex, "characters_list")
}
\seealso{
\code{\link{get_trees}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta.R
\name{slot,ResourceMeta-method}
\alias{slot,ResourceMeta-method}
\alias{slot-ResourceMeta}
\alias{slot<-,ResourceMeta-method}
\title{Access or set slot of S4 object}
\usage{
\S4method{slot}{ResourceMeta}(object, name)

\S4method{slot}{ResourceMeta}(object, name) <- value
}
\arguments{
\item{object}{the object}

\item{name}{name of the slot}

\item{value}{the new value}
}
\description{
See \code{\link[methods:slot]{methods::slot()}}. This version allows using "property" consistently
for both LiteralMeta and ResourceMeta (which internally uses "rel" because
RDFa does), which is easier to program. It also allows using "meta"
as an alias for "children" for ResourceMeta, to be consistent with the
corresponding slot for instances of \code{Annotated}.
}
\seealso{
\code{\link[methods:slot]{methods::slot()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{get_characters_list}
\alias{get_characters_list}
\title{Extract the character matrix}
\usage{
get_characters_list(nexml, rownames_as_col = FALSE)
}
\arguments{
\item{nexml}{nexml object (e.g. from read.nexml)}

\item{rownames_as_col}{option to return character matrix rownames
(with taxon ids) as it's own column in the data.frame. Default is FALSE
for compatibility with geiger and similar packages.}
}
\value{
the list of taxa
}
\description{
Extract the character matrix
}
\examples{
comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
nex <- nexml_read(comp_analysis)
get_characters_list(nex)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_trees.R
\name{get_trees_list}
\alias{get_trees_list}
\title{extract all phylogenetic trees in ape format}
\usage{
get_trees_list(nexml)
}
\arguments{
\item{nexml}{a representation of the nexml object from
which the data is to be retrieved}
}
\value{
returns a list of lists of multiphylo trees, even if all trees
are in the same \code{trees} node (and hence the outer list will be of length
\enumerate{
\item or if there is only a single tree (and hence the inner list will also
be of length 1.  This ensures a consistent return type regardless of
the number of trees present in the nexml file, and also preserves any
grouping of trees.
}
}
\description{
extract all phylogenetic trees in ape format
}
\examples{
comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
nex <- nexml_read(comp_analysis)
get_trees_list(nex)
}
\seealso{
\code{\link{get_trees}} \code{\link{get_flat_trees}} \code{\link{get_item}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_trees.R
\name{flatten_multiphylo}
\alias{flatten_multiphylo}
\title{Flatten a multiphylo object}
\usage{
flatten_multiphylo(object)
}
\arguments{
\item{object}{a list of multiphylo objects}
}
\description{
Flatten a multiphylo object
}
\details{
NeXML has the concept of multiple \verb{<trees>} nodes, each with multiple child \verb{<tree>} nodes.
This maps naturally to a list of multiphylo  objects.  Sometimes
this hierarchy conveys important structural information, so it is not discarded by default.
Occasionally it is useful to flatten the structure though, hence this function.  Note that this
discards the original structure, and the nexml file must be parsed again to recover it.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S4-utils.R
\name{.sigLabel}
\alias{.sigLabel}
\title{Create a label for a method signature}
\usage{
.sigLabel(signature)
}
\arguments{
\item{signature}{the signature for which to create a label, as a vector
or list of strings, or as an instance of \code{signature}.}
}
\value{
a character string
}
\description{
Creates a label for a signature mirroring the result of \code{.sigLabel()}
in the \code{methods} package, which unfortunately does not export the function.
This is needed, for example, for the \code{excluded} slot in the
\code{MethodWithNext} class.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nexml_publish.R
\name{nexml_figshare}
\alias{nexml_figshare}
\title{publish nexml to figshare}
\usage{
nexml_figshare(
  nexml,
  file = "nexml.xml",
  categories = "Evolutionary Biology",
  tags = list("phylogeny", "NeXML"),
  visibility = c("public", "private", "draft"),
  id = NULL,
  ...
)
}
\arguments{
\item{nexml}{a nexml object (or file path to a nexml file)}

\item{file}{The filename desired for the object, if nexml is not already a file.
if the first argument is already a path, this value is ignored.}

\item{categories}{The figshare categories, must match available set. see \code{fs_add_categories}}

\item{tags}{Any keyword tags you want to add to the data.}

\item{visibility}{whether the results should be published (public), or kept private,
or kept as a draft for further editing before publication.  (New versions can be updated,
but any former versions that was once made public will always be archived and cannot be removed).}

\item{id}{an existing figshare id (e.g. from fs_create), to which this file can be appended.}

\item{...}{additional arguments}
}
\value{
the figshare id of the object
}
\description{
publish nexml to figshare
}
\examples{
\dontrun{
data(bird.orders)
birds <- add_trees(bird.orders)
doi <- nexml_figshare(birds, visibility = "public", repository="figshare")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_characters.R
\name{add_characters}
\alias{add_characters}
\title{Add character data to a nexml object}
\usage{
add_characters(x, nexml = new("nexml"), append_to_existing_otus = FALSE)
}
\arguments{
\item{x}{character data, in which character traits labels are column names
and taxon labels are row names.  x can be in matrix or data.frame
format.}

\item{nexml}{a nexml object, if appending character table to an existing
nexml object.  If omitted will initiate a new nexml object.}

\item{append_to_existing_otus}{logical. If TRUE, will add any new taxa
(taxa not matching any existing otus block) to the existing (first)
otus block.  Otherwise (default), a new otus block is created, even
though it may contain duplicate taxa to those already present.  While
FALSE is the safe option, TRUE may be appropriate when building nexml
files from scratch with both characters and trees.}
}
\description{
Add character data to a nexml object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta.R
\name{c,meta-method}
\alias{c,meta-method}
\alias{c-meta}
\alias{c,ListOfmeta-method}
\alias{c-ListOfmeta}
\title{Concatenate meta elements into a ListOfmeta}
\usage{
\S4method{c}{meta}(x, ..., recursive = TRUE)

\S4method{c}{ListOfmeta}(x, ..., recursive = TRUE)
}
\arguments{
\item{x, ...}{\code{meta} and \code{ListOfmeta} elements to be concatenated, see \code{\link{meta}}}

\item{recursive}{logical, if 'recursive=TRUE', the function recursively
descends through lists and combines their elements into a flat vector.
This method does not support \code{recursive=FALSE}, use \link[base:list]{list}
instead.}
}
\value{
a ListOfmeta object containing a flat list of meta elements.
}
\description{
Concatenate meta elements into a ListOfmeta

Concatenate ListOfmeta elements into a flat ListOfmeta
}
\examples{
c(meta(content="example", property="dc:title"),
  meta(content="Carl", property="dc:creator"))
metalist <- c(meta(content="example", property="dc:title"),
              meta(content="Carl", property="dc:creator"))
out <- c(metalist, metalist) 
out <- c(metalist, meta(content="a", property="b")) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{coalesce_}
\alias{coalesce_}
\title{Front-end to dplyr::coalesce to deal with NULL vectors}
\usage{
coalesce_(...)
}
\arguments{
\item{...}{the vectors to coalesce on NA}
}
\value{
a vector of the same type and length as the last argument
}
\description{
Replaces any NULL argument with a vector of \code{NA}, and casts every vector
to the same type as the last vector. After that, calls \code{\link[dplyr:coalesce]{dplyr::coalesce()}}.
}
\seealso{
\code{\link[dplyr:coalesce]{dplyr::coalesce()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_level.R
\name{get_level}
\alias{get_level}
\title{get_level}
\usage{
get_level(nex, level)
}
\arguments{
\item{nex}{a nexml object}

\item{level}{a character vector indicating the class of node, see details}
}
\value{
Returns the attributes of specified class of nodes as a data.frame
}
\description{
get a data.frame of attribute values of a given node
}
\details{
level should be a character vector giving the path to the specified node
group.  For instance, \code{otus}, \code{characters}, and \code{trees} are top-level blocks (e.g.
child nodes of the root nexml block), and can be specified directly.  To get metadata
for all "char" elements from all characters blocks, you must specify that \code{char} nodes
are child nodes to \code{character} nodes: e.g. \code{get_level(nex, "characters/char")},
or similarly for states: \code{get_level(nex, characters/states)}.

The return object is a data frame whose columns are the attribute names of the elements
specified. The column names match the attribute names except for "id" attribute, for which the column
is renamed using the node itself. (Thus \verb{<otus id="os2">} would be rendered in a data.frame with column
called "otus" instead of "id"). Additional columns are
added for each parent element in the path; e.g. \code{get_level(nex, "otus/otu")} would include a column
named "otus" with the id of each otus block.  Even though the method always returns the data frame
for all matching nodes in all blocks, these ids let you see which otu values came from which
otus block.  This is identical to the function call \code{get_taxa()}.
Similarly, \code{get_level(nex, "otus/otu/meta")} would return additional columns 'otus' and
also a column, 'otu', with the otu parent ids of each metadata block.  (This is identical to a
function call to \code{get_metadata}).  This makes it easier to join data.frames as well, see examples
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_trees.R
\name{get_flat_trees}
\alias{get_flat_trees}
\title{get_flat_trees}
\usage{
get_flat_trees(nexml)
}
\arguments{
\item{nexml}{a representation of the nexml object from  which the data is to be retrieved}
}
\value{
a multiPhylo object (list of ape::phylo objects).  See details.
}
\description{
extract a single multiPhylo object containing all trees in the nexml
}
\details{
Note that this method collapses any hierarchical structure that may have been present as multiple \code{trees} nodes in the original nexml (though such a feature is rarely used).  To preserve that structure, use \code{\link{get_trees}} instead.
}
\examples{
comp_analysis <- system.file("examples", "comp_analysis.xml", package="RNeXML")
nex <- nexml_read(comp_analysis)
get_flat_trees(nex)
}
\seealso{
\code{\link{get_trees}} \code{\link{get_trees}} \code{\link{get_item}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_trees.R
\name{add_trees}
\alias{add_trees}
\title{add_trees}
\usage{
add_trees(phy, nexml = new("nexml"), append_to_existing_otus = FALSE)
}
\arguments{
\item{phy}{a phylo object, multiPhylo object, or list of
multiPhylo to be added to the nexml}

\item{nexml}{a nexml object to which we should append this phylo.
By default, a new nexml object will be created.}

\item{append_to_existing_otus}{logical, indicating if we should
make a new OTU block (default) or append to the existing one.}
}
\value{
a nexml object containing the phy in nexml format.
}
\description{
add_trees
}
